@@ -1,6 +1,6 @@
 #[cfg(all(
     any(feature = "http", feature = "gateway"),
-    not(any(feature = "rustls_backend_marker", feature = "native_tls_backend_marker"))
+    not(any(feature = "rustls_backend", feature = "native_tls_backend"))
 ))]
 compile_error!(
     "You have the `http` or `gateway` feature enabled, \
