@@ -1,14 +1,12 @@
 use std::collections::HashMap;
 
-use serde_json::Value;
-
+use crate::json;
+use crate::json::prelude::*;
 use crate::model::interactions::application_command::ApplicationCommandPermissionType;
-use crate::utils;
 
 /// A builder for creating several [`ApplicationCommandPermission`].
 ///
 /// [`ApplicationCommandPermission`]: crate::model::interactions::application_command::ApplicationCommandPermission
-/// [`kind`]: Self::kind
 #[derive(Clone, Debug, Default)]
 pub struct CreateApplicationCommandsPermissions(pub Vec<Value>);
 
@@ -33,7 +31,7 @@ impl CreateApplicationCommandsPermissions {
         &mut self,
         application_command: CreateApplicationCommandPermissions,
     ) -> &mut Self {
-        let new_data = Value::Object(utils::hashmap_to_json_map(application_command.0));
+        let new_data = Value::from(json::hashmap_to_json_map(application_command.0));
 
         self.0.push(new_data);
 
@@ -45,10 +43,8 @@ impl CreateApplicationCommandsPermissions {
         &mut self,
         application_commands: Vec<CreateApplicationCommandPermissions>,
     ) -> &mut Self {
-        let new_application_commands = application_commands
-            .into_iter()
-            .map(|f| Value::Object(utils::hashmap_to_json_map(f.0)))
-            .collect::<Vec<Value>>();
+        let new_application_commands =
+            application_commands.into_iter().map(|f| Value::from(json::hashmap_to_json_map(f.0)));
 
         for application_command in new_application_commands {
             self.0.push(application_command);
@@ -60,7 +56,6 @@ impl CreateApplicationCommandsPermissions {
 /// A builder for creating an [`ApplicationCommandPermission`].
 ///
 /// [`ApplicationCommandPermission`]: crate::model::interactions::application_command::ApplicationCommandPermission
-/// [`kind`]: Self::kind
 #[derive(Clone, Debug, Default)]
 pub struct CreateApplicationCommandPermissions(pub HashMap<&'static str, Value>);
 
@@ -69,7 +64,7 @@ impl CreateApplicationCommandPermissions {
     ///
     /// [`CommandId`]: crate::model::id::CommandId
     pub fn id(&mut self, application_command_id: u64) -> &mut Self {
-        self.0.insert("id", Value::String(application_command_id.to_string()));
+        self.0.insert("id", Value::from(application_command_id.to_string()));
         self
     }
 
@@ -93,12 +88,13 @@ impl CreateApplicationCommandPermissions {
         &mut self,
         permission: CreateApplicationCommandPermissionData,
     ) -> &mut Self {
-        let new_data = utils::hashmap_to_json_map(permission.0);
-        let permissions = self.0.entry("permissions").or_insert_with(|| Value::Array(Vec::new()));
+        let new_data = json::hashmap_to_json_map(permission.0);
+        let permissions =
+            self.0.entry("permissions").or_insert_with(|| Value::from(Vec::<Value>::new()));
 
         let permissions_array = permissions.as_array_mut().expect("Must be an array");
 
-        permissions_array.push(Value::Object(new_data));
+        permissions_array.push(Value::from(new_data));
 
         self
     }
@@ -110,10 +106,10 @@ impl CreateApplicationCommandPermissions {
     ) -> &mut Self {
         let new_permissions = permissions
             .into_iter()
-            .map(|f| Value::Object(utils::hashmap_to_json_map(f.0)))
+            .map(|f| Value::from(json::hashmap_to_json_map(f.0)))
             .collect::<Vec<Value>>();
 
-        self.0.insert("permissions", Value::Array(new_permissions));
+        self.0.insert("permissions", Value::from(new_permissions));
 
         self
     }
@@ -122,7 +118,6 @@ impl CreateApplicationCommandPermissions {
 /// A builder for creating several [`ApplicationCommandPermissionData`].
 ///
 /// [`ApplicationCommandPermissionData`]: crate::model::interactions::application_command::ApplicationCommandPermissionData
-/// [`kind`]: Self::kind
 #[derive(Clone, Debug, Default)]
 pub struct CreateApplicationCommandPermissionsData(pub HashMap<&'static str, Value>);
 
@@ -147,12 +142,13 @@ impl CreateApplicationCommandPermissionsData {
         &mut self,
         permission: CreateApplicationCommandPermissionData,
     ) -> &mut Self {
-        let new_data = utils::hashmap_to_json_map(permission.0);
-        let permissions = self.0.entry("permissions").or_insert_with(|| Value::Array(Vec::new()));
+        let new_data = json::hashmap_to_json_map(permission.0);
+        let permissions =
+            self.0.entry("permissions").or_insert_with(|| Value::from(Vec::<Value>::new()));
 
         let permissions_array = permissions.as_array_mut().expect("Must be an array");
 
-        permissions_array.push(Value::Object(new_data));
+        permissions_array.push(Value::from(new_data));
 
         self
     }
@@ -164,10 +160,10 @@ impl CreateApplicationCommandPermissionsData {
     ) -> &mut Self {
         let new_permissions = permissions
             .into_iter()
-            .map(|f| Value::Object(utils::hashmap_to_json_map(f.0)))
+            .map(|f| Value::from(json::hashmap_to_json_map(f.0)))
             .collect::<Vec<Value>>();
 
-        self.0.insert("permissions", Value::Array(new_permissions));
+        self.0.insert("permissions", Value::from(new_permissions));
 
         self
     }
@@ -178,7 +174,6 @@ impl CreateApplicationCommandPermissionsData {
 /// All fields are required.
 ///
 /// [`ApplicationCommandPermissionData`]: crate::model::interactions::application_command::ApplicationCommandPermissionData
-/// [`kind`]: Self::kind
 #[derive(Clone, Debug, Default)]
 pub struct CreateApplicationCommandPermissionData(pub HashMap<&'static str, Value>);
 
@@ -187,7 +182,7 @@ impl CreateApplicationCommandPermissionData {
     ///
     /// [`ApplicationCommandPermissionData`]: crate::model::interactions::application_command::ApplicationCommandPermissionData
     pub fn kind(&mut self, kind: ApplicationCommandPermissionType) -> &mut Self {
-        self.0.insert("type", Value::Number(serde_json::Number::from(kind as u8)));
+        self.0.insert("type", from_number(kind as u8));
         self
     }
 
@@ -195,7 +190,7 @@ impl CreateApplicationCommandPermissionData {
     ///
     /// [`ApplicationCommandPermissionData`]: crate::model::interactions::application_command::ApplicationCommandPermissionData
     pub fn id(&mut self, id: u64) -> &mut Self {
-        self.0.insert("id", Value::String(id.to_string()));
+        self.0.insert("id", Value::from(id.to_string()));
         self
     }
 
@@ -206,7 +201,7 @@ impl CreateApplicationCommandPermissionData {
     ///
     /// [`ApplicationCommandPermissionData`]: crate::model::interactions::application_command::ApplicationCommandPermissionData
     pub fn permission(&mut self, permission: bool) -> &mut Self {
-        self.0.insert("permission", Value::Bool(permission));
+        self.0.insert("permission", Value::from(permission));
         self
     }
 }
