@@ -15,14 +15,10 @@
 //! [here]: https://discord.com/developers/docs/resources/channel#embed-object
 
 use std::collections::HashMap;
-use std::fmt::Display;
 
-use chrono::{DateTime, TimeZone};
-use serde_json::{json, Value};
-
-use crate::internal::prelude::*;
+use crate::json::{self, from_number, json, Value};
 use crate::model::channel::Embed;
-use crate::utils;
+use crate::model::Timestamp;
 #[cfg(feature = "utils")]
 use crate::utils::Colour;
 
@@ -56,9 +52,9 @@ impl CreateEmbed {
 
     /// Set the author of the embed.
     pub fn set_author(&mut self, author: CreateEmbedAuthor) -> &mut Self {
-        let map = utils::hashmap_to_json_map(author.0);
+        let map = json::hashmap_to_json_map(author.0);
 
-        self.0.insert("author", Value::Object(map));
+        self.0.insert("author", Value::from(map));
         self
     }
 
@@ -82,7 +78,7 @@ impl CreateEmbed {
 
     #[cfg(feature = "utils")]
     fn _colour(&mut self, colour: Colour) {
-        self.0.insert("color", Value::Number(Number::from(u64::from(colour.0))));
+        self.0.insert("color", from_number(u64::from(colour.0)));
     }
 
     /// Set the colour of the left-hand side of the embed.
@@ -98,7 +94,7 @@ impl CreateEmbed {
     /// Set the colour of the left-hand side of the embed.
     #[cfg(not(feature = "utils"))]
     pub fn colour(&mut self, colour: u32) -> &mut Self {
-        self.0.insert("color", Value::Number(Number::from(colour)));
+        self.0.insert("color", from_number(colour));
         self
     }
 
@@ -107,7 +103,7 @@ impl CreateEmbed {
     /// **Note**: This can't be longer than 4096 characters.
     #[inline]
     pub fn description<D: ToString>(&mut self, description: D) -> &mut Self {
-        self.0.insert("description", Value::String(description.to_string()));
+        self.0.insert("description", Value::from(description.to_string()));
         self
     }
 
@@ -128,7 +124,7 @@ impl CreateEmbed {
 
     fn _field(&mut self, name: String, value: String, inline: bool) {
         {
-            let entry = self.0.entry("fields").or_insert_with(|| Value::Array(vec![]));
+            let entry = self.0.entry("fields").or_insert_with(|| Value::from(Vec::<Value>::new()));
 
             if let Value::Array(ref mut inner) = *entry {
                 inner.push(json!({
@@ -172,9 +168,9 @@ impl CreateEmbed {
     /// Set the footer of the embed.
     pub fn set_footer(&mut self, create_embed_footer: CreateEmbedFooter) -> &mut Self {
         let footer = create_embed_footer.0;
-        let map = utils::hashmap_to_json_map(footer);
+        let map = json::hashmap_to_json_map(footer);
 
-        self.0.insert("footer", Value::Object(map));
+        self.0.insert("footer", Value::from(map));
         self
     }
 
@@ -234,16 +230,15 @@ impl CreateEmbed {
     ///             let _ = msg
     ///                 .channel_id
     ///                 .send_message(&context.http, |m| {
-    ///                     m.embed(|e| e.title("hello").timestamp("2004-06-08T16:04:23"));
-    ///
-    ///                     m
+    ///                     m.embed(|e| e.title("hello").timestamp("2004-06-08T16:04:23"))
     ///                 })
     ///                 .await;
     ///         }
     ///     }
     /// }
     ///
-    /// let mut client = Client::builder("token").event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder("token", GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// client.start().await?;
     /// #     Ok(())
@@ -265,7 +260,8 @@ impl CreateEmbed {
     ///
     /// #[serenity::async_trait]
     /// impl EventHandler for Handler {
-    ///     async fn guild_member_addition(&self, context: Context, guild_id: GuildId, member: Member) {
+    ///     async fn guild_member_addition(&self, context: Context, member: Member) {
+    ///         let guild_id = member.guild_id;
     ///         if let Ok(guild) = guild_id.to_partial_guild(&context).await {
     ///             let channels = guild.channels(&context).await.unwrap();
     ///
@@ -277,14 +273,11 @@ impl CreateEmbed {
     ///                 let _ = channel
     ///                     .send_message(&context, |m| {
     ///                         m.embed(|e| {
-    ///                             e.author(|a| a.icon_url(&user.face()).name(&user.name));
-    ///                             e.title("Member Join");
-    ///
     ///                             if let Some(ref joined_at) = member.joined_at {
     ///                                 e.timestamp(joined_at);
     ///                             }
-    ///
-    ///                             e
+    ///                             e.author(|a| a.icon_url(&user.face()).name(&user.name))
+    ///                                 .title("Member Join")
     ///                         })
     ///                     })
     ///                     .await;
@@ -293,7 +286,8 @@ impl CreateEmbed {
     ///     }
     /// }
     ///
-    /// let mut client = Client::builder("token").event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder("token", GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// client.start().await?;
     /// #     Ok(())
@@ -306,20 +300,20 @@ impl CreateEmbed {
     }
 
     fn _timestamp(&mut self, timestamp: Timestamp) {
-        self.0.insert("timestamp", Value::String(timestamp.ts));
+        self.0.insert("timestamp", Value::from(timestamp.to_string()));
     }
 
     /// Set the title of the embed.
     #[inline]
     pub fn title<D: ToString>(&mut self, title: D) -> &mut Self {
-        self.0.insert("title", Value::String(title.to_string()));
+        self.0.insert("title", Value::from(title.to_string()));
         self
     }
 
     /// Set the URL to direct to when clicking on the title.
     #[inline]
     pub fn url<S: ToString>(&mut self, url: S) -> &mut Self {
-        self.0.insert("url", Value::String(url.to_string()));
+        self.0.insert("url", Value::from(url.to_string()));
         self
     }
 
@@ -343,7 +337,7 @@ impl Default for CreateEmbed {
     /// Creates a builder with default values, setting the `type` to `rich`.
     fn default() -> CreateEmbed {
         let mut map = HashMap::new();
-        map.insert("type", Value::String("rich".to_string()));
+        map.insert("type", Value::from("rich".to_string()));
 
         CreateEmbed(map)
     }
@@ -355,7 +349,10 @@ impl From<Embed> for CreateEmbed {
     /// Some values - such as Proxy URLs - are not preserved.
     fn from(embed: Embed) -> Self {
         let mut b = CreateEmbed::default();
-        b.colour(embed.colour);
+
+        if let Some(colour) = embed.colour {
+            b.colour(colour);
+        }
 
         if let Some(author) = embed.author {
             b.author(move |a| {
@@ -403,13 +400,10 @@ impl From<Embed> for CreateEmbed {
 
         if let Some(footer) = embed.footer {
             b.footer(move |f| {
-                f.text(&footer.text);
-
                 if let Some(icon_url) = footer.icon_url {
                     f.icon_url(&icon_url);
                 }
-
-                f
+                f.text(&footer.text)
             });
         }
 
@@ -429,19 +423,19 @@ pub struct CreateEmbedAuthor(pub HashMap<&'static str, Value>);
 impl CreateEmbedAuthor {
     /// Set the URL of the author's icon.
     pub fn icon_url<S: ToString>(&mut self, icon_url: S) -> &mut Self {
-        self.0.insert("icon_url", Value::String(icon_url.to_string()));
+        self.0.insert("icon_url", Value::from(icon_url.to_string()));
         self
     }
 
     /// Set the author's name.
     pub fn name<S: ToString>(&mut self, name: S) -> &mut Self {
-        self.0.insert("name", Value::String(name.to_string()));
+        self.0.insert("name", Value::from(name.to_string()));
         self
     }
 
     /// Set the author's URL.
     pub fn url<S: ToString>(&mut self, url: S) -> &mut Self {
-        self.0.insert("url", Value::String(url.to_string()));
+        self.0.insert("url", Value::from(url.to_string()));
         self
     }
 }
@@ -458,75 +452,29 @@ pub struct CreateEmbedFooter(pub HashMap<&'static str, Value>);
 impl CreateEmbedFooter {
     /// Set the icon URL's value. This only supports HTTP(S).
     pub fn icon_url<S: ToString>(&mut self, icon_url: S) -> &mut Self {
-        self.0.insert("icon_url", Value::String(icon_url.to_string()));
+        self.0.insert("icon_url", Value::from(icon_url.to_string()));
         self
     }
 
     /// Set the footer's text.
     pub fn text<S: ToString>(&mut self, text: S) -> &mut Self {
-        self.0.insert("text", Value::String(text.to_string()));
+        self.0.insert("text", Value::from(text.to_string()));
         self
     }
 }
 
-#[derive(Clone, Debug)]
-pub struct Timestamp {
-    pub ts: String,
-}
-
-impl From<String> for Timestamp {
-    fn from(ts: String) -> Self {
-        Self {
-            ts,
-        }
-    }
-}
-
-impl<'a> From<&'a str> for Timestamp {
-    fn from(ts: &'a str) -> Self {
-        Self {
-            ts: ts.to_string(),
-        }
-    }
-}
-
-impl<Tz: TimeZone> From<DateTime<Tz>> for Timestamp
-where
-    Tz::Offset: Display,
-{
-    fn from(dt: DateTime<Tz>) -> Self {
-        Self {
-            ts: dt.to_rfc3339(),
-        }
-    }
-}
-
-impl<'a, Tz: TimeZone> From<&'a DateTime<Tz>> for Timestamp
-where
-    Tz::Offset: Display,
-{
-    fn from(dt: &'a DateTime<Tz>) -> Self {
-        Self {
-            ts: dt.to_rfc3339(),
-        }
-    }
-}
-
 #[cfg(test)]
 mod test {
-    use serde_json::{json, Value};
-
     use super::CreateEmbed;
-    use crate::{
-        model::channel::{Embed, EmbedField, EmbedFooter, EmbedImage, EmbedVideo},
-        utils::{self, Colour},
-    };
+    use crate::json::{self, json, Value};
+    use crate::model::channel::{Embed, EmbedField, EmbedFooter, EmbedImage, EmbedVideo};
+    use crate::utils::Colour;
 
     #[test]
     fn test_from_embed() {
         let embed = Embed {
             author: None,
-            colour: Colour::new(0xFF0011),
+            colour: Some(Colour::new(0xFF0011)),
             description: Some("This is a test description".to_string()),
             fields: vec![
                 EmbedField {
@@ -541,26 +489,27 @@ mod test {
                 },
             ],
             footer: Some(EmbedFooter {
+                text: "This is a hakase footer".to_string(),
                 icon_url: Some("https://i.imgur.com/XfWpfCV.gif".to_string()),
                 proxy_icon_url: None,
-                text: "This is a hakase footer".to_string(),
             }),
             image: Some(EmbedImage {
-                height: 213,
-                proxy_url: "a".to_string(),
                 url: "https://i.imgur.com/XfWpfCV.gif".to_string(),
-                width: 224,
+                proxy_url: Some("a".to_string()),
+                height: Some(213),
+                width: Some(224),
             }),
-            kind: "rich".to_string(),
+            kind: Some("rich".to_string()),
             provider: None,
             thumbnail: None,
             timestamp: None,
             title: Some("hakase".to_string()),
             url: Some("https://i.imgur.com/XfWpfCV.gif".to_string()),
             video: Some(EmbedVideo {
-                height: 213,
                 url: "https://i.imgur.com/XfWpfCV.mp4".to_string(),
-                width: 224,
+                proxy_url: Some("a".to_string()),
+                height: Some(213),
+                width: Some(224),
             }),
         };
 
@@ -571,7 +520,7 @@ mod test {
         builder.title("still a hakase");
         builder.url("https://i.imgur.com/XfWpfCV.gif");
 
-        let built = Value::Object(utils::hashmap_to_json_map(builder.0));
+        let built = Value::from(json::hashmap_to_json_map(builder.0));
 
         let obj = json!({
             "color": 0xFF0011,
