@@ -1,8 +1,6 @@
 use std::collections::HashMap;
 
-use serde_json::Value;
-
-use crate::internal::prelude::*;
+use crate::json::{from_number, Value, NULL};
 use crate::model::id::{ApplicationId, UserId};
 use crate::model::invite::InviteTargetType;
 
@@ -28,7 +26,7 @@ use crate::model::invite::InviteTargetType;
 /// impl EventHandler for Handler {
 ///     async fn message(&self, context: Context, msg: Message) {
 ///         if msg.content == "!createinvite" {
-///             let channel = match context.cache.guild_channel(msg.channel_id).await {
+///             let channel = match context.cache.guild_channel(msg.channel_id) {
 ///                 Some(channel) => channel,
 ///                 None => {
 ///                     let _ = msg.channel_id.say(&context, "Error creating invite").await;
@@ -59,7 +57,8 @@ use crate::model::invite::InviteTargetType;
 ///     }
 /// }
 ///
-/// let mut client = Client::builder("token").event_handler(Handler).await?;
+/// let mut client =
+///     Client::builder("token", GatewayIntents::default()).event_handler(Handler).await?;
 ///
 /// client.start().await?;
 /// #     Ok(())
@@ -92,17 +91,14 @@ impl CreateInvite {
     /// # #[cfg(all(feature = "cache", feature = "client", feature = "framework", feature = "http"))]
     /// # #[command]
     /// # async fn example(context: &Context) -> CommandResult {
-    /// #     let channel = context.cache.guild_channel(81384788765712384).await.unwrap();
+    /// #     let channel = context.cache.guild_channel(81384788765712384).unwrap();
     /// #
-    /// let invite = channel.create_invite(context, |i| {
-    ///     i.max_age(3600)
-    /// })
-    /// .await?;
+    /// let invite = channel.create_invite(context, |i| i.max_age(3600)).await?;
     /// #     Ok(())
     /// # }
     /// ```
     pub fn max_age(&mut self, max_age: u64) -> &mut Self {
-        self.0.insert("max_age", Value::Number(Number::from(max_age)));
+        self.0.insert("max_age", from_number(max_age));
         self
     }
 
@@ -126,17 +122,14 @@ impl CreateInvite {
     /// # #[cfg(all(feature = "cache", feature = "client", feature = "framework", feature = "http"))]
     /// # #[command]
     /// # async fn example(context: &Context) -> CommandResult {
-    /// #     let channel = context.cache.guild_channel(81384788765712384).await.unwrap();
+    /// #     let channel = context.cache.guild_channel(81384788765712384).unwrap();
     /// #
-    /// let invite = channel.create_invite(context, |i| {
-    ///     i.max_uses(5)
-    /// })
-    /// .await?;
+    /// let invite = channel.create_invite(context, |i| i.max_uses(5)).await?;
     /// #     Ok(())
     /// # }
     /// ```
     pub fn max_uses(&mut self, max_uses: u64) -> &mut Self {
-        self.0.insert("max_uses", Value::Number(Number::from(max_uses)));
+        self.0.insert("max_uses", from_number(max_uses));
         self
     }
 
@@ -158,19 +151,16 @@ impl CreateInvite {
     /// # #[cfg(all(feature = "cache", feature = "client", feature = "framework", feature = "http"))]
     /// # #[command]
     /// # async fn example(context: &Context) -> CommandResult {
-    /// #     let channel = context.cache.guild_channel(81384788765712384).await.unwrap();
+    /// #     let channel = context.cache.guild_channel(81384788765712384).unwrap();
     /// #
-    /// let invite = channel.create_invite(context, |i| {
-    ///     i.temporary(true)
-    /// })
-    /// .await?;
+    /// let invite = channel.create_invite(context, |i| i.temporary(true)).await?;
     /// #     Ok(())
     /// # }
     /// #
     /// # fn main() {}
     /// ```
     pub fn temporary(&mut self, temporary: bool) -> &mut Self {
-        self.0.insert("temporary", Value::Bool(temporary));
+        self.0.insert("temporary", Value::from(temporary));
         self
     }
 
@@ -192,23 +182,20 @@ impl CreateInvite {
     /// # #[cfg(all(feature = "cache", feature = "client", feature = "framework", feature = "http"))]
     /// # #[command]
     /// # async fn example(context: &Context) -> CommandResult {
-    /// #     let channel = context.cache.guild_channel(81384788765712384).await.unwrap();
+    /// #     let channel = context.cache.guild_channel(81384788765712384).unwrap();
     /// #
-    /// let invite = channel.create_invite(context, |i| {
-    ///     i.unique(true)
-    /// })
-    /// .await?;
+    /// let invite = channel.create_invite(context, |i| i.unique(true)).await?;
     /// #     Ok(())
     /// # }
     /// ```
     pub fn unique(&mut self, unique: bool) -> &mut Self {
-        self.0.insert("unique", Value::Bool(unique));
+        self.0.insert("unique", Value::from(unique));
         self
     }
 
     /// The type of target for this voice channel invite.
     pub fn target_type(&mut self, target_type: InviteTargetType) -> &mut Self {
-        self.0.insert("target_type", Value::Number(Number::from(target_type as u8)));
+        self.0.insert("target_type", from_number(target_type as u8));
         self
     }
 
@@ -216,7 +203,7 @@ impl CreateInvite {
     /// `Stream`
     /// The user must be streaming in the channel.
     pub fn target_user_id(&mut self, target_user_id: UserId) -> &mut Self {
-        self.0.insert("target_user_id", Value::Number(Number::from(target_user_id.0)));
+        self.0.insert("target_user_id", from_number(target_user_id.0));
         self
     }
 
@@ -235,8 +222,7 @@ impl CreateInvite {
     /// poker: `755827207812677713`
     /// chess: `832012774040141894`
     pub fn target_application_id(&mut self, target_application_id: ApplicationId) -> &mut Self {
-        self.0
-            .insert("target_application_id", Value::Number(Number::from(target_application_id.0)));
+        self.0.insert("target_application_id", from_number(target_application_id.0));
         self
     }
 }
@@ -255,7 +241,7 @@ impl Default for CreateInvite {
     /// ```
     fn default() -> CreateInvite {
         let mut map = HashMap::new();
-        map.insert("validate", Value::Null);
+        map.insert("validate", NULL);
 
         CreateInvite(map)
     }
