@@ -1,8 +1,6 @@
 use std::collections::HashMap;
 
-use serde_json::{json, Value};
-
-use crate::internal::prelude::*;
+use crate::json::{from_number, json, Value, NULL};
 use crate::model::channel::{PermissionOverwrite, PermissionOverwriteType, VideoQualityMode};
 use crate::model::id::ChannelId;
 
@@ -18,7 +16,7 @@ use crate::model::id::ChannelId;
 /// # use serenity::{http::Http, model::id::ChannelId};
 /// #
 /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-/// #     let http = Http::default();
+/// #     let http = Http::new("token");
 /// #     let mut channel = ChannelId(0);
 /// // assuming a channel has already been bound
 /// if let Err(why) = channel.edit(&http, |c| c.name("new name").topic("a test topic")).await {
@@ -40,7 +38,7 @@ impl EditChannel {
     ///
     /// [voice]: crate::model::channel::ChannelType::Voice
     pub fn bitrate(&mut self, bitrate: u64) -> &mut Self {
-        self.0.insert("bitrate", Value::Number(Number::from(bitrate)));
+        self.0.insert("bitrate", from_number(bitrate));
         self
     }
 
@@ -50,7 +48,7 @@ impl EditChannel {
     ///
     /// [voice]: crate::model::channel::ChannelType::Voice
     pub fn video_quality_mode(&mut self, quality: VideoQualityMode) -> &mut Self {
-        self.0.insert("video_quality_mode", Value::Number(Number::from(quality as u8)));
+        self.0.insert("video_quality_mode", from_number(quality as u8));
         self
     }
 
@@ -62,8 +60,8 @@ impl EditChannel {
     /// [voice]: crate::model::channel::ChannelType::Voice
     pub fn voice_region(&mut self, id: Option<String>) -> &mut Self {
         self.0.insert("rtc_region", match id {
-            Some(region) => Value::String(region),
-            None => Value::Null,
+            Some(region) => Value::from(region),
+            None => NULL,
         });
         self
     }
@@ -72,13 +70,13 @@ impl EditChannel {
     ///
     /// Must be between 2 and 100 characters long.
     pub fn name<S: ToString>(&mut self, name: S) -> &mut Self {
-        self.0.insert("name", Value::String(name.to_string()));
+        self.0.insert("name", Value::from(name.to_string()));
         self
     }
 
     /// The position of the channel in the channel list.
     pub fn position(&mut self, position: u64) -> &mut Self {
-        self.0.insert("position", Value::Number(Number::from(position)));
+        self.0.insert("position", from_number(position));
         self
     }
 
@@ -90,7 +88,7 @@ impl EditChannel {
     ///
     /// [text]: crate::model::channel::ChannelType::Text
     pub fn topic<S: ToString>(&mut self, topic: S) -> &mut Self {
-        self.0.insert("topic", Value::String(topic.to_string()));
+        self.0.insert("topic", Value::from(topic.to_string()));
         self
     }
 
@@ -100,7 +98,7 @@ impl EditChannel {
     ///
     /// [text]: crate::model::channel::ChannelType::Text
     pub fn nsfw(&mut self, nsfw: bool) -> &mut Self {
-        self.0.insert("nsfw", Value::Bool(nsfw));
+        self.0.insert("nsfw", Value::from(nsfw));
 
         self
     }
@@ -111,7 +109,7 @@ impl EditChannel {
     ///
     /// [voice]: crate::model::channel::ChannelType::Voice
     pub fn user_limit(&mut self, user_limit: u64) -> &mut Self {
-        self.0.insert("user_limit", Value::Number(Number::from(user_limit)));
+        self.0.insert("user_limit", from_number(user_limit));
         self
     }
 
@@ -129,20 +127,11 @@ impl EditChannel {
 
     fn _category(&mut self, category: Option<ChannelId>) {
         self.0.insert("parent_id", match category {
-            Some(c) => Value::Number(Number::from(c.0)),
-            None => Value::Null,
+            Some(c) => Value::from(c.0),
+            None => NULL,
         });
     }
 
-    /// The seconds a user has to wait before sending another message.
-    ///
-    /// **Note**: Must be between 0 and 21600 seconds (360 minutes or 6 hours).
-    #[deprecated(note = "replaced by `rate_limit_per_user`")]
-    #[inline]
-    pub fn slow_mode_rate(&mut self, seconds: u64) -> &mut Self {
-        self.rate_limit_per_user(seconds)
-    }
-
     /// How many seconds must a user wait before sending another message.
     ///
     /// Bots, or users with the [`MANAGE_MESSAGES`] and/or [`MANAGE_CHANNELS`] permissions are exempt
@@ -154,7 +143,7 @@ impl EditChannel {
     /// [`MANAGE_CHANNELS`]: crate::model::permissions::Permissions::MANAGE_CHANNELS
     #[doc(alias = "slowmode")]
     pub fn rate_limit_per_user(&mut self, seconds: u64) -> &mut Self {
-        self.0.insert("rate_limit_per_user", Value::Number(Number::from(seconds)));
+        self.0.insert("rate_limit_per_user", from_number(seconds));
 
         self
     }
@@ -164,14 +153,14 @@ impl EditChannel {
     ///
     /// # Example
     ///
-    /// Inheriting permissions from an exisiting channel:
+    /// Inheriting permissions from an existing channel:
     ///
     /// ```rust,no_run
     /// # use serenity::{http::Http, model::id::ChannelId};
     /// # use std::sync::Arc;
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// #     let http = Arc::new(Http::default());
+    /// #     let http = Arc::new(Http::new("token"));
     /// #     let mut channel = ChannelId(0);
     /// use serenity::model::channel::{PermissionOverwrite, PermissionOverwriteType};
     /// use serenity::model::id::UserId;
@@ -179,7 +168,7 @@ impl EditChannel {
     ///
     /// // Assuming a channel has already been bound.
     /// let permissions = vec![PermissionOverwrite {
-    ///     allow: Permissions::READ_MESSAGES,
+    ///     allow: Permissions::VIEW_CHANNEL,
     ///     deny: Permissions::SEND_TTS_MESSAGES,
     ///     kind: PermissionOverwriteType::Member(UserId(1234)),
     /// }];
@@ -207,9 +196,9 @@ impl EditChannel {
                     "type": kind,
                 })
             })
-            .collect();
+            .collect::<Vec<_>>();
 
-        self.0.insert("permission_overwrites", Value::Array(overwrites));
+        self.0.insert("permission_overwrites", Value::from(overwrites));
 
         self
     }
