@@ -1,6 +1,7 @@
 use std::collections::HashMap;
 
 use crate::internal::prelude::*;
+use crate::json::{from_number, NULL};
 use crate::model::prelude::*;
 
 /// A builder to optionally edit certain fields of a [`Guild`]. This is meant
@@ -30,15 +31,15 @@ impl EditGuild {
 
     fn _afk_channel(&mut self, channel: Option<ChannelId>) {
         self.0.insert("afk_channel_id", match channel {
-            Some(channel) => Value::Number(Number::from(channel.0)),
-            None => Value::Null,
+            Some(channel) => from_number(channel.0),
+            None => NULL,
         });
     }
 
     /// Set the amount of time a user is to be moved to the AFK channel -
     /// configured via [`Self::afk_channel`] - after being AFK.
     pub fn afk_timeout(&mut self, timeout: u64) -> &mut Self {
-        self.0.insert("afk_timeout", Value::Number(Number::from(timeout)));
+        self.0.insert("afk_timeout", from_number(timeout));
         self
     }
 
@@ -53,7 +54,7 @@ impl EditGuild {
     /// # use serenity::{http::Http, model::id::GuildId};
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// #     let http = Http::default();
+    /// #     let http = Http::new("token");
     /// #     let mut guild = GuildId(0).to_partial_guild(&http).await?;
     /// use serenity::utils;
     ///
@@ -61,22 +62,22 @@ impl EditGuild {
     ///
     /// let base64_icon = utils::read_image("./guild_icon.png")?;
     ///
-    /// guild.edit(&http, |mut g| g.icon(Some(&base64_icon))).await?;
+    /// guild.edit(&http, |g| g.icon(Some(&base64_icon))).await?;
     /// #     Ok(())
     /// # }
     /// ```
     ///
     /// [`utils::read_image`]: crate::utils::read_image
     pub fn icon(&mut self, icon: Option<&str>) -> &mut Self {
-        self.0.insert("icon", icon.map_or_else(|| Value::Null, |x| Value::String(x.to_string())));
+        self.0.insert("icon", icon.map_or_else(|| NULL, |x| Value::from(x.to_string())));
         self
     }
 
     /// Set the name of the guild.
     ///
-    /// **Note**: Must be between (and including) 2-100 chracters.
+    /// **Note**: Must be between (and including) 2-100 characters.
     pub fn name<S: ToString>(&mut self, name: S) -> &mut Self {
-        self.0.insert("name", Value::String(name.to_string()));
+        self.0.insert("name", Value::from(name.to_string()));
         self
     }
 
@@ -87,7 +88,7 @@ impl EditGuild {
     ///
     /// [`features`]: crate::model::guild::Guild::features
     pub fn description<S: ToString>(&mut self, name: S) -> &mut Self {
-        self.0.insert("name", Value::String(name.to_string()));
+        self.0.insert("name", Value::from(name.to_string()));
         self
     }
 
@@ -101,10 +102,10 @@ impl EditGuild {
         let mut values: Vec<Value> = vec![];
 
         for value in features {
-            values.push(Value::String(value));
+            values.push(Value::from(value));
         }
 
-        self.0.insert("features", Value::Array(values));
+        self.0.insert("features", Value::from(values));
         self
     }
 
@@ -118,37 +119,10 @@ impl EditGuild {
     }
 
     fn _owner(&mut self, user_id: UserId) {
-        let id = Value::Number(Number::from(user_id.0));
+        let id = from_number(user_id.0);
         self.0.insert("owner_id", id);
     }
 
-    /// Set the voice region of the server.
-    ///
-    /// # Examples
-    ///
-    /// Setting the region to [`Region::UsWest`]:
-    ///
-    /// ```rust,no_run
-    /// # use serenity::{http::Http, model::id::GuildId};
-    /// #
-    /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// #     let http = Http::default();
-    /// #     let mut guild = GuildId(0).to_partial_guild(&http).await?;
-    /// use serenity::model::guild::Region;
-    ///
-    /// // assuming a `guild` has already been bound
-    ///
-    /// guild.edit(&http, |g| g.region(Region::UsWest)).await?;
-    /// #     Ok(())
-    /// # }
-    /// ```
-    #[deprecated(note = "Regions are now set per voice channel instead of globally.")]
-    #[allow(deprecated)]
-    pub fn region(&mut self, region: Region) -> &mut Self {
-        self.0.insert("region", Value::String(region.name().to_string()));
-        self
-    }
-
     /// Set the splash image of the guild on the invitation page.
     ///
     /// The `splash` must be base64-encoded 1024x1024 png/jpeg/gif image-data.
@@ -158,7 +132,7 @@ impl EditGuild {
     ///
     /// [`features`]: crate::model::guild::Guild::features
     pub fn splash(&mut self, splash: Option<&str>) -> &mut Self {
-        let splash = splash.map_or(Value::Null, |x| Value::String(x.to_string()));
+        let splash = splash.map_or(NULL, |x| Value::from(x.to_string()));
         self.0.insert("splash", splash);
         self
     }
@@ -172,7 +146,7 @@ impl EditGuild {
     ///
     /// [`features`]: crate::model::guild::Guild::features
     pub fn discovery_splash(&mut self, splash: Option<&str>) -> &mut Self {
-        let splash = splash.map_or(Value::Null, |x| Value::String(x.to_string()));
+        let splash = splash.map_or(NULL, |x| Value::from(x.to_string()));
         self.0.insert("splash", splash);
         self
     }
@@ -186,7 +160,7 @@ impl EditGuild {
     ///
     /// [`features`]: crate::model::guild::Guild::features
     pub fn banner(&mut self, banner: Option<&str>) -> &mut Self {
-        let banner = banner.map_or(Value::Null, |x| Value::String(x.to_string()));
+        let banner = banner.map_or(NULL, |x| Value::from(x.to_string()));
         self.0.insert("banner", banner);
         self
     }
@@ -194,7 +168,7 @@ impl EditGuild {
     /// Set the channel ID where welcome messages and boost events will be
     /// posted.
     pub fn system_channel_id(&mut self, channel_id: Option<ChannelId>) -> &mut Self {
-        let channel_id = channel_id.map_or(Value::Null, |x| Value::from(x.0));
+        let channel_id = channel_id.map_or(NULL, |x| Value::from(x.0));
         self.0.insert("system_channel_id", channel_id);
         self
     }
@@ -204,7 +178,7 @@ impl EditGuild {
     /// **Note**:
     /// This feature is for Community guilds only.
     pub fn rules_channel_id(&mut self, channel_id: Option<ChannelId>) -> &mut Self {
-        let channel_id = channel_id.map_or(Value::Null, |x| Value::from(x.0));
+        let channel_id = channel_id.map_or(NULL, |x| Value::from(x.0));
         self.0.insert("rules_channel_id", channel_id);
         self
     }
@@ -215,7 +189,7 @@ impl EditGuild {
     /// **Note**:
     /// This feature is for Community guilds only.
     pub fn public_updates_channel_id(&mut self, channel_id: Option<ChannelId>) -> &mut Self {
-        let channel_id = channel_id.map_or(Value::Null, |x| Value::from(x.0));
+        let channel_id = channel_id.map_or(NULL, |x| Value::from(x.0));
         self.0.insert("public_updates_channel_id", channel_id);
         self
     }
@@ -228,14 +202,14 @@ impl EditGuild {
     /// **Note**:
     /// This feature is for Community guilds only.
     pub fn preferred_locale(&mut self, locale: Option<&str>) -> &mut Self {
-        let locale = locale.map_or(Value::Null, |x| Value::String(x.to_string()));
+        let locale = locale.map_or(NULL, |x| Value::from(x.to_string()));
         self.0.insert("preferred_locale", locale);
         self
     }
 
     /// Set the content filter level.
     pub fn explicit_content_filter(&mut self, level: Option<ExplicitContentFilter>) -> &mut Self {
-        let level = level.map_or(Value::Null, |x| Value::from(x as u8));
+        let level = level.map_or(NULL, |x| Value::from(x as u8));
         self.0.insert("explicit_content_filter", level);
         self
     }
@@ -245,7 +219,7 @@ impl EditGuild {
         &mut self,
         level: Option<DefaultMessageNotificationLevel>,
     ) -> &mut Self {
-        let level = level.map_or(Value::Null, |x| Value::from(x as u8));
+        let level = level.map_or(NULL, |x| Value::from(x as u8));
         self.0.insert("default_message_notifications", level);
         self
     }
@@ -265,7 +239,7 @@ impl EditGuild {
     /// # use serenity::{http::Http, model::id::GuildId};
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// #     let http = Http::default();
+    /// #     let http = Http::new("token");
     /// #     let mut guild = GuildId(0).to_partial_guild(&http).await?;
     /// use serenity::model::guild::VerificationLevel;
     ///
@@ -289,7 +263,7 @@ impl EditGuild {
     }
 
     fn _verification_level(&mut self, verification_level: VerificationLevel) {
-        let num = Value::Number(Number::from(verification_level.num()));
+        let num = from_number(verification_level.num());
         self.0.insert("verification_level", num);
     }
 
@@ -299,7 +273,7 @@ impl EditGuild {
     /// # use serenity::{http::Http, model::id::GuildId};
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// #     let http = Http::default();
+    /// #     let http = Http::new("token");
     /// #     let mut guild = GuildId(0).to_partial_guild(&http).await?;
     /// use serenity::model::guild::SystemChannelFlags;
     ///
