@@ -1,10 +1,9 @@
 use std::collections::HashMap;
-use std::fmt::Display;
-
-use chrono::{DateTime, TimeZone};
 
 use crate::internal::prelude::*;
+use crate::json::{from_number, NULL};
 use crate::model::id::{ChannelId, RoleId};
+use crate::model::Timestamp;
 
 /// A builder which edits the properties of a [`Member`], to be used in
 /// conjunction with [`Member::edit`].
@@ -21,7 +20,7 @@ impl EditMember {
     ///
     /// [Deafen Members]: crate::model::permissions::Permissions::DEAFEN_MEMBERS
     pub fn deafen(&mut self, deafen: bool) -> &mut Self {
-        self.0.insert("deaf", Value::Bool(deafen));
+        self.0.insert("deaf", Value::from(deafen));
         self
     }
 
@@ -31,7 +30,7 @@ impl EditMember {
     ///
     /// [Mute Members]: crate::model::permissions::Permissions::MUTE_MEMBERS
     pub fn mute(&mut self, mute: bool) -> &mut Self {
-        self.0.insert("mute", Value::Bool(mute));
+        self.0.insert("mute", Value::from(mute));
         self
     }
 
@@ -42,7 +41,7 @@ impl EditMember {
     ///
     /// [Manage Nicknames]: crate::model::permissions::Permissions::MANAGE_NICKNAMES
     pub fn nickname<S: ToString>(&mut self, nickname: S) -> &mut Self {
-        self.0.insert("nick", Value::String(nickname.to_string()));
+        self.0.insert("nick", Value::from(nickname.to_string()));
         self
     }
 
@@ -52,15 +51,14 @@ impl EditMember {
     ///
     /// [Manage Roles]: crate::model::permissions::Permissions::MANAGE_ROLES
     pub fn roles<T: AsRef<RoleId>, It: IntoIterator<Item = T>>(&mut self, roles: It) -> &mut Self {
-        let role_ids =
-            roles.into_iter().map(|x| Value::Number(Number::from(x.as_ref().0))).collect();
+        let role_ids = roles.into_iter().map(|x| from_number(x.as_ref().0)).collect();
 
         self._roles(role_ids);
         self
     }
 
     fn _roles(&mut self, roles: Vec<Value>) {
-        self.0.insert("roles", Value::Array(roles));
+        self.0.insert("roles", Value::from(roles));
     }
 
     /// The Id of the voice channel to move the member to.
@@ -76,7 +74,7 @@ impl EditMember {
     }
 
     fn _voice_channel(&mut self, channel_id: ChannelId) {
-        let num = Value::Number(Number::from(channel_id.0));
+        let num = from_number(channel_id.0);
         self.0.insert("channel_id", num);
     }
 
@@ -86,7 +84,7 @@ impl EditMember {
     ///
     /// [Move Members]: crate::model::permissions::Permissions::MOVE_MEMBERS
     pub fn disconnect_member(&mut self) -> &mut Self {
-        self.0.insert("channel_id", Value::Null);
+        self.0.insert("channel_id", NULL);
 
         self
     }
@@ -101,7 +99,7 @@ impl EditMember {
     /// [Moderate Members]: crate::model::permissions::Permissions::MODERATE_MEMBERS
     #[doc(alias = "timeout")]
     pub fn disable_communication_until(&mut self, time: String) -> &mut Self {
-        self.0.insert("communication_disabled_until", Value::String(time));
+        self.0.insert("communication_disabled_until", Value::from(time));
         self
     }
 
@@ -112,13 +110,8 @@ impl EditMember {
     ///
     /// [Moderate Members]: crate::model::permissions::Permissions::MODERATE_MEMBERS
     #[doc(alias = "timeout")]
-    pub fn disable_communication_until_datetime<Tz>(&mut self, time: DateTime<Tz>) -> &mut Self
-    where
-        Tz: TimeZone,
-        Tz::Offset: Display,
-    {
-        let timestamp = time.to_rfc3339();
-        self.0.insert("communication_disabled_until", Value::String(timestamp));
+    pub fn disable_communication_until_datetime(&mut self, time: Timestamp) -> &mut Self {
+        self.0.insert("communication_disabled_until", Value::from(time.to_string()));
         self
     }
 
@@ -129,7 +122,7 @@ impl EditMember {
     /// [Moderate Members]: crate::model::permissions::Permissions::MODERATE_MEMBERS
     #[doc(alias = "timeout")]
     pub fn enable_communication(&mut self) -> &mut Self {
-        self.0.insert("communication_disabled_until", Value::Null);
+        self.0.insert("communication_disabled_until", NULL);
         self
     }
 }
