@@ -1,12 +1,13 @@
 use std::collections::HashMap;
 
-use bytes::buf::Buf;
-use reqwest::Url;
-use tokio::{fs::File, io::AsyncReadExt};
-
-use crate::http::{AttachmentType, Http};
+#[cfg(feature = "model")]
+use crate::http::Http;
 use crate::internal::prelude::*;
-use crate::model::{guild::Role, Permissions};
+use crate::json::from_number;
+#[cfg(feature = "model")]
+use crate::model::channel::AttachmentType;
+use crate::model::guild::Role;
+use crate::model::Permissions;
 
 /// A builder to create or edit a [`Role`] for use via a number of model methods.
 ///
@@ -29,7 +30,7 @@ use crate::model::{guild::Role, Permissions};
 /// # use serenity::{model::id::{ChannelId, GuildId}, http::Http};
 /// # use std::sync::Arc;
 /// #
-/// # let http = Arc::new(Http::default());
+/// # let http = Arc::new(Http::new("token"));
 /// # let (channel_id, guild_id) = (ChannelId(1), GuildId(2));
 /// #
 /// // assuming a `channel_id` and `guild_id` has been bound
@@ -47,25 +48,26 @@ pub struct EditRole(pub HashMap<&'static str, Value>);
 
 impl EditRole {
     /// Creates a new builder with the values of the given [`Role`].
+    #[must_use]
     pub fn new(role: &Role) -> Self {
         let mut map = HashMap::with_capacity(9);
 
         #[cfg(feature = "utils")]
         {
-            map.insert("color", Value::Number(Number::from(role.colour.0)));
+            map.insert("color", from_number(role.colour.0));
         }
 
         #[cfg(not(feature = "utils"))]
         {
-            map.insert("color", Value::Number(Number::from(role.colour)));
+            map.insert("color", from_number(role.colour));
         }
 
-        map.insert("hoist", Value::Bool(role.hoist));
-        map.insert("managed", Value::Bool(role.managed));
-        map.insert("mentionable", Value::Bool(role.mentionable));
-        map.insert("name", Value::String(role.name.clone()));
-        map.insert("permissions", Value::Number(Number::from(role.permissions.bits())));
-        map.insert("position", Value::Number(Number::from(role.position)));
+        map.insert("hoist", Value::from(role.hoist));
+        map.insert("managed", Value::from(role.managed));
+        map.insert("mentionable", Value::from(role.mentionable));
+        map.insert("name", Value::from(role.name.clone()));
+        map.insert("permissions", from_number(role.permissions.bits()));
+        map.insert("position", from_number(role.position));
 
         if let Some(unicode_emoji) = &role.unicode_emoji {
             map.insert("unicode_emoji", Value::String(unicode_emoji.clone()));
@@ -80,39 +82,39 @@ impl EditRole {
 
     /// Sets the colour of the role.
     pub fn colour(&mut self, colour: u64) -> &mut Self {
-        self.0.insert("color", Value::Number(Number::from(colour)));
+        self.0.insert("color", from_number(colour));
         self
     }
 
     /// Whether or not to hoist the role above lower-positioned role in the user
     /// list.
     pub fn hoist(&mut self, hoist: bool) -> &mut Self {
-        self.0.insert("hoist", Value::Bool(hoist));
+        self.0.insert("hoist", Value::from(hoist));
         self
     }
 
     /// Whether or not to make the role mentionable, notifying its users.
     pub fn mentionable(&mut self, mentionable: bool) -> &mut Self {
-        self.0.insert("mentionable", Value::Bool(mentionable));
+        self.0.insert("mentionable", Value::from(mentionable));
         self
     }
 
     /// The name of the role to set.
     pub fn name<S: ToString>(&mut self, name: S) -> &mut Self {
-        self.0.insert("name", Value::String(name.to_string()));
+        self.0.insert("name", Value::from(name.to_string()));
         self
     }
 
     /// The set of permissions to assign the role.
     pub fn permissions(&mut self, permissions: Permissions) -> &mut Self {
-        self.0.insert("permissions", Value::Number(Number::from(permissions.bits())));
+        self.0.insert("permissions", from_number(permissions.bits()));
         self
     }
 
     /// The position to assign the role in the role list. This correlates to the
     /// role's position in the user list.
     pub fn position(&mut self, position: u8) -> &mut Self {
-        self.0.insert("position", Value::Number(Number::from(position)));
+        self.0.insert("position", from_number(position));
         self
     }
 
@@ -130,45 +132,16 @@ impl EditRole {
     ///
     /// May error if the icon is a URL and the HTTP request fails, or if the icon is a file
     /// on a path that doesn't exist.
+    #[cfg(feature = "model")]
     pub async fn icon<'a>(
         &mut self,
         http: impl AsRef<Http>,
         icon: impl Into<AttachmentType<'a>>,
     ) -> Result<&mut Self> {
-        let icon = match icon.into() {
-            AttachmentType::Bytes {
-                data,
-                filename: _,
-            } => "data:image/png;base64,".to_string() + &base64::encode(&data.into_owned()),
-            AttachmentType::File {
-                file,
-                filename: _,
-            } => {
-                let mut buf = Vec::new();
-                file.try_clone().await?.read_to_end(&mut buf).await?;
-
-                "data:image/png;base64,".to_string() + &base64::encode(&buf)
-            },
-            AttachmentType::Path(path) => {
-                let mut file = File::open(path).await?;
-                let mut buf = vec![];
-                file.read_to_end(&mut buf).await?;
-
-                "data:image/png;base64,".to_string() + &base64::encode(&buf)
-            },
-            AttachmentType::Image(url) => {
-                let url = Url::parse(url).map_err(|_| Error::Url(url.to_string()))?;
-                let response = http.as_ref().client.get(url).send().await?;
-                let mut bytes = response.bytes().await?;
-                let mut picture: Vec<u8> = vec![0; bytes.len()];
-                bytes.copy_to_slice(&mut picture[..]);
-
-                "data:image/png;base64,".to_string() + &base64::encode(&picture)
-            },
-        };
-
+        let icon_data = icon.into().data(&http.as_ref().client).await?;
+        let icon_string = format!("data:image/png;base64,{}", base64::encode(icon_data));
         self.0.remove("unicode_emoji");
-        self.0.insert("icon", Value::String(icon));
+        self.0.insert("icon", Value::from(icon_string));
 
         Ok(self)
     }
