@@ -1,6 +1,6 @@
 use std::collections::HashMap;
 
-use serde_json::Value;
+use crate::json::Value;
 
 /// Edits a [`StageInstance`].
 ///
@@ -11,7 +11,7 @@ pub struct EditStageInstance(pub HashMap<&'static str, Value>);
 impl EditStageInstance {
     /// Sets the topic of the stage channel instance.
     pub fn topic<D: ToString>(&mut self, topic: D) -> &mut Self {
-        self.0.insert("topic", Value::String(topic.to_string()));
+        self.0.insert("topic", Value::from(topic.to_string()));
 
         self
     }
