@@ -1,8 +1,12 @@
 use std::collections::HashMap;
+#[cfg(not(feature = "model"))]
+use std::marker::PhantomData;
 
-use serde_json::Value;
-
-use crate::http::AttachmentType;
+use super::CreateAllowedMentions;
+use crate::builder::CreateComponents;
+use crate::json::{self, from_number, Value};
+#[cfg(feature = "model")]
+use crate::model::channel::AttachmentType;
 use crate::model::channel::MessageFlags;
 
 /// A builder to create the inner content of a [`Webhook`]'s execution.
@@ -24,10 +28,9 @@ use crate::model::channel::MessageFlags;
 /// use serenity::utils::Colour;
 ///
 /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-/// # let http = Http::default();
-/// let id = 245037420704169985;
-/// let token = "ig5AO-wdVWpCBtUUMxmgsWryqgsW3DChbKYOINftJ4DCrUbnkedoYZD0VOH1QLr-S3sV";
-/// let webhook = http.get_webhook_with_token(id, token).await?;
+/// # let http = Http::new("token");
+/// let url = "https://discord.com/api/webhooks/245037420704169985/ig5AO-wdVWpCBtUUMxmgsWryqgsW3DChbKYOINftJ4DCrUbnkedoYZD0VOH1QLr-S3sV";
+/// let webhook = http.get_webhook_from_url(url).await?;
 ///
 /// let website = Embed::fake(|e| {
 ///     e.title("The Rust Language Website")
@@ -56,7 +59,11 @@ use crate::model::channel::MessageFlags;
 /// [`Webhook::execute`]: crate::model::webhook::Webhook::execute
 /// [`execute_webhook`]: crate::http::client::Http::execute_webhook
 #[derive(Clone, Debug)]
-pub struct ExecuteWebhook<'a>(pub HashMap<&'static str, Value>, pub Vec<AttachmentType<'a>>);
+pub struct ExecuteWebhook<'a>(
+    pub HashMap<&'static str, Value>,
+    #[cfg(feature = "model")] pub Vec<AttachmentType<'a>>,
+    #[cfg(not(feature = "model"))] PhantomData<&'a ()>,
+);
 
 impl<'a> ExecuteWebhook<'a> {
     /// Override the default avatar of the webhook with an image URL.
@@ -69,7 +76,7 @@ impl<'a> ExecuteWebhook<'a> {
     /// # use serenity::http::Http;
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// # let http = Http::default();
+    /// # let http = Http::new("token");
     /// # let webhook = http.get_webhook_with_token(0, "").await?;
     /// #
     /// let avatar_url = "https://i.imgur.com/KTs6whd.jpg";
@@ -79,7 +86,7 @@ impl<'a> ExecuteWebhook<'a> {
     /// # }
     /// ```
     pub fn avatar_url<S: ToString>(&mut self, avatar_url: S) -> &mut Self {
-        self.0.insert("avatar_url", Value::String(avatar_url.to_string()));
+        self.0.insert("avatar_url", Value::from(avatar_url.to_string()));
         self
     }
 
@@ -96,7 +103,7 @@ impl<'a> ExecuteWebhook<'a> {
     /// # use serenity::http::Http;
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// # let http = Http::default();
+    /// # let http = Http::new("token");
     /// # let webhook = http.get_webhook_with_token(0, "").await?;
     /// #
     /// let execution = webhook.execute(&http, false, |w| w.content("foo")).await;
@@ -108,22 +115,24 @@ impl<'a> ExecuteWebhook<'a> {
     /// # }
     /// ```
     pub fn content<S: ToString>(&mut self, content: S) -> &mut Self {
-        self.0.insert("content", Value::String(content.to_string()));
+        self.0.insert("content", Value::from(content.to_string()));
         self
     }
 
     /// Appends a file to the webhook message.
+    #[cfg(feature = "model")]
     pub fn add_file<T: Into<AttachmentType<'a>>>(&mut self, file: T) -> &mut Self {
         self.1.push(file.into());
         self
     }
 
     /// Appends a list of files to the webhook message.
+    #[cfg(feature = "model")]
     pub fn add_files<T: Into<AttachmentType<'a>>, It: IntoIterator<Item = T>>(
         &mut self,
         files: It,
     ) -> &mut Self {
-        self.1.extend(files.into_iter().map(|f| f.into()));
+        self.1.extend(files.into_iter().map(Into::into));
         self
     }
 
@@ -131,11 +140,52 @@ impl<'a> ExecuteWebhook<'a> {
     ///
     /// Calling this multiple times will overwrite the file list.
     /// To append files, call [`Self::add_file`] or [`Self::add_files`] instead.
+    #[cfg(feature = "model")]
     pub fn files<T: Into<AttachmentType<'a>>, It: IntoIterator<Item = T>>(
         &mut self,
         files: It,
     ) -> &mut Self {
-        self.1 = files.into_iter().map(|f| f.into()).collect();
+        self.1 = files.into_iter().map(Into::into).collect();
+        self
+    }
+
+    /// Set the allowed mentions for the message.
+    pub fn allowed_mentions<F>(&mut self, f: F) -> &mut Self
+    where
+        F: FnOnce(&mut CreateAllowedMentions) -> &mut CreateAllowedMentions,
+    {
+        let mut allowed_mentions = CreateAllowedMentions::default();
+        f(&mut allowed_mentions);
+        let map = json::hashmap_to_json_map(allowed_mentions.0);
+        let allowed_mentions = Value::from(map);
+
+        self.0.insert("allowed_mentions", allowed_mentions);
+        self
+    }
+
+    /// Creates components for this message. Requires an application-owned webhook, meaning either
+    /// the webhook's `kind` field is set to [`WebhookType::Application`], or it was created by an
+    /// application (and has kind [`WebhookType::Incoming`]).
+    ///
+    /// [`WebhookType::Application`]: crate::model::webhook::WebhookType
+    /// [`WebhookType::Incoming`]: crate::model::webhook::WebhookType
+    pub fn components<F>(&mut self, f: F) -> &mut Self
+    where
+        F: FnOnce(&mut CreateComponents) -> &mut CreateComponents,
+    {
+        let mut components = CreateComponents::default();
+        f(&mut components);
+
+        self.0.insert("components", Value::from(components.0));
+        self
+    }
+
+    /// Sets the components of this message. Requires an application-owned webhook. See
+    /// [`components`] for details.
+    ///
+    /// [`components`]: crate::builder::ExecuteWebhook::components
+    pub fn set_components(&mut self, components: CreateComponents) -> &mut Self {
+        self.0.insert("components", Value::Array(components.0));
         self
     }
 
@@ -153,7 +203,7 @@ impl<'a> ExecuteWebhook<'a> {
     /// [`Webhook::execute`]: crate::model::webhook::Webhook::execute
     /// [struct-level documentation]: #examples
     pub fn embeds(&mut self, embeds: Vec<Value>) -> &mut Self {
-        self.0.insert("embeds", Value::Array(embeds));
+        self.0.insert("embeds", Value::from(embeds));
         self
     }
 
@@ -167,7 +217,7 @@ impl<'a> ExecuteWebhook<'a> {
     /// # use serenity::http::Http;
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// # let http = Http::default();
+    /// # let http = Http::new("token");
     /// # let webhook = http.get_webhook_with_token(0, "").await?;
     /// #
     /// let execution = webhook.execute(&http, false, |w| w.content("hello").tts(true)).await;
@@ -179,7 +229,7 @@ impl<'a> ExecuteWebhook<'a> {
     /// # }
     /// ```
     pub fn tts(&mut self, tts: bool) -> &mut Self {
-        self.0.insert("tts", Value::Bool(tts));
+        self.0.insert("tts", Value::from(tts));
         self
     }
 
@@ -193,7 +243,7 @@ impl<'a> ExecuteWebhook<'a> {
     /// # use serenity::http::Http;
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// # let http = Http::default();
+    /// # let http = Http::new("token");
     /// # let webhook = http.get_webhook_with_token(0, "").await?;
     /// #
     /// let execution = webhook.execute(&http, false, |w| w.content("hello").username("hakase")).await;
@@ -205,7 +255,7 @@ impl<'a> ExecuteWebhook<'a> {
     /// # }
     /// ```
     pub fn username<S: ToString>(&mut self, username: S) -> &mut Self {
-        self.0.insert("username", Value::String(username.to_string()));
+        self.0.insert("username", Value::from(username.to_string()));
         self
     }
 
@@ -213,14 +263,14 @@ impl<'a> ExecuteWebhook<'a> {
     ///
     /// # Examples
     ///
-    /// Supressing an embed on the message.
+    /// Suppressing an embed on the message.
     ///
     /// ```rust,no_run
     /// # use serenity::http::Http;
     /// # use serenity::model::channel::MessageFlags;
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// # let http = Http::default();
+    /// # let http = Http::new("token");
     /// # let webhook = http.get_webhook_with_token(0, "").await?;
     /// #
     /// let execution = webhook
@@ -237,7 +287,7 @@ impl<'a> ExecuteWebhook<'a> {
     /// # }
     /// ```
     pub fn flags(&mut self, flags: MessageFlags) -> &mut Self {
-        self.0.insert("flags", Value::Number(serde_json::Number::from(flags.bits)));
+        self.0.insert("flags", from_number(flags.bits()));
         self
     }
 }
@@ -260,8 +310,8 @@ impl<'a> Default for ExecuteWebhook<'a> {
     /// [`Webhook`]: crate::model::webhook::Webhook
     fn default() -> ExecuteWebhook<'a> {
         let mut map = HashMap::new();
-        map.insert("tts", Value::Bool(false));
+        map.insert("tts", Value::from(false));
 
-        ExecuteWebhook(map, vec![])
+        ExecuteWebhook(map, Vec::default())
     }
 }
