@@ -10,7 +10,7 @@
 /// let mut settings = CacheSettings::new();
 /// settings.max_messages(10);
 /// ```
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Default)]
 #[non_exhaustive]
 pub struct Settings {
     /// The maximum number of messages to store in a channel's message cache.
@@ -19,17 +19,10 @@ pub struct Settings {
     pub max_messages: usize,
 }
 
-impl Default for Settings {
-    fn default() -> Self {
-        Settings {
-            max_messages: usize::default(),
-        }
-    }
-}
-
 impl Settings {
     /// Creates new settings to be used with a cache.
     #[inline]
+    #[must_use]
     pub fn new() -> Self {
         Self::default()
     }
