@@ -1,10 +1,9 @@
-use std::{borrow::Cow, sync::Arc};
+use std::borrow::Cow;
+use std::sync::Arc;
 
-use async_tungstenite::tungstenite::{
-    self,
-    error::Error as TungsteniteError,
-    protocol::frame::CloseFrame,
-};
+use async_tungstenite::tungstenite;
+use async_tungstenite::tungstenite::error::Error as TungsteniteError;
+use async_tungstenite::tungstenite::protocol::frame::CloseFrame;
 use futures::channel::mpsc::{self, UnboundedReceiver as Receiver, UnboundedSender as Sender};
 use futures::{SinkExt, StreamExt};
 use serde::Deserialize;
@@ -18,18 +17,24 @@ use super::{ShardClientMessage, ShardId, ShardManagerMessage, ShardRunnerMessage
 use crate::client::bridge::voice::VoiceGatewayManager;
 use crate::client::dispatch::{dispatch, DispatchEvent};
 use crate::client::{EventHandler, RawEventHandler};
-#[cfg(all(feature = "unstable_discord_api", feature = "collector"))]
-use crate::collector::ComponentInteractionFilter;
 #[cfg(feature = "collector")]
-use crate::collector::{EventFilter, LazyArc, LazyReactionAction, MessageFilter, ReactionFilter};
+use crate::collector::{
+    ComponentInteractionFilter,
+    EventFilter,
+    LazyArc,
+    LazyReactionAction,
+    MessageFilter,
+    ModalInteractionFilter,
+    ReactionFilter,
+};
 #[cfg(feature = "framework")]
 use crate::framework::Framework;
 use crate::gateway::{GatewayError, InterMessage, ReconnectType, Shard, ShardAction};
 use crate::internal::prelude::*;
 use crate::internal::ws_impl::{ReceiverExt, SenderExt};
 use crate::model::event::{Event, GatewayEvent};
-#[cfg(all(feature = "unstable_discord_api", feature = "collector"))]
-use crate::model::interactions::{Interaction, InteractionType};
+#[cfg(feature = "collector")]
+use crate::model::interactions::Interaction;
 use crate::CacheAndHttp;
 
 /// A runner for managing a [`Shard`] and its respective WebSocket client.
@@ -38,7 +43,7 @@ pub struct ShardRunner {
     event_handler: Option<Arc<dyn EventHandler>>,
     raw_event_handler: Option<Arc<dyn RawEventHandler>>,
     #[cfg(feature = "framework")]
-    framework: Arc<Box<dyn Framework + Send + Sync>>,
+    framework: Arc<dyn Framework + Send + Sync>,
     manager_tx: Sender<ShardManagerMessage>,
     // channel to receive messages from the shard manager and dispatches
     runner_rx: Receiver<InterMessage>,
@@ -54,8 +59,10 @@ pub struct ShardRunner {
     message_filters: Vec<MessageFilter>,
     #[cfg(feature = "collector")]
     reaction_filters: Vec<ReactionFilter>,
-    #[cfg(all(feature = "unstable_discord_api", feature = "collector"))]
+    #[cfg(feature = "collector")]
     component_interaction_filters: Vec<ComponentInteractionFilter>,
+    #[cfg(feature = "collector")]
+    modal_interaction_filters: Vec<ModalInteractionFilter>,
 }
 
 impl ShardRunner {
@@ -82,8 +89,10 @@ impl ShardRunner {
             message_filters: Vec::new(),
             #[cfg(feature = "collector")]
             reaction_filters: Vec::new(),
-            #[cfg(all(feature = "unstable_discord_api", feature = "collector"))]
+            #[cfg(feature = "collector")]
             component_interaction_filters: vec![],
+            #[cfg(feature = "collector")]
+            modal_interaction_filters: vec![],
         }
     }
 
@@ -231,17 +240,22 @@ impl ShardRunner {
                 let mut reaction = LazyReactionAction::new(&reaction_event.reaction, false);
                 retain(&mut self.reaction_filters, |f| f.send_reaction(&mut reaction));
             },
-            #[cfg(all(feature = "unstable_discord_api", feature = "collector"))]
+            #[cfg(feature = "collector")]
             Event::InteractionCreate(ref interaction_event) => {
-                if interaction_event.interaction.kind() == InteractionType::MessageComponent {
-                    if let Interaction::MessageComponent(ref interaction) =
-                        interaction_event.interaction
-                    {
+                match &interaction_event.interaction {
+                    Interaction::MessageComponent(interaction) => {
                         let mut interaction = LazyArc::new(interaction);
                         retain(&mut self.component_interaction_filters, |f| {
                             f.send_interaction(&mut interaction)
                         });
-                    }
+                    },
+                    Interaction::ModalSubmit(interaction) => {
+                        let mut interaction = LazyArc::new(interaction);
+                        retain(&mut self.modal_interaction_filters, |f| {
+                            f.send_interaction(&mut interaction)
+                        });
+                    },
+                    _ => (),
                 }
             },
             _ => {},
@@ -293,15 +307,15 @@ impl ShardRunner {
         }
 
         // Send a Close Frame to Discord, which allows a bot to "log off"
-        #[allow(clippy::let_underscore_must_use)]
-        let _ = self
-            .shard
-            .client
-            .close(Some(CloseFrame {
-                code: close_code.into(),
-                reason: Cow::from(""),
-            }))
-            .await;
+        drop(
+            self.shard
+                .client
+                .close(Some(CloseFrame {
+                    code: close_code.into(),
+                    reason: Cow::from(""),
+                }))
+                .await,
+        );
 
         // In return, we wait for either a Close Frame response, or an error, after which this WS is deemed
         // disconnected from Discord.
@@ -370,26 +384,22 @@ impl ShardRunner {
 
                     true
                 },
-                ShardClientMessage::Manager(ShardManagerMessage::ShardUpdate {
-                    ..
-                }) => {
+                ShardClientMessage::Manager(
+                    ShardManagerMessage::ShardUpdate {
+                        ..
+                    }
+                    | ShardManagerMessage::ShutdownInitiated
+                    | ShardManagerMessage::ShutdownFinished(_),
+                ) => {
                     // nb: not sent here
 
                     true
                 },
-                ShardClientMessage::Manager(ShardManagerMessage::ShutdownInitiated) => {
-                    // nb: not sent here
-
-                    true
-                },
-                ShardClientMessage::Manager(ShardManagerMessage::ShutdownFinished(_)) => {
-                    // nb: not sent here
-
-                    true
-                },
-                ShardClientMessage::Manager(ShardManagerMessage::ShardDisallowedGatewayIntents)
-                | ShardClientMessage::Manager(ShardManagerMessage::ShardInvalidAuthentication)
-                | ShardClientMessage::Manager(ShardManagerMessage::ShardInvalidGatewayIntents) => {
+                ShardClientMessage::Manager(
+                    ShardManagerMessage::ShardDisallowedGatewayIntents
+                    | ShardManagerMessage::ShardInvalidAuthentication
+                    | ShardManagerMessage::ShardInvalidGatewayIntents,
+                ) => {
                     // These variants should never be received.
                     warn!("[ShardRunner {:?}] Received a ShardError?", self.shard.shard_info(),);
 
@@ -404,7 +414,7 @@ impl ShardRunner {
                     self.shard.chunk_guild(guild_id, limit, filter, nonce.as_deref()).await.is_ok()
                 },
                 ShardClientMessage::Runner(ShardRunnerMessage::Close(code, reason)) => {
-                    let reason = reason.unwrap_or_else(String::new);
+                    let reason = reason.unwrap_or_default();
                     let close = CloseFrame {
                         code: code.into(),
                         reason: Cow::from(reason),
@@ -460,12 +470,20 @@ impl ShardRunner {
 
                     true
                 },
-                #[cfg(all(feature = "unstable_discord_api", feature = "collector"))]
+                #[cfg(feature = "collector")]
                 ShardClientMessage::Runner(ShardRunnerMessage::SetComponentInteractionFilter(
                     collector,
                 )) => {
                     self.component_interaction_filters.push(collector);
 
+                    true
+                },
+                #[cfg(feature = "collector")]
+                ShardClientMessage::Runner(ShardRunnerMessage::SetModalInteractionFilter(
+                    collector,
+                )) => {
+                    self.modal_interaction_filters.push(collector);
+
                     true
                 },
             },
@@ -492,7 +510,7 @@ impl ShardRunner {
                     }
                 },
                 Event::VoiceStateUpdate(ref event) => {
-                    if let Some(guild_id) = event.guild_id {
+                    if let Some(guild_id) = event.voice_state.guild_id {
                         voice_manager.state_update(guild_id, &event.voice_state).await;
                     }
                 },
@@ -525,9 +543,7 @@ impl ShardRunner {
                         self.shard.shard_info(),
                     );
 
-                    #[allow(clippy::let_underscore_must_use)]
-                    let _ = self.request_restart().await;
-
+                    drop(self.request_restart().await);
                     return Ok(false);
                 },
                 Err(_) => break,
@@ -627,7 +643,7 @@ impl ShardRunner {
         #[cfg(feature = "voice")]
         {
             if let Ok(GatewayEvent::Dispatch(_, ref event)) = event {
-                self.handle_voice_event(&event).await;
+                self.handle_voice_event(event).await;
             }
         }
 
@@ -661,12 +677,11 @@ impl ShardRunner {
 
     #[instrument(skip(self))]
     fn update_manager(&self) {
-        #[allow(clippy::let_underscore_must_use)]
-        let _ = self.manager_tx.unbounded_send(ShardManagerMessage::ShardUpdate {
+        drop(self.manager_tx.unbounded_send(ShardManagerMessage::ShardUpdate {
             id: ShardId(self.shard.shard_info()[0]),
             latency: self.shard.latency(),
             stage: self.shard.stage(),
-        });
+        }));
     }
 }
 
@@ -676,7 +691,7 @@ pub struct ShardRunnerOptions {
     pub event_handler: Option<Arc<dyn EventHandler>>,
     pub raw_event_handler: Option<Arc<dyn RawEventHandler>>,
     #[cfg(feature = "framework")]
-    pub framework: Arc<Box<dyn Framework + Send + Sync>>,
+    pub framework: Arc<dyn Framework + Send + Sync>,
     pub manager_tx: Sender<ShardManagerMessage>,
     pub shard: Shard,
     #[cfg(feature = "voice")]
