@@ -2,20 +2,17 @@
 use std::fmt;
 use std::sync::Arc;
 
-use futures::{
-    channel::mpsc::UnboundedSender as Sender,
-    future::{BoxFuture, FutureExt},
-};
+use futures::channel::mpsc::UnboundedSender as Sender;
+use futures::future::{BoxFuture, FutureExt};
 use tokio::sync::RwLock;
 use tracing::instrument;
 use typemap_rev::TypeMap;
 
-use super::Context;
 #[cfg(feature = "gateway")]
-use super::{
-    bridge::gateway::event::ClientEvent,
-    event_handler::{EventHandler, RawEventHandler},
-};
+use super::bridge::gateway::event::ClientEvent;
+#[cfg(feature = "gateway")]
+use super::event_handler::{EventHandler, RawEventHandler};
+use super::Context;
 #[cfg(feature = "cache")]
 use crate::cache::{Cache, CacheUpdate};
 #[cfg(feature = "framework")]
@@ -23,27 +20,25 @@ use crate::framework::Framework;
 use crate::gateway::InterMessage;
 use crate::http::Http;
 use crate::internal::tokio::spawn_named;
+use crate::model::channel::{Channel, Message};
+use crate::model::event::Event;
+use crate::model::guild::Member;
 #[cfg(feature = "cache")]
 use crate::model::id::GuildId;
-use crate::model::{
-    channel::{Channel, Message},
-    event::Event,
-    guild::Member,
-};
 use crate::CacheAndHttp;
 
 #[inline]
 #[cfg(feature = "cache")]
-async fn update<E: CacheUpdate + fmt::Debug>(
+fn update<E: CacheUpdate + fmt::Debug>(
     cache_and_http: &Arc<CacheAndHttp>,
     event: &mut E,
 ) -> Option<E::Output> {
-    cache_and_http.cache.update(event).await
+    cache_and_http.cache.update(event)
 }
 
 #[inline]
 #[cfg(not(feature = "cache"))]
-async fn update<E>(_cache_and_http: &Arc<CacheAndHttp>, _event: &mut E) -> Option<()> {
+fn update<E>(_cache_and_http: &Arc<CacheAndHttp>, _event: &mut E) -> Option<()> {
     None
 }
 
@@ -78,69 +73,81 @@ pub(crate) enum DispatchEvent {
 
 impl DispatchEvent {
     #[instrument(skip(self, cache_and_http))]
-    async fn update(&mut self, cache_and_http: &Arc<CacheAndHttp>) {
+    fn update(&mut self, cache_and_http: &Arc<CacheAndHttp>) {
         match self {
             Self::Model(Event::ChannelCreate(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::ChannelDelete(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::ChannelUpdate(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::GuildCreate(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::GuildDelete(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::GuildEmojisUpdate(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::GuildMemberAdd(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::GuildMemberRemove(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::GuildMemberUpdate(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::GuildMembersChunk(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::GuildRoleCreate(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::GuildRoleDelete(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::GuildRoleUpdate(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
+            },
+            Self::Model(Event::GuildStickersUpdate(ref mut event)) => {
+                update(cache_and_http, event);
             },
             Self::Model(Event::GuildUnavailable(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             // Already handled by the framework check macro
             Self::Model(Event::MessageCreate(_)) => {},
             Self::Model(Event::MessageUpdate(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::PresencesReplace(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::PresenceUpdate(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::Ready(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::UserUpdate(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
             },
             Self::Model(Event::VoiceStateUpdate(ref mut event)) => {
-                update(cache_and_http, event).await;
+                update(cache_and_http, event);
+            },
+            Self::Model(Event::ThreadCreate(ref mut event)) => {
+                update(cache_and_http, event);
+            },
+            Self::Model(Event::ThreadUpdate(ref mut event)) => {
+                update(cache_and_http, event);
+            },
+            Self::Model(Event::ThreadDelete(ref mut event)) => {
+                update(cache_and_http, event);
             },
             _ => (),
         }
@@ -151,7 +158,7 @@ impl DispatchEvent {
 pub(crate) fn dispatch<'rec>(
     // #[allow(unused_variables)]
     mut event: DispatchEvent,
-    #[cfg(feature = "framework")] framework: &'rec Arc<Box<dyn Framework + Send + Sync>>,
+    #[cfg(feature = "framework")] framework: &'rec Arc<dyn Framework + Send + Sync>,
     data: &'rec Arc<RwLock<TypeMap>>,
     event_handler: &'rec Option<Arc<dyn EventHandler>>,
     raw_event_handler: &'rec Option<Arc<dyn RawEventHandler>>,
@@ -162,33 +169,31 @@ pub(crate) fn dispatch<'rec>(
     async move {
         match (event_handler, raw_event_handler) {
             (None, None) => {
-                event.update(&cache_and_http).await;
+                event.update(&cache_and_http);
 
+                #[cfg(feature = "framework")]
                 if let DispatchEvent::Model(Event::MessageCreate(event)) = event {
-                    #[cfg(feature = "framework")]
-                    {
-                        #[cfg(not(feature = "cache"))]
-                        let context = context(data, runner_tx, shard_id, &cache_and_http.http);
-                        #[cfg(feature = "cache")]
-                        let context = context(
-                            data,
-                            runner_tx,
-                            shard_id,
-                            &cache_and_http.http,
-                            &cache_and_http.cache,
-                        );
+                    #[cfg(not(feature = "cache"))]
+                    let context = context(data, runner_tx, shard_id, &cache_and_http.http);
+                    #[cfg(feature = "cache")]
+                    let context = context(
+                        data,
+                        runner_tx,
+                        shard_id,
+                        &cache_and_http.http,
+                        &cache_and_http.cache,
+                    );
 
-                        let framework = Arc::clone(framework);
+                    let framework = Arc::clone(framework);
 
-                        spawn_named("dispatch::framework::message", async move {
-                            framework.dispatch(context, event.message).await;
-                        });
-                    }
+                    spawn_named("dispatch::framework::message", async move {
+                        framework.dispatch(context, event.message).await;
+                    });
                 }
             },
             (Some(ref h), None) => match event {
                 DispatchEvent::Model(Event::MessageCreate(mut event)) => {
-                    update(&cache_and_http, &mut event).await;
+                    update(&cache_and_http, &mut event);
 
                     #[cfg(not(feature = "cache"))]
                     let context = context(data, runner_tx, shard_id, &cache_and_http.http);
@@ -223,7 +228,7 @@ pub(crate) fn dispatch<'rec>(
                 },
             },
             (None, Some(ref rh)) => {
-                event.update(&cache_and_http).await;
+                event.update(&cache_and_http);
 
                 if let DispatchEvent::Model(event) = event {
                     let event_handler = Arc::clone(rh);
@@ -298,7 +303,7 @@ pub(crate) fn dispatch<'rec>(
                     },
                     other => {
                         handle_event(other, data, handler, runner_tx, shard_id, cache_and_http)
-                            .await
+                            .await;
                     },
                 }
             },
@@ -325,6 +330,7 @@ async fn dispatch_message(
 }
 // Once we can use `Box` as part of a pattern, we will reconsider boxing.
 #[allow(clippy::too_many_arguments)]
+#[cfg_attr(feature = "cache", allow(clippy::used_underscore_binding))]
 #[instrument(skip(event, data, event_handler, cache_and_http))]
 async fn handle_event(
     event: DispatchEvent,
@@ -347,8 +353,20 @@ async fn handle_event(
                 event_handler.shard_stage_update(context, event).await;
             });
         },
+        DispatchEvent::Model(Event::ApplicationCommandPermissionsUpdate(event)) => {
+            let event_handler = Arc::clone(event_handler);
+
+            spawn_named(
+                "dispatch::event_handler::application_command_permissions_update",
+                async move {
+                    event_handler
+                        .application_command_permissions_update(context, event.permission)
+                        .await;
+                },
+            );
+        },
         DispatchEvent::Model(Event::ChannelCreate(mut event)) => {
-            update(&cache_and_http, &mut event).await;
+            update(&cache_and_http, &mut event);
             match event.channel {
                 Channel::Guild(channel) => {
                     let event_handler = Arc::clone(event_handler);
@@ -369,7 +387,7 @@ async fn handle_event(
             }
         },
         DispatchEvent::Model(Event::ChannelDelete(mut event)) => {
-            update(&cache_and_http, &mut event).await;
+            update(&cache_and_http, &mut event);
 
             match event.channel {
                 Channel::Private(_) => {},
@@ -401,12 +419,12 @@ async fn handle_event(
 
             spawn_named("dispatch::event_handler::channel_update", async move {
                 feature_cache! {{
-                    let old_channel = cache_and_http.cache.as_ref().channel(event.channel.id()).await;
-                    update(&cache_and_http, &mut event).await;
+                    let old_channel = cache_and_http.cache.as_ref().channel(event.channel.id());
+                    update(&cache_and_http, &mut event);
 
                     event_handler.channel_update(context, old_channel, event.channel).await;
                 } else {
-                    update(&cache_and_http, &mut event).await;
+                    update(&cache_and_http, &mut event);
 
                     event_handler.channel_update(context, event.channel).await;
                 }}
@@ -428,23 +446,20 @@ async fn handle_event(
         },
         DispatchEvent::Model(Event::GuildCreate(mut event)) => {
             #[cfg(feature = "cache")]
-            let _is_new =
-                { !cache_and_http.cache.unavailable_guilds.read().await.contains(&event.guild.id) };
+            let _is_new = !cache_and_http.cache.unavailable_guilds.contains(&event.guild.id);
 
-            update(&cache_and_http, &mut event).await;
+            update(&cache_and_http, &mut event);
 
             #[cfg(feature = "cache")]
             {
                 let context = context.clone();
 
-                if cache_and_http.cache.unavailable_guilds.read().await.is_empty() {
+                if cache_and_http.cache.unavailable_guilds.is_empty() {
                     let guild_amount = cache_and_http
                         .cache
                         .guilds
-                        .read()
-                        .await
                         .iter()
-                        .map(|(&id, _)| id)
+                        .map(|i| *i.key())
                         .collect::<Vec<GuildId>>();
                     let event_handler = Arc::clone(event_handler);
 
@@ -465,7 +480,7 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::GuildDelete(mut event)) => {
-            let _full = update(&cache_and_http, &mut event).await;
+            let _full = update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::guild_delete", async move {
@@ -477,7 +492,7 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::GuildEmojisUpdate(mut event)) => {
-            update(&cache_and_http, &mut event).await;
+            update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::guild_emojis_update", async move {
@@ -492,16 +507,16 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::GuildMemberAdd(mut event)) => {
-            update(&cache_and_http, &mut event).await;
+            update(&cache_and_http, &mut event);
 
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::guild_member_addition", async move {
-                event_handler.guild_member_addition(context, event.guild_id, event.member).await;
+                event_handler.guild_member_addition(context, event.member).await;
             });
         },
         DispatchEvent::Model(Event::GuildMemberRemove(mut event)) => {
-            let _member = update(&cache_and_http, &mut event).await;
+            let _member = update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::guild_member_removal", async move {
@@ -513,9 +528,9 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::GuildMemberUpdate(mut event)) => {
-            let _before = update(&cache_and_http, &mut event).await;
+            let _before = update(&cache_and_http, &mut event);
             let _after: Option<Member> = feature_cache! {{
-                cache_and_http.cache.member(event.guild_id, event.user.id).await
+                cache_and_http.cache.member(event.guild_id, event.user.id)
             } else {
                 None
             }};
@@ -533,7 +548,7 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::GuildMembersChunk(mut event)) => {
-            update(&cache_and_http, &mut event).await;
+            update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::guild_members_chunk", async move {
@@ -541,15 +556,15 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::GuildRoleCreate(mut event)) => {
-            update(&cache_and_http, &mut event).await;
+            update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::guild_role_create", async move {
-                event_handler.guild_role_create(context, event.guild_id, event.role).await;
+                event_handler.guild_role_create(context, event.role).await;
             });
         },
         DispatchEvent::Model(Event::GuildRoleDelete(mut event)) => {
-            let _role = update(&cache_and_http, &mut event).await;
+            let _role = update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::guild_role_delete", async move {
@@ -561,19 +576,27 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::GuildRoleUpdate(mut event)) => {
-            let _before = update(&cache_and_http, &mut event).await;
+            let _before = update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::guild_role_update", async move {
                 feature_cache! {{
-                    event_handler.guild_role_update(context, event.guild_id, _before, event.role).await;
+                    event_handler.guild_role_update(context, _before, event.role).await;
                 } else {
-                    event_handler.guild_role_update(context, event.guild_id, event.role).await;
+                    event_handler.guild_role_update(context, event.role).await;
                 }}
             });
         },
+        DispatchEvent::Model(Event::GuildStickersUpdate(mut event)) => {
+            update(&cache_and_http, &mut event);
+            let event_handler = Arc::clone(event_handler);
+
+            tokio::spawn(async move {
+                event_handler.guild_stickers_update(context, event.guild_id, event.stickers).await;
+            });
+        },
         DispatchEvent::Model(Event::GuildUnavailable(mut event)) => {
-            update(&cache_and_http, &mut event).await;
+            update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::guild_unavailable", async move {
@@ -586,14 +609,13 @@ async fn handle_event(
             spawn_named("dispatch::event_handler::guild_update", async move {
                 feature_cache! {{
                     let before = cache_and_http.cache
-                        .guild(&event.guild.id)
-                        .await;
+                        .guild(&event.guild.id);
 
-                    update(&cache_and_http, &mut event).await;
+                    update(&cache_and_http, &mut event);
 
                     event_handler.guild_update(context, before, event.guild).await;
                 } else {
-                    update(&cache_and_http, &mut event).await;
+                    update(&cache_and_http, &mut event);
 
                     event_handler.guild_update(context, event.guild).await;
                 }}
@@ -634,12 +656,12 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::MessageUpdate(mut event)) => {
-            let _before = update(&cache_and_http, &mut event).await;
+            let _before = update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::message_update", async move {
                 feature_cache! {{
-                    let _after = cache_and_http.cache.message(event.channel_id, event.id).await;
+                    let _after = cache_and_http.cache.message(event.channel_id, event.id);
                     event_handler.message_update(context, _before, _after, event).await;
                 } else {
                     event_handler.message_update(context, event).await;
@@ -647,7 +669,7 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::PresencesReplace(mut event)) => {
-            update(&cache_and_http, &mut event).await;
+            update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::presence_replace", async move {
@@ -655,12 +677,12 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::PresenceUpdate(mut event)) => {
-            update(&cache_and_http, &mut event).await;
+            update(&cache_and_http, &mut event);
 
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::presence_update", async move {
-                event_handler.presence_update(context, event).await;
+                event_handler.presence_update(context, event.presence).await;
             });
         },
         DispatchEvent::Model(Event::ReactionAdd(event)) => {
@@ -687,7 +709,7 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::Ready(mut event)) => {
-            update(&cache_and_http, &mut event).await;
+            update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::ready", async move {
@@ -716,7 +738,7 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::UserUpdate(mut event)) => {
-            let _before = update(&cache_and_http, &mut event).await;
+            let _before = update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::user_update", async move {
@@ -735,14 +757,14 @@ async fn handle_event(
             });
         },
         DispatchEvent::Model(Event::VoiceStateUpdate(mut event)) => {
-            let _before = update(&cache_and_http, &mut event).await;
+            let _before = update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::voice_state_update", async move {
                 feature_cache! {{
-                    event_handler.voice_state_update(context, event.guild_id, _before, event.voice_state).await;
+                    event_handler.voice_state_update(context, _before, event.voice_state).await;
                 } else {
-                    event_handler.voice_state_update(context, event.guild_id, event.voice_state).await;
+                    event_handler.voice_state_update(context, event.voice_state).await;
                 }}
             });
         },
@@ -753,7 +775,6 @@ async fn handle_event(
                 event_handler.webhook_update(context, event.guild_id, event.channel_id).await;
             });
         },
-        #[cfg(feature = "unstable_discord_api")]
         DispatchEvent::Model(Event::InteractionCreate(event)) => {
             let event_handler = Arc::clone(event_handler);
 
@@ -761,7 +782,6 @@ async fn handle_event(
                 event_handler.interaction_create(context, event.interaction).await;
             });
         },
-        #[cfg(feature = "unstable_discord_api")]
         DispatchEvent::Model(Event::IntegrationCreate(event)) => {
             let event_handler = Arc::clone(event_handler);
 
@@ -769,7 +789,6 @@ async fn handle_event(
                 event_handler.integration_create(context, event.integration).await;
             });
         },
-        #[cfg(feature = "unstable_discord_api")]
         DispatchEvent::Model(Event::IntegrationUpdate(event)) => {
             let event_handler = Arc::clone(event_handler);
 
@@ -777,7 +796,6 @@ async fn handle_event(
                 event_handler.integration_update(context, event.integration).await;
             });
         },
-        #[cfg(feature = "unstable_discord_api")]
         DispatchEvent::Model(Event::IntegrationDelete(event)) => {
             let event_handler = Arc::clone(event_handler);
 
@@ -787,33 +805,6 @@ async fn handle_event(
                     .await;
             });
         },
-        #[cfg(feature = "unstable_discord_api")]
-        #[allow(deprecated)]
-        DispatchEvent::Model(Event::ApplicationCommandCreate(event)) => {
-            let event_handler = Arc::clone(event_handler);
-
-            spawn_named("dispatch::event_handler::application_command_create", async move {
-                event_handler.application_command_create(context, event.application_command).await;
-            });
-        },
-        #[cfg(feature = "unstable_discord_api")]
-        #[allow(deprecated)]
-        DispatchEvent::Model(Event::ApplicationCommandUpdate(event)) => {
-            let event_handler = Arc::clone(event_handler);
-
-            spawn_named("dispatch::event_handler::application_command_update", async move {
-                event_handler.application_command_update(context, event.application_command).await;
-            });
-        },
-        #[cfg(feature = "unstable_discord_api")]
-        #[allow(deprecated)]
-        DispatchEvent::Model(Event::ApplicationCommandDelete(event)) => {
-            let event_handler = Arc::clone(event_handler);
-
-            spawn_named("dispatch::event_handler::application_command_delete", async move {
-                event_handler.application_command_delete(context, event.application_command).await;
-            });
-        },
         DispatchEvent::Model(Event::StageInstanceCreate(event)) => {
             let event_handler = Arc::clone(event_handler);
 
@@ -835,21 +826,24 @@ async fn handle_event(
                 event_handler.stage_instance_delete(context, event.stage_instance).await;
             });
         },
-        DispatchEvent::Model(Event::ThreadCreate(event)) => {
+        DispatchEvent::Model(Event::ThreadCreate(mut event)) => {
+            update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::thread_create", async move {
                 event_handler.thread_create(context, event.thread).await;
             });
         },
-        DispatchEvent::Model(Event::ThreadUpdate(event)) => {
+        DispatchEvent::Model(Event::ThreadUpdate(mut event)) => {
+            update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::thread_update", async move {
                 event_handler.thread_update(context, event.thread).await;
             });
         },
-        DispatchEvent::Model(Event::ThreadDelete(event)) => {
+        DispatchEvent::Model(Event::ThreadDelete(mut event)) => {
+            update(&cache_and_http, &mut event);
             let event_handler = Arc::clone(event_handler);
 
             spawn_named("dispatch::event_handler::thread_delete", async move {
