@@ -24,18 +24,11 @@ mod dispatch;
 mod error;
 #[cfg(feature = "gateway")]
 mod event_handler;
-#[cfg(feature = "gateway")]
-mod extras;
 
-#[cfg(all(feature = "cache", feature = "gateway"))]
-use std::time::Duration;
-use std::{
-    boxed::Box,
-    future::Future,
-    pin::Pin,
-    sync::Arc,
-    task::{Context as FutContext, Poll},
-};
+use std::future::Future;
+use std::pin::Pin;
+use std::sync::Arc;
+use std::task::{Context as FutContext, Poll};
 
 use futures::future::BoxFuture;
 use tokio::sync::{Mutex, RwLock};
@@ -44,7 +37,6 @@ use typemap_rev::{TypeMap, TypeMapKey};
 
 #[cfg(feature = "gateway")]
 use self::bridge::gateway::{
-    GatewayIntents,
     ShardManager,
     ShardManagerError,
     ShardManagerMonitor,
@@ -52,12 +44,10 @@ use self::bridge::gateway::{
 };
 #[cfg(feature = "voice")]
 use self::bridge::voice::VoiceGatewayManager;
-pub use self::{context::Context, error::Error as ClientError};
+pub use self::context::Context;
+pub use self::error::Error as ClientError;
 #[cfg(feature = "gateway")]
-pub use self::{
-    event_handler::{EventHandler, RawEventHandler},
-    extras::Extras,
-};
+pub use self::event_handler::{EventHandler, RawEventHandler};
 #[cfg(feature = "gateway")]
 use super::gateway::GatewayError;
 #[cfg(feature = "cache")]
@@ -68,27 +58,25 @@ use crate::cache::Settings as CacheSettings;
 use crate::framework::Framework;
 use crate::http::Http;
 use crate::internal::prelude::*;
-#[cfg(feature = "unstable_discord_api")]
+#[cfg(feature = "gateway")]
+use crate::model::gateway::GatewayIntents;
 use crate::model::id::ApplicationId;
-use crate::model::id::UserId;
 pub use crate::CacheAndHttp;
 
 /// A builder implementing [`Future`] building a [`Client`] to interact with Discord.
 #[cfg(feature = "gateway")]
-pub struct ClientBuilder<'a> {
+#[must_use = "Builders do nothing unless they are awaited"]
+pub struct ClientBuilder {
+    // TODO: data, http and cache_settings are Options in order to take() them out in the Future impl.
+    // This should be changed after the stabilization of std::future::IntoFuture.
     data: Option<TypeMap>,
-    http: Http,
-    fut: Option<BoxFuture<'a, Result<Client>>>,
+    http: Option<Http>,
+    fut: Option<BoxFuture<'static, Result<Client>>>,
     intents: GatewayIntents,
-    #[cfg(feature = "unstable_discord_api")]
-    application_id: Option<ApplicationId>,
     #[cfg(feature = "cache")]
-    timeout: Option<Duration>,
-    #[cfg(feature = "cache")]
-    // Option in order to take() it out in the Future impl
     cache_settings: Option<CacheSettings>,
     #[cfg(feature = "framework")]
-    framework: Option<Arc<Box<dyn Framework + Send + Sync + 'static>>>,
+    framework: Option<Arc<dyn Framework + Send + Sync + 'static>>,
     #[cfg(feature = "voice")]
     voice_manager: Option<Arc<dyn VoiceGatewayManager + Send + Sync + 'static>>,
     event_handler: Option<Arc<dyn EventHandler>>,
@@ -96,17 +84,13 @@ pub struct ClientBuilder<'a> {
 }
 
 #[cfg(feature = "gateway")]
-impl<'a> ClientBuilder<'a> {
-    fn _new(http: Http) -> Self {
+impl ClientBuilder {
+    fn _new(http: Http, intents: GatewayIntents) -> Self {
         Self {
             data: Some(TypeMap::new()),
-            http,
+            http: Some(http),
             fut: None,
-            intents: GatewayIntents::non_privileged(),
-            #[cfg(feature = "unstable_discord_api")]
-            application_id: None,
-            #[cfg(feature = "cache")]
-            timeout: None,
+            intents,
             #[cfg(feature = "cache")]
             cache_settings: Some(CacheSettings::new()),
             #[cfg(feature = "framework")]
@@ -125,8 +109,8 @@ impl<'a> ClientBuilder<'a> {
     /// If you have enabled the `framework`-feature (on by default), you must specify
     /// a framework via the [`Self::framework`] or [`Self::framework_arc`] method,
     /// otherwise awaiting the builder will cause a panic.
-    pub fn new(token: impl AsRef<str>) -> Self {
-        Self::_new(Http::new_with_token(token.as_ref()))
+    pub fn new(token: impl AsRef<str>, intents: GatewayIntents) -> Self {
+        Self::_new(Http::new(token.as_ref()), intents)
     }
 
     /// Construct a new builder with a [`Http`] instance to calls methods on
@@ -136,41 +120,36 @@ impl<'a> ClientBuilder<'a> {
     /// If you have enabled the `framework`-feature (on by default), you must specify
     /// a framework via the [`Self::framework`] or [`Self::framework_arc`] method,
     /// otherwise awaiting the builder will cause a panic.
-    ///
-    /// [`Http`]: crate::http::Http
-    pub fn new_with_http(http: Http) -> Self {
-        Self::_new(http)
+    pub fn new_with_http(http: Http, intents: GatewayIntents) -> Self {
+        Self::_new(http, intents)
     }
 
     /// Sets a token for the bot. If the token is not prefixed "Bot ",
     /// this method will automatically do so.
     pub fn token(mut self, token: impl AsRef<str>) -> Self {
-        self.http = Http::new_with_token(token.as_ref());
+        self.http = Some(Http::new(token.as_ref()));
 
         self
     }
 
     /// Gets the current token used for the [`Http`] client.
-    ///
-    /// [`Http`]: crate::http::Http
-    pub fn get_token(&self) -> &str {
-        &self.http.token
+    /// This can be unwrapped safely unless used after awaiting the builder.
+    pub fn get_token(&self) -> Option<&str> {
+        self.http.as_ref().map(|http| http.token.as_str())
     }
 
     /// Sets the application id.
-    #[cfg(feature = "unstable_discord_api")]
-    pub fn application_id(mut self, application_id: u64) -> Self {
-        self.application_id = Some(ApplicationId(application_id));
-
-        self.http = Http::new_with_token_application_id(self.get_token(), application_id);
+    pub fn application_id(self, application_id: u64) -> Self {
+        if let Some(http) = &self.http {
+            http.set_application_id(application_id);
+        }
 
         self
     }
 
     /// Gets the application ID, if already initialized. See [`Self::application_id`] for more info.
-    #[cfg(feature = "unstable_discord_api")]
     pub fn get_application_id(&self) -> Option<ApplicationId> {
-        self.application_id
+        self.http.as_ref().and_then(|h| h.application_id().map(ApplicationId))
     }
 
     /// Sets the entire [`TypeMap`] that will be available in [`Context`]s.
@@ -182,7 +161,8 @@ impl<'a> ClientBuilder<'a> {
         self
     }
 
-    /// Gets the type map, if already initialized. See [`Self::type_map`] for more info.
+    /// Gets the type map. See [`Self::type_map`] for more info.
+    /// This can be unwrapped safely unless used after awaiting the builder.
     pub fn get_type_map(&self) -> Option<&TypeMap> {
         self.data.as_ref()
     }
@@ -197,31 +177,10 @@ impl<'a> ClientBuilder<'a> {
         self
     }
 
-    /// Sets how long - if wanted to begin with - a cache update shall
-    /// be attempted for. After the `timeout` ran out, the update will be
-    /// skipped.
-    ///
-    /// By default, a cache update will never timeout and potentially
-    /// cause a deadlock.
-    /// A timeout however, will invalidate the cache.
-    #[cfg(feature = "cache")]
-    pub fn cache_update_timeout(mut self, timeout: Duration) -> Self {
-        self.timeout = Some(timeout);
-
-        self
-    }
-
-    /// Gets the cache update timeout, if already initialized. See [`Self::cache_update_timeout`]
-    /// for more info.
-    #[cfg(feature = "cache")]
-    pub fn get_cache_update_timeout(&self) -> Option<Duration> {
-        self.timeout
-    }
-
     /// Sets the settings of the cache.
     /// Refer to [`Settings`] for more information.
     ///
-    /// [`Settings`]: self::CacheSettings
+    /// [`Settings`]: CacheSettings
     #[cfg(feature = "cache")]
     pub fn cache_settings<F>(mut self, f: F) -> Self
     where
@@ -235,12 +194,10 @@ impl<'a> ClientBuilder<'a> {
     }
 
     /// Gets the cache settings. See [`Self::cache_settings`] for more info.
+    /// This can be unwrapped safely unless used after awaiting the builder.
     #[cfg(feature = "cache")]
-    pub fn get_cache_settings(&self) -> &CacheSettings {
-        // unwrap() is ok because cache_settings will only ever be None in the middle of being
-        // .await'ed
-        #[allow(clippy::unwrap_used)]
-        self.cache_settings.as_ref().unwrap()
+    pub fn get_cache_settings(&self) -> Option<&CacheSettings> {
+        self.cache_settings.as_ref()
     }
 
     /// Sets the command framework to be used. It will receive messages sent
@@ -255,7 +212,7 @@ impl<'a> ClientBuilder<'a> {
     where
         F: Framework + Send + Sync + 'static,
     {
-        self.framework = Some(Arc::new(Box::new(framework)));
+        self.framework = Some(Arc::new(framework));
 
         self
     }
@@ -265,18 +222,18 @@ impl<'a> ClientBuilder<'a> {
     /// extra control.
     /// You can provide a clone and keep the original to manually dispatch.
     #[cfg(feature = "framework")]
-    pub fn framework_arc(
+    pub fn framework_arc<T: Framework + Send + Sync + 'static>(
         mut self,
-        framework: Arc<Box<dyn Framework + Send + Sync + 'static>>,
+        framework: Arc<T>,
     ) -> Self {
-        self.framework = Some(framework);
+        self.framework = Some(framework as Arc<dyn Framework + Send + Sync + 'static>);
 
         self
     }
 
     /// Gets the framework, if already initialized. See [`Self::framework`] for more info.
     #[cfg(feature = "framework")]
-    pub fn get_framework(&self) -> Option<Arc<Box<dyn Framework + Send + Sync>>> {
+    pub fn get_framework(&self) -> Option<Arc<dyn Framework + Send + Sync>> {
         self.framework.clone()
     }
 
@@ -302,7 +259,7 @@ impl<'a> ClientBuilder<'a> {
     /// extra control.
     /// You can provide a clone and keep the original to manually dispatch.
     ///
-    /// [`voice_manager`]: #method.voice_manager
+    /// [`voice_manager`]: Self::voice_manager
     #[cfg(feature = "voice")]
     pub fn voice_manager_arc(
         mut self,
@@ -333,8 +290,8 @@ impl<'a> ClientBuilder<'a> {
     ///
     /// # Privileged Intents
     ///
-    /// The intents [`GatewayIntents::GUILD_PRESENCES`] and [`GatewayIntents::GUILD_MEMBERS`]
-    /// are *privileged*.
+    /// The intents [`GatewayIntents::GUILD_PRESENCES`], [`GatewayIntents::GUILD_MEMBERS`]
+    /// and [`GatewayIntents::MESSAGE_CONTENT`] are *privileged*.
     /// [Privileged intents] need to be enabled in the *developer portal*.
     /// Once the bot is in 100 guilds or more, [the bot must be verified] in
     /// order to use privileged intents.
@@ -342,8 +299,6 @@ impl<'a> ClientBuilder<'a> {
     /// [gateway intent]: https://discord.com/developers/docs/topics/gateway#privileged-intents
     /// [Privileged intents]: https://discord.com/developers/docs/topics/gateway#privileged-intents
     /// [the bot must be verified]: https://support.discord.com/hc/en-us/articles/360040720412-Bot-Verification-and-Data-Whitelisting
-    /// [`GatewayIntents::GUILD_PRESENCES`]: crate::client::bridge::gateway::GatewayIntents::GUILD_PRESENCES
-    /// [`GatewayIntents::GUILD_MEMBERS`]: crate::client::bridge::gateway::GatewayIntents::GUILD_MEMBERS
     pub fn intents(mut self, intents: GatewayIntents) -> Self {
         self.intents = intents;
 
@@ -362,6 +317,16 @@ impl<'a> ClientBuilder<'a> {
         self
     }
 
+    /// Sets an event handler with multiple methods for each possible event. Passed by Arc.
+    pub fn event_handler_arc<H: EventHandler + 'static>(
+        mut self,
+        event_handler_arc: Arc<H>,
+    ) -> Self {
+        self.event_handler = Some(event_handler_arc);
+
+        self
+    }
+
     /// Gets the event handler, if already initialized. See [`Self::event_handler`] for more info.
     pub fn get_event_handler(&self) -> Option<Arc<dyn EventHandler>> {
         self.event_handler.clone()
@@ -383,7 +348,7 @@ impl<'a> ClientBuilder<'a> {
 }
 
 #[cfg(feature = "gateway")]
-impl<'a> Future for ClientBuilder<'a> {
+impl Future for ClientBuilder {
     type Output = Result<Client>;
 
     #[allow(clippy::unwrap_used)] // Allowing unwrap because all should be Some() by this point
@@ -398,12 +363,7 @@ impl<'a> Future for ClientBuilder<'a> {
             let event_handler = self.event_handler.take();
             let raw_event_handler = self.raw_event_handler.take();
             let intents = self.intents;
-            let http = Arc::new(std::mem::take(&mut self.http));
-
-            #[cfg(feature = "unstable_discord_api")]
-            if http.application_id == 0 {
-                panic!("Please provide an Application Id in order to use interactions features.");
-            }
+            let http = Arc::new(self.http.take().unwrap());
 
             #[cfg(feature = "voice")]
             let voice_manager = self.voice_manager.take();
@@ -411,13 +371,11 @@ impl<'a> Future for ClientBuilder<'a> {
             let cache_and_http = Arc::new(CacheAndHttp {
                 #[cfg(feature = "cache")]
                 cache: Arc::new(Cache::new_with_settings(self.cache_settings.take().unwrap())),
-                #[cfg(feature = "cache")]
-                update_cache_timeout: self.timeout.take(),
                 http: Arc::clone(&http),
             });
 
             self.fut = Some(Box::pin(async move {
-                let url = Arc::new(Mutex::new(http.get_gateway().await?.url));
+                let ws_url = Arc::new(Mutex::new(http.get_gateway().await?.url));
 
                 let (shard_manager, shard_manager_worker) = {
                     ShardManager::new(ShardManagerOptions {
@@ -431,7 +389,7 @@ impl<'a> Future for ClientBuilder<'a> {
                         shard_total: 0,
                         #[cfg(feature = "voice")]
                         voice_manager: &voice_manager,
-                        ws_url: &url,
+                        ws_url: &ws_url,
                         cache_and_http: &cache_and_http,
                         intents,
                     })
@@ -439,15 +397,15 @@ impl<'a> Future for ClientBuilder<'a> {
                 };
 
                 Ok(Client {
-                    ws_uri: url,
                     data,
                     shard_manager,
                     shard_manager_worker,
                     #[cfg(feature = "voice")]
                     voice_manager,
+                    ws_url,
                     cache_and_http,
                 })
-            }))
+            }));
         }
 
         self.fut.as_mut().unwrap().as_mut().poll(ctx)
@@ -490,7 +448,8 @@ impl<'a> Future for ClientBuilder<'a> {
 /// }
 ///
 /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-/// let mut client = Client::builder("my token here").event_handler(Handler).await?;
+/// let mut client =
+///     Client::builder("my token here", GatewayIntents::default()).event_handler(Handler).await?;
 ///
 /// client.start().await?;
 /// #   Ok(())
@@ -524,11 +483,12 @@ pub struct Client {
     /// - [`Event::MessageUpdate`]
     ///
     /// ```rust,ignore
-    /// use serenity::prelude::*;
-    /// use serenity::model::prelude::*;
     /// use std::collections::HashMap;
     /// use std::env;
     ///
+    /// use serenity::model::prelude::*;
+    /// use serenity::prelude::*;
+    ///
     /// struct MessageEventCounter;
     ///
     /// impl TypeMapKey for MessageEventCounter {
@@ -557,7 +517,13 @@ pub struct Client {
     ///     }
     ///
     ///     #[cfg(feature = "cache")]
-    ///     async fn message_update(&self, ctx: Context, _old: Option<Message>, _new: Option<Message>, _: MessageUpdateEvent) {
+    ///     async fn message_update(
+    ///         &self,
+    ///         ctx: Context,
+    ///         _old: Option<Message>,
+    ///         _new: Option<Message>,
+    ///         _: MessageUpdateEvent,
+    ///     ) {
     ///         reg(ctx, "MessageUpdate").await
     ///     }
     ///
@@ -569,7 +535,7 @@ pub struct Client {
     ///
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
     /// let token = std::env::var("DISCORD_TOKEN")?;
-    /// let mut client = Client::builder(&token).event_handler(Handler).await?;
+    /// let mut client = Client::builder(&token, GatewayIntents::default()).event_handler(Handler).await?;
     /// {
     ///     let mut data = client.data.write().await;
     ///     data.insert::<MessageEventCounter>(HashMap::default());
@@ -606,7 +572,7 @@ pub struct Client {
     /// 5 seconds:
     ///
     /// ```rust,no_run
-    /// # use serenity::client::{Client, EventHandler};
+    /// # use serenity::prelude::*;
     /// # use std::time::Duration;
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
@@ -615,7 +581,8 @@ pub struct Client {
     /// impl EventHandler for Handler {}
     ///
     /// let token = std::env::var("DISCORD_TOKEN")?;
-    /// let mut client = Client::builder(&token).event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder(&token, GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// let shard_manager = client.shard_manager.clone();
     ///
@@ -638,14 +605,15 @@ pub struct Client {
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
     /// use std::time::Duration;
     ///
-    /// use serenity::client::{Client, EventHandler};
+    /// use serenity::prelude::*;
     ///
     /// struct Handler;
     ///
     /// impl EventHandler for Handler {}
     ///
     /// let token = std::env::var("DISCORD_TOKEN")?;
-    /// let mut client = Client::builder(&token).event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder(&token, GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// // Create a clone of the `Arc` containing the shard manager.
     /// let shard_manager = client.shard_manager.clone();
@@ -672,22 +640,21 @@ pub struct Client {
     /// connections.
     #[cfg(feature = "voice")]
     pub voice_manager: Option<Arc<dyn VoiceGatewayManager + Send + Sync + 'static>>,
-    /// URI that the client's shards will use to connect to the gateway.
+    /// URL that the client's shards will use to connect to the gateway.
     ///
     /// This is likely not important for production usage and is, at best, used
     /// for debugging.
     ///
     /// This is wrapped in an `Arc<Mutex<T>>` so all shards will have an updated
     /// value available.
-    pub ws_uri: Arc<Mutex<String>>,
+    pub ws_url: Arc<Mutex<String>>,
     /// A container for an optional cache and HTTP client.
-    /// It also contains the cache update timeout.
     pub cache_and_http: Arc<CacheAndHttp>,
 }
 
 impl Client {
-    pub fn builder<'a>(token: impl AsRef<str>) -> ClientBuilder<'a> {
-        ClientBuilder::new(token)
+    pub fn builder(token: impl AsRef<str>, intents: GatewayIntents) -> ClientBuilder {
+        ClientBuilder::new(token, intents)
     }
 
     /// Establish the connection and start listening for events.
@@ -708,7 +675,7 @@ impl Client {
     ///
     /// ```rust,no_run
     /// # use std::error::Error;
-    /// # use serenity::prelude::EventHandler;
+    /// # use serenity::prelude::*;
     /// use serenity::Client;
     ///
     /// struct Handler;
@@ -717,7 +684,8 @@ impl Client {
     ///
     /// # async fn run() -> Result<(), Box<dyn Error>> {
     /// let token = std::env::var("DISCORD_TOKEN")?;
-    /// let mut client = Client::builder(&token).event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder(&token, GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// if let Err(why) = client.start().await {
     ///     println!("Err with client: {:?}", why);
@@ -750,7 +718,7 @@ impl Client {
     ///
     /// ```rust,no_run
     /// # use std::error::Error;
-    /// # use serenity::prelude::EventHandler;
+    /// # use serenity::prelude::*;
     /// use serenity::Client;
     ///
     /// struct Handler;
@@ -759,7 +727,8 @@ impl Client {
     ///
     /// # async fn run() -> Result<(), Box<dyn Error>> {
     /// let token = std::env::var("DISCORD_TOKEN")?;
-    /// let mut client = Client::builder(&token).event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder(&token, GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// if let Err(why) = client.start_autosharded().await {
     ///     println!("Err with client: {:?}", why);
@@ -803,7 +772,7 @@ impl Client {
     ///
     /// ```rust,no_run
     /// # use std::error::Error;
-    /// # use serenity::prelude::EventHandler;
+    /// # use serenity::prelude::*;
     /// use serenity::Client;
     ///
     /// struct Handler;
@@ -812,7 +781,8 @@ impl Client {
     ///
     /// # async fn run() -> Result<(), Box<dyn Error>> {
     /// let token = std::env::var("DISCORD_TOKEN")?;
-    /// let mut client = Client::builder(&token).event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder(&token, GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// if let Err(why) = client.start_shard(3, 5).await {
     ///     println!("Err with client: {:?}", why);
@@ -826,7 +796,7 @@ impl Client {
     ///
     /// ```rust,no_run
     /// # use std::error::Error;
-    /// # use serenity::prelude::EventHandler;
+    /// # use serenity::prelude::*;
     /// use serenity::Client;
     ///
     /// struct Handler;
@@ -835,7 +805,8 @@ impl Client {
     ///
     /// # async fn run() -> Result<(), Box<dyn Error>> {
     /// let token = std::env::var("DISCORD_TOKEN")?;
-    /// let mut client = Client::builder(&token).event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder(&token, GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// if let Err(why) = client.start_shard(0, 1).await {
     ///     println!("Err with client: {:?}", why);
@@ -873,7 +844,7 @@ impl Client {
     ///
     /// ```rust,no_run
     /// # use std::error::Error;
-    /// # use serenity::prelude::EventHandler;
+    /// # use serenity::prelude::*;
     /// use serenity::Client;
     ///
     /// struct Handler;
@@ -882,7 +853,8 @@ impl Client {
     ///
     /// # async fn run() -> Result<(), Box<dyn Error>> {
     /// let token = std::env::var("DISCORD_TOKEN")?;
-    /// let mut client = Client::builder(&token).event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder(&token, GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// if let Err(why) = client.start_shards(8).await {
     ///     println!("Err with client: {:?}", why);
@@ -921,7 +893,7 @@ impl Client {
     ///
     /// ```rust,no_run
     /// # use std::error::Error;
-    /// # use serenity::prelude::EventHandler;
+    /// # use serenity::prelude::*;
     /// use serenity::Client;
     ///
     /// struct Handler;
@@ -930,7 +902,8 @@ impl Client {
     ///
     /// # async fn run() -> Result<(), Box<dyn Error>> {
     /// let token = std::env::var("DISCORD_TOKEN")?;
-    /// let mut client = Client::builder(&token).event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder(&token, GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// if let Err(why) = client.start_shard_range([4, 7], 10).await {
     ///     println!("Err with client: {:?}", why);
@@ -1003,84 +976,3 @@ impl Client {
         Ok(())
     }
 }
-
-/// Validates that a token is likely in a valid format.
-///
-/// This performs the following checks on a given token:
-///
-/// - At least one character long;
-/// - Contains 3 parts (split by the period char `'.'`);
-/// - The second part of the token is at least 6 characters long;
-/// - The token does not contain any whitespace prior to or after the token.
-///
-/// # Examples
-///
-/// Validate that a token is valid and that a number of invalid tokens are
-/// actually invalid:
-///
-/// ```rust,no_run
-/// use serenity::client::validate_token;
-///
-/// // ensure a valid token is in fact valid:
-/// assert!(validate_token("Mjg4NzYwMjQxMzYzODc3ODg4.C_ikow.j3VupLBuE1QWZng3TMGH0z_UAwg").is_ok());
-///
-/// // helpful to prevent typos
-/// assert!(validate_token("Njg4NzYwMjQxMzYzODc3ODg4.C_ikow.j3VupLBuE1QWZng3TMGH0z_UAwg").is_err());
-/// ```
-///
-/// # Errors
-///
-/// Returns a [`ClientError::InvalidToken`] when one of the above checks fail.
-/// The type of failure is not specified.
-pub fn validate_token(token: impl AsRef<str>) -> Result<()> {
-    if parse_token(token.as_ref()).is_some() {
-        Ok(())
-    } else {
-        Err(Error::Client(ClientError::InvalidToken))
-    }
-}
-
-/// Part of the data contained within a Discord bot token. Returned by [`parse_token`].
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
-pub struct TokenComponents {
-    pub bot_user_id: UserId,
-    pub creation_time: chrono::NaiveDateTime,
-}
-
-/// Verifies that the token adheres to the Discord token format and extracts the bot user ID and the
-/// token generation timestamp
-pub fn parse_token(token: impl AsRef<str>) -> Option<TokenComponents> {
-    // The token consists of three base64-encoded parts
-    let parts: Vec<&str> = token.as_ref().split('.').collect();
-    let base64_config = base64::Config::new(base64::CharacterSet::UrlSafe, true);
-
-    // First part must be a base64-encoded stringified user ID
-    let user_id = base64::decode_config(parts.get(0)?, base64_config).ok()?;
-    let user_id = UserId(std::str::from_utf8(&user_id).ok()?.parse().ok()?);
-
-    // Second part must be a base64-encoded token generation timestamp
-    let timestamp_base64 = parts.get(1)?;
-    // The base64-encoded timestamp must be at least 6 characters
-    if timestamp_base64.len() < 6 {
-        return None;
-    }
-    let timestamp_bytes = base64::decode_config(timestamp_base64, base64_config).ok()?;
-    let mut timestamp = 0;
-    for byte in timestamp_bytes {
-        timestamp *= 256;
-        timestamp += byte as u64;
-    }
-    // Some timestamps are based on the Discord epoch. Convert to Unix epoch
-    if timestamp < 1293840000 {
-        timestamp += 1293840000;
-    }
-    let timestamp = chrono::NaiveDateTime::from_timestamp_opt(timestamp as i64, 0)?;
-
-    // Third part is a base64-encoded HMAC that's not interesting on its own
-    let _ = base64::decode_config(parts.get(2)?, base64_config).ok()?;
-
-    Some(TokenComponents {
-        bot_user_id: user_id,
-        creation_time: timestamp,
-    })
-}
