@@ -7,16 +7,16 @@ use std::sync::Arc;
 mod error;
 pub use error::Error as CollectorError;
 
-#[cfg(feature = "unstable_discord_api")]
 pub mod component_interaction_collector;
 pub mod event_collector;
 pub mod message_collector;
+pub mod modal_interaction_collector;
 pub mod reaction_collector;
 
-#[cfg(feature = "unstable_discord_api")]
 pub use component_interaction_collector::*;
 pub use event_collector::*;
 pub use message_collector::*;
+pub use modal_interaction_collector::*;
 pub use reaction_collector::*;
 
 /// Wraps a &T and clones the value into an Arc<T> lazily. Used with collectors to allow inspecting
