@@ -1,7 +1,8 @@
 use std::borrow::Cow;
 use std::error::Error as StdError;
+use std::fmt;
 use std::marker::PhantomData;
-use std::{fmt, str::FromStr};
+use std::str::FromStr;
 
 use uwl::Stream;
 
@@ -23,11 +24,9 @@ impl<E> From<E> for Error<E> {
 
 impl<E: fmt::Display> fmt::Display for Error<E> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        use self::Error::*;
-
         match *self {
-            Eos => f.write_str(r#"ArgError("end of string")"#),
-            Parse(ref e) => write!(f, "ArgError(\"{}\")", e),
+            Self::Eos => f.write_str(r#"ArgError("end of string")"#),
+            Self::Parse(ref e) => write!(f, "ArgError(\"{}\")", e),
         }
     }
 }
@@ -82,6 +81,7 @@ impl<'a> From<&'a str> for Delimiter {
 }
 
 #[derive(Debug, Clone, Copy, PartialEq)]
+#[allow(clippy::enum_variant_names)]
 enum TokenKind {
     Argument,
     QuotedArgument,
@@ -341,6 +341,7 @@ impl Args {
     /// // We shall not see `the quick brown` again.
     /// assert_eq!(args.rest(), "fox jumps over the lazy");
     /// ```
+    #[must_use]
     pub fn new(message: &str, possible_delimiters: &[Delimiter]) -> Self {
         let delims = possible_delimiters
             .iter()
@@ -348,15 +349,20 @@ impl Args {
                 Delimiter::Single(c) => message.contains(*c),
                 Delimiter::Multiple(s) => message.contains(s),
             })
-            .map(|delim| delim.to_str())
+            .map(Delimiter::to_str)
             .collect::<Vec<_>>();
 
-        let args = if delims.is_empty() && !message.is_empty() {
-            let kind =
-                if is_quoted(message) { TokenKind::QuotedArgument } else { TokenKind::Argument };
+        let args = if delims.is_empty() {
+            let msg = message.trim();
+            let kind = if is_quoted(msg) { TokenKind::QuotedArgument } else { TokenKind::Argument };
 
-            // If there are no delimiters, then the only possible argument is the whole message.
-            vec![Token::new(kind, 0, message.len())]
+            if msg.is_empty() {
+                Vec::new()
+            } else {
+                // If there are no delimiters, then the only possible argument is the whole
+                // message.
+                vec![Token::new(kind, 0, message.len())]
+            }
         } else {
             let mut args = Vec::new();
             let mut stream = Stream::new(message);
@@ -484,6 +490,7 @@ impl Args {
     /// assert_eq!(args.current(), None);
     /// ```
     #[inline]
+    #[must_use]
     pub fn current(&self) -> Option<&str> {
         if self.is_empty() {
             return None;
@@ -716,6 +723,7 @@ impl Args {
     /// assert_eq!(protagonists, "Harry, Hermione, Ronald");
     /// ```
     #[inline]
+    #[must_use]
     pub fn raw(&self) -> RawArguments<'_> {
         RawArguments {
             tokens: &self.args,
@@ -738,6 +746,7 @@ impl Args {
     /// assert_eq!(&*horror_movies, &["Saw", "The Mist", "A Quiet Place"]);
     /// ```
     #[inline]
+    #[must_use]
     pub fn raw_quoted(&self) -> RawArguments<'_> {
         let mut raw = self.raw();
         raw.quoted = true;
@@ -778,12 +787,11 @@ impl Args {
         let before = self.offset;
         self.restore();
 
-        let pos = match self.iter::<T>().quoted().position(|res| res.is_ok()) {
-            Some(p) => p,
-            None => {
-                self.offset = before;
-                return Err(Error::Eos);
-            },
+        let pos = if let Some(p) = self.iter::<T>().quoted().position(|res| res.is_ok()) {
+            p
+        } else {
+            self.offset = before;
+            return Err(Error::Eos);
         };
 
         self.offset = pos;
@@ -826,12 +834,11 @@ impl Args {
         let before = self.offset;
         self.restore();
 
-        let pos = match self.iter::<T>().quoted().position(|res| res.is_ok()) {
-            Some(p) => p,
-            None => {
-                self.offset = before;
-                return Err(Error::Eos);
-            },
+        let pos = if let Some(p) = self.iter::<T>().quoted().position(|res| res.is_ok()) {
+            p
+        } else {
+            self.offset = before;
+            return Err(Error::Eos);
         };
 
         self.offset = pos;
@@ -844,12 +851,14 @@ impl Args {
 
     /// Get the original, unmodified message passed to the command.
     #[inline]
+    #[must_use]
     pub fn message(&self) -> &str {
         &self.message
     }
 
     /// Starting from the offset, return the remainder of available arguments.
     #[inline]
+    #[must_use]
     pub fn rest(&self) -> &str {
         self.remains().unwrap_or_default()
     }
@@ -858,6 +867,7 @@ impl Args {
     ///
     /// Returns [`None`] if there are no remaining arguments.
     #[inline]
+    #[must_use]
     pub fn remains(&self) -> Option<&str> {
         if self.is_empty() {
             return None;
@@ -874,20 +884,23 @@ impl Args {
     /// # Note
     ///
     /// The value returned is to be assumed to stay static.
-    /// However, if [`Self::find`] was called previously, and was successful, then the value is substracted by one.
+    /// However, if [`Self::find`] was called previously, and was successful, then the value is subtracted by one.
     #[inline]
+    #[must_use]
     pub fn len(&self) -> usize {
         self.args.len()
     }
 
     /// Assert that there are no more arguments left.
     #[inline]
+    #[must_use]
     pub fn is_empty(&self) -> bool {
         self.offset >= self.len()
     }
 
     /// Return the amount of arguments still available.
     #[inline]
+    #[must_use]
     pub fn remaining(&self) -> usize {
         if self.is_empty() {
             return 0;
@@ -930,7 +943,7 @@ impl<'a, T: FromStr> Iter<'a, T> {
         self
     }
 
-    /// Trim leading and trailling whitespace off all arguments.
+    /// Trim leading and trailing whitespace off all arguments.
     #[inline]
     pub fn trimmed(&mut self) -> &mut Self {
         match self.state {
