@@ -4,10 +4,8 @@ use futures::future::BoxFuture;
 
 use super::Delimiter;
 use crate::client::Context;
-use crate::model::{
-    channel::Message,
-    id::{ChannelId, GuildId, UserId},
-};
+use crate::model::channel::Message;
+use crate::model::id::{ChannelId, GuildId, UserId};
 
 type DynamicPrefixHook =
     for<'fut> fn(&'fut Context, &'fut Message) -> BoxFuture<'fut, Option<String>>;
@@ -78,7 +76,7 @@ impl From<(bool, bool, bool)> for WithWhiteSpace {
 /// Responding to mentions and setting a command prefix of `"~"`:
 ///
 /// ```rust,no_run
-/// # use serenity::prelude::EventHandler;
+/// # use serenity::prelude::*;
 /// struct Handler;
 ///
 /// impl EventHandler for Handler {}
@@ -92,7 +90,10 @@ impl From<(bool, bool, bool)> for WithWhiteSpace {
 /// let framework =
 ///     StandardFramework::new().configure(|c| c.on_mention(Some(UserId(5))).prefix("~"));
 ///
-/// let mut client = Client::builder(&token).event_handler(Handler).framework(framework).await?;
+/// let mut client = Client::builder(&token, GatewayIntents::default())
+///     .event_handler(Handler)
+///     .framework(framework)
+///     .await?;
 /// #     Ok(())
 /// # }
 /// ```
@@ -333,16 +334,12 @@ impl Configuration {
     /// use serenity::framework::StandardFramework;
     ///
     /// let framework = StandardFramework::new().configure(|c| {
-    ///     c
-    ///        .dynamic_prefix(|_, msg| Box::pin(async move {
-    ///             Some(if msg.channel_id.0 % 5 == 0 {
-    ///                 "!"
-    ///             } else {
-    ///                 "*"
-    ///             }.to_string())
-    ///         }))
-    ///         // This disables the default prefix "~"
-    ///         .prefix("")
+    ///     c.dynamic_prefix(|_, msg| {
+    ///         Box::pin(
+    ///             async move { Some(if msg.channel_id.0 % 5 == 0 { "!" } else { "*" }.to_string()) },
+    ///         )
+    ///     })
+    ///     .prefix("") // This disables the default prefix "~"
     /// });
     /// ```
     ///
@@ -546,7 +543,7 @@ impl Configuration {
         It: IntoIterator<Item = T>,
     {
         self.delimiters.clear();
-        self.delimiters.extend(delimiters.into_iter().map(|s| s.into()));
+        self.delimiters.extend(delimiters.into_iter().map(Into::into));
 
         self
     }
