@@ -1,12 +1,8 @@
-use std::{
-    sync::Arc,
-    time::{Duration as StdDuration, Instant},
-};
+use std::sync::Arc;
+use std::time::{Duration as StdDuration, Instant};
 
-use async_tungstenite::tungstenite::{
-    error::Error as TungsteniteError,
-    protocol::frame::CloseFrame,
-};
+use async_tungstenite::tungstenite::error::Error as TungsteniteError;
+use async_tungstenite::tungstenite::protocol::frame::CloseFrame;
 use tokio::sync::Mutex;
 use tracing::{debug, error, info, instrument, trace, warn};
 use url::Url;
@@ -20,19 +16,15 @@ use super::{
     WebSocketGatewayClientExt,
     WsStream,
 };
-use crate::client::bridge::gateway::{ChunkGuildFilter, GatewayIntents};
+use crate::client::bridge::gateway::ChunkGuildFilter;
 use crate::constants::{self, close_codes};
+use crate::http::Http;
 use crate::internal::prelude::*;
-#[cfg(feature = "native_tls_backend_marker")]
-use crate::internal::ws_impl::create_native_tls_client;
-#[cfg(all(feature = "rustls_backend_marker", not(feature = "native_tls_backend_marker")))]
-use crate::internal::ws_impl::create_rustls_client;
-use crate::model::{
-    event::{Event, GatewayEvent},
-    gateway::Activity,
-    id::GuildId,
-    user::OnlineStatus,
-};
+use crate::internal::ws_impl::create_client;
+use crate::model::event::{Event, GatewayEvent};
+use crate::model::gateway::{Activity, GatewayIntents};
+use crate::model::id::GuildId;
+use crate::model::user::OnlineStatus;
 
 /// A Shard is a higher-level handler for a websocket connection to Discord's
 /// gateway. The shard allows for sending and receiving messages over the
@@ -79,6 +71,7 @@ pub struct Shard {
     /// [`latency`]: fn@Self::latency
     heartbeat_instants: (Option<Instant>, Option<Instant>),
     heartbeat_interval: Option<u64>,
+    http: Option<Arc<Http>>,
     /// This is used by the heartbeater to determine whether the last
     /// heartbeat was sent without an acknowledgement, and whether to reconnect.
     // This _must_ be set to `true` in `Shard::handle_event`'s
@@ -87,8 +80,6 @@ pub struct Shard {
     seq: u64,
     session_id: Option<String>,
     shard_info: [u64; 2],
-    /// Whether the shard has permanently shutdown.
-    shutdown: bool,
     stage: ConnectionStage,
     /// Instant of when the shard was started.
     // This acts as a timeout to determine if the shard has - for some reason -
@@ -116,10 +107,10 @@ impl Shard {
     /// use tokio::sync::Mutex;
     /// #
     /// # use serenity::http::Http;
-    /// # use serenity::client::bridge::gateway::GatewayIntents;
+    /// # use serenity::model::gateway::GatewayIntents;
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// #     let http = Arc::new(Http::default());
+    /// #     let http = Arc::new(Http::new("token"));
     /// let token = std::env::var("DISCORD_BOT_TOKEN")?;
     /// // retrieve the gateway response, which contains the URL to connect to
     /// let gateway = Arc::new(Mutex::new(http.get_gateway().await?.url));
@@ -132,6 +123,7 @@ impl Shard {
     /// ```
     ///
     /// # Errors
+    ///
     /// On Error, will return either [`Error::Gateway`], [`Error::Tungstenite`]
     /// or a Rustls/native TLS error.
     ///
@@ -154,11 +146,11 @@ impl Shard {
         let session_id = None;
 
         Ok(Shard {
-            shutdown: false,
             client,
             current_presence,
             heartbeat_instants,
             heartbeat_interval,
+            http: None,
             last_heartbeat_acknowledged,
             seq,
             stage,
@@ -171,18 +163,19 @@ impl Shard {
         })
     }
 
+    /// Sets the associated [`Http`] client.
+    ///
+    /// This will update the client's application id after the shard receives a READY payload.
+    pub fn set_http(&mut self, http: Arc<Http>) {
+        self.http = Some(http);
+    }
+
     /// Retrieves the current presence of the shard.
     #[inline]
     pub fn current_presence(&self) -> &CurrentPresence {
         &self.current_presence
     }
 
-    /// Whether the shard has permanently shutdown.
-    #[inline]
-    pub fn is_shutdown(&self) -> bool {
-        self.shutdown
-    }
-
     /// Retrieves the heartbeat instants of the shard.
     ///
     /// This is the time of when a heartbeat was sent and when an
@@ -304,7 +297,7 @@ impl Shard {
     /// ```rust,no_run
     /// # use serenity::gateway::Shard;
     /// # use serenity::prelude::Mutex;
-    /// # use serenity::client::bridge::gateway::GatewayIntents;
+    /// # use serenity::model::gateway::GatewayIntents;
     /// # use std::sync::Arc;
     /// #
     /// # #[cfg(feature = "model")]
@@ -339,6 +332,10 @@ impl Shard {
 
                 self.session_id = Some(ready.ready.session_id.clone());
                 self.stage = ConnectionStage::Connected;
+
+                if let Some(ref http) = self.http {
+                    http.set_application_id(ready.ready.application.id.0);
+                }
             },
             Event::Resumed(_) => {
                 info!("[Shard {:?}] Resumed", self.shard_info);
@@ -370,14 +367,13 @@ impl Shard {
                 self.stage = ConnectionStage::Identifying;
 
                 return ShardAction::Identify;
-            } else {
-                warn!(
-                    "[Shard {:?}] Heartbeat during non-Handshake; auto-reconnecting",
-                    self.shard_info
-                );
-
-                return ShardAction::Reconnect(self.reconnection_type());
             }
+            warn!(
+                "[Shard {:?}] Heartbeat during non-Handshake; auto-reconnecting",
+                self.shard_info
+            );
+
+            return ShardAction::Reconnect(self.reconnection_type());
         }
 
         ShardAction::Heartbeat
@@ -432,7 +428,7 @@ impl Shard {
 
                 return Err(Error::Gateway(GatewayError::OverloadedShard));
             },
-            Some(4006) | Some(close_codes::SESSION_TIMEOUT) => {
+            Some(4006 | close_codes::SESSION_TIMEOUT) => {
                 info!("[Shard {:?}] Invalid session.", self.shard_info);
 
                 self.session_id = None;
@@ -462,8 +458,7 @@ impl Shard {
         }
 
         let resume = num
-            .map(|x| x != close_codes::AUTHENTICATION_FAILED && self.session_id.is_some())
-            .unwrap_or(true);
+            .map_or(true, |x| x != close_codes::AUTHENTICATION_FAILED && self.session_id.is_some());
 
         Ok(Some(if resume {
             ShardAction::Reconnect(ReconnectType::Resume)
@@ -671,7 +666,8 @@ impl Shard {
     ///
     /// ```rust,no_run
     /// # use tokio::sync::Mutex;
-    /// # use serenity::client::bridge::gateway::{GatewayIntents, ChunkGuildFilter};
+    /// # use serenity::model::gateway::GatewayIntents;
+    /// # use serenity::client::bridge::gateway::ChunkGuildFilter;
     /// # use serenity::gateway::Shard;
     /// # use std::sync::Arc;
     /// #
@@ -693,7 +689,8 @@ impl Shard {
     /// ```rust,no_run
     /// # use tokio::sync::Mutex;
     /// # use serenity::gateway::Shard;
-    /// # use serenity::client::bridge::gateway::{GatewayIntents, ChunkGuildFilter};
+    /// # use serenity::model::gateway::GatewayIntents;
+    /// # use serenity::client::bridge::gateway::ChunkGuildFilter;
     /// # use std::error::Error;
     /// # use std::sync::Arc;
     /// #
@@ -813,24 +810,13 @@ impl Shard {
     }
 }
 
-#[cfg(all(feature = "rustls_backend_marker", not(feature = "native_tls_backend_marker")))]
 async fn connect(base_url: &str) -> Result<WsStream> {
-    let url = build_gateway_url(base_url)?;
+    let url =
+        Url::parse(&format!("{}?v={}", base_url, constants::GATEWAY_VERSION)).map_err(|why| {
+            warn!("Error building gateway URL with base `{}`: {:?}", base_url, why);
 
-    Ok(create_rustls_client(url).await?)
-}
+            Error::Gateway(GatewayError::BuildingUrl)
+        })?;
 
-#[cfg(feature = "native_tls_backend_marker")]
-async fn connect(base_url: &str) -> Result<WsStream> {
-    let url = build_gateway_url(base_url)?;
-
-    Ok(create_native_tls_client(url).await?)
-}
-
-fn build_gateway_url(base: &str) -> Result<Url> {
-    Url::parse(&format!("{}?v={}", base, constants::GATEWAY_VERSION)).map_err(|why| {
-        warn!("Error building gateway URL with base `{}`: {:?}", base, why);
-
-        Error::Gateway(GatewayError::BuildingUrl)
-    })
+    create_client(url).await
 }
