@@ -15,12 +15,12 @@ pub fn deserialize_errors<'de, D: Deserializer<'de>>(
 
     let mut errors = Vec::new();
 
-    loop_errors(map, &mut errors, Vec::new());
+    loop_errors(&map, &mut errors, &[]);
 
     Ok(errors)
 }
 
-fn loop_errors(value: Value, errors: &mut Vec<DiscordJsonSingleError>, path: Vec<String>) {
+fn loop_errors(value: &Value, errors: &mut Vec<DiscordJsonSingleError>, path: &[String]) {
     for (key, looped) in value.as_object().expect("expected object").iter() {
         let object = looped.as_object().expect("expected object");
         if object.contains_key("_errors") {
@@ -29,10 +29,19 @@ fn loop_errors(value: Value, errors: &mut Vec<DiscordJsonSingleError>, path: Vec
                 .expect("expected _errors")
                 .as_array()
                 .expect("expected array")
-                .to_owned();
+                .clone();
+
+            // FIXME: This is a workaround to a bug in the {Create,Edit}ScheduledEvent endpoints.
+            // Occasionally there will be an additional nested _errors field. See the relevant
+            // github issue: https://github.com/discord/discord-api-docs/issues/4811
+            let found_errors = match found_errors[0].get("_errors") {
+                Some(array) => array.as_array().expect("expected array"),
+                None => &found_errors,
+            };
+
             for error in found_errors {
                 let error_object = error.as_object().expect("expected object");
-                let mut object_path = path.clone();
+                let mut object_path = path.to_owned();
 
                 object_path.push(key.to_string());
 
@@ -55,9 +64,9 @@ fn loop_errors(value: Value, errors: &mut Vec<DiscordJsonSingleError>, path: Vec
             continue;
         }
 
-        let mut new_path = path.clone();
+        let mut new_path = path.to_owned();
         new_path.push(key.to_string());
 
-        loop_errors(looped.clone(), errors, new_path);
+        loop_errors(looped, errors, &new_path);
     }
 }
