@@ -13,7 +13,7 @@ macro_rules! cdn {
 #[cfg(feature = "http")]
 macro_rules! api {
     ($e:expr) => {
-        concat!("https://discord.com/api/v9", $e)
+        concat!("https://discord.com/api/v10", $e)
     };
     ($e:expr, $($rest:tt)*) => {
         format!(api!($e), $($rest)*)
@@ -43,9 +43,10 @@ macro_rules! feature_cache {
 }
 
 macro_rules! enum_number {
-    ($name:ident { $($variant:ident $(,)? )* }) => {
+    ($name:ident { $($(#[$attr:meta])? $variant:ident $(,)? )* }) => {
         impl $name {
             #[inline]
+            #[must_use]
             pub fn num(&self) -> u64 {
                 *self as u64
             }
@@ -80,7 +81,7 @@ macro_rules! enum_number {
                         // Rust does not come with a simple way of converting a
                         // number to an enum, so use a big `match`.
                         match value {
-                            $( v if v == $name::$variant as u64 => Ok($name::$variant), )*
+                            $( $(#[$attr])? v if v == $name::$variant as u64 => Ok($name::$variant), )*
                             _ => {
                                 tracing::warn!("Unknown {} value: {}", stringify!($name), value);
 
@@ -96,3 +97,53 @@ macro_rules! enum_number {
         }
     }
 }
+
+/// The macro forwards the generation to the `bitflags::bitflags!` macro and implements
+/// the default (de)serialization for Discord's bitmask values.
+///
+/// The flags are created with `T::from_bits_truncate` for the deserialized integer value.
+///
+/// Use the `bitflags::bitflags! macro directly if a different serde implementation is required.
+macro_rules! bitflags {
+    (
+        $(#[$outer:meta])*
+        $vis:vis struct $BitFlags:ident: $T:ty {
+            $(
+                $(#[$inner:ident $($args:tt)*])*
+                const $Flag:ident = $value:expr;
+            )*
+        }
+
+        $($t:tt)*
+    ) => {
+        bitflags::bitflags! {
+            $(#[$outer])*
+            $vis struct $BitFlags: $T {
+                $(
+                    $(#[$inner $($args)*])*
+                    const $Flag = $value;
+                )*
+            }
+        }
+
+        bitflags!(__impl_serde $BitFlags: $T);
+
+        bitflags! {
+            $($t)*
+        }
+    };
+    (__impl_serde $BitFlags:ident: $T:tt) => {
+        impl<'de> serde::de::Deserialize<'de> for $BitFlags {
+            fn deserialize<D: serde::de::Deserializer<'de>>(deserializer: D) -> std::result::Result<Self, D::Error> {
+                Ok(Self::from_bits_truncate(<$T>::deserialize(deserializer)?))
+            }
+        }
+
+        impl serde::ser::Serialize for $BitFlags {
+            fn serialize<S: serde::ser::Serializer>(&self, serializer: S) -> std::result::Result<S::Ok, S::Error> {
+                self.bits().serialize(serializer)
+            }
+        }
+    };
+    () => {};
+}
