@@ -6,9 +6,5 @@ pub mod prelude;
 #[cfg(feature = "gateway")]
 pub mod ws_impl;
 
-#[cfg(any(feature = "tokio", feature = "tokio_compat"))]
+#[cfg(feature = "tokio")]
 pub mod tokio;
-
-pub fn is_false(b: &bool) -> bool {
-    !b
-}
