@@ -4,13 +4,12 @@
 //! These are not publicly re-exported to the end user, and must stay as a
 //! private module.
 
-pub type JsonMap = Map<String, Value>;
-
 pub use std::result::Result as StdResult;
 
-pub use serde_json::{Map, Number, Value};
+#[cfg(feature = "simd-json")]
+pub use simd_json::{Mutable, Value as ValueTrait, ValueAccess};
 
-pub use super::is_false;
 #[cfg(feature = "client")]
 pub use crate::client::ClientError;
 pub use crate::error::{Error, Result};
+pub use crate::json::{JsonMap, Value};
