@@ -1,6 +1,6 @@
 use std::future::Future;
 
-#[cfg(all(tokio_unstable, not(feature = "tokio_compat")))]
+#[cfg(all(tokio_unstable, feature = "tokio_task_builder"))]
 pub fn spawn_named<F, T>(name: &str, future: F) -> tokio::task::JoinHandle<T>
 where
     F: Future<Output = T> + Send + 'static,
@@ -9,7 +9,7 @@ where
     tokio::task::Builder::new().name(&*format!("serenity::{}", name)).spawn(future)
 }
 
-#[cfg(any(not(tokio_unstable), feature = "tokio_compat"))]
+#[cfg(not(all(tokio_unstable, feature = "tokio_task_builder")))]
 pub fn spawn_named<F, T>(_name: &str, future: F) -> tokio::task::JoinHandle<T>
 where
     F: Future<Output = T> + Send + 'static,
