@@ -1,10 +1,10 @@
 #[cfg(feature = "model")]
 use reqwest::Client as ReqwestClient;
 
-use crate::internal::is_false;
 #[cfg(feature = "model")]
 use crate::internal::prelude::*;
 use crate::model::id::AttachmentId;
+use crate::model::utils::is_false;
 
 /// A file uploaded with a message. Not to be confused with [`Embed`]s.
 ///
@@ -45,6 +45,7 @@ pub struct Attachment {
 impl Attachment {
     /// If this attachment is an image, then a tuple of the width and height
     /// in pixels is returned.
+    #[must_use]
     pub fn dimensions(&self) -> Option<(u64, u64)> {
         self.width.and_then(|width| self.height.map(|height| (width, height)))
     }
@@ -111,7 +112,8 @@ impl Attachment {
     ///     }
     /// }
     /// let token = std::env::var("DISCORD_TOKEN")?;
-    /// let mut client = Client::builder(&token).event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder(&token, GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// client.start().await?;
     /// #     Ok(())
