@@ -1,18 +1,10 @@
 #[cfg(feature = "model")]
-use std::fmt::Write as FmtWrite;
+use std::fmt::Write as _;
 #[cfg(feature = "model")]
 use std::sync::Arc;
 
-#[cfg(feature = "model")]
-use bytes::buf::Buf;
 #[cfg(feature = "model")]
 use futures::stream::Stream;
-#[cfg(feature = "model")]
-use reqwest::Url;
-#[cfg(feature = "model")]
-use serde_json::json;
-#[cfg(feature = "model")]
-use tokio::{fs::File, io::AsyncReadExt};
 
 #[cfg(feature = "model")]
 use crate::builder::{
@@ -38,10 +30,12 @@ use crate::collector::{
     ReactionCollectorBuilder,
 };
 #[cfg(feature = "model")]
-use crate::http::{AttachmentType, CacheHttp, Http, Typing};
+use crate::http::{CacheHttp, Http, Typing};
+#[cfg(feature = "model")]
+use crate::json::{self, json};
+#[cfg(feature = "model")]
+use crate::model::channel::AttachmentType;
 use crate::model::prelude::*;
-#[cfg(all(feature = "model", feature = "utils"))]
-use crate::utils;
 
 #[cfg(feature = "model")]
 impl ChannelId {
@@ -62,7 +56,7 @@ impl ChannelId {
     /// use serenity::model::id::ChannelId;
     ///
     /// # async fn run() {
-    /// # let http = serenity::http::Http::default();
+    /// # let http = serenity::http::Http::new("token");
     /// let _successful = ChannelId(7).broadcast_typing(&http).await;
     /// # }
     /// ```
@@ -80,14 +74,13 @@ impl ChannelId {
 
     /// Creates an invite leading to the given channel.
     ///
-    /// **Note**: Requres the [Create Invite] permission.
+    /// **Note**: Requires the [Create Instant Invite] permission.
     ///
     /// # Errors
     ///
     /// Returns [`Error::Http`] if the current user lacks permission.
     ///
-    /// [Create Invite]: Permissions::CREATE_INVITE
-    #[cfg(feature = "utils")]
+    /// [Create Instant Invite]: Permissions::CREATE_INSTANT_INVITE
     pub async fn create_invite<F>(&self, http: impl AsRef<Http>, f: F) -> Result<RichInvite>
     where
         F: FnOnce(&mut CreateInvite) -> &mut CreateInvite,
@@ -95,9 +88,9 @@ impl ChannelId {
         let mut invite = CreateInvite::default();
         f(&mut invite);
 
-        let map = utils::hashmap_to_json_map(invite.0);
+        let map = json::hashmap_to_json_map(invite.0);
 
-        http.as_ref().create_invite(self.0, &map).await
+        http.as_ref().create_invite(self.0, &map, None).await
     }
 
     /// Creates a [permission overwrite][`PermissionOverwrite`] for either a
@@ -322,7 +315,7 @@ impl ChannelId {
     /// # async fn run() {
     /// #     use serenity::http::Http;
     /// #     use serenity::model::id::ChannelId;
-    /// #     let http = Http::default();
+    /// #     let http = Http::new("token");
     /// #     let channel_id = ChannelId(1234);
     /// channel_id.edit(&http, |c| c.name("test").bitrate(64000)).await;
     /// # }
@@ -334,7 +327,6 @@ impl ChannelId {
     /// or if an invalid value is set.
     ///
     /// [Manage Channel]: Permissions::MANAGE_CHANNELS
-    #[cfg(feature = "utils")]
     #[inline]
     pub async fn edit<F>(self, http: impl AsRef<Http>, f: F) -> Result<GuildChannel>
     where
@@ -343,9 +335,9 @@ impl ChannelId {
         let mut channel = EditChannel::default();
         f(&mut channel);
 
-        let map = utils::hashmap_to_json_map(channel.0);
+        let map = json::hashmap_to_json_map(channel.0);
 
-        http.as_ref().edit_channel(self.0, &map).await
+        http.as_ref().edit_channel(self.0, &map, None).await
     }
 
     /// Edits a [`Message`] in the channel given its Id.
@@ -363,58 +355,71 @@ impl ChannelId {
     /// is over the [`the limit`], containing the number of unicode code points
     /// over the limit.
     ///
-    /// [`EditMessage`]: crate::builder::EditMessage
     /// [`the limit`]: crate::builder::EditMessage::content
-    #[cfg(feature = "utils")]
     #[inline]
-    pub async fn edit_message<F>(
+    pub async fn edit_message<'a, F>(
         self,
         http: impl AsRef<Http>,
         message_id: impl Into<MessageId>,
         f: F,
     ) -> Result<Message>
     where
-        F: FnOnce(&mut EditMessage) -> &mut EditMessage,
+        F: for<'b> FnOnce(&'b mut EditMessage<'a>) -> &'b mut EditMessage<'a>,
     {
         let mut msg = EditMessage::default();
         f(&mut msg);
 
-        if let Some(Value::String(ref content)) = msg.0.get("content") {
+        if let Some(Value::String(content)) = msg.0.get("content") {
             if let Some(length_over) = Message::overflow_length(content) {
                 return Err(Error::Model(ModelError::MessageTooLong(length_over)));
             }
         }
 
-        let map = utils::hashmap_to_json_map(msg.0);
+        let map = json::hashmap_to_json_map(msg.0);
 
-        http.as_ref().edit_message(self.0, message_id.into().0, &Value::Object(map)).await
+        http.as_ref()
+            .edit_message_and_attachments(self.0, message_id.into().0, &Value::from(map), msg.1)
+            .await
     }
 
     /// Attempts to find a [`Channel`] by its Id in the cache.
     #[cfg(feature = "cache")]
     #[inline]
-    pub async fn to_channel_cached(self, cache: impl AsRef<Cache>) -> Option<Channel> {
-        cache.as_ref().channel(self).await
+    pub fn to_channel_cached(self, cache: impl AsRef<Cache>) -> Option<Channel> {
+        cache.as_ref().channel(self)
     }
 
     /// First attempts to find a [`Channel`] by its Id in the cache,
     /// upon failure requests it via the REST API.
     ///
-    /// **Note**: If the `cache`-feature is enabled permissions will be checked and upon
-    /// owning the required permissions the HTTP-request will be issued.
+    /// **Note**: If the `cache`-feature is enabled permissions will be checked and upon owning the
+    /// required permissions the HTTP-request will be issued. Additionally, you might want to
+    /// enable the `temp_cache` feature to cache channel data retrieved by this function for a
+    /// short duration.
     #[allow(clippy::missing_errors_doc)]
     #[inline]
     pub async fn to_channel(self, cache_http: impl CacheHttp) -> Result<Channel> {
         #[cfg(feature = "cache")]
         {
             if let Some(cache) = cache_http.cache() {
-                if let Some(channel) = cache.channel(self).await {
+                if let Some(channel) = cache.channel(self) {
                     return Ok(channel);
                 }
             }
         }
 
-        cache_http.http().get_channel(self.0).await
+        let channel = cache_http.http().get_channel(self.0).await?;
+
+        #[cfg(all(feature = "cache", feature = "temp_cache"))]
+        {
+            if let Some(cache) = cache_http.cache() {
+                if let Channel::Guild(guild_channel) = &channel {
+                    cache.temp_channels.insert(guild_channel.id, guild_channel.clone());
+                }
+            }
+        }
+
+        Ok(channel)
     }
 
     /// Gets all of the channel's invites.
@@ -465,7 +470,6 @@ impl ChannelId {
     /// Returns [`Error::Http`] if the current user does not have
     /// permission to view the channel.
     ///
-    /// [`GetMessages`]: crate::builder::GetMessages
     /// [Read Message History]: Permissions::READ_MESSAGE_HISTORY
     pub async fn messages<F>(self, http: impl AsRef<Http>, builder: F) -> Result<Vec<Message>>
     where
@@ -511,7 +515,7 @@ impl ChannelId {
     /// #
     /// # async fn run() {
     /// # let channel_id = ChannelId::default();
-    /// # let ctx = Http::default();
+    /// # let ctx = Http::new("token");
     /// use serenity::futures::StreamExt;
     /// use serenity::model::channel::MessagesIter;
     ///
@@ -531,7 +535,7 @@ impl ChannelId {
     /// Returns the name of whatever channel this id holds.
     #[cfg(feature = "cache")]
     pub async fn name(self, cache: impl AsRef<Cache>) -> Option<String> {
-        let channel = self.to_channel_cached(cache).await?;
+        let channel = self.to_channel_cached(cache)?;
 
         Some(match channel {
             Channel::Guild(channel) => channel.name().to_string(),
@@ -552,7 +556,7 @@ impl ChannelId {
     /// [Manage Messages]: Permissions::MANAGE_MESSAGES
     #[inline]
     pub async fn pin(self, http: impl AsRef<Http>, message_id: impl Into<MessageId>) -> Result<()> {
-        http.as_ref().pin_message(self.0, message_id.into().0).await
+        http.as_ref().pin_message(self.0, message_id.into().0, None).await
     }
 
     /// Crossposts a [`Message`].
@@ -666,7 +670,7 @@ impl ChannelId {
     /// # use std::sync::Arc;
     /// #
     /// # async fn run() {
-    /// # let http = Arc::new(Http::default());
+    /// # let http = Arc::new(Http::new("token"));
     /// use serenity::model::id::ChannelId;
     ///
     /// let channel_id = ChannelId(7);
@@ -684,7 +688,7 @@ impl ChannelId {
     /// # use std::sync::Arc;
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// # let http = Arc::new(Http::default());
+    /// # let http = Arc::new(Http::new("token"));
     /// use serenity::model::id::ChannelId;
     /// use tokio::fs::File;
     ///
@@ -715,7 +719,6 @@ impl ChannelId {
     /// [Attach Files]: Permissions::ATTACH_FILES
     /// [Send Messages]: Permissions::SEND_MESSAGES
     /// [`File`]: tokio::fs::File
-    #[cfg(feature = "utils")]
     pub async fn send_files<'a, F, T, It>(
         self,
         http: impl AsRef<Http>,
@@ -730,12 +733,11 @@ impl ChannelId {
         let mut create_message = CreateMessage::default();
         let msg = f(&mut create_message);
 
-        let map = utils::hashmap_to_json_map(msg.0.clone());
+        let map = json::hashmap_to_json_map(msg.0.clone());
 
-        Message::check_content_length(&map)?;
-        Message::check_embed_length(&map)?;
+        Message::check_lengths(&map)?;
 
-        http.as_ref().send_files(self.0, files, map).await
+        http.as_ref().send_files(self.0, files, &map).await
     }
 
     /// Sends a message to the channel.
@@ -756,28 +758,28 @@ impl ChannelId {
     /// Returns [`Error::Http`] if the current user lacks permission to
     /// send a message in this channel.
     ///
-    /// [`CreateMessage`]: crate::builder::CreateMessage
     /// [Send Messages]: Permissions::SEND_MESSAGES
-    #[cfg(feature = "utils")]
     pub async fn send_message<'a, F>(self, http: impl AsRef<Http>, f: F) -> Result<Message>
     where
         for<'b> F: FnOnce(&'b mut CreateMessage<'a>) -> &'b mut CreateMessage<'a>,
     {
         let mut create_message = CreateMessage::default();
-        let msg = f(&mut create_message);
+        f(&mut create_message);
+        self._send_message(http.as_ref(), create_message).await
+    }
 
-        let map = utils::hashmap_to_json_map(msg.0.clone());
+    async fn _send_message<'a>(self, http: &Http, msg: CreateMessage<'a>) -> Result<Message> {
+        let map = json::hashmap_to_json_map(msg.0);
 
-        Message::check_content_length(&map)?;
-        Message::check_embed_length(&map)?;
+        Message::check_lengths(&map)?;
 
         let message = if msg.2.is_empty() {
-            http.as_ref().send_message(self.0, &Value::Object(map)).await?
+            http.as_ref().send_message(self.0, &Value::from(map)).await?
         } else {
-            http.as_ref().send_files(self.0, msg.2.clone(), map).await?
+            http.as_ref().send_files(self.0, msg.2, &map).await?
         };
 
-        if let Some(reactions) = msg.1.clone() {
+        if let Some(reactions) = msg.1 {
             for reaction in reactions {
                 self.create_reaction(&http, message.id, reaction).await?;
             }
@@ -807,7 +809,7 @@ impl ChannelId {
     /// #
     /// # fn long_process() {}
     /// # fn main() -> Result<()> {
-    /// # let http = Arc::new(Http::default());
+    /// # let http = Arc::new(Http::new("token"));
     /// // Initiate typing (assuming http is `Arc<Http>`)
     /// let typing = ChannelId(7).start_typing(&http)?;
     ///
@@ -844,7 +846,7 @@ impl ChannelId {
         http: impl AsRef<Http>,
         message_id: impl Into<MessageId>,
     ) -> Result<()> {
-        http.as_ref().unpin_message(self.0, message_id.into().0).await
+        http.as_ref().unpin_message(self.0, message_id.into().0, None).await
     }
 
     /// Retrieves the channel's webhooks.
@@ -871,11 +873,11 @@ impl ChannelId {
         http: impl AsRef<Http>,
         name: impl std::fmt::Display,
     ) -> Result<Webhook> {
-        let map = serde_json::json!({
+        let map = json!({
             "name": name.to_string(),
         });
 
-        http.as_ref().create_webhook(self.0, &map).await
+        http.as_ref().create_webhook(self.0, &map, None).await
     }
 
     /// Creates a webhook with a name and an avatar.
@@ -890,82 +892,43 @@ impl ChannelId {
         name: impl std::fmt::Display,
         avatar: impl Into<AttachmentType<'a>>,
     ) -> Result<Webhook> {
-        let name = name.to_string();
-        let avatar = avatar.into();
-
-        let avatar = match avatar {
-            AttachmentType::Bytes {
-                data,
-                filename: _,
-            } => "data:image/png;base64,".to_string() + &base64::encode(&data.into_owned()),
-            AttachmentType::File {
-                file,
-                filename: _,
-            } => {
-                let mut buf = Vec::new();
-                file.try_clone().await?.read_to_end(&mut buf).await?;
-
-                "data:image/png;base64,".to_string() + &base64::encode(&buf)
-            },
-            AttachmentType::Path(path) => {
-                let mut file = File::open(path).await?;
-                let mut buf = vec![];
-                file.read_to_end(&mut buf).await?;
-
-                "data:image/png;base64,".to_string() + &base64::encode(&buf)
-            },
-            AttachmentType::Image(url) => {
-                let url = Url::parse(url).map_err(|_| Error::Url(url.to_string()))?;
-                let response = http.as_ref().client.get(url).send().await?;
-                let mut bytes = response.bytes().await?;
-                let mut picture: Vec<u8> = vec![0; bytes.len()];
-                bytes.copy_to_slice(&mut picture[..]);
-
-                "data:image/png;base64,".to_string() + &base64::encode(&picture)
-            },
-        };
-
-        let map = serde_json::json!({
-            "name": name,
-            "avatar": avatar
+        let http = http.as_ref();
+        let data = avatar.into().data(&http.client).await?;
+        let map = json!({
+            "name": name.to_string(),
+            "avatar": format!("data:image/png;base64,{}", base64::encode(data)),
         });
 
-        http.as_ref().create_webhook(self.0, &map).await
+        http.create_webhook(self.0, &map, None).await
     }
 
     /// Returns a future that will await one message sent in this channel.
     #[cfg(feature = "collector")]
-    pub fn await_reply<'a>(
-        &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> CollectReply<'a> {
+    pub fn await_reply(&self, shard_messenger: impl AsRef<ShardMessenger>) -> CollectReply {
         CollectReply::new(shard_messenger).channel_id(self.0)
     }
 
     /// Returns a stream builder which can be awaited to obtain a stream of messages in this channel.
     #[cfg(feature = "collector")]
-    pub fn await_replies<'a>(
+    pub fn await_replies(
         &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> MessageCollectorBuilder<'a> {
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> MessageCollectorBuilder {
         MessageCollectorBuilder::new(shard_messenger).channel_id(self.0)
     }
 
     /// Await a single reaction in this guild.
     #[cfg(feature = "collector")]
-    pub fn await_reaction<'a>(
-        &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> CollectReaction<'a> {
+    pub fn await_reaction(&self, shard_messenger: impl AsRef<ShardMessenger>) -> CollectReaction {
         CollectReaction::new(shard_messenger).channel_id(self.0)
     }
 
     /// Returns a stream builder which can be awaited to obtain a stream of reactions sent in this channel.
     #[cfg(feature = "collector")]
-    pub fn await_reactions<'a>(
+    pub fn await_reactions(
         &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> ReactionCollectorBuilder<'a> {
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> ReactionCollectorBuilder {
         ReactionCollectorBuilder::new(shard_messenger).channel_id(self.0)
     }
 
@@ -996,9 +959,9 @@ impl ChannelId {
         let mut instance = CreateStageInstance::default();
         f(&mut instance);
 
-        let map = utils::hashmap_to_json_map(instance.0);
+        let map = json::hashmap_to_json_map(instance.0);
 
-        http.as_ref().create_stage_instance(&Value::Object(map)).await
+        http.as_ref().create_stage_instance(&Value::from(map)).await
     }
 
     /// Edits a stage instance.
@@ -1018,9 +981,9 @@ impl ChannelId {
         let mut instance = EditStageInstance::default();
         f(&mut instance);
 
-        let map = utils::hashmap_to_json_map(instance.0);
+        let map = json::hashmap_to_json_map(instance.0);
 
-        http.as_ref().edit_stage_instance(self.0, &Value::Object(map)).await
+        http.as_ref().edit_stage_instance(self.0, &Value::from(map)).await
     }
 
     /// Edits a thread.
@@ -1035,7 +998,7 @@ impl ChannelId {
         let mut instance = EditThread::default();
         f(&mut instance);
 
-        let map = utils::hashmap_to_json_map(instance.0);
+        let map = json::hashmap_to_json_map(instance.0);
 
         http.as_ref().edit_thread(self.0, &map).await
     }
@@ -1067,7 +1030,7 @@ impl ChannelId {
         let mut instance = CreateThread::default();
         f(&mut instance);
 
-        let map = utils::hashmap_to_json_map(instance.0);
+        let map = json::hashmap_to_json_map(instance.0);
 
         http.as_ref().create_public_thread(self.0, message_id.into().0, &map).await
     }
@@ -1089,7 +1052,7 @@ impl ChannelId {
         instance.kind(ChannelType::PrivateThread);
         f(&mut instance);
 
-        let map = utils::hashmap_to_json_map(instance.0);
+        let map = json::hashmap_to_json_map(instance.0);
 
         http.as_ref().create_private_thread(self.0, &map).await
     }
@@ -1143,18 +1106,6 @@ impl ChannelId {
         http.as_ref().remove_thread_channel_member(self.0, user_id.into()).await
     }
 
-    /// Gets all active threads of a channel.
-    ///
-    /// # Errors
-    ///
-    /// It may return an [`Error::Http`] if the bot doesn't have the
-    /// permission to get it.
-    #[deprecated(note = "Use GuildId::get_active_threads instead")]
-    pub async fn get_active_threads(&self, http: impl AsRef<Http>) -> Result<ThreadsData> {
-        #[allow(deprecated)]
-        http.as_ref().get_channel_active_threads(self.0).await
-    }
-
     /// Gets private archived threads of a channel.
     ///
     /// # Errors
@@ -1326,7 +1277,7 @@ impl<H: AsRef<Http>> MessagesIter<H> {
     /// #
     /// # async fn run() {
     /// # let channel_id = ChannelId::default();
-    /// # let ctx = Http::default();
+    /// # let ctx = Http::new("token");
     /// use serenity::futures::StreamExt;
     /// use serenity::model::channel::MessagesIter;
     ///
