@@ -1,9 +1,7 @@
-#[cfg(feature = "model")]
-use std::fmt::{Display, Formatter, Result as FmtResult};
+use std::fmt;
 #[cfg(feature = "model")]
 use std::sync::Arc;
 
-use chrono::{DateTime, Utc};
 #[cfg(feature = "cache")]
 use futures::stream::StreamExt;
 
@@ -33,19 +31,20 @@ use crate::collector::{
     ReactionCollectorBuilder,
 };
 #[cfg(feature = "model")]
-use crate::http::AttachmentType;
-#[cfg(feature = "model")]
 use crate::http::{CacheHttp, Http, Typing};
 #[cfg(all(feature = "cache", feature = "model"))]
 use crate::internal::prelude::*;
+#[cfg(feature = "model")]
+use crate::json::{self, from_number};
+#[cfg(feature = "model")]
+use crate::model::channel::AttachmentType;
 use crate::model::prelude::*;
-#[cfg(all(feature = "model", feature = "utils"))]
-use crate::utils as serenity_utils;
+use crate::model::Timestamp;
 
 /// Represents a guild's text, news, or voice channel. Some methods are available
 /// only for voice channels and some are only available for text channels.
 /// News channels are a subset of text channels and lack slow mode hence
-/// [`Self::slow_mode_rate`] will be [`None`].
+/// [`Self::rate_limit_per_user`] will be [`None`].
 #[derive(Clone, Debug, Deserialize, Serialize)]
 #[non_exhaustive]
 pub struct GuildChannel {
@@ -57,9 +56,10 @@ pub struct GuildChannel {
     ///
     /// **Note**: This is only available for voice and stage channels.
     pub bitrate: Option<u64>,
-    /// Whether this guild channel belongs in a category.
-    #[serde(rename = "parent_id")]
-    pub category_id: Option<ChannelId>,
+    /// The Id of the parent category for a channel, or of the parent text channel for a thread.
+    ///
+    /// **Note**: This is only available for channels in a category and thread channels.
+    pub parent_id: Option<ChannelId>,
     /// The Id of the guild the channel is located in.
     ///
     /// If this matches with the [`id`], then this is the default text channel.
@@ -77,7 +77,7 @@ pub struct GuildChannel {
     /// The timestamp of the time a pin was most recently made.
     ///
     /// **Note**: This is only available for text channels.
-    pub last_pin_timestamp: Option<DateTime<Utc>>,
+    pub last_pin_timestamp: Option<Timestamp>,
     /// The name of the channel.
     pub name: String,
     /// Permission overwrites for [`Member`]s and for [`Role`]s.
@@ -108,9 +108,8 @@ pub struct GuildChannel {
     /// **Note**: This is only available for text channels excluding news
     /// channels.
     #[doc(alias = "slowmode")]
-    #[deprecated(note = "will be renamed to `rate_limit_per_user` with serenity 0.11")]
-    #[serde(default, rename = "rate_limit_per_user")]
-    pub slow_mode_rate: Option<u64>,
+    #[serde(default)]
+    pub rate_limit_per_user: Option<u64>,
     /// The region override.
     ///
     /// **Note**: This is only available for voice and stage channels. [`None`]
@@ -144,6 +143,7 @@ pub struct GuildChannel {
 impl GuildChannel {
     /// Whether or not this channel is text-based, meaning that
     /// it is possible to send messages.
+    #[must_use]
     pub fn is_text_based(&self) -> bool {
         matches!(self.kind, ChannelType::Text | ChannelType::News)
     }
@@ -167,7 +167,7 @@ impl GuildChannel {
 
     /// Creates an invite leading to the given channel.
     ///
-    /// **Note**: Requres the [Create Invite] permission.
+    /// **Note**: Requires the [Create Instant Invite] permission.
     ///
     /// # Examples
     ///
@@ -184,7 +184,7 @@ impl GuildChannel {
     ///
     /// Otherwise returns [`Error::Http`] if the current user lacks permission.
     ///
-    /// [Create Instant Invite]: Permissions::CREATE_INVITE
+    /// [Create Instant Invite]: Permissions::CREATE_INSTANT_INVITE
     #[inline]
     #[cfg(feature = "utils")]
     pub async fn create_invite<F>(&self, cache_http: impl CacheHttp, f: F) -> Result<RichInvite>
@@ -198,9 +198,8 @@ impl GuildChannel {
                     cache,
                     self.id,
                     Some(self.guild_id),
-                    Permissions::CREATE_INVITE,
-                )
-                .await?;
+                    Permissions::CREATE_INSTANT_INVITE,
+                )?;
             }
         }
 
@@ -226,10 +225,9 @@ impl GuildChannel {
     /// # #[cfg(feature = "cache")]
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
     /// # use serenity::{cache::Cache, http::Http, model::id::{ChannelId, UserId}};
-    /// # use tokio::sync::RwLock;
     /// # use std::sync::Arc;
     /// #
-    /// #     let http = Arc::new(Http::default());
+    /// #     let http = Arc::new(Http::new("token"));
     /// #     let cache = Cache::default();
     /// #     let (channel_id, user_id) = (ChannelId(0), UserId(0));
     /// #
@@ -243,7 +241,7 @@ impl GuildChannel {
     ///     kind: PermissionOverwriteType::Member(user_id),
     /// };
     /// // assuming the cache has been unlocked
-    /// let channel = cache.guild_channel(channel_id).await.ok_or(ModelError::ItemMissing)?;
+    /// let channel = cache.guild_channel(channel_id).ok_or(ModelError::ItemMissing)?;
     ///
     /// channel.create_permission(&http, &overwrite).await?;
     /// #   Ok(())
@@ -259,15 +257,14 @@ impl GuildChannel {
     /// # #[cfg(feature = "cache")]
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
     /// # use serenity::{cache::Cache, http::Http, model::id::{ChannelId, UserId}};
-    /// # use tokio::sync::RwLock;
     /// # use std::sync::Arc;
     /// #
-    /// #   let http = Arc::new(Http::default());
+    /// #   let http = Arc::new(Http::new("token"));
     /// #   let cache = Cache::default();
     /// #   let (channel_id, user_id) = (ChannelId(0), UserId(0));
     /// #
-    /// use serenity::model::channel::{PermissionOverwrite, PermissionOverwriteType};
-    /// use serenity::model::{channel::Channel, ModelError, Permissions};
+    /// use serenity::model::channel::{Channel, PermissionOverwrite, PermissionOverwriteType};
+    /// use serenity::model::{ModelError, Permissions};
     ///
     /// let allow = Permissions::SEND_MESSAGES;
     /// let deny = Permissions::SEND_TTS_MESSAGES | Permissions::ATTACH_FILES;
@@ -277,7 +274,7 @@ impl GuildChannel {
     ///     kind: PermissionOverwriteType::Member(user_id),
     /// };
     ///
-    /// let channel = cache.guild_channel(channel_id).await.ok_or(ModelError::ItemMissing)?;
+    /// let channel = cache.guild_channel(channel_id).ok_or(ModelError::ItemMissing)?;
     ///
     /// channel.create_permission(&http, &overwrite).await?;
     /// #     Ok(())
@@ -304,7 +301,7 @@ impl GuildChannel {
 
     /// Deletes this channel, returning the channel on a successful deletion.
     ///
-    /// **Note**: Requres the [Manage Channels] permission.
+    /// **Note**: Requires the [Manage Channels] permission.
     ///
     /// # Errors
     ///
@@ -323,8 +320,7 @@ impl GuildChannel {
                     self.id,
                     Some(self.guild_id),
                     Permissions::MANAGE_CHANNELS,
-                )
-                .await?;
+                )?;
             }
         }
 
@@ -416,7 +412,6 @@ impl GuildChannel {
     /// if the current user lacks permission to edit the channel.
     ///
     /// Otherwise returns [`Error::Http`] if the current user lacks permission.
-    #[cfg(feature = "utils")]
     pub async fn edit<F>(&mut self, cache_http: impl CacheHttp, f: F) -> Result<()>
     where
         F: FnOnce(&mut EditChannel) -> &mut EditChannel,
@@ -429,20 +424,19 @@ impl GuildChannel {
                     self.id,
                     Some(self.guild_id),
                     Permissions::MANAGE_CHANNELS,
-                )
-                .await?;
+                )?;
             }
         }
 
         let mut map = HashMap::new();
-        map.insert("name", Value::String(self.name.clone()));
-        map.insert("position", Value::Number(Number::from(self.position)));
+        map.insert("name", Value::from(self.name.clone()));
+        map.insert("position", from_number(self.position));
 
         let mut edit_channel = EditChannel::default();
         f(&mut edit_channel);
-        let edited = serenity_utils::hashmap_to_json_map(edit_channel.0);
+        let edited = json::hashmap_to_json_map(edit_channel.0);
 
-        *self = cache_http.http().edit_channel(self.id.0, &edited).await?;
+        *self = cache_http.http().edit_channel(self.id.0, &edited, None).await?;
 
         Ok(())
     }
@@ -462,17 +456,16 @@ impl GuildChannel {
     /// is over the [`the limit`], containing the number of unicode code points
     /// over the limit.
     ///
-    /// [`EditMessage`]: crate::builder::EditMessage
     /// [`the limit`]: crate::builder::EditMessage::content
     #[inline]
-    pub async fn edit_message<F>(
+    pub async fn edit_message<'a, F>(
         &self,
         http: impl AsRef<Http>,
         message_id: impl Into<MessageId>,
         f: F,
     ) -> Result<Message>
     where
-        F: FnOnce(&mut EditMessage) -> &mut EditMessage,
+        F: for<'b> FnOnce(&'b mut EditMessage<'a>) -> &'b mut EditMessage<'a>,
     {
         self.id.edit_message(&http, message_id, f).await
     }
@@ -508,14 +501,14 @@ impl GuildChannel {
     /// # use std::sync::Arc;
     /// # use serenity::{cache::Cache, http::Http, model::id::{ChannelId, UserId}};
     /// #
-    /// #     let http = Arc::new(Http::default());
+    /// #     let http = Arc::new(Http::new("token"));
     /// #     let cache = Cache::default();
     /// #     let (channel_id, user_id) = (ChannelId(0), UserId(0));
     /// #
     /// use serenity::model::ModelError;
     ///
     /// // assuming the cache has been unlocked
-    /// let channel = cache.guild_channel(channel_id).await.ok_or(ModelError::ItemMissing)?;
+    /// let channel = cache.guild_channel(channel_id).ok_or(ModelError::ItemMissing)?;
     ///
     /// channel.edit_voice_state(&http, user_id, |v| v.suppress(false)).await?;
     /// #   Ok(())
@@ -558,14 +551,14 @@ impl GuildChannel {
     /// # use std::sync::Arc;
     /// # use serenity::{cache::Cache, http::Http, model::id::ChannelId};
     /// #
-    /// #     let http = Arc::new(Http::default());
+    /// #     let http = Arc::new(Http::new("token"));
     /// #     let cache = Cache::default();
     /// #     let channel_id = ChannelId(0);
     /// #
     /// use serenity::model::ModelError;
     ///
     /// // assuming the cache has been unlocked
-    /// let channel = cache.guild_channel(channel_id).await.ok_or(ModelError::ItemMissing)?;
+    /// let channel = cache.guild_channel(channel_id).ok_or(ModelError::ItemMissing)?;
     ///
     /// // Send a request to speak
     /// channel.edit_own_voice_state(&http, |v| v.request_to_speak(true)).await?;
@@ -606,9 +599,9 @@ impl GuildChannel {
         let mut voice_state = EditVoiceState::default();
         f(&mut voice_state);
 
-        voice_state.0.insert("channel_id", Value::String(self.id.0.to_string()));
+        voice_state.0.insert("channel_id", Value::from(self.id.0.to_string()));
 
-        let map = serenity_utils::hashmap_to_json_map(voice_state.0);
+        let map = json::hashmap_to_json_map(voice_state.0);
 
         if let Some(id) = user_id {
             http.as_ref().edit_voice_state(self.guild_id.0, id.into().0, &map).await
@@ -620,8 +613,8 @@ impl GuildChannel {
     /// Attempts to find this channel's guild in the Cache.
     #[cfg(feature = "cache")]
     #[inline]
-    pub async fn guild(&self, cache: impl AsRef<Cache>) -> Option<Guild> {
-        cache.as_ref().guild(self.guild_id).await
+    pub fn guild(&self, cache: impl AsRef<Cache>) -> Option<Guild> {
+        cache.as_ref().guild(self.guild_id)
     }
 
     /// Gets all of the channel's invites.
@@ -643,6 +636,7 @@ impl GuildChannel {
     /// Only [text channels][`ChannelType::Text`] are taken into consideration
     /// as being NSFW. [voice channels][`ChannelType::Voice`] are never NSFW.
     #[inline]
+    #[must_use]
     pub fn is_nsfw(&self) -> bool {
         self.kind == ChannelType::Text && self.nsfw
     }
@@ -679,7 +673,6 @@ impl GuildChannel {
     /// Returns [`Error::Http`] if the current user lacks permission to
     /// view the channel.
     ///
-    /// [`GetMessages`]: crate::builder::GetMessages
     /// [Read Message History]: Permissions::READ_MESSAGE_HISTORY
     #[inline]
     pub async fn messages<F>(&self, http: impl AsRef<Http>, builder: F) -> Result<Vec<Message>>
@@ -690,6 +683,7 @@ impl GuildChannel {
     }
 
     /// Returns the name of the guild channel.
+    #[must_use]
     pub fn name(&self) -> &str {
         &self.name
     }
@@ -712,20 +706,20 @@ impl GuildChannel {
     /// #[serenity::async_trait]
     /// impl EventHandler for Handler {
     ///     async fn message(&self, context: Context, msg: Message) {
-    ///         let channel = match context.cache.guild_channel(msg.channel_id).await {
+    ///         let channel = match context.cache.guild_channel(msg.channel_id) {
     ///             Some(channel) => channel,
     ///             None => return,
     ///         };
     ///
-    ///         if let Ok(permissions) = channel.permissions_for_user(&context.cache, &msg.author).await
-    ///         {
+    ///         if let Ok(permissions) = channel.permissions_for_user(&context.cache, &msg.author) {
     ///             println!("The user's permissions: {:?}", permissions);
     ///         }
     ///     }
     /// }
     ///
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// let mut client = Client::builder("token").event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder("token", GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// client.start().await?;
     /// #     Ok(())
@@ -747,15 +741,13 @@ impl GuildChannel {
     /// #[serenity::async_trait]
     /// impl EventHandler for Handler {
     ///     async fn message(&self, context: Context, mut msg: Message) {
-    ///         let channel = match context.cache.guild_channel(msg.channel_id).await {
+    ///         let channel = match context.cache.guild_channel(msg.channel_id) {
     ///             Some(channel) => channel,
     ///             None => return,
     ///         };
     ///
-    ///         let current_user_id = context.cache.current_user().await.id;
-    ///         if let Ok(permissions) =
-    ///             channel.permissions_for_user(&context.cache, current_user_id).await
-    ///         {
+    ///         let current_user_id = context.cache.current_user_id();
+    ///         if let Ok(permissions) = channel.permissions_for_user(&context.cache, current_user_id) {
     ///             if !permissions.contains(Permissions::ATTACH_FILES | Permissions::SEND_MESSAGES) {
     ///                 return;
     ///             }
@@ -771,9 +763,8 @@ impl GuildChannel {
     ///
     ///             let _ = msg
     ///                 .channel_id
-    ///                 .send_files(&context.http, vec![(&file, "cat.png")], |mut m| {
-    ///                     m.content("here's a cat");
-    ///                     m
+    ///                 .send_files(&context.http, vec![(&file, "cat.png")], |m| {
+    ///                     m.content("here's a cat")
     ///                 })
     ///                 .await;
     ///         }
@@ -781,7 +772,8 @@ impl GuildChannel {
     /// }
     ///
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// let mut client = Client::builder("token").event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder("token", GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// client.start().await?;
     /// #     Ok(())
@@ -797,12 +789,12 @@ impl GuildChannel {
     /// [Send Messages]: Permissions::SEND_MESSAGES
     #[cfg(feature = "cache")]
     #[inline]
-    pub async fn permissions_for_user(
+    pub fn permissions_for_user(
         &self,
         cache: impl AsRef<Cache>,
         user_id: impl Into<UserId>,
     ) -> Result<Permissions> {
-        let guild = self.guild(&cache).await.ok_or(Error::Model(ModelError::GuildNotFound))?;
+        let guild = self.guild(&cache).ok_or(Error::Model(ModelError::GuildNotFound))?;
         let member =
             guild.members.get(&user_id.into()).ok_or(Error::Model(ModelError::MemberNotFound))?;
         guild.user_permissions_in(self, member)
@@ -822,12 +814,12 @@ impl GuildChannel {
     /// be found in the [`Cache`].
     #[cfg(feature = "cache")]
     #[inline]
-    pub async fn permissions_for_role(
+    pub fn permissions_for_role(
         &self,
         cache: impl AsRef<Cache>,
         role_id: impl Into<RoleId>,
     ) -> Result<Permissions> {
-        let guild = self.guild(&cache).await.ok_or(Error::Model(ModelError::GuildNotFound))?;
+        let guild = self.guild(&cache).ok_or(Error::Model(ModelError::GuildNotFound))?;
         let role =
             guild.roles.get(&role_id.into()).ok_or(Error::Model(ModelError::RoleNotFound))?;
         guild.role_permissions_in(self, role)
@@ -947,8 +939,6 @@ impl GuildChannel {
 
     /// Sends a message to the channel with the given content.
     ///
-    /// **Note**: This will only work when a [`Message`] is received.
-    ///
     /// **Note**: Requires the [Send Messages] permission.
     ///
     /// # Errors
@@ -972,8 +962,7 @@ impl GuildChannel {
             if let Some(cache) = cache_http.cache() {
                 let req = Permissions::SEND_MESSAGES;
 
-                if let Ok(false) =
-                    utils::user_has_perms(&cache, self.id, Some(self.guild_id), req).await
+                if let Ok(false) = utils::user_has_perms(&cache, self.id, Some(self.guild_id), req)
                 {
                     return Err(Error::Model(ModelError::InvalidPermissions(req)));
                 }
@@ -1010,11 +999,11 @@ impl GuildChannel {
     /// # use std::sync::Arc;
     /// #
     /// # fn long_process() {}
-    /// # let http = Arc::new(Http::default());
+    /// # let http = Arc::new(Http::new("token"));
     /// # let cache = Cache::default();
     /// # let channel = cache
     /// #    .guild_channel(ChannelId(7))
-    /// #    .await.ok_or(ModelError::ItemMissing)?;
+    /// #    .ok_or(ModelError::ItemMissing)?;
     /// // Initiate typing (assuming http is `Arc<Http>` and `channel` is bound)
     /// let typing = channel.start_typing(&http)?;
     ///
@@ -1080,7 +1069,7 @@ impl GuildChannel {
     #[inline]
     pub async fn members(&self, cache: impl AsRef<Cache>) -> Result<Vec<Member>> {
         let cache = cache.as_ref();
-        let guild = cache.guild(self.guild_id).await.ok_or(ModelError::GuildNotFound)?;
+        let guild = cache.guild(self.guild_id).ok_or(ModelError::GuildNotFound)?;
 
         match self.kind {
             ChannelType::Voice | ChannelType::Stage => Ok(guild
@@ -1101,8 +1090,7 @@ impl GuildChannel {
                     .filter_map(|e| async move {
                         if self
                             .permissions_for_user(cache, e.0)
-                            .await
-                            .map(|p| p.contains(Permissions::READ_MESSAGES))
+                            .map(|p| p.contains(Permissions::VIEW_CHANNEL))
                             .unwrap_or(false)
                         {
                             Some(e.1.clone())
@@ -1119,37 +1107,31 @@ impl GuildChannel {
 
     /// Returns a future that will await one message by this guild channel.
     #[cfg(feature = "collector")]
-    pub fn await_reply<'a>(
-        &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> CollectReply<'a> {
+    pub fn await_reply(&self, shard_messenger: impl AsRef<ShardMessenger>) -> CollectReply {
         CollectReply::new(shard_messenger).channel_id(self.id.0)
     }
 
     /// Returns a stream builder which can be awaited to obtain a stream of messages sent by this guild channel.
     #[cfg(feature = "collector")]
-    pub fn await_replies<'a>(
+    pub fn await_replies(
         &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> MessageCollectorBuilder<'a> {
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> MessageCollectorBuilder {
         MessageCollectorBuilder::new(shard_messenger).channel_id(self.id.0)
     }
 
     /// Await a single reaction by this guild channel.
     #[cfg(feature = "collector")]
-    pub fn await_reaction<'a>(
-        &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> CollectReaction<'a> {
+    pub fn await_reaction(&self, shard_messenger: impl AsRef<ShardMessenger>) -> CollectReaction {
         CollectReaction::new(shard_messenger).channel_id(self.id.0)
     }
 
     /// Returns a stream builder which can be awaited to obtain a stream of reactions sent by this guild channel.
     #[cfg(feature = "collector")]
-    pub fn await_reactions<'a>(
+    pub fn await_reactions(
         &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> ReactionCollectorBuilder<'a> {
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> ReactionCollectorBuilder {
         ReactionCollectorBuilder::new(shard_messenger).channel_id(self.id.0)
     }
 
@@ -1193,7 +1175,6 @@ impl GuildChannel {
         avatar: impl Into<AttachmentType<'a>>,
     ) -> Result<Webhook> {
         let name = name.to_string();
-        let avatar = avatar.into();
 
         if name.len() < 2 {
             return Err(Error::Model(ModelError::NameTooShort));
@@ -1310,10 +1291,10 @@ impl GuildChannel {
     }
 }
 
-impl Display for GuildChannel {
+impl fmt::Display for GuildChannel {
     /// Formats the channel, creating a mention of it.
-    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
-        Display::fmt(&self.id.mention(), f)
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt::Display::fmt(&self.id.mention(), f)
     }
 }
 
@@ -1324,9 +1305,8 @@ pub struct PartialGuildChannel {
     pub id: ChannelId,
     /// The channel guild Id.
     pub guild_id: GuildId,
-    /// The channel category Id.
-    #[serde(rename = "parent_id")]
-    pub category_id: ChannelId,
+    /// The channel category Id,  or the parent text channel Id for a thread.
+    pub parent_id: ChannelId,
     /// The channel type.
     #[serde(rename = "type")]
     pub kind: ChannelType,
