@@ -2,20 +2,6 @@
 
 #[cfg(feature = "model")]
 use std::fmt::Display;
-#[cfg(all(feature = "cache", feature = "model"))]
-use std::fmt::Write;
-#[cfg(feature = "model")]
-use std::result::Result as StdResult;
-
-#[cfg(feature = "model")]
-use bitflags::__impl_bitflags;
-use chrono::{DateTime, Utc};
-#[cfg(feature = "model")]
-use serde::{
-    de::{Deserialize, Deserializer},
-    ser::{Serialize, Serializer},
-};
-use serde_json::Value;
 
 #[cfg(all(feature = "model", feature = "utils"))]
 use crate::builder::{CreateEmbed, EditMessage};
@@ -23,21 +9,31 @@ use crate::builder::{CreateEmbed, EditMessage};
 use crate::cache::Cache;
 #[cfg(feature = "collector")]
 use crate::client::bridge::gateway::ShardMessenger;
-#[cfg(all(feature = "unstable_discord_api", feature = "collector"))]
-use crate::collector::{CollectComponentInteraction, ComponentInteractionCollectorBuilder};
 #[cfg(feature = "collector")]
-use crate::collector::{CollectReaction, ReactionCollectorBuilder};
+use crate::collector::{
+    CollectComponentInteraction,
+    CollectModalInteraction,
+    CollectReaction,
+    ComponentInteractionCollectorBuilder,
+    ModalInteractionCollectorBuilder,
+    ReactionCollectorBuilder,
+};
 #[cfg(feature = "model")]
 use crate::http::{CacheHttp, Http};
-#[cfg(feature = "unstable_discord_api")]
-use crate::model::interactions::{message_component::ActionRow, MessageInteraction};
-use crate::model::prelude::*;
 #[cfg(feature = "model")]
-use crate::model::utils::U64Visitor;
+use crate::json;
+use crate::json::prelude::*;
+use crate::model::interactions::message_component::ActionRow;
+use crate::model::interactions::MessageInteraction;
+use crate::model::prelude::*;
 #[cfg(feature = "model")]
 use crate::{
     constants,
-    model::id::{ApplicationId, ChannelId, GuildId, MessageId},
+    model::{
+        id::{ApplicationId, ChannelId, GuildId, MessageId},
+        sticker::StickerItem,
+        timestamp::Timestamp,
+    },
 };
 
 /// A representation of a message over a guild's text channel, a group, or a
@@ -57,7 +53,7 @@ pub struct Message {
     /// The content of the message.
     pub content: String,
     /// The timestamp of the last time the message was updated, if it was.
-    pub edited_timestamp: Option<DateTime<Utc>>,
+    pub edited_timestamp: Option<Timestamp>,
     /// Array of embeds sent with the message.
     pub embeds: Vec<Embed>,
     /// The Id of the [`Guild`] that the message was sent in. This value will
@@ -102,7 +98,7 @@ pub struct Message {
     #[serde(default)]
     pub reactions: Vec<MessageReaction>,
     /// Initial message creation timestamp, calculated from its Id.
-    pub timestamp: DateTime<Utc>,
+    pub timestamp: Timestamp,
     /// Indicator of whether the command is to be played back via
     /// text-to-speech.
     ///
@@ -118,18 +114,16 @@ pub struct Message {
     pub message_reference: Option<MessageReference>,
     /// Bit flags describing extra features of the message.
     pub flags: Option<MessageFlags>,
-    /// Array of stickers sent with the message.
+    /// Array of message sticker item objects.
     #[serde(default)]
-    pub stickers: Vec<Sticker>,
+    pub sticker_items: Vec<StickerItem>,
     /// The message that was replied to using this message.
-    pub referenced_message: Option<Box<Message>>, // Boxed to avoid recusion
+    pub referenced_message: Option<Box<Message>>, // Boxed to avoid recursion
     /// Sent if the message is a response to an [`Interaction`].
     ///
     /// [`Interaction`]: crate::model::interactions::Interaction
-    #[cfg(feature = "unstable_discord_api")]
     pub interaction: Option<MessageInteraction>,
     /// The components of this message
-    #[cfg(feature = "unstable_discord_api")]
     #[serde(default)]
     pub components: Vec<ActionRow>,
 }
@@ -157,14 +151,13 @@ impl Message {
         #[cfg(feature = "cache")]
         {
             if let Some(cache) = cache_http.cache() {
-                if self.author.id != cache.current_user_id().await && self.guild_id.is_some() {
+                if self.author.id != cache.current_user_id() && self.guild_id.is_some() {
                     utils::user_has_perms_cache(
                         cache,
                         self.channel_id,
                         self.guild_id,
                         Permissions::MANAGE_MESSAGES,
-                    )
-                    .await?;
+                    )?;
                 }
             }
         }
@@ -182,20 +175,25 @@ impl Message {
         self.channel_id.crosspost(cache_http.http(), self.id.0).await
     }
 
-    /// Retrieves the related channel located in the cache.
+    /// First attempts to find a [`Channel`] by its Id in the cache,
+    /// upon failure requests it via the REST API.
     ///
-    /// Returns [`None`] if the channel is not in the cache.
-    #[cfg(feature = "cache")]
+    /// **Note**: If the `cache`-feature is enabled permissions will be checked and upon
+    /// owning the required permissions the HTTP-request will be issued.
+    ///
+    /// # Errors
+    ///
+    /// Can return an error if the HTTP request fails.
     #[inline]
-    pub async fn channel(&self, cache: impl AsRef<Cache>) -> Option<Channel> {
-        cache.as_ref().channel(self.channel_id).await
+    pub async fn channel(&self, cache_http: impl CacheHttp) -> Result<Channel> {
+        self.channel_id.to_channel(cache_http).await
     }
 
     /// A util function for determining whether this message was sent by someone else, or the
     /// bot.
-    #[cfg(all(feature = "cache", feature = "utils"))]
-    pub async fn is_own(&self, cache: impl AsRef<Cache>) -> bool {
-        self.author.id == cache.as_ref().current_user().await.id
+    #[cfg(feature = "cache")]
+    pub fn is_own(&self, cache: impl AsRef<Cache>) -> bool {
+        self.author.id == cache.as_ref().current_user().id
     }
 
     /// Deletes the message.
@@ -214,18 +212,16 @@ impl Message {
         #[cfg(feature = "cache")]
         {
             if let Some(cache) = cache_http.cache() {
-                if self.author.id != cache.current_user_id().await {
+                if self.author.id != cache.current_user_id() {
                     if self.is_private() {
                         return Err(Error::Model(ModelError::NotAuthor));
-                    } else {
-                        utils::user_has_perms_cache(
-                            cache,
-                            self.channel_id,
-                            self.guild_id,
-                            Permissions::MANAGE_MESSAGES,
-                        )
-                        .await?;
                     }
+                    utils::user_has_perms_cache(
+                        cache,
+                        self.channel_id,
+                        self.guild_id,
+                        Permissions::MANAGE_MESSAGES,
+                    )?;
                 }
             }
         }
@@ -253,8 +249,7 @@ impl Message {
                     self.channel_id,
                     self.guild_id,
                     Permissions::MANAGE_MESSAGES,
-                )
-                .await?;
+                )?;
             }
         }
 
@@ -285,8 +280,7 @@ impl Message {
                     self.channel_id,
                     self.guild_id,
                     Permissions::MANAGE_MESSAGES,
-                )
-                .await?;
+                )?;
             }
         }
 
@@ -325,22 +319,25 @@ impl Message {
     /// is over [`the limit`], containing the number of unicode code points
     /// over the limit.
     ///
-    /// [`EditMessage`]: crate::builder::EditMessage
     /// [`the limit`]: crate::builder::EditMessage::content
-    #[cfg(feature = "utils")]
-    pub async fn edit<F>(&mut self, cache_http: impl CacheHttp, f: F) -> Result<()>
+    pub async fn edit<'a, F>(&mut self, cache_http: impl CacheHttp, f: F) -> Result<()>
     where
-        F: FnOnce(&mut EditMessage) -> &mut EditMessage,
+        F: for<'b> FnOnce(&'b mut EditMessage<'a>) -> &'b mut EditMessage<'a>,
     {
         #[cfg(feature = "cache")]
         {
             if let Some(cache) = cache_http.cache() {
-                if self.author.id != cache.current_user_id().await {
+                if self.author.id != cache.current_user_id() {
                     return Err(Error::Model(ModelError::InvalidUser));
                 }
             }
         }
+        let mut builder = self._prepare_edit_builder();
+        f(&mut builder);
+        self._send_edit(cache_http.http(), builder).await
+    }
 
+    fn _prepare_edit_builder<'a>(&self) -> EditMessage<'a> {
         let mut builder = EditMessage::default();
 
         if !self.content.is_empty() {
@@ -350,15 +347,23 @@ impl Message {
         let embeds: Vec<_> = self.embeds.iter().map(|e| CreateEmbed::from(e.clone())).collect();
         builder.set_embeds(embeds);
 
-        f(&mut builder);
+        for attachment in &self.attachments {
+            builder.add_existing_attachment(attachment.id);
+        }
+        builder
+    }
 
-        let map = crate::utils::hashmap_to_json_map(builder.0);
+    async fn _send_edit<'a>(&mut self, http: &Http, builder: EditMessage<'a>) -> Result<()> {
+        let map = json::hashmap_to_json_map(builder.0);
 
-        *self = cache_http
-            .http()
-            .edit_message(self.channel_id.0, self.id.0, &Value::Object(map))
+        *self = http
+            .edit_message_and_attachments(
+                self.channel_id.0,
+                self.id.0,
+                &Value::from(map),
+                builder.1,
+            )
             .await?;
-
         Ok(())
     }
 
@@ -369,7 +374,7 @@ impl Message {
                     format!("{} pinned a message to this channel. See all the pins.", self.author);
             },
             MessageType::MemberJoin => {
-                let sec = self.timestamp.timestamp() as usize;
+                let sec = self.timestamp.unix_timestamp() as usize;
                 let chosen = constants::JOIN_MESSAGES[sec % constants::JOIN_MESSAGES.len()];
 
                 self.content = if chosen.contains("$user") {
@@ -385,7 +390,7 @@ impl Message {
     /// Returns message content, but with user and role mentions replaced with
     /// names and everyone/here mentions cancelled.
     #[cfg(feature = "cache")]
-    pub async fn content_safe(&self, cache: impl AsRef<Cache>) -> String {
+    pub fn content_safe(&self, cache: impl AsRef<Cache>) -> String {
         let mut result = self.content.clone();
 
         // First replace all user mentions.
@@ -395,8 +400,7 @@ impl Message {
             at_distinct.push_str(&u.name);
             at_distinct.push('#');
 
-            #[allow(clippy::let_underscore_must_use)]
-            let _ = write!(at_distinct, "{:04}", u.discriminator);
+            at_distinct.push_str(&format!("{:04}", u.discriminator));
 
             let mut m = u.mention().to_string();
             // Check whether we're replacing a nickname mention or a normal mention.
@@ -412,7 +416,7 @@ impl Message {
         for id in &self.mention_roles {
             let mention = id.mention().to_string();
 
-            if let Some(role) = id.to_role_cached(&cache).await {
+            if let Some(role) = id.to_role_cached(&cache) {
                 result = result.replace(&mention, &format!("@{}", role.name));
             } else {
                 result = result.replace(&mention, "@deleted-role");
@@ -458,8 +462,8 @@ impl Message {
     ///
     /// Requires the `cache` feature be enabled.
     #[cfg(feature = "cache")]
-    pub async fn guild(&self, cache: impl AsRef<Cache>) -> Option<Guild> {
-        cache.as_ref().guild(self.guild_id?).await
+    pub fn guild(&self, cache: impl AsRef<Cache>) -> Option<Guild> {
+        cache.as_ref().guild(self.guild_id?)
     }
 
     /// Returns a field to the [`Guild`] for the message if one is in the cache.
@@ -470,7 +474,7 @@ impl Message {
     ///
     /// Requires the `cache` feature be enabled.
     #[cfg(feature = "cache")]
-    pub async fn guild_field<Ret, Fun>(
+    pub fn guild_field<Ret, Fun>(
         &self,
         cache: impl AsRef<Cache>,
         field_accessor: Fun,
@@ -479,11 +483,12 @@ impl Message {
         Ret: Clone,
         Fun: FnOnce(&Guild) -> Ret,
     {
-        cache.as_ref().guild_field(self.guild_id?, field_accessor).await
+        cache.as_ref().guild_field(self.guild_id?, field_accessor)
     }
 
     /// True if message was sent using direct messages.
     #[inline]
+    #[must_use]
     pub fn is_private(&self) -> bool {
         self.guild_id.is_none()
     }
@@ -506,7 +511,7 @@ impl Message {
         #[cfg(feature = "cache")]
         {
             if let Some(cache) = cache_http.cache() {
-                if let Some(member) = cache.member(guild_id, self.author.id).await {
+                if let Some(member) = cache.member(guild_id, self.author.id) {
                     return Ok(member);
                 }
             }
@@ -521,6 +526,7 @@ impl Message {
     /// Returns [`None`] if the message is within the limit, otherwise returns
     /// [`Some`] with an inner value of how many unicode code points the message
     /// is over.
+    #[must_use]
     pub fn overflow_length(content: &str) -> Option<usize> {
         // Check if the content is over the maximum number of unicode code
         // points.
@@ -554,8 +560,7 @@ impl Message {
                         self.channel_id,
                         self.guild_id,
                         Permissions::MANAGE_MESSAGES,
-                    )
-                    .await?;
+                    )?;
                 }
             }
         }
@@ -601,11 +606,10 @@ impl Message {
                         self.channel_id,
                         self.guild_id,
                         Permissions::ADD_REACTIONS,
-                    )
-                    .await?;
+                    )?;
                 }
 
-                user_id = Some(cache.current_user().await.id);
+                user_id = Some(cache.current_user_id());
             }
         }
 
@@ -720,8 +724,7 @@ impl Message {
                         self.channel_id,
                         self.guild_id,
                         Permissions::SEND_MESSAGES,
-                    )
-                    .await?;
+                    )?;
                 }
             }
         }
@@ -757,7 +760,6 @@ impl Message {
     /// Otherwise returns [`Error::Http`] if the current user lacks permission.
     ///
     /// [Manage Messages]: Permissions::MANAGE_MESSAGES
-    #[cfg(feature = "utils")]
     pub async fn suppress_embeds(&mut self, cache_http: impl CacheHttp) -> Result<()> {
         #[cfg(feature = "cache")]
         {
@@ -767,10 +769,9 @@ impl Message {
                     self.channel_id,
                     self.guild_id,
                     Permissions::MANAGE_MESSAGES,
-                )
-                .await?;
+                )?;
 
-                if self.author.id != cache.current_user_id().await {
+                if self.author.id != cache.current_user_id() {
                     return Err(Error::Model(ModelError::NotAuthor));
                 }
             }
@@ -779,12 +780,10 @@ impl Message {
         let mut suppress = EditMessage::default();
         suppress.suppress_embeds(true);
 
-        let map = crate::utils::hashmap_to_json_map(suppress.0);
+        let map = json::hashmap_to_json_map(suppress.0);
 
-        *self = cache_http
-            .http()
-            .edit_message(self.channel_id.0, self.id.0, &Value::Object(map))
-            .await?;
+        *self =
+            cache_http.http().edit_message(self.channel_id.0, self.id.0, &Value::from(map)).await?;
 
         Ok(())
     }
@@ -798,6 +797,7 @@ impl Message {
 
     /// Checks whether the message mentions passed [`User`].
     #[inline]
+    #[must_use]
     pub fn mentions_user(&self, user: &User) -> bool {
         self.mentions_user_id(user.id)
     }
@@ -812,7 +812,7 @@ impl Message {
         #[cfg(feature = "cache")]
         {
             if let Some(cache) = cache_http.cache() {
-                return Ok(self.mentions_user_id(cache.user.read().await.id));
+                return Ok(self.mentions_user_id(cache.current_user_id()));
             }
         }
 
@@ -841,13 +841,12 @@ impl Message {
                         self.channel_id,
                         self.guild_id,
                         Permissions::MANAGE_MESSAGES,
-                    )
-                    .await?;
+                    )?;
                 }
             }
         }
 
-        cache_http.http().unpin_message(self.channel_id.0, self.id.0).await
+        cache_http.http().unpin_message(self.channel_id.0, self.id.0, None).await
     }
 
     /// Tries to return author's nickname in the current channel's guild.
@@ -861,6 +860,7 @@ impl Message {
     /// Returns a link referencing this message. When clicked, users will jump to the message.
     /// The link will be valid for messages in either private channels or guilds.
     #[inline]
+    #[must_use]
     pub fn link(&self) -> String {
         self.id.link(self.channel_id, self.guild_id)
     }
@@ -876,48 +876,71 @@ impl Message {
 
     /// Await a single reaction on this message.
     #[cfg(feature = "collector")]
-    pub fn await_reaction<'a>(
-        &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> CollectReaction<'a> {
+    pub fn await_reaction(&self, shard_messenger: impl AsRef<ShardMessenger>) -> CollectReaction {
         CollectReaction::new(shard_messenger).message_id(self.id.0)
     }
 
     /// Returns a stream builder which can be awaited to obtain a stream of reactions on this message.
     #[cfg(feature = "collector")]
-    pub fn await_reactions<'a>(
+    pub fn await_reactions(
         &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> ReactionCollectorBuilder<'a> {
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> ReactionCollectorBuilder {
         ReactionCollectorBuilder::new(shard_messenger).message_id(self.id.0)
     }
 
     /// Await a single component interaction on this message.
-    #[cfg(all(feature = "unstable_discord_api", feature = "collector"))]
-    pub fn await_component_interaction<'a>(
+    #[cfg(feature = "collector")]
+    pub fn await_component_interaction(
         &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> CollectComponentInteraction<'a> {
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> CollectComponentInteraction {
         CollectComponentInteraction::new(shard_messenger).message_id(self.id.0)
     }
 
     /// Returns a stream builder which can be awaited to obtain a stream of component interactions on this message.
-    #[cfg(all(feature = "unstable_discord_api", feature = "collector"))]
-    pub fn await_component_interactions<'a>(
+    #[cfg(feature = "collector")]
+    pub fn await_component_interactions(
         &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> ComponentInteractionCollectorBuilder<'a> {
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> ComponentInteractionCollectorBuilder {
         ComponentInteractionCollectorBuilder::new(shard_messenger).message_id(self.id.0)
     }
 
+    /// Await a single modal submit interaction on this message.
+    #[cfg(feature = "collector")]
+    pub fn await_modal_interaction(
+        &self,
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> CollectModalInteraction {
+        CollectModalInteraction::new(shard_messenger).message_id(self.id.0)
+    }
+
+    /// Returns a stream builder which can be awaited to obtain a stream of modal submit interactions on this message.
+    #[cfg(feature = "collector")]
+    pub fn await_modal_interactions(
+        &self,
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> ModalInteractionCollectorBuilder {
+        ModalInteractionCollectorBuilder::new(shard_messenger).message_id(self.id.0)
+    }
+
     /// Retrieves the message channel's category ID if the channel has one.
     #[cfg(feature = "cache")]
-    pub async fn category_id(&self, cache: impl AsRef<Cache>) -> Option<ChannelId> {
-        cache.as_ref().channel_category_id(self.channel_id).await
+    pub fn category_id(&self, cache: impl AsRef<Cache>) -> Option<ChannelId> {
+        cache.as_ref().channel_category_id(self.channel_id)
+    }
+
+    pub(crate) fn check_lengths(map: &JsonMap) -> Result<()> {
+        Self::check_content_length(map)?;
+        Self::check_embed_length(map)?;
+        Self::check_sticker_ids_length(map)?;
+
+        Ok(())
     }
 
     pub(crate) fn check_content_length(map: &JsonMap) -> Result<()> {
-        if let Some(Value::String(ref content)) = map.get("content") {
+        if let Some(Value::String(content)) = map.get("content") {
             if let Some(length_over) = Message::overflow_length(content) {
                 return Err(Error::Model(ModelError::MessageTooLong(length_over)));
             }
@@ -981,6 +1004,16 @@ impl Message {
 
         Ok(())
     }
+
+    pub(crate) fn check_sticker_ids_length(map: &JsonMap) -> Result<()> {
+        if let Some(Value::Array(sticker_ids)) = map.get("sticker_ids") {
+            if sticker_ids.len() > constants::STICKER_MAX_COUNT {
+                return Err(Error::Model(ModelError::StickerAmount));
+            }
+        }
+
+        Ok(())
+    }
 }
 
 impl AsRef<MessageId> for Message {
@@ -1065,8 +1098,7 @@ pub enum MessageType {
     /// A message reply.
     InlineReply = 19,
     /// A slash command.
-    // TODO: Rename to chat input command in next
-    ApplicationCommand = 20,
+    ChatInputCommand = 20,
     /// A thread start message.
     ThreadStarterMessage = 21,
     /// Server setup tips.
@@ -1097,7 +1129,7 @@ enum_number!(MessageType {
     GuildDiscoveryGracePeriodFinalWarning
     ThreadCreated,
     InlineReply,
-    ApplicationCommand,
+    ChatInputCommand,
     ThreadStarterMessage,
     GuildInviteReminder,
     ContextMenuCommand,
@@ -1197,52 +1229,28 @@ pub struct ChannelMention {
     pub name: String,
 }
 
-/// Describes extra features of the message.
-#[derive(Copy, PartialEq, Eq, Clone, PartialOrd, Ord, Hash)]
-#[cfg_attr(not(feature = "model"), derive(Debug, Deserialize, Serialize))]
-pub struct MessageFlags {
-    pub bits: u64,
-}
-
-#[cfg(feature = "model")]
-__impl_bitflags! {
-    MessageFlags: u64 {
+bitflags! {
+    /// Describes extra features of the message.
+    #[derive(Default)]
+    pub struct MessageFlags: u64 {
         /// This message has been published to subscribed channels (via Channel Following).
-        CROSSPOSTED = 1 << 0;
+        const CROSSPOSTED = 1 << 0;
         /// This message originated from a message in another channel (via Channel Following).
-        IS_CROSSPOST = 1 << 1;
+        const IS_CROSSPOST = 1 << 1;
         /// Do not include any embeds when serializing this message.
-        SUPPRESS_EMBEDS = 1 << 2;
+        const SUPPRESS_EMBEDS = 1 << 2;
         /// The source message for this crosspost has been deleted (via Channel Following).
-        SOURCE_MESSAGE_DELETED = 1 << 3;
+        const SOURCE_MESSAGE_DELETED = 1 << 3;
         /// This message came from the urgent message system.
-        URGENT = 1 << 4;
+        const URGENT = 1 << 4;
         /// This message has an associated thread, with the same id as the message.
-        HAS_THREAD = 1 << 5;
+        const HAS_THREAD = 1 << 5;
         /// This message is only visible to the user who invoked the Interaction.
-        EPHEMERAL = 1 << 6;
+        const EPHEMERAL = 1 << 6;
         /// This message is an Interaction Response and the bot is "thinking".
-        LOADING = 1 << 7;
-    }
-}
-
-#[cfg(feature = "model")]
-impl<'de> Deserialize<'de> for MessageFlags {
-    fn deserialize<D>(deserializer: D) -> StdResult<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
-        Ok(MessageFlags::from_bits_truncate(deserializer.deserialize_u64(U64Visitor)?))
-    }
-}
-
-#[cfg(feature = "model")]
-impl Serialize for MessageFlags {
-    fn serialize<S>(&self, serializer: S) -> StdResult<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u64(self.bits())
+        const LOADING = 1 << 7;
+        /// This message failed to mention some roles and add their members to the thread.
+        const FAILED_TO_MENTION_SOME_ROLES_IN_THREAD = 1 << 8;
     }
 }
 
@@ -1250,12 +1258,12 @@ impl Serialize for MessageFlags {
 impl MessageId {
     /// Returns a link referencing this message. When clicked, users will jump to the message.
     /// The link will be valid for messages in either private channels or guilds.
+    #[must_use]
     pub fn link(&self, channel_id: ChannelId, guild_id: Option<GuildId>) -> String {
-        match guild_id {
-            Some(guild_id) => {
-                format!("https://discord.com/channels/{}/{}/{}", guild_id.0, channel_id.0, self.0)
-            },
-            None => format!("https://discord.com/channels/@me/{}/{}", channel_id.0, self.0),
+        if let Some(guild_id) = guild_id {
+            format!("https://discord.com/channels/{}/{}/{}", guild_id, channel_id, self)
+        } else {
+            format!("https://discord.com/channels/@me/{}/{}", channel_id, self)
         }
     }
 
