@@ -1,6 +1,7 @@
 //! Models relating to channels and types within channels.
 
 mod attachment;
+mod attachment_type;
 mod channel_category;
 mod channel_id;
 mod embed;
@@ -9,18 +10,16 @@ mod message;
 mod partial_channel;
 mod private_channel;
 mod reaction;
-mod sticker;
-
-#[cfg(feature = "model")]
-use std::fmt::{Display, Formatter, Result as FmtResult};
 
 #[cfg(all(feature = "cache", feature = "model", feature = "utils"))]
-use async_trait::async_trait;
-use chrono::{DateTime, Utc};
+use std::error::Error as StdError;
+use std::fmt;
+
 use serde::de::{Error as DeError, Unexpected};
 use serde::ser::{Serialize, SerializeStruct, Serializer};
 
 pub use self::attachment::*;
+pub use self::attachment_type::*;
 pub use self::channel_category::*;
 pub use self::channel_id::*;
 pub use self::embed::*;
@@ -29,18 +28,16 @@ pub use self::message::*;
 pub use self::partial_channel::*;
 pub use self::private_channel::*;
 pub use self::reaction::*;
-pub use self::sticker::*;
-use super::utils::deserialize_u64;
 #[cfg(all(feature = "cache", feature = "model"))]
 use crate::cache::Cache;
 #[cfg(all(feature = "cache", feature = "model", feature = "utils"))]
 use crate::cache::FromStrAndCache;
 #[cfg(feature = "model")]
 use crate::http::CacheHttp;
-use crate::internal::is_false;
-#[cfg(all(feature = "cache", feature = "model", feature = "utils"))]
-use crate::model::misc::ChannelParseError;
+use crate::json::prelude::*;
 use crate::model::prelude::*;
+use crate::model::utils::is_false;
+use crate::model::Timestamp;
 #[cfg(all(feature = "cache", feature = "model", feature = "utils"))]
 use crate::utils::parse_channel;
 
@@ -48,11 +45,12 @@ use crate::utils::parse_channel;
 #[derive(Clone, Debug)]
 #[non_exhaustive]
 pub enum Channel {
-    /// A [text], [voice], or [stage] channel within a [`Guild`].
+    /// A [text], [voice], [stage] or [directory] channel within a [`Guild`].
     ///
     /// [text]: ChannelType::Text
     /// [voice]: ChannelType::Voice
     /// [stage]: ChannelType::Stage
+    /// [directory]: ChannelType::Directory
     Guild(GuildChannel),
     /// A private channel to another [`User`]. No other users may access the
     /// channel. For multi-user "private channels", use a group.
@@ -61,7 +59,6 @@ pub enum Channel {
     Category(ChannelCategory),
 }
 
-#[cfg(feature = "model")]
 impl Channel {
     /// Converts from [`Channel`] to `Option<GuildChannel>`.
     ///
@@ -74,14 +71,8 @@ impl Channel {
     /// Basic usage:
     ///
     /// ```rust,no_run
-    /// # #[cfg(all(feature = "model", feature = "cache"))]
-    /// # async fn run() {
-    /// # use serenity::{cache::Cache, model::id::ChannelId};
-    /// # use tokio::sync::RwLock;
-    /// # use std::sync::Arc;
-    /// #
-    /// #   let cache = Cache::default();
-    /// #   let channel = ChannelId(0).to_channel_cached(&cache).await.unwrap();
+    /// # use serenity::model::channel::Channel;
+    /// # fn run(channel: Channel) {
     /// #
     /// match channel.guild() {
     ///     Some(guild_channel) => {
@@ -93,6 +84,7 @@ impl Channel {
     /// }
     /// # }
     /// ```
+    #[must_use]
     pub fn guild(self) -> Option<GuildChannel> {
         match self {
             Channel::Guild(lock) => Some(lock),
@@ -111,14 +103,8 @@ impl Channel {
     /// Basic usage:
     ///
     /// ```rust,no_run
-    /// # #[cfg(all(feature = "model", feature = "cache"))]
-    /// # async fn run() {
-    /// # use serenity::{cache::Cache, model::id::ChannelId};
-    /// # use tokio::sync::RwLock;
-    /// # use std::sync::Arc;
-    /// #
-    /// #   let cache = Cache::default();
-    /// #   let channel = ChannelId(0).to_channel_cached(&cache).await.unwrap();
+    /// # use serenity::model::channel::Channel;
+    /// # fn run(channel: Channel) {
     /// #
     /// match channel.private() {
     ///     Some(private) => {
@@ -130,6 +116,7 @@ impl Channel {
     /// }
     /// # }
     /// ```
+    #[must_use]
     pub fn private(self) -> Option<PrivateChannel> {
         match self {
             Channel::Private(lock) => Some(lock),
@@ -148,14 +135,8 @@ impl Channel {
     /// Basic usage:
     ///
     /// ```rust,no_run
-    /// # #[cfg(all(feature = "model", feature = "cache"))]
-    /// # async fn run() {
-    /// # use serenity::{cache::Cache, model::id::ChannelId};
-    /// # use tokio::sync::RwLock;
-    /// # use std::sync::Arc;
-    /// #
-    /// #   let cache = Cache::default();
-    /// #   let channel = ChannelId(0).to_channel_cached(&cache).await.unwrap();
+    /// # use serenity::model::channel::Channel;
+    /// # fn run(channel: Channel) {
     /// #
     /// match channel.category() {
     ///     Some(category) => {
@@ -165,9 +146,9 @@ impl Channel {
     ///         println!("It's not a category!");
     ///     },
     /// }
-    /// #
     /// # }
     /// ```
+    #[must_use]
     pub fn category(self) -> Option<ChannelCategory> {
         match self {
             Channel::Category(lock) => Some(lock),
@@ -184,6 +165,7 @@ impl Channel {
     ///
     /// Otherwise will return [`Error::Http`] if the current user does not
     /// have permission.
+    #[cfg(feature = "http")]
     pub async fn delete(&self, cache_http: impl CacheHttp) -> Result<()> {
         match self {
             Channel::Guild(public_channel) => {
@@ -202,6 +184,8 @@ impl Channel {
 
     /// Determines if the channel is NSFW.
     #[inline]
+    #[must_use]
+    #[cfg(feature = "model")]
     pub fn is_nsfw(&self) -> bool {
         match self {
             Channel::Guild(channel) => channel.is_nsfw(),
@@ -213,6 +197,7 @@ impl Channel {
     /// Retrieves the Id of the inner [`GuildChannel`], or
     /// [`PrivateChannel`].
     #[inline]
+    #[must_use]
     pub fn id(&self) -> ChannelId {
         match self {
             Channel::Guild(ch) => ch.id,
@@ -226,10 +211,11 @@ impl Channel {
     ///
     /// If other channel types are used it will return None.
     #[inline]
+    #[must_use]
     pub fn position(&self) -> Option<i64> {
         match self {
             Channel::Guild(channel) => Some(channel.position),
-            Channel::Category(catagory) => Some(catagory.position),
+            Channel::Category(category) => Some(category.position),
             _ => None,
         }
     }
@@ -253,15 +239,13 @@ impl<'de> Deserialize<'de> for Channel {
         };
 
         match kind {
-            0 | 2 | 5 | 6 | 10 | 11 | 12 | 13 => {
-                serde_json::from_value::<GuildChannel>(Value::Object(v))
-                    .map(Channel::Guild)
-                    .map_err(DeError::custom)
-            },
-            1 => serde_json::from_value::<PrivateChannel>(Value::Object(v))
+            0 | 2 | 5 | 10 | 11 | 12 | 13 | 14 | 15 => from_value::<GuildChannel>(Value::from(v))
+                .map(Channel::Guild)
+                .map_err(DeError::custom),
+            1 => from_value::<PrivateChannel>(Value::from(v))
                 .map(Channel::Private)
                 .map_err(DeError::custom),
-            4 => serde_json::from_value::<ChannelCategory>(Value::Object(v))
+            4 => from_value::<ChannelCategory>(Value::from(v))
                 .map(Channel::Category)
                 .map_err(DeError::custom),
             _ => Err(DeError::custom("Unknown channel type")),
@@ -282,7 +266,7 @@ impl Serialize for Channel {
     }
 }
 
-impl Display for Channel {
+impl fmt::Display for Channel {
     /// Formats the channel into a "mentioned" string.
     ///
     /// This will return a different format for each type of channel:
@@ -290,11 +274,11 @@ impl Display for Channel {
     /// - [`PrivateChannel`]s: the recipient's name;
     /// - [`GuildChannel`]s: a string mentioning the channel that users who can
     /// see the channel can click on.
-    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            Channel::Guild(ch) => Display::fmt(&ch.id.mention(), f),
-            Channel::Private(ch) => Display::fmt(&ch.recipient.name, f),
-            Channel::Category(ch) => Display::fmt(&ch.name, f),
+            Channel::Guild(ch) => fmt::Display::fmt(&ch.id.mention(), f),
+            Channel::Private(ch) => fmt::Display::fmt(&ch.recipient.name, f),
+            Channel::Category(ch) => fmt::Display::fmt(&ch.name, f),
         }
     }
 }
@@ -316,18 +300,21 @@ pub enum ChannelType {
     ///
     /// Note: `NewsChannel` is serialized into a [`GuildChannel`]
     News = 5,
-    /// An indicator that the channel is a `StoreChannel`
-    ///
-    /// Note: `StoreChannel` is serialized into a [`GuildChannel`]
-    Store = 6,
     /// An indicator that the channel is a news thread [`GuildChannel`].
     NewsThread = 10,
-    /// An indicator that the channel is a news thread [`GuildChannel`].
+    /// An indicator that the channel is a public thread [`GuildChannel`].
     PublicThread = 11,
-    /// An indicator that the channel is a news thread [`GuildChannel`].
+    /// An indicator that the channel is a private thread [`GuildChannel`].
     PrivateThread = 12,
     /// An indicator that the channel is a stage [`GuildChannel`].
     Stage = 13,
+    /// An indicator that the channel is a directory [`GuildChannel`] in a [hub].
+    ///
+    /// [hub]: https://support.discord.com/hc/en-us/articles/4406046651927-Discord-Student-Hubs-FAQ
+    Directory = 14,
+    /// An indicator that the channel is a forum [`GuildChannel`].
+    #[cfg(feature = "unstable_discord_api")]
+    Forum = 15,
     /// An indicator that the channel is of unknown type.
     Unknown = !0,
 }
@@ -338,15 +325,18 @@ enum_number!(ChannelType {
     Voice,
     Category,
     News,
-    Store,
     NewsThread,
     PublicThread,
     PrivateThread,
-    Stage
+    Stage,
+    Directory,
+    #[cfg(feature = "unstable_discord_api")]
+    Forum,
 });
 
 impl ChannelType {
     #[inline]
+    #[must_use]
     pub fn name(&self) -> &str {
         match *self {
             ChannelType::Private => "private",
@@ -354,11 +344,13 @@ impl ChannelType {
             ChannelType::Voice => "voice",
             ChannelType::Category => "category",
             ChannelType::News => "news",
-            ChannelType::Store => "store",
             ChannelType::NewsThread => "news_thread",
             ChannelType::PublicThread => "public_thread",
             ChannelType::PrivateThread => "private_thread",
             ChannelType::Stage => "stage",
+            ChannelType::Directory => "directory",
+            #[cfg(feature = "unstable_discord_api")]
+            ChannelType::Forum => "forum",
             ChannelType::Unknown => "unknown",
         }
     }
@@ -368,14 +360,14 @@ impl ChannelType {
 struct PermissionOverwriteData {
     allow: Permissions,
     deny: Permissions,
-    #[serde(serialize_with = "serialize_u64", deserialize_with = "deserialize_u64")]
+    #[serde(with = "snowflake")]
     id: u64,
     #[serde(rename = "type")]
     kind: u8,
 }
 
 /// A channel-specific permission overwrite for a member or role.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq)]
 pub struct PermissionOverwrite {
     pub allow: Permissions,
     pub deny: Permissions,
@@ -408,8 +400,8 @@ impl Serialize for PermissionOverwrite {
         S: Serializer,
     {
         let (id, kind) = match self.kind {
-            PermissionOverwriteType::Role(id) => (id.0, 0),
-            PermissionOverwriteType::Member(id) => (id.0, 1),
+            PermissionOverwriteType::Role(id) => (GenericId(id.0), 0),
+            PermissionOverwriteType::Member(id) => (GenericId(id.0), 1),
         };
 
         let mut state = serializer.serialize_struct("PermissionOverwrite", 4)?;
@@ -476,14 +468,14 @@ pub struct ThreadMetadata {
     /// **Note**: It can currently only be set to 60, 1440, 4320, 10080.
     pub auto_archive_duration: Option<u64>,
     /// Timestamp when the thread's archive status was last changed, used for calculating recent activity.
-    pub archive_timestamp: Option<DateTime<Utc>>,
+    pub archive_timestamp: Option<Timestamp>,
     /// When a thread is locked, only users with `MANAGE_THREADS` permission can unarchive it.
     #[serde(default)]
     pub locked: bool,
     /// Timestamp when the thread was created.
     ///
     /// **Note**: only populated for threads created after 2022-01-09
-    pub create_timestamp: Option<DateTime<Utc>>,
+    pub create_timestamp: Option<Timestamp>,
     /// Whether non-moderators can add other non-moderators to a thread.
     ///
     /// **Note**: Only available on private threads.
@@ -514,7 +506,7 @@ mod test {
             GuildChannel {
                 id: ChannelId(1),
                 bitrate: None,
-                category_id: None,
+                parent_id: None,
                 guild_id: GuildId(2),
                 kind: ChannelType::Text,
                 last_message_id: None,
@@ -525,7 +517,7 @@ mod test {
                 topic: None,
                 user_limit: None,
                 nsfw: false,
-                slow_mode_rate: Some(0),
+                rate_limit_per_user: Some(0),
                 rtc_region: None,
                 video_quality_mode: None,
                 message_count: None,
@@ -588,16 +580,35 @@ mod test {
 }
 
 #[cfg(all(feature = "cache", feature = "model", feature = "utils"))]
-#[async_trait]
+#[derive(Debug)]
+pub enum ChannelParseError {
+    NotPresentInCache,
+    InvalidChannel,
+}
+
+#[cfg(all(feature = "cache", feature = "model", feature = "utils"))]
+impl fmt::Display for ChannelParseError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            ChannelParseError::NotPresentInCache => f.write_str("not present in cache"),
+            ChannelParseError::InvalidChannel => f.write_str("invalid channel"),
+        }
+    }
+}
+
+#[cfg(all(feature = "cache", feature = "model", feature = "utils"))]
+impl StdError for ChannelParseError {}
+
+#[cfg(all(feature = "cache", feature = "model", feature = "utils"))]
 impl FromStrAndCache for Channel {
     type Err = ChannelParseError;
 
-    async fn from_str<C>(cache: C, s: &str) -> StdResult<Self, Self::Err>
+    fn from_str<C>(cache: C, s: &str) -> StdResult<Self, Self::Err>
     where
         C: AsRef<Cache> + Send + Sync,
     {
         match parse_channel(s) {
-            Some(x) => match ChannelId(x).to_channel_cached(&cache).await {
+            Some(x) => match ChannelId(x).to_channel_cached(&cache) {
                 Some(channel) => Ok(channel),
                 _ => Err(ChannelParseError::NotPresentInCache),
             },
