@@ -1,26 +1,22 @@
 #[cfg(feature = "model")]
 use std::borrow::Cow;
-#[cfg(all(feature = "cache", feature = "model"))]
+#[cfg(feature = "cache")]
 use std::cmp::Reverse;
-use std::fmt::{Display, Formatter, Result as FmtResult};
-
-use bitflags::__impl_bitflags;
-use chrono::{DateTime, Utc};
-use serde::{Serialize, Serializer};
+use std::fmt;
 
 #[cfg(feature = "model")]
 use crate::builder::EditMember;
-#[cfg(all(feature = "cache"))]
+#[cfg(feature = "cache")]
 use crate::cache::Cache;
 #[cfg(feature = "model")]
 use crate::http::{CacheHttp, Http};
 #[cfg(all(feature = "cache", feature = "model"))]
 use crate::internal::prelude::*;
-#[cfg(feature = "unstable_discord_api")]
+#[cfg(feature = "model")]
+use crate::json;
 use crate::model::permissions::Permissions;
 use crate::model::prelude::*;
-#[cfg(feature = "model")]
-use crate::utils;
+use crate::model::Timestamp;
 #[cfg(all(feature = "cache", feature = "model", feature = "utils"))]
 use crate::utils::Colour;
 
@@ -33,7 +29,7 @@ pub struct Member {
     /// The unique Id of the guild that the member is a part of.
     pub guild_id: GuildId,
     /// Timestamp representing the date when the member joined.
-    pub joined_at: Option<DateTime<Utc>>,
+    pub joined_at: Option<Timestamp>,
     /// Indicator of whether the member can speak in voice channels.
     pub mute: bool,
     /// The member's nickname, if present.
@@ -48,20 +44,59 @@ pub struct Member {
     #[serde(default)]
     pub pending: bool,
     /// Timestamp representing the date since the member is boosting the guild.
-    pub premium_since: Option<DateTime<Utc>>,
+    pub premium_since: Option<Timestamp>,
     /// The total permissions of the member in a channel, including overrides.
     ///
     /// This is only [`Some`] when returned in an [`Interaction`] object.
     ///
     /// [`Interaction`]: crate::model::interactions::Interaction
-    #[cfg(feature = "unstable_discord_api")]
     pub permissions: Option<Permissions>,
     /// The guild avatar hash
     pub avatar: Option<String>,
-    /// 	When the user's timeout will expire and the user will be able to communicate in the guild again.
+    /// When the user's timeout will expire and the user will be able to communicate in the guild again.
     ///
-    /// 	Will be None or a time in the past if the user is not timed out.
-    pub communication_disabled_until: Option<DateTime<Utc>>,
+    /// Will be None or a time in the past if the user is not timed out.
+    pub communication_disabled_until: Option<Timestamp>,
+}
+
+/// Helper for deserialization without a `GuildId` but then later updated to the correct `GuildId`.
+///
+/// The only difference to `Member` is `#[serde(default)]` on `guild_id`.
+#[derive(Deserialize)]
+pub(crate) struct InterimMember {
+    pub deaf: bool,
+    #[serde(default)]
+    pub guild_id: GuildId,
+    pub joined_at: Option<Timestamp>,
+    pub mute: bool,
+    pub nick: Option<String>,
+    pub roles: Vec<RoleId>,
+    pub user: User,
+    #[serde(default)]
+    pub pending: bool,
+    pub premium_since: Option<Timestamp>,
+    pub permissions: Option<Permissions>,
+    pub avatar: Option<String>,
+    pub communication_disabled_until: Option<Timestamp>,
+}
+
+impl From<InterimMember> for Member {
+    fn from(m: InterimMember) -> Self {
+        Self {
+            deaf: m.deaf,
+            guild_id: m.guild_id,
+            joined_at: m.joined_at,
+            mute: m.mute,
+            nick: m.nick,
+            roles: m.roles,
+            user: m.user,
+            pending: m.pending,
+            premium_since: m.premium_since,
+            permissions: m.permissions,
+            avatar: m.avatar,
+            communication_disabled_until: m.communication_disabled_until,
+        }
+    }
 }
 
 #[cfg(feature = "model")]
@@ -91,7 +126,8 @@ impl Member {
             return Ok(());
         }
 
-        match http.as_ref().add_member_role(self.guild_id.0, self.user.id.0, role_id.0).await {
+        match http.as_ref().add_member_role(self.guild_id.0, self.user.id.0, role_id.0, None).await
+        {
             Ok(()) => {
                 self.roles.push(role_id);
 
@@ -121,9 +157,9 @@ impl Member {
 
         let mut builder = EditMember::default();
         builder.roles(&self.roles);
-        let map = utils::hashmap_to_json_map(builder.0);
+        let map = json::hashmap_to_json_map(builder.0);
 
-        match http.as_ref().edit_member(self.guild_id.0, self.user.id.0, &map).await {
+        match http.as_ref().edit_member(self.guild_id.0, self.user.id.0, &map, None).await {
             Ok(member) => Ok(member.roles),
             Err(why) => {
                 self.roles.retain(|r| !role_ids.contains(r));
@@ -168,8 +204,8 @@ impl Member {
 
     /// Determines the member's colour.
     #[cfg(feature = "cache")]
-    pub async fn colour(&self, cache: impl AsRef<Cache>) -> Option<Colour> {
-        let guild_roles = cache.as_ref().guild_field(self.guild_id, |g| g.roles.clone()).await?;
+    pub fn colour(&self, cache: impl AsRef<Cache>) -> Option<Colour> {
+        let guild_roles = cache.as_ref().guild_field(self.guild_id, |g| g.roles.clone())?;
 
         let mut roles = self
             .roles
@@ -188,14 +224,16 @@ impl Member {
     /// (This returns the first channel that can be read by the member, if there isn't
     /// one returns [`None`])
     #[cfg(feature = "cache")]
-    pub async fn default_channel(&self, cache: impl AsRef<Cache>) -> Option<GuildChannel> {
-        let guild = self.guild_id.to_guild_cached(cache).await?;
+    pub fn default_channel(&self, cache: impl AsRef<Cache>) -> Option<GuildChannel> {
+        let guild = self.guild_id.to_guild_cached(cache)?;
 
         let member = guild.members.get(&self.user.id)?;
 
         for channel in guild.channels.values() {
-            if guild.user_permissions_in(channel, member).ok()?.read_messages() {
-                return Some(channel.clone());
+            if let Channel::Guild(channel) = channel {
+                if guild.user_permissions_in(channel, member).ok()?.view_channel() {
+                    return Some(channel.clone());
+                }
             }
         }
 
@@ -218,13 +256,12 @@ impl Member {
     pub async fn disable_communication_until_datetime(
         &mut self,
         http: impl AsRef<Http>,
-        time: DateTime<Utc>,
+        time: Timestamp,
     ) -> Result<()> {
         match self
             .guild_id
             .edit_member(http, self.user.id, |member| {
-                member.disable_communication_until_datetime(time);
-                member
+                member.disable_communication_until_datetime(time)
             })
             .await
         {
@@ -246,6 +283,7 @@ impl Member {
 
     /// Returns the DiscordTag of a Member, taking possible nickname into account.
     #[inline]
+    #[must_use]
     pub fn distinct(&self) -> String {
         format!("{}#{:04}", self.display_name(), self.user.discriminator)
     }
@@ -259,17 +297,15 @@ impl Member {
     /// # Errors
     ///
     /// Returns [`Error::Http`] if the current user lacks necessary permissions.
-    ///
-    /// [`EditMember`]: crate::builder::EditMember
     pub async fn edit<F>(&self, http: impl AsRef<Http>, f: F) -> Result<Member>
     where
         F: FnOnce(&mut EditMember) -> &mut EditMember,
     {
         let mut edit_member = EditMember::default();
         f(&mut edit_member);
-        let map = utils::hashmap_to_json_map(edit_member.0);
+        let map = json::hashmap_to_json_map(edit_member.0);
 
-        http.as_ref().edit_member(self.guild_id.0, self.user.id.0, &map).await
+        http.as_ref().edit_member(self.guild_id.0, self.user.id.0, &map, None).await
     }
 
     /// Allow a user to communicate, removing their timeout, if there is one.
@@ -283,13 +319,7 @@ impl Member {
     /// [Moderate Members]: Permissions::MODERATE_MEMBERS
     #[doc(alias = "timeout")]
     pub async fn enable_communication(&mut self, http: impl AsRef<Http>) -> Result<()> {
-        match self
-            .guild_id
-            .edit_member(&http, self.user.id, |member| {
-                member.enable_communication();
-                member
-            })
-            .await
+        match self.guild_id.edit_member(&http, self.user.id, EditMember::enable_communication).await
         {
             Ok(_) => {
                 self.communication_disabled_until = None;
@@ -312,8 +342,8 @@ impl Member {
     /// position. If two or more roles have the same highest position, then the
     /// role with the lowest ID is the highest.
     #[cfg(feature = "cache")]
-    pub async fn highest_role_info(&self, cache: impl AsRef<Cache>) -> Option<(RoleId, i64)> {
-        let guild_roles = cache.as_ref().guild_field(self.guild_id, |g| g.roles.clone()).await?;
+    pub fn highest_role_info(&self, cache: impl AsRef<Cache>) -> Option<(RoleId, i64)> {
+        let guild_roles = cache.as_ref().guild_field(self.guild_id, |g| g.roles.clone())?;
 
         let mut highest = None;
 
@@ -405,14 +435,14 @@ impl Member {
         #[cfg(feature = "cache")]
         {
             if let Some(cache) = cache_http.cache() {
-                if let Some(guild) = cache.guilds.read().await.get(&self.guild_id) {
+                if let Some(guild) = cache.guilds.get(&self.guild_id) {
                     let req = Permissions::KICK_MEMBERS;
 
                     if !guild.has_perms(&cache_http, req).await {
                         return Err(Error::Model(ModelError::InvalidPermissions(req)));
                     }
 
-                    guild.check_hierarchy(cache, self.user.id).await?;
+                    guild.check_hierarchy(cache, self.user.id)?;
                 }
             }
         }
@@ -459,7 +489,7 @@ impl Member {
     /// ```rust,ignore
     /// // assuming there's a `member` variable gotten from anything.
     /// println!("The permission bits for the member are: {}",
-    /// member.permissions().expect("permissions").bits);
+    /// member.permissions(&cache).expect("permissions").bits());
     /// ```
     ///
     /// # Errors
@@ -470,14 +500,10 @@ impl Member {
     /// And/or returns [`ModelError::ItemMissing`] if the "default channel" of the guild is not
     /// found.
     #[cfg(feature = "cache")]
-    pub async fn permissions(
-        &self,
-        cache_http: impl CacheHttp + AsRef<Cache>,
-    ) -> Result<Permissions> {
-        let perms_opt = cache_http
+    pub fn permissions(&self, cache: impl AsRef<Cache>) -> Result<Permissions> {
+        let perms_opt = cache
             .as_ref()
-            .guild_field(self.guild_id, |guild| guild._member_permission_from_member(self))
-            .await;
+            .guild_field(self.guild_id, |guild| guild._member_permission_from_member(self));
 
         match perms_opt {
             Some(perms) => Ok(perms),
@@ -507,7 +533,11 @@ impl Member {
             return Ok(());
         }
 
-        match http.as_ref().remove_member_role(self.guild_id.0, self.user.id.0, role_id.0).await {
+        match http
+            .as_ref()
+            .remove_member_role(self.guild_id.0, self.user.id.0, role_id.0, None)
+            .await
+        {
             Ok(()) => {
                 self.roles.retain(|r| r.0 != role_id.0);
 
@@ -537,9 +567,9 @@ impl Member {
 
         let mut builder = EditMember::default();
         builder.roles(&self.roles);
-        let map = utils::hashmap_to_json_map(builder.0);
+        let map = json::hashmap_to_json_map(builder.0);
 
-        match http.as_ref().edit_member(self.guild_id.0, self.user.id.0, &map).await {
+        match http.as_ref().edit_member(self.guild_id.0, self.user.id.0, &map, None).await {
             Ok(member) => Ok(member.roles),
             Err(why) => {
                 self.roles.extend_from_slice(role_ids);
@@ -555,12 +585,11 @@ impl Member {
     ///
     /// If role data can not be found for the member, then [`None`] is returned.
     #[cfg(feature = "cache")]
-    pub async fn roles(&self, cache: impl AsRef<Cache>) -> Option<Vec<Role>> {
+    pub fn roles(&self, cache: impl AsRef<Cache>) -> Option<Vec<Role>> {
         Some(
             cache
                 .as_ref()
-                .guild_field(self.guild_id, |g| g.roles.clone())
-                .await?
+                .guild_field(self.guild_id, |g| g.roles.clone())?
                 .into_iter()
                 .map(|(_, v)| v)
                 .filter(|role| self.roles.contains(&role.id))
@@ -580,13 +609,14 @@ impl Member {
     /// [Ban Members]: Permissions::BAN_MEMBERS
     #[inline]
     pub async fn unban(&self, http: impl AsRef<Http>) -> Result<()> {
-        http.as_ref().remove_ban(self.guild_id.0, self.user.id.0).await
+        http.as_ref().remove_ban(self.guild_id.0, self.user.id.0, None).await
     }
 
     /// Returns the formatted URL of the member's per guild avatar, if one exists.
     ///
     /// This will produce a WEBP image URL, or GIF if the member has a GIF avatar.
     #[inline]
+    #[must_use]
     pub fn avatar_url(&self) -> Option<String> {
         avatar_url(self.guild_id, self.user.id, self.avatar.as_ref())
     }
@@ -597,12 +627,13 @@ impl Member {
     /// This will call [`Self::avatar_url`] first, and if that returns [`None`],
     /// it then falls back to [`User::face()`].
     #[inline]
+    #[must_use]
     pub fn face(&self) -> String {
         self.avatar_url().unwrap_or_else(|| self.user.face())
     }
 }
 
-impl Display for Member {
+impl fmt::Display for Member {
     /// Mentions the user so that they receive a notification.
     ///
     /// # Examples
@@ -613,8 +644,8 @@ impl Display for Member {
     /// ```
     ///
     /// This is in the format of `<@USER_ID>`.
-    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
-        Display::fmt(&self.user.mention(), f)
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt::Display::fmt(&self.user.mention(), f)
     }
 }
 
@@ -628,7 +659,7 @@ pub struct PartialMember {
     #[serde(default)]
     pub deaf: bool,
     /// Timestamp representing the date when the member joined.
-    pub joined_at: Option<DateTime<Utc>>,
+    pub joined_at: Option<Timestamp>,
     /// Indicator of whether the member can speak in voice channels
     #[serde(default)]
     pub mute: bool,
@@ -642,7 +673,7 @@ pub struct PartialMember {
     #[serde(default)]
     pub pending: bool,
     /// Timestamp representing the date since the member is boosting the guild.
-    pub premium_since: Option<DateTime<Utc>>,
+    pub premium_since: Option<Timestamp>,
     /// The unique Id of the guild that the member is a part of.
     pub guild_id: Option<GuildId>,
     /// Attached User struct.
@@ -652,7 +683,6 @@ pub struct PartialMember {
     /// This is only [`Some`] when returned in an [`Interaction`] object.
     ///
     /// [`Interaction`]: crate::model::interactions::Interaction
-    #[cfg(feature = "unstable_discord_api")]
     pub permissions: Option<Permissions>,
 }
 
@@ -673,38 +703,16 @@ pub struct ThreadMember {
     /// The id of the user.
     pub user_id: Option<UserId>,
     /// The time the current user last joined the thread.
-    pub join_timestamp: DateTime<Utc>,
+    pub join_timestamp: Timestamp,
     /// Any user-thread settings, currently only used for notifications
     pub flags: ThreadMemberFlags,
 }
 
-/// Describes extra features of the message.
-#[derive(Copy, PartialEq, Eq, Clone, PartialOrd, Ord, Hash)]
-pub struct ThreadMemberFlags {
-    pub bits: u64,
-}
-
-__impl_bitflags! {
-    ThreadMemberFlags: u64 {
+bitflags! {
+    /// Describes extra features of the message.
+    #[derive(Default)]
+    pub struct ThreadMemberFlags: u64 {
         // Not documented.
-        NOTIFICATIONS = 1 << 0;
-    }
-}
-
-impl<'de> Deserialize<'de> for ThreadMemberFlags {
-    fn deserialize<D>(deserializer: D) -> StdResult<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
-        Ok(ThreadMemberFlags::from_bits_truncate(deserializer.deserialize_u64(U64Visitor)?))
-    }
-}
-
-impl Serialize for ThreadMemberFlags {
-    fn serialize<S>(&self, serializer: S) -> StdResult<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u64(self.bits())
+        const NOTIFICATIONS = 1 << 0;
     }
 }
