@@ -1,6 +1,3 @@
-// FIXME: Remove after the removal of the `̀nsfw` field.
-#![allow(deprecated)]
-
 use serde::de::Error as DeError;
 #[cfg(feature = "cache")]
 use tracing::{error, warn};
@@ -8,11 +5,13 @@ use tracing::{error, warn};
 #[cfg(feature = "model")]
 use crate::builder::{
     CreateChannel,
+    CreateSticker,
     EditGuild,
     EditGuildWelcomeScreen,
     EditGuildWidget,
     EditMember,
     EditRole,
+    EditSticker,
 };
 #[cfg(all(feature = "cache", feature = "utils", feature = "client"))]
 use crate::cache::Cache;
@@ -27,9 +26,11 @@ use crate::collector::{
 };
 #[cfg(feature = "model")]
 use crate::http::{CacheHttp, Http};
+use crate::json::prelude::*;
+use crate::json::{from_number, from_value};
 use crate::model::prelude::*;
-use crate::model::utils::{deserialize_emojis, deserialize_roles};
-#[cfg(all(feature = "model", feature = "unstable_discord_api"))]
+use crate::model::utils::{emojis, roles, stickers};
+#[cfg(feature = "model")]
 use crate::{
     builder::{
         CreateApplicationCommand,
@@ -49,7 +50,7 @@ pub struct PartialGuild {
     pub application_id: Option<ApplicationId>,
     /// The unique Id identifying the guild.
     ///
-    /// This is equivilant to the Id of the default role (`@everyone`) and also
+    /// This is equivalent to the Id of the default role (`@everyone`) and also
     /// that of the default channel (typically `#general`).
     pub id: GuildId,
     /// Id of a voice channel that's considered the AFK channel.
@@ -65,7 +66,7 @@ pub struct PartialGuild {
     /// The channel id that the widget will generate an invite to, or null if set to no invite
     pub widget_channel_id: Option<ChannelId>,
     /// All of the guild's custom emojis.
-    #[serde(serialize_with = "serialize_emojis", deserialize_with = "deserialize_emojis")]
+    #[serde(with = "emojis")]
     pub emojis: HashMap<EmojiId, Emoji>,
     /// Features enabled for the guild.
     ///
@@ -85,11 +86,8 @@ pub struct PartialGuild {
     /// Whether or not the user is the owner of the guild.
     #[serde(default)]
     pub owner: bool,
-    /// The region that the voice servers that the guild uses are located in.
-    #[deprecated(note = "Regions are now set per voice channel instead of globally.")]
-    pub region: String,
     /// A mapping of the guild's roles.
-    #[serde(serialize_with = "serialize_roles", deserialize_with = "deserialize_roles")]
+    #[serde(with = "roles")]
     pub roles: HashMap<RoleId, Role>,
     /// An identifying hash of the guild's splash icon.
     ///
@@ -134,12 +132,6 @@ pub struct PartialGuild {
     pub approximate_member_count: Option<u64>,
     /// Approximate number of non-offline members in this guild.
     pub approximate_presence_count: Option<u64>,
-    /// Whether or not this guild is designated as NSFW. See [`discord support article`].
-    ///
-    /// [`discord support article`]: https://support.discord.com/hc/en-us/articles/1500005389362-NSFW-Server-Designation
-    #[deprecated(note = "Removed in favor of Guild::nsfw_level.")]
-    #[serde(default)]
-    pub nsfw: bool,
     /// The guild NSFW state. See [`discord support article`].
     ///
     /// [`discord support article`]: https://support.discord.com/hc/en-us/articles/1500005389362-NSFW-Server-Designation
@@ -154,6 +146,9 @@ pub struct PartialGuild {
     pub max_members: Option<u64>,
     /// The user permissions in the guild.
     pub permissions: Option<String>,
+    /// All of the guild's custom stickers.
+    #[serde(with = "stickers")]
+    pub stickers: HashMap<StickerId, Sticker>,
 }
 
 #[cfg(feature = "model")]
@@ -258,15 +253,17 @@ impl PartialGuild {
     }
 
     #[cfg(feature = "cache")]
-    pub async fn channel_id_from_name(
+    pub fn channel_id_from_name(
         &self,
         cache: impl AsRef<Cache>,
         name: impl AsRef<str>,
     ) -> Option<ChannelId> {
         let name = name.as_ref();
-        let guild_channels = cache.as_ref().guild_channels(&self.id).await?;
+        let guild_channels = cache.as_ref().guild_channels(&self.id)?;
+
+        for channel_entry in guild_channels.iter() {
+            let (id, channel) = channel_entry.pair();
 
-        for (id, channel) in guild_channels.iter() {
             if channel.name == name {
                 return Some(*id);
             }
@@ -312,7 +309,7 @@ impl PartialGuild {
     /// Refer to the documentation for [`Guild::create_emoji`] for more
     /// information.
     ///
-    /// Requires the [Manage Emojis] permission.
+    /// Requires the [Manage Emojis and Stickers] permission.
     ///
     /// # Examples
     ///
@@ -327,7 +324,7 @@ impl PartialGuild {
     ///
     /// [`EditProfile::avatar`]: crate::builder::EditProfile::avatar
     /// [`utils::read_image`]: crate::utils::read_image
-    /// [Manage Emojis]: Permissions::MANAGE_EMOJIS
+    /// [Manage Emojis and Stickers]: Permissions::MANAGE_EMOJIS_AND_STICKERS
     #[inline]
     pub async fn create_emoji(
         &self,
@@ -366,7 +363,6 @@ impl PartialGuild {
     /// Returns the same possible errors as `create_global_application_command`.
     ///
     /// [`ApplicationCommand`]: crate::model::interactions::application_command::ApplicationCommand
-    #[cfg(feature = "unstable_discord_api")]
     #[allow(clippy::missing_errors_doc)]
     #[inline]
     pub async fn create_application_command<F>(
@@ -390,7 +386,6 @@ impl PartialGuild {
     ///
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
-    #[cfg(feature = "unstable_discord_api")]
     pub async fn set_application_commands<F>(
         &self,
         http: impl AsRef<Http>,
@@ -414,7 +409,6 @@ impl PartialGuild {
     ///
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
-    #[cfg(feature = "unstable_discord_api")]
     pub async fn create_application_command_permission<F>(
         &self,
         http: impl AsRef<Http>,
@@ -440,7 +434,6 @@ impl PartialGuild {
     ///
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
-    #[cfg(feature = "unstable_discord_api")]
     pub async fn set_application_commands_permissions<F>(
         &self,
         http: impl AsRef<Http>,
@@ -462,7 +455,6 @@ impl PartialGuild {
     ///
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
-    #[cfg(feature = "unstable_discord_api")]
     pub async fn get_application_commands(
         &self,
         http: impl AsRef<Http>,
@@ -478,7 +470,6 @@ impl PartialGuild {
     ///
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
-    #[cfg(feature = "unstable_discord_api")]
     pub async fn get_application_command(
         &self,
         http: impl AsRef<Http>,
@@ -495,7 +486,6 @@ impl PartialGuild {
     ///
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
-    #[cfg(feature = "unstable_discord_api")]
     pub async fn edit_application_command<F>(
         &self,
         http: impl AsRef<Http>,
@@ -516,7 +506,6 @@ impl PartialGuild {
     ///
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
-    #[cfg(feature = "unstable_discord_api")]
     pub async fn delete_application_command(
         &self,
         http: impl AsRef<Http>,
@@ -533,7 +522,6 @@ impl PartialGuild {
     ///
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
-    #[cfg(feature = "unstable_discord_api")]
     pub async fn get_application_commands_permissions(
         &self,
         http: impl AsRef<Http>,
@@ -549,7 +537,6 @@ impl PartialGuild {
     ///
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
-    #[cfg(feature = "unstable_discord_api")]
     pub async fn get_application_command_permissions(
         &self,
         http: impl AsRef<Http>,
@@ -578,6 +565,34 @@ impl PartialGuild {
         self.id.create_role(&http, f).await
     }
 
+    /// Creates a new sticker in the guild with the data set, if any.
+    ///
+    /// **Note**: Requires the [Manage Emojis and Stickers] permission.
+    ///
+    /// # Errors
+    ///
+    /// If the `cache` is enabled, returns a [`ModelError::InvalidPermissions`]
+    /// if the current user does not have permission to manage roles.
+    ///
+    /// [Manage Emojis and Stickers]: crate::model::permissions::Permissions::MANAGE_EMOJIS_AND_STICKERS
+    pub async fn create_sticker<'a, F>(&self, cache_http: impl CacheHttp, f: F) -> Result<Sticker>
+    where
+        for<'b> F: FnOnce(&'b mut CreateSticker<'a>) -> &'b mut CreateSticker<'a>,
+    {
+        #[cfg(feature = "cache")]
+        {
+            if cache_http.cache().is_some() {
+                let req = Permissions::MANAGE_EMOJIS_AND_STICKERS;
+
+                if !self.has_perms(&cache_http, req).await {
+                    return Err(Error::Model(ModelError::InvalidPermissions(req)));
+                }
+            }
+        }
+
+        self.id.create_sticker(cache_http.http(), f).await
+    }
+
     /// Deletes the current guild if the current user is the owner of the
     /// guild.
     ///
@@ -594,14 +609,14 @@ impl PartialGuild {
 
     /// Deletes an [`Emoji`] from the guild.
     ///
-    /// Requires the [Manage Emojis] permission.
+    /// Requires the [Manage Emojis and Stickers] permission.
     ///
     /// # Errors
     ///
     /// Returns [`Error::Http`] if the current user lacks permission,
     /// or if an emoji with that Id does not exist in the guild.
     ///
-    /// [Manage Emojis]: Permissions::MANAGE_EMOJIS
+    /// [Manage Emojis and Stickers]: Permissions::MANAGE_EMOJIS_AND_STICKERS
     #[inline]
     pub async fn delete_emoji(
         &self,
@@ -652,6 +667,25 @@ impl PartialGuild {
         self.id.delete_role(&http, role_id).await
     }
 
+    /// Deletes a [`Sticker`] by Id from the guild.
+    ///
+    /// Requires the [Manage Emojis and Stickers] permission.
+    ///
+    /// # Errors
+    ///
+    /// Returns [`Error::Http`] if the current user lacks permission
+    /// to delete the sticker.
+    ///
+    /// [Manage Emojis and Stickers]: crate::model::permissions::Permissions::MANAGE_EMOJIS_AND_STICKERS
+    #[inline]
+    pub async fn delete_sticker(
+        &self,
+        http: impl AsRef<Http>,
+        sticker_id: impl Into<StickerId>,
+    ) -> Result<()> {
+        self.id.delete_sticker(&http, sticker_id).await
+    }
+
     /// Edits the current guild with new data where specified.
     ///
     /// **Note**: Requires the current user to have the [Manage Guild]
@@ -678,12 +712,6 @@ impl PartialGuild {
                 self.mfa_level = guild.mfa_level;
                 self.name = guild.name;
                 self.owner_id = guild.owner_id;
-
-                #[allow(deprecated)]
-                {
-                    self.region = guild.region;
-                }
-
                 self.roles = guild.roles;
                 self.splash = guild.splash;
                 self.verification_level = guild.verification_level;
@@ -699,14 +727,14 @@ impl PartialGuild {
     /// Also see [`Emoji::edit`] if you have the `cache` and `methods` features
     /// enabled.
     ///
-    /// Requires the [Manage Emojis] permission.
+    /// Requires the [Manage Emojis and Stickers] permission.
     ///
     /// # Errors
     ///
     /// Returns [`Error::Http`] if the current user lacks permission,
     /// or if an emoji with that Id does not exist in the guild.
     ///
-    /// [Manage Emojis]: Permissions::MANAGE_EMOJIS
+    /// [Manage Emojis and Stickers]: Permissions::MANAGE_EMOJIS_AND_STICKERS
     #[inline]
     pub async fn edit_emoji(
         &self,
@@ -828,6 +856,37 @@ impl PartialGuild {
         self.id.edit_role_position(&http, role_id, position).await
     }
 
+    /// Edits a sticker, optionally setting its fields.
+    ///
+    /// Requires the [Manage Emojis and Stickers] permission.
+    ///
+    /// # Examples
+    ///
+    /// Rename a sticker:
+    ///
+    /// ```rust,ignore
+    /// guild.edit_sticker(&context, StickerId(7), |r| r.name("Bun bun meow"));
+    /// ```
+    ///
+    /// # Errors
+    ///
+    /// Returns [`Error::Http`] if the current user lacks permission.
+    ///
+    /// [`Error::Http`]: crate::error::Error::Http
+    /// [Manage Emojis and Stickers]: crate::model::permissions::Permissions::MANAGE_EMOJIS_AND_STICKERS
+    #[inline]
+    pub async fn edit_sticker<F>(
+        &self,
+        http: impl AsRef<Http>,
+        sticker_id: impl Into<StickerId>,
+        f: F,
+    ) -> Result<Sticker>
+    where
+        F: FnOnce(&mut EditSticker) -> &mut EditSticker,
+    {
+        self.id.edit_sticker(&http, sticker_id, f).await
+    }
+
     /// Edits the [`GuildWelcomeScreen`].
     ///
     /// # Errors
@@ -835,7 +894,6 @@ impl PartialGuild {
     /// Returns an [`Error::Http`] if some mandatory fields are not provided.
     ///
     /// [`Error::Http`]: crate::error::Error::Http
-    /// [`GuildWelcomeScreen`]: super::guild::GuildWelcomeScreen
     pub async fn edit_welcome_screen<F>(
         &self,
         http: impl AsRef<Http>,
@@ -855,7 +913,6 @@ impl PartialGuild {
     /// permission.
     ///
     /// [`Error::Http`]: crate::error::Error::Http
-    /// [`GuildWelcomeScreen`]: super::guild::GuildWelcomeScreen
     pub async fn edit_widget<F>(&self, http: impl AsRef<Http>, f: F) -> Result<GuildWidget>
     where
         F: FnOnce(&mut EditGuildWidget) -> &mut EditGuildWidget,
@@ -919,21 +976,17 @@ impl PartialGuild {
 
         let lhs = cache
             .as_ref()
-            .guild(self.id)
-            .await?
+            .guild(self.id)?
             .members
             .get(&lhs_id)?
             .highest_role_info(&cache)
-            .await
             .unwrap_or((RoleId(0), 0));
         let rhs = cache
             .as_ref()
-            .guild(self.id)
-            .await?
+            .guild(self.id)?
             .members
             .get(&rhs_id)?
             .highest_role_info(&cache)
-            .await
             .unwrap_or((RoleId(0), 0));
 
         // If LHS and RHS both have no top position or have the same role ID,
@@ -993,13 +1046,12 @@ impl PartialGuild {
             return Ok(Permissions::all());
         }
 
-        let everyone = match self.roles.get(&RoleId(self.id.0)) {
-            Some(everyone) => everyone,
-            None => {
-                error!("@everyone role ({}) missing in '{}'", self.id, self.name,);
+        let everyone = if let Some(everyone) = self.roles.get(&RoleId(self.id.0)) {
+            everyone
+        } else {
+            error!("@everyone role ({}) missing in '{}'", self.id, self.name,);
 
-                return Ok(Permissions::empty());
-            },
+            return Ok(Permissions::empty());
         };
 
         let member = self.member(cache_http, &user_id).await?;
@@ -1104,7 +1156,7 @@ impl PartialGuild {
     #[cfg(feature = "cache")]
     async fn has_perms(&self, cache_http: impl CacheHttp, mut permissions: Permissions) -> bool {
         if let Some(cache) = cache_http.cache() {
-            let user_id = cache.current_user().await.id;
+            let user_id = cache.current_user().id;
 
             if let Ok(perms) = self.member_permissions(&cache_http, user_id).await {
                 permissions.remove(perms);
@@ -1133,11 +1185,11 @@ impl PartialGuild {
         self.id.kick(&http, user_id).await
     }
 
-    #[inline]
     /// # Errors
     ///
     /// In addition to the reasons [`Self::kick`] may return an error,
     /// can also return an error if the reason is too long.
+    #[inline]
     pub async fn kick_with_reason(
         &self,
         http: impl AsRef<Http>,
@@ -1148,13 +1200,15 @@ impl PartialGuild {
     }
 
     /// Returns a formatted URL of the guild's icon, if the guild has an icon.
+    #[must_use]
     pub fn icon_url(&self) -> Option<String> {
-        self.icon.as_ref().map(|icon| format!(cdn!("/icons/{}/{}.webp"), self.id, icon))
+        self.icon.as_ref().map(|icon| cdn!("/icons/{}/{}.webp", self.id, icon))
     }
 
     /// Returns a formatted URL of the guild's banner, if the guild has a banner.
+    #[must_use]
     pub fn banner_url(&self) -> Option<String> {
-        self.banner.as_ref().map(|banner| format!(cdn!("/banners/{}/{}.webp"), self.id, banner))
+        self.banner.as_ref().map(|banner| cdn!("/banners/{}/{}.webp", self.id, banner))
     }
 
     /// Gets all [`Emoji`]s of this guild via HTTP.
@@ -1245,7 +1299,7 @@ impl PartialGuild {
     /// may also return [`Error::NotInRange`] if the input is
     /// not within range.
     ///
-    /// [`User`]: ../user/struct.User.html
+    /// [`User`]: crate::model::user::User
     #[inline]
     pub async fn members(
         &self,
@@ -1331,8 +1385,8 @@ impl PartialGuild {
     /// [`utils::shard_id`]: crate::utils::shard_id
     #[cfg(all(feature = "cache", feature = "utils"))]
     #[inline]
-    pub async fn shard_id(&self, cache: impl AsRef<Cache>) -> u64 {
-        self.id.shard_id(cache).await
+    pub fn shard_id(&self, cache: impl AsRef<Cache>) -> u64 {
+        self.id.shard_id(cache)
     }
 
     /// Returns the Id of the shard associated with the guild.
@@ -1357,16 +1411,15 @@ impl PartialGuild {
     /// ```
     #[cfg(all(feature = "utils", not(feature = "cache")))]
     #[inline]
-    pub async fn shard_id(&self, shard_count: u64) -> u64 {
-        self.id.shard_id(shard_count).await
+    pub fn shard_id(&self, shard_count: u64) -> u64 {
+        self.id.shard_id(shard_count)
     }
 
     /// Returns the formatted URL of the guild's splash image, if one exists.
     #[inline]
+    #[must_use]
     pub fn splash_url(&self) -> Option<String> {
-        self.splash
-            .as_ref()
-            .map(|splash| format!(cdn!("/splashes/{}/{}.webp?size=4096"), self.id, splash))
+        self.splash.as_ref().map(|splash| cdn!("/splashes/{}/{}.webp?size=4096", self.id, splash))
     }
 
     /// Starts an integration sync for the given integration Id.
@@ -1454,7 +1507,7 @@ impl PartialGuild {
     /// impl EventHandler for Handler {
     ///     async fn message(&self, context: Context, msg: Message) {
     ///         if let Some(guild_id) = msg.guild_id {
-    ///             if let Some(guild) = guild_id.to_guild_cached(&context).await {
+    ///             if let Some(guild) = guild_id.to_guild_cached(&context) {
     ///                 if let Some(role) = guild.role_by_name("role_name") {
     ///                     println!("Obtained role's reference: {:?}", role);
     ///                 }
@@ -1463,50 +1516,46 @@ impl PartialGuild {
     ///     }
     /// }
     ///
-    /// let mut client = Client::builder("token").event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder("token", GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// client.start().await?;
     /// #    Ok(())
     /// # }
     /// ```
     #[inline]
+    #[must_use]
     pub fn role_by_name(&self, role_name: &str) -> Option<&Role> {
         self.roles.values().find(|role| role_name == role.name)
     }
 
     /// Returns a future that will await one message sent in this guild.
     #[cfg(feature = "collector")]
-    pub fn await_reply<'a>(
-        &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> CollectReply<'a> {
+    pub fn await_reply(&self, shard_messenger: impl AsRef<ShardMessenger>) -> CollectReply {
         CollectReply::new(shard_messenger).guild_id(self.id.0)
     }
 
     /// Returns a stream builder which can be awaited to obtain a stream of messages in this guild.
     #[cfg(feature = "collector")]
-    pub fn await_replies<'a>(
+    pub fn await_replies(
         &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> MessageCollectorBuilder<'a> {
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> MessageCollectorBuilder {
         MessageCollectorBuilder::new(shard_messenger).guild_id(self.id.0)
     }
 
     /// Await a single reaction in this guild.
     #[cfg(feature = "collector")]
-    pub fn await_reaction<'a>(
-        &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> CollectReaction<'a> {
+    pub fn await_reaction(&self, shard_messenger: impl AsRef<ShardMessenger>) -> CollectReaction {
         CollectReaction::new(shard_messenger).guild_id(self.id.0)
     }
 
     /// Returns a stream builder which can be awaited to obtain a stream of reactions sent in this guild.
     #[cfg(feature = "collector")]
-    pub fn await_reactions<'a>(
+    pub fn await_reactions(
         &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> ReactionCollectorBuilder<'a> {
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> ReactionCollectorBuilder {
         ReactionCollectorBuilder::new(shard_messenger).guild_id(self.id.0)
     }
 
@@ -1525,20 +1574,20 @@ impl<'de> Deserialize<'de> for PartialGuild {
     fn deserialize<D: Deserializer<'de>>(deserializer: D) -> StdResult<Self, D::Error> {
         let mut map = JsonMap::deserialize(deserializer)?;
 
-        let id = map.get("id").and_then(|x| x.as_str()).and_then(|x| x.parse::<u64>().ok());
+        let id = map.get("id").and_then(Value::as_str).and_then(|x| x.parse::<u64>().ok());
 
         if let Some(guild_id) = id {
-            if let Some(array) = map.get_mut("roles").and_then(|x| x.as_array_mut()) {
+            if let Some(array) = map.get_mut("roles").and_then(Value::as_array_mut) {
                 for value in array {
                     if let Some(role) = value.as_object_mut() {
-                        role.insert("guild_id".to_string(), Value::Number(Number::from(guild_id)));
+                        role.insert("guild_id".to_string(), from_number(guild_id));
                     }
                 }
             }
         }
 
         let afk_channel_id = match map.remove("afk_channel_id") {
-            Some(v) => serde_json::from_value::<Option<ChannelId>>(v).map_err(DeError::custom)?,
+            Some(v) => from_value::<Option<ChannelId>>(v).map_err(DeError::custom)?,
             None => None,
         };
         let afk_timeout = map
@@ -1554,12 +1603,12 @@ impl<'de> Deserialize<'de> for PartialGuild {
         let emojis = map
             .remove("emojis")
             .ok_or_else(|| DeError::custom("expected guild emojis"))
-            .and_then(deserialize_emojis)
+            .and_then(emojis::deserialize)
             .map_err(DeError::custom)?;
         let features = map
             .remove("features")
-            .ok_or_else(|| DeError::custom("expected guild features"))
-            .and_then(serde_json::from_value::<Vec<String>>)
+            .ok_or(Error::Other("expected guild features"))
+            .and_then(from_value::<Vec<String>>)
             .map_err(DeError::custom)?;
         let icon = match map.remove("icon") {
             Some(v) => Option::<String>::deserialize(v).map_err(DeError::custom)?,
@@ -1585,15 +1634,10 @@ impl<'de> Deserialize<'de> for PartialGuild {
             .ok_or_else(|| DeError::custom("expected guild owner_id"))
             .and_then(UserId::deserialize)
             .map_err(DeError::custom)?;
-        let region = map
-            .remove("region")
-            .ok_or_else(|| DeError::custom("expected guild region"))
-            .and_then(String::deserialize)
-            .map_err(DeError::custom)?;
         let roles = map
             .remove("roles")
             .ok_or_else(|| DeError::custom("expected guild roles"))
-            .and_then(deserialize_roles)
+            .and_then(roles::deserialize)
             .map_err(DeError::custom)?;
         let splash = match map.remove("splash") {
             Some(v) => Option::<String>::deserialize(v).map_err(DeError::custom)?,
@@ -1613,7 +1657,10 @@ impl<'de> Deserialize<'de> for PartialGuild {
             None => PremiumTier::default(),
         };
         let premium_subscription_count = match map.remove("premium_subscription_count") {
+            #[cfg(not(feature = "simd-json"))]
             Some(Value::Null) | None => 0,
+            #[cfg(feature = "simd-json")]
+            Some(Value::Static(StaticNode::Null)) | None => 0,
             Some(v) => u64::deserialize(v).map_err(DeError::custom)?,
         };
         let banner = match map.remove("banner") {
@@ -1629,71 +1676,44 @@ impl<'de> Deserialize<'de> for PartialGuild {
             None => None,
         };
 
-        let approximate_member_count = match map.contains_key("approximate_member_count") {
-            true => Some(
-                map.remove("approximate_member_count")
-                    .ok_or_else(|| DeError::custom("expected approximate_member_count"))
-                    .and_then(u64::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
-        };
-
-        let approximate_presence_count = match map.contains_key("approximate_presence_count") {
-            true => Some(
-                map.remove("approximate_presence_count")
-                    .ok_or_else(|| DeError::custom("expected approximate_presence_count"))
-                    .and_then(u64::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
-        };
-
-        let owner = match map.contains_key("owner") {
-            true => map
-                .remove("owner")
-                .ok_or_else(|| DeError::custom("expected owner"))
-                .and_then(bool::deserialize)
-                .map_err(DeError::custom)?,
-            false => false,
-        };
-
-        let nsfw = map
-            .remove("nsfw")
-            .ok_or_else(|| DeError::custom("expected approximate_presence_count"))
-            .and_then(bool::deserialize)
+        let approximate_member_count = map
+            .remove("approximate_member_count")
+            .map(u64::deserialize)
+            .transpose()
             .map_err(DeError::custom)?;
 
+        let approximate_presence_count = map
+            .remove("approximate_presence_count")
+            .map(u64::deserialize)
+            .transpose()
+            .map_err(DeError::custom)?;
+
+        let owner = map
+            .remove("owner")
+            .map(bool::deserialize)
+            .transpose()
+            .map_err(DeError::custom)?
+            .unwrap_or_default();
+
         let nsfw_level = map
             .remove("nsfw_level")
             .ok_or_else(|| DeError::custom("expected nsfw_level"))
             .and_then(NsfwLevel::deserialize)
             .map_err(DeError::custom)?;
 
-        let max_video_channel_users = match map.contains_key("max_video_channel_users") {
-            true => Some(
-                map.remove("max_video_channel_users")
-                    .ok_or_else(|| DeError::custom("expected max_video_channel_users"))
-                    .and_then(u64::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
-        };
+        let max_video_channel_users = map
+            .remove("max_video_channel_users")
+            .map(u64::deserialize)
+            .transpose()
+            .map_err(DeError::custom)?;
 
         let max_presences = match map.remove("max_presences") {
             Some(v) => Option::<u64>::deserialize(v).map_err(DeError::custom)?,
             None => None,
         };
 
-        let max_members = match map.contains_key("max_members") {
-            true => Some(
-                map.remove("max_members")
-                    .ok_or_else(|| DeError::custom("expected max_members"))
-                    .and_then(u64::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
-        };
+        let max_members =
+            map.remove("max_members").map(u64::deserialize).transpose().map_err(DeError::custom)?;
 
         let discovery_splash = match map.remove("discovery_splash") {
             Some(v) => Option::<String>::deserialize(v).map_err(DeError::custom)?,
@@ -1736,17 +1756,18 @@ impl<'de> Deserialize<'de> for PartialGuild {
             None => None,
         };
 
-        let welcome_screen = match map.contains_key("welcome_screen") {
-            true => Some(
-                map.remove("welcome_screen")
-                    .ok_or_else(|| DeError::custom("expected welcome_screen"))
-                    .and_then(GuildWelcomeScreen::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
-        };
+        let welcome_screen = map
+            .remove("welcome_screen")
+            .map(GuildWelcomeScreen::deserialize)
+            .transpose()
+            .map_err(DeError::custom)?;
+
+        let stickers = map
+            .remove("stickers")
+            .ok_or_else(|| DeError::custom("expected guild stickers"))
+            .and_then(stickers::deserialize)
+            .map_err(DeError::custom)?;
 
-        #[allow(deprecated)]
         Ok(Self {
             application_id,
             id,
@@ -1762,7 +1783,6 @@ impl<'de> Deserialize<'de> for PartialGuild {
             name,
             owner_id,
             owner,
-            region,
             roles,
             splash,
             discovery_splash,
@@ -1779,12 +1799,61 @@ impl<'de> Deserialize<'de> for PartialGuild {
             welcome_screen,
             approximate_member_count,
             approximate_presence_count,
-            nsfw,
             nsfw_level,
             max_video_channel_users,
             max_presences,
             max_members,
             permissions,
+            stickers,
         })
     }
 }
+
+impl From<Guild> for PartialGuild {
+    /// Converts this [`Guild`] instance into a [`PartialGuild`]
+    ///
+    /// [`PartialGuild`] is not a strict subset and contains some data specific to the current user
+    /// that [`Guild`] does not contain. Therefore, this method needs access to cache and HTTP to
+    /// generate the missing data
+    fn from(guild: Guild) -> Self {
+        Self {
+            application_id: guild.application_id,
+            id: guild.id,
+            afk_channel_id: guild.afk_channel_id,
+            afk_timeout: guild.afk_timeout,
+            default_message_notifications: guild.default_message_notifications,
+            widget_enabled: guild.widget_enabled,
+            widget_channel_id: guild.widget_channel_id,
+            emojis: guild.emojis,
+            features: guild.features,
+            icon: guild.icon,
+            mfa_level: guild.mfa_level,
+            name: guild.name,
+            owner_id: guild.owner_id,
+            // Not very good, but PartialGuild deserialization uses .unwrap_or_default() too
+            owner: Default::default(),
+            roles: guild.roles,
+            splash: guild.splash,
+            discovery_splash: guild.discovery_splash,
+            system_channel_id: guild.system_channel_id,
+            system_channel_flags: guild.system_channel_flags,
+            rules_channel_id: guild.rules_channel_id,
+            public_updates_channel_id: guild.public_updates_channel_id,
+            verification_level: guild.verification_level,
+            description: guild.description,
+            premium_tier: guild.premium_tier,
+            premium_subscription_count: guild.premium_subscription_count,
+            banner: guild.banner,
+            vanity_url_code: guild.vanity_url_code,
+            welcome_screen: guild.welcome_screen,
+            approximate_member_count: guild.approximate_member_count,
+            approximate_presence_count: guild.approximate_presence_count,
+            nsfw_level: guild.nsfw_level,
+            max_video_channel_users: guild.max_video_channel_users,
+            max_presences: guild.max_presences,
+            max_members: guild.max_members,
+            permissions: None,
+            stickers: guild.stickers,
+        }
+    }
+}
