@@ -4,16 +4,22 @@ use serde::de::Error as DeError;
 use serde::{Deserialize, Deserializer};
 
 use super::prelude::*;
+#[cfg(feature = "http")]
+use crate::builder::CreateApplicationCommand;
+#[cfg(feature = "http")]
 use crate::builder::{
-    CreateApplicationCommand,
     CreateApplicationCommands,
     CreateInteractionResponse,
     CreateInteractionResponseFollowup,
     EditInteractionResponse,
 };
+#[cfg(feature = "http")]
 use crate::http::Http;
-use crate::internal::prelude::{JsonMap, StdResult, Value};
-use crate::model::channel::{ChannelType, PartialChannel};
+use crate::internal::prelude::StdResult;
+#[cfg(feature = "http")]
+use crate::json;
+use crate::json::prelude::*;
+use crate::model::channel::{Attachment, ChannelType, PartialChannel};
 use crate::model::guild::{Member, PartialMember, Role};
 use crate::model::id::{
     ApplicationId,
@@ -26,16 +32,7 @@ use crate::model::id::{
 };
 use crate::model::interactions::InteractionType;
 use crate::model::prelude::User;
-use crate::model::utils::{
-    deserialize_channels_map,
-    deserialize_messages_map,
-    deserialize_options,
-    deserialize_options_with_resolved,
-    deserialize_partial_members_map,
-    deserialize_roles_map,
-    deserialize_users,
-};
-use crate::utils;
+use crate::model::utils::deserialize_options_with_resolved;
 
 /// An interaction when a user invokes a slash command.
 #[derive(Clone, Debug, Serialize)]
@@ -72,6 +69,7 @@ pub struct ApplicationCommandInteraction {
     pub locale: String,
 }
 
+#[cfg(feature = "http")]
 impl ApplicationCommandInteraction {
     /// Gets the interaction response.
     ///
@@ -98,19 +96,43 @@ impl ApplicationCommandInteraction {
     /// [`Error::Model`]: crate::error::Error::Model
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
-    pub async fn create_interaction_response<F>(&self, http: impl AsRef<Http>, f: F) -> Result<()>
+    pub async fn create_interaction_response<'a, F>(
+        &self,
+        http: impl AsRef<Http>,
+        f: F,
+    ) -> Result<()>
     where
-        F: FnOnce(&mut CreateInteractionResponse) -> &mut CreateInteractionResponse,
+        for<'b> F:
+            FnOnce(&'b mut CreateInteractionResponse<'a>) -> &'b mut CreateInteractionResponse<'a>,
     {
         let mut interaction_response = CreateInteractionResponse::default();
         f(&mut interaction_response);
+        self._create_interaction_response(http.as_ref(), interaction_response).await
+    }
 
-        let map = utils::hashmap_to_json_map(interaction_response.0);
+    async fn _create_interaction_response<'a>(
+        &self,
+        http: &Http,
+        interaction_response: CreateInteractionResponse<'a>,
+    ) -> Result<()> {
+        let map = json::hashmap_to_json_map(interaction_response.0);
 
-        Message::check_content_length(&map)?;
-        Message::check_embed_length(&map)?;
+        Message::check_lengths(&map)?;
 
-        http.as_ref().create_interaction_response(self.id.0, &self.token, &Value::Object(map)).await
+        if interaction_response.1.is_empty() {
+            http.as_ref()
+                .create_interaction_response(self.id.0, &self.token, &Value::from(map))
+                .await
+        } else {
+            http.as_ref()
+                .create_interaction_response_with_files(
+                    self.id.0,
+                    &self.token,
+                    &Value::from(map),
+                    interaction_response.1,
+                )
+                .await
+        }
     }
 
     /// Edits the initial interaction response.
@@ -121,8 +143,6 @@ impl ApplicationCommandInteraction {
     ///
     /// **Note**:   Message contents must be under 2000 unicode code points, does not work on ephemeral messages.
     ///
-    /// [`UserId`]: crate::model::id::UserId
-    ///
     /// # Errors
     ///
     /// Returns [`Error::Model`] if the edited content is too long.
@@ -143,12 +163,11 @@ impl ApplicationCommandInteraction {
         let mut interaction_response = EditInteractionResponse::default();
         f(&mut interaction_response);
 
-        let map = utils::hashmap_to_json_map(interaction_response.0);
+        let map = json::hashmap_to_json_map(interaction_response.0);
 
-        Message::check_content_length(&map)?;
-        Message::check_embed_length(&map)?;
+        Message::check_lengths(&map)?;
 
-        http.as_ref().edit_original_interaction_response(&self.token, &Value::Object(map)).await
+        http.as_ref().edit_original_interaction_response(&self.token, &Value::from(map)).await
     }
 
     /// Deletes the initial interaction response.
@@ -186,13 +205,29 @@ impl ApplicationCommandInteraction {
     {
         let mut interaction_response = CreateInteractionResponseFollowup::default();
         f(&mut interaction_response);
+        self._create_followup_message(http.as_ref(), interaction_response).await
+    }
 
-        let map = utils::hashmap_to_json_map(interaction_response.0);
+    async fn _create_followup_message<'a>(
+        &self,
+        http: &Http,
+        interaction_response: CreateInteractionResponseFollowup<'a>,
+    ) -> Result<Message> {
+        let map = json::hashmap_to_json_map(interaction_response.0);
 
-        Message::check_content_length(&map)?;
-        Message::check_embed_length(&map)?;
+        Message::check_lengths(&map)?;
 
-        http.as_ref().create_followup_message(&self.token, &Value::Object(map)).await
+        if interaction_response.1.is_empty() {
+            http.as_ref().create_followup_message(&self.token, &Value::from(map)).await
+        } else {
+            http.as_ref()
+                .create_followup_message_with_files(
+                    &self.token,
+                    &Value::from(map),
+                    interaction_response.1,
+                )
+                .await
+        }
     }
 
     /// Edits a followup response to the response sent.
@@ -222,14 +257,24 @@ impl ApplicationCommandInteraction {
         let mut interaction_response = CreateInteractionResponseFollowup::default();
         f(&mut interaction_response);
 
-        let map = utils::hashmap_to_json_map(interaction_response.0);
+        let map = json::hashmap_to_json_map(interaction_response.0);
 
-        Message::check_content_length(&map)?;
-        Message::check_embed_length(&map)?;
+        Message::check_lengths(&map)?;
 
-        http.as_ref()
-            .edit_followup_message(&self.token, message_id.into().into(), &Value::Object(map))
-            .await
+        let message_id = message_id.into().into();
+
+        if interaction_response.1.is_empty() {
+            http.as_ref().edit_followup_message(&self.token, message_id, &Value::from(map)).await
+        } else {
+            http.as_ref()
+                .edit_followup_message_and_attachments(
+                    &self.token,
+                    message_id,
+                    &Value::from(map),
+                    interaction_response.1,
+                )
+                .await
+        }
     }
 
     /// Deletes a followup message.
@@ -268,8 +313,6 @@ impl ApplicationCommandInteraction {
     /// or an [`Error::Json`] if there is an error in deserializing the
     /// API response.
     ///
-    /// # Errors
-    ///
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
     pub async fn defer(&self, http: impl AsRef<Http>) -> Result<()> {
@@ -284,11 +327,11 @@ impl<'de> Deserialize<'de> for ApplicationCommandInteraction {
     fn deserialize<D: Deserializer<'de>>(deserializer: D) -> StdResult<Self, D::Error> {
         let mut map = JsonMap::deserialize(deserializer)?;
 
-        let id = map.get("guild_id").and_then(|x| x.as_str()).and_then(|x| x.parse::<u64>().ok());
+        let id = map.get("guild_id").and_then(Value::as_str).and_then(|x| x.parse::<u64>().ok());
 
         if let Some(guild_id) = id {
-            if let Some(member) = map.get_mut("member").and_then(|x| x.as_object_mut()) {
-                member.insert("guild_id".to_string(), Value::Number(Number::from(guild_id)));
+            if let Some(member) = map.get_mut("member").and_then(Value::as_object_mut) {
+                member.insert("guild_id".to_string(), from_number(guild_id));
             }
 
             if let Some(data) = map.get_mut("data") {
@@ -298,7 +341,7 @@ impl<'de> Deserialize<'de> for ApplicationCommandInteraction {
                             for value in values.values_mut() {
                                 value.as_object_mut().expect("couldn't deserialize").insert(
                                     "guild_id".to_string(),
-                                    Value::String(guild_id.to_string()),
+                                    Value::from(guild_id.to_string()),
                                 );
                             }
                         }
@@ -312,7 +355,7 @@ impl<'de> Deserialize<'de> for ApplicationCommandInteraction {
                                     .expect("couldn't deserialize application command")
                                     .insert(
                                         "guild_id".to_string(),
-                                        Value::String(guild_id.to_string()),
+                                        Value::from(guild_id.to_string()),
                                     );
                             }
                         }
@@ -345,15 +388,11 @@ impl<'de> Deserialize<'de> for ApplicationCommandInteraction {
             .and_then(ApplicationCommandInteractionData::deserialize)
             .map_err(DeError::custom)?;
 
-        let guild_id = match map.contains_key("guild_id") {
-            true => Some(
-                map.remove("guild_id")
-                    .ok_or_else(|| DeError::custom("expected guild_id"))
-                    .and_then(GuildId::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
-        };
+        let guild_id = map
+            .remove("guild_id")
+            .map(GuildId::deserialize)
+            .transpose()
+            .map_err(DeError::custom)?;
 
         let channel_id = map
             .remove("channel_id")
@@ -361,24 +400,15 @@ impl<'de> Deserialize<'de> for ApplicationCommandInteraction {
             .and_then(ChannelId::deserialize)
             .map_err(DeError::custom)?;
 
-        let member = match map.contains_key("member") {
-            true => Some(
-                map.remove("member")
-                    .ok_or_else(|| DeError::custom("expected member"))
-                    .and_then(Member::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
-        };
+        let member =
+            map.remove("member").map(Member::deserialize).transpose().map_err(DeError::custom)?;
 
-        let user = match map.contains_key("user") {
-            true => map
-                .remove("user")
-                .ok_or_else(|| DeError::custom("expected user"))
-                .and_then(User::deserialize)
-                .map_err(DeError::custom)?,
-            false => member.as_ref().expect("expected user or member").user.clone(),
-        };
+        let user =
+            map.remove("user").map(User::deserialize).transpose().map_err(DeError::custom)?;
+
+        let user = user
+            .or_else(|| member.as_ref().map(|m| m.user.clone()))
+            .ok_or_else(|| DeError::custom("expected user or member"))?;
 
         let token = map
             .remove("token")
@@ -392,15 +422,11 @@ impl<'de> Deserialize<'de> for ApplicationCommandInteraction {
             .and_then(u8::deserialize)
             .map_err(DeError::custom)?;
 
-        let guild_locale = match map.contains_key("guild_locale") {
-            true => Some(
-                map.remove("guild_locale")
-                    .ok_or_else(|| DeError::custom("expected guild_locale"))
-                    .and_then(String::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
-        };
+        let guild_locale = map
+            .remove("guild_locale")
+            .map(String::deserialize)
+            .transpose()
+            .map_err(DeError::custom)?;
 
         let locale = map
             .remove("locale")
@@ -442,6 +468,9 @@ pub struct ApplicationCommandInteractionData {
     /// The converted objects from the given options.
     #[serde(default)]
     pub resolved: ApplicationCommandInteractionDataResolved,
+    /// The Id of the guild the command is registered to.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub guild_id: Option<GuildId>,
     /// The targeted user or message, if the triggered application command type
     /// is [`User`] or [`Message`].
     ///
@@ -451,11 +480,32 @@ pub struct ApplicationCommandInteractionData {
     /// [`User`]: ApplicationCommandType::User
     /// [`Message`]: ApplicationCommandType::Message
     pub target_id: Option<TargetId>,
+}
+
+impl ApplicationCommandInteractionData {
     /// The target resolved data of [`target_id`]
     ///
     /// [`target_id`]: Self::target_id
-    #[serde(skip_serializing)]
-    pub target: Option<ResolvedTarget>,
+    #[must_use]
+    pub fn target(&self) -> Option<ResolvedTarget> {
+        match (self.kind, self.target_id) {
+            (ApplicationCommandType::User, Some(id)) => {
+                let user_id = id.to_user_id();
+
+                let user = self.resolved.users.get(&user_id).cloned()?;
+                let member = self.resolved.members.get(&user_id).cloned();
+
+                Some(ResolvedTarget::User(user, member.map(Box::new)))
+            },
+            (ApplicationCommandType::Message, Some(id)) => {
+                let message_id = id.to_message_id();
+                let message = self.resolved.messages.get(&message_id).cloned()?;
+
+                Some(ResolvedTarget::Message(Box::new(message)))
+            },
+            _ => None,
+        }
+    }
 }
 
 impl<'de> Deserialize<'de> for ApplicationCommandInteractionData {
@@ -474,23 +524,19 @@ impl<'de> Deserialize<'de> for ApplicationCommandInteractionData {
             .and_then(CommandId::deserialize)
             .map_err(DeError::custom)?;
 
-        let resolved = match map.contains_key("resolved") {
-            true => map
-                .remove("resolved")
-                .ok_or_else(|| DeError::custom("expected resolved"))
-                .and_then(ApplicationCommandInteractionDataResolved::deserialize)
-                .map_err(DeError::custom)?,
-            false => ApplicationCommandInteractionDataResolved::default(),
-        };
+        let resolved = map
+            .remove("resolved")
+            .map(ApplicationCommandInteractionDataResolved::deserialize)
+            .transpose()
+            .map_err(DeError::custom)?
+            .unwrap_or_default();
 
-        let options = match map.contains_key("options") {
-            true => map
-                .remove("options")
-                .ok_or_else(|| DeError::custom("expected options"))
-                .and_then(|deserializer| deserialize_options_with_resolved(deserializer, &resolved))
-                .map_err(DeError::custom)?,
-            false => vec![],
-        };
+        let options = map
+            .remove("options")
+            .map(|deserializer| deserialize_options_with_resolved(deserializer, &resolved))
+            .transpose()
+            .map_err(DeError::custom)?
+            .unwrap_or_default();
 
         let kind = map
             .remove("type")
@@ -498,46 +544,24 @@ impl<'de> Deserialize<'de> for ApplicationCommandInteractionData {
             .and_then(ApplicationCommandType::deserialize)
             .map_err(DeError::custom)?;
 
-        let target_id = match kind != ApplicationCommandType::ChatInput {
-            true => Some(
-                map.remove("target_id")
-                    .ok_or_else(|| DeError::custom("expected resolved"))
-                    .and_then(TargetId::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
+        let guild_id = match map.remove("guild_id") {
+            Some(id) => Option::<GuildId>::deserialize(id).map_err(DeError::custom)?,
+            None => None,
         };
 
-        let target = match target_id {
-            Some(id) => {
-                if kind == ApplicationCommandType::Message {
-                    let resolved = resolved
-                        .messages
-                        .get(&id.to_message_id())
-                        .expect("expected message object")
-                        .to_owned();
-
-                    Some(ResolvedTarget::Message(resolved))
-                } else {
-                    let user_id = id.to_user_id();
-
-                    let user = resolved.users.get(&user_id).expect("expected user").to_owned();
-                    let member = resolved.members.get(&user_id).map(|m| m.to_owned());
-
-                    Some(ResolvedTarget::User(user, member))
-                }
-            },
+        let target_id = match map.remove("target_id") {
+            Some(id) => Option::<TargetId>::deserialize(id).map_err(DeError::custom)?,
             None => None,
         };
 
         Ok(Self {
-            name,
             id,
+            name,
             kind,
             options,
             resolved,
+            guild_id,
             target_id,
-            target,
         })
     }
 }
@@ -547,84 +571,33 @@ impl<'de> Deserialize<'de> for ApplicationCommandInteractionData {
 #[non_exhaustive]
 #[repr(u8)]
 pub enum ResolvedTarget {
-    User(User, Option<PartialMember>),
-    Message(Message),
+    User(User, Option<Box<PartialMember>>),
+    Message(Box<Message>),
 }
 
 /// The resolved data of a command data interaction payload.
 /// It contains the objects of [`ApplicationCommandInteractionDataOption`]s.
-#[derive(Clone, Debug, Serialize, Default)]
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
 #[non_exhaustive]
 pub struct ApplicationCommandInteractionDataResolved {
     /// The resolved users.
+    #[serde(default, skip_serializing_if = "HashMap::is_empty")]
     pub users: HashMap<UserId, User>,
     /// The resolved partial members.
+    #[serde(default, skip_serializing_if = "HashMap::is_empty")]
     pub members: HashMap<UserId, PartialMember>,
     /// The resolved roles.
+    #[serde(default, skip_serializing_if = "HashMap::is_empty")]
     pub roles: HashMap<RoleId, Role>,
     /// The resolved partial channels.
+    #[serde(default, skip_serializing_if = "HashMap::is_empty")]
     pub channels: HashMap<ChannelId, PartialChannel>,
     /// The resolved messages.
+    #[serde(default, skip_serializing_if = "HashMap::is_empty")]
     pub messages: HashMap<MessageId, Message>,
-}
-
-impl<'de> Deserialize<'de> for ApplicationCommandInteractionDataResolved {
-    fn deserialize<D: Deserializer<'de>>(deserializer: D) -> StdResult<Self, D::Error> {
-        let mut map = JsonMap::deserialize(deserializer)?;
-
-        let members = match map.contains_key("members") {
-            true => map
-                .remove("members")
-                .ok_or_else(|| DeError::custom("expected members"))
-                .and_then(deserialize_partial_members_map)
-                .map_err(DeError::custom)?,
-            false => HashMap::new(),
-        };
-
-        let users = match map.contains_key("users") {
-            true => map
-                .remove("users")
-                .ok_or_else(|| DeError::custom("expected users"))
-                .and_then(deserialize_users)
-                .map_err(DeError::custom)?,
-            false => HashMap::new(),
-        };
-
-        let roles = match map.contains_key("roles") {
-            true => map
-                .remove("roles")
-                .ok_or_else(|| DeError::custom("expected roles"))
-                .and_then(deserialize_roles_map)
-                .map_err(DeError::custom)?,
-            false => HashMap::new(),
-        };
-
-        let channels = match map.contains_key("channels") {
-            true => map
-                .remove("channels")
-                .ok_or_else(|| DeError::custom("expected channels"))
-                .and_then(deserialize_channels_map)
-                .map_err(DeError::custom)?,
-            false => HashMap::new(),
-        };
-
-        let messages = match map.contains_key("messages") {
-            true => map
-                .remove("messages")
-                .ok_or_else(|| DeError::custom("expected messages"))
-                .and_then(deserialize_messages_map)
-                .map_err(DeError::custom)?,
-            false => HashMap::new(),
-        };
-
-        Ok(Self {
-            users,
-            members,
-            roles,
-            channels,
-            messages,
-        })
-    }
+    /// The resolved attachments.
+    #[serde(default, skip_serializing_if = "HashMap::is_empty")]
+    pub attachments: HashMap<AttachmentId, Attachment>,
 }
 
 /// A set of a parameter and a value from the user.
@@ -668,15 +641,7 @@ impl<'de> Deserialize<'de> for ApplicationCommandInteractionDataOption {
             .and_then(String::deserialize)
             .map_err(DeError::custom)?;
 
-        let value = match map.contains_key("value") {
-            true => Some(
-                map.remove("value")
-                    .ok_or_else(|| DeError::custom("expected value"))
-                    .and_then(Value::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
-        };
+        let value = map.remove("value");
 
         let kind = map
             .remove("type")
@@ -684,14 +649,12 @@ impl<'de> Deserialize<'de> for ApplicationCommandInteractionDataOption {
             .and_then(ApplicationCommandOptionType::deserialize)
             .map_err(DeError::custom)?;
 
-        let options = match map.contains_key("options") {
-            true => map
-                .remove("options")
-                .ok_or_else(|| DeError::custom("expected type"))
-                .and_then(deserialize_options)
-                .map_err(DeError::custom)?,
-            false => vec![],
-        };
+        let options = map
+            .remove("options")
+            .map(Vec::deserialize)
+            .transpose()
+            .map_err(DeError::custom)?
+            .unwrap_or_default();
 
         let focused = match map.get("focused") {
             Some(value) => value.as_bool().ok_or_else(|| DeError::custom("expected bool"))?,
@@ -721,6 +684,7 @@ pub enum ApplicationCommandInteractionDataOptionValue {
     Channel(PartialChannel),
     Role(Role),
     Number(f64),
+    Attachment(Attachment),
 }
 
 fn default_permission_value() -> bool {
@@ -749,14 +713,22 @@ pub struct ApplicationCommand {
     /// The parameters for the command.
     #[serde(default)]
     pub options: Vec<ApplicationCommandOption>,
+    /// The default permissions required to execute the command.
+    pub default_member_permissions: Option<Permissions>,
+    /// Indicates whether the command is available in DMs with the app, only for globally-scoped commands.
+    /// By default, commands are visible.
+    #[serde(default)]
+    pub dm_permission: Option<bool>,
     /// Whether the command is enabled by default when
     /// the application is added to a guild.
     #[serde(default = "self::default_permission_value")]
+    #[deprecated(note = "replaced by `default_member_permissions`")]
     pub default_permission: bool,
     /// An autoincremented version identifier updated during substantial record changes.
     pub version: CommandVersionId,
 }
 
+#[cfg(feature = "http")]
 impl ApplicationCommand {
     /// Creates a global [`ApplicationCommand`],
     /// overriding an existing one with the same name if it exists.
@@ -778,11 +750,9 @@ impl ApplicationCommand {
     /// # use std::sync::Arc;
     /// #
     /// # async fn run() {
-    /// # let http = Arc::new(Http::default());
-    /// use serenity::model::{
-    ///     id::ApplicationId,
-    ///     interactions::application_command::ApplicationCommand,
-    /// };
+    /// # let http = Arc::new(Http::new("token"));
+    /// use serenity::model::id::ApplicationId;
+    /// use serenity::model::interactions::application_command::ApplicationCommand;
     ///
     /// let _ = ApplicationCommand::create_global_application_command(&http, |command| {
     ///     command.name("ping").description("A simple ping command")
@@ -798,10 +768,11 @@ impl ApplicationCommand {
     /// # use std::sync::Arc;
     /// #
     /// # async fn run() {
-    /// # let http = Arc::new(Http::default());
-    /// use serenity::model::{
-    ///     id::ApplicationId,
-    ///     interactions::application_command::{ApplicationCommand, ApplicationCommandOptionType},
+    /// # let http = Arc::new(Http::new("token"));
+    /// use serenity::model::id::ApplicationId;
+    /// use serenity::model::interactions::application_command::{
+    ///     ApplicationCommand,
+    ///     ApplicationCommandOptionType,
     /// };
     ///
     /// let _ = ApplicationCommand::create_global_application_command(&http, |command| {
@@ -839,7 +810,7 @@ impl ApplicationCommand {
         F: FnOnce(&mut CreateApplicationCommand) -> &mut CreateApplicationCommand,
     {
         let map = ApplicationCommand::build_application_command(f);
-        http.as_ref().create_global_application_command(&Value::Object(map)).await
+        http.as_ref().create_global_application_command(&Value::from(map)).await
     }
 
     /// Overrides all global application commands.
@@ -863,7 +834,7 @@ impl ApplicationCommand {
 
         f(&mut array);
 
-        http.as_ref().create_global_application_commands(&Value::Array(array.0)).await
+        http.as_ref().create_global_application_commands(&Value::from(array.0)).await
     }
 
     /// Edits a global command by its Id.
@@ -883,7 +854,7 @@ impl ApplicationCommand {
         F: FnOnce(&mut CreateApplicationCommand) -> &mut CreateApplicationCommand,
     {
         let map = ApplicationCommand::build_application_command(f);
-        http.as_ref().edit_global_application_command(command_id.into(), &Value::Object(map)).await
+        http.as_ref().edit_global_application_command(command_id.into(), &Value::from(map)).await
     }
 
     /// Gets all global commands.
@@ -929,15 +900,18 @@ impl ApplicationCommand {
     ) -> Result<()> {
         http.as_ref().delete_global_application_command(command_id.into()).await
     }
+}
 
+#[cfg(feature = "http")]
+impl ApplicationCommand {
     #[inline]
-    pub(crate) fn build_application_command<F>(f: F) -> Map<String, Value>
+    pub(crate) fn build_application_command<F>(f: F) -> JsonMap
     where
         F: FnOnce(&mut CreateApplicationCommand) -> &mut CreateApplicationCommand,
     {
         let mut create_application_command = CreateApplicationCommand::default();
         f(&mut create_application_command);
-        utils::hashmap_to_json_map(create_application_command.0)
+        json::hashmap_to_json_map(create_application_command.0)
     }
 }
 
@@ -1020,9 +994,6 @@ pub struct ApplicationCommandPermission {
 #[non_exhaustive]
 pub struct ApplicationCommandPermissionData {
     /// The [`RoleId`] or [`UserId`], depends on `kind` value.
-    ///
-    /// [`RoleId`]: crate::model::id::RoleId
-    /// [`UserId`]: crate::model::id::UserId
     pub id: CommandPermissionId,
     /// The type of data this permissions applies to.
     #[serde(rename = "type")]
@@ -1033,11 +1004,13 @@ pub struct ApplicationCommandPermissionData {
 
 impl CommandPermissionId {
     /// Converts this [`CommandPermissionId`] to [`UserId`].
+    #[must_use]
     pub fn to_user_id(self) -> UserId {
         self.0.into()
     }
 
     /// Converts this [`CommandPermissionId`] to [`RoleId`].
+    #[must_use]
     pub fn to_role_id(self) -> RoleId {
         self.0.into()
     }
@@ -1080,12 +1053,14 @@ impl From<CommandPermissionId> for UserId {
 }
 
 impl TargetId {
-    /// Converts this [`CommandPermissionId`] to [`UserId`].
+    /// Converts this [`TargetId`] to [`UserId`].
+    #[must_use]
     pub fn to_user_id(self) -> UserId {
         self.0.into()
     }
 
-    /// Converts this [`CommandPermissionId`] to [`MessageId`].
+    /// Converts this [`TargetId`] to [`MessageId`].
+    #[must_use]
     pub fn to_message_id(self) -> MessageId {
         self.0.into()
     }
@@ -1142,6 +1117,7 @@ pub enum ApplicationCommandOptionType {
     Role = 8,
     Mentionable = 9,
     Number = 10,
+    Attachment = 11,
     Unknown = !0,
 }
 
@@ -1155,7 +1131,8 @@ enum_number!(ApplicationCommandOptionType {
     Channel,
     Role,
     Mentionable,
-    Number
+    Number,
+    Attachment
 });
 
 /// The type of an [`ApplicationCommandPermissionData`].
@@ -1165,12 +1142,14 @@ enum_number!(ApplicationCommandOptionType {
 pub enum ApplicationCommandPermissionType {
     Role = 1,
     User = 2,
+    Channel = 3,
     Unknown = !0,
 }
 
 enum_number!(ApplicationCommandPermissionType {
     Role,
-    User
+    User,
+    Channel,
 });
 
 /// The only valid values a user can pick in an [`ApplicationCommandOption`].
