@@ -1,17 +1,19 @@
-use std::convert::TryFrom;
-
 use serde::de::Error as DeError;
 use serde::{Serialize, Serializer};
 
 use super::prelude::*;
+#[cfg(feature = "http")]
 use crate::builder::{
     CreateInteractionResponse,
     CreateInteractionResponseFollowup,
     EditInteractionResponse,
 };
+#[cfg(feature = "http")]
 use crate::http::Http;
+#[cfg(feature = "http")]
+use crate::json;
+use crate::json::prelude::*;
 use crate::model::interactions::InteractionType;
-use crate::utils;
 
 /// An interaction triggered by a message component.
 #[derive(Clone, Debug, Serialize)]
@@ -51,6 +53,7 @@ pub struct MessageComponentInteraction {
     pub locale: String,
 }
 
+#[cfg(feature = "http")]
 impl MessageComponentInteraction {
     /// Gets the interaction response.
     ///
@@ -74,24 +77,40 @@ impl MessageComponentInteraction {
     /// or an [`Error::Json`] if there is an error in deserializing the
     /// API response.
     ///
-    /// # Errors
-    ///
     /// [`Error::Model`]: crate::error::Error::Model
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
-    pub async fn create_interaction_response<F>(&self, http: impl AsRef<Http>, f: F) -> Result<()>
+    pub async fn create_interaction_response<'a, F>(
+        &self,
+        http: impl AsRef<Http>,
+        f: F,
+    ) -> Result<()>
     where
-        F: FnOnce(&mut CreateInteractionResponse) -> &mut CreateInteractionResponse,
+        for<'b> F:
+            FnOnce(&'b mut CreateInteractionResponse<'a>) -> &'b mut CreateInteractionResponse<'a>,
     {
         let mut interaction_response = CreateInteractionResponse::default();
         f(&mut interaction_response);
 
-        let map = utils::hashmap_to_json_map(interaction_response.0);
+        let map = json::hashmap_to_json_map(interaction_response.0);
 
         Message::check_content_length(&map)?;
         Message::check_embed_length(&map)?;
 
-        http.as_ref().create_interaction_response(self.id.0, &self.token, &Value::Object(map)).await
+        if interaction_response.1.is_empty() {
+            http.as_ref()
+                .create_interaction_response(self.id.0, &self.token, &Value::from(map))
+                .await
+        } else {
+            http.as_ref()
+                .create_interaction_response_with_files(
+                    self.id.0,
+                    &self.token,
+                    &Value::from(map),
+                    interaction_response.1,
+                )
+                .await
+        }
     }
 
     /// Edits the initial interaction response.
@@ -124,12 +143,12 @@ impl MessageComponentInteraction {
         let mut interaction_response = EditInteractionResponse::default();
         f(&mut interaction_response);
 
-        let map = utils::hashmap_to_json_map(interaction_response.0);
+        let map = json::hashmap_to_json_map(interaction_response.0);
 
         Message::check_content_length(&map)?;
         Message::check_embed_length(&map)?;
 
-        http.as_ref().edit_original_interaction_response(&self.token, &Value::Object(map)).await
+        http.as_ref().edit_original_interaction_response(&self.token, &Value::from(map)).await
     }
 
     /// Deletes the initial interaction response.
@@ -168,12 +187,12 @@ impl MessageComponentInteraction {
         let mut interaction_response = CreateInteractionResponseFollowup::default();
         f(&mut interaction_response);
 
-        let map = utils::hashmap_to_json_map(interaction_response.0);
+        let map = json::hashmap_to_json_map(interaction_response.0);
 
         Message::check_content_length(&map)?;
         Message::check_embed_length(&map)?;
 
-        http.as_ref().create_followup_message(&self.token, &Value::Object(map)).await
+        http.as_ref().create_followup_message(&self.token, &Value::from(map)).await
     }
 
     /// Edits a followup response to the response sent.
@@ -203,13 +222,13 @@ impl MessageComponentInteraction {
         let mut interaction_response = CreateInteractionResponseFollowup::default();
         f(&mut interaction_response);
 
-        let map = utils::hashmap_to_json_map(interaction_response.0);
+        let map = json::hashmap_to_json_map(interaction_response.0);
 
         Message::check_content_length(&map)?;
         Message::check_embed_length(&map)?;
 
         http.as_ref()
-            .edit_followup_message(&self.token, message_id.into().into(), &Value::Object(map))
+            .edit_followup_message(&self.token, message_id.into().into(), &Value::from(map))
             .await
     }
 
@@ -265,11 +284,11 @@ impl<'de> Deserialize<'de> for MessageComponentInteraction {
     fn deserialize<D: Deserializer<'de>>(deserializer: D) -> StdResult<Self, D::Error> {
         let mut map = JsonMap::deserialize(deserializer)?;
 
-        let id = map.get("guild_id").and_then(|x| x.as_str()).and_then(|x| x.parse::<u64>().ok());
+        let id = map.get("guild_id").and_then(Value::as_str).and_then(|x| x.parse::<u64>().ok());
 
         if let Some(guild_id) = id {
-            if let Some(member) = map.get_mut("member").and_then(|x| x.as_object_mut()) {
-                member.insert("guild_id".to_string(), Value::Number(Number::from(guild_id)));
+            if let Some(member) = map.get_mut("member").and_then(Value::as_object_mut) {
+                member.insert("guild_id".to_string(), from_number(guild_id));
             }
 
             if let Some(data) = map.get_mut("data") {
@@ -282,7 +301,7 @@ impl<'de> Deserialize<'de> for MessageComponentInteraction {
                                     .expect("couldn't deserialize message component")
                                     .insert(
                                         "guild_id".to_string(),
-                                        Value::String(guild_id.to_string()),
+                                        Value::from(guild_id.to_string()),
                                     );
                             }
                         }
@@ -298,7 +317,7 @@ impl<'de> Deserialize<'de> for MessageComponentInteraction {
                                     )
                                     .insert(
                                         "guild_id".to_string(),
-                                        Value::String(guild_id.to_string()),
+                                        Value::from(guild_id.to_string()),
                                     );
                             }
                         }
@@ -331,15 +350,11 @@ impl<'de> Deserialize<'de> for MessageComponentInteraction {
             .and_then(MessageComponentInteractionData::deserialize)
             .map_err(DeError::custom)?;
 
-        let guild_id = match map.contains_key("guild_id") {
-            true => Some(
-                map.remove("guild_id")
-                    .ok_or_else(|| DeError::custom("expected guild_id"))
-                    .and_then(GuildId::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
-        };
+        let guild_id = map
+            .remove("guild_id")
+            .map(GuildId::deserialize)
+            .transpose()
+            .map_err(DeError::custom)?;
 
         let channel_id = map
             .remove("channel_id")
@@ -347,24 +362,15 @@ impl<'de> Deserialize<'de> for MessageComponentInteraction {
             .and_then(ChannelId::deserialize)
             .map_err(DeError::custom)?;
 
-        let member = match map.contains_key("member") {
-            true => Some(
-                map.remove("member")
-                    .ok_or_else(|| DeError::custom("expected member"))
-                    .and_then(Member::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
-        };
+        let member =
+            map.remove("member").map(Member::deserialize).transpose().map_err(DeError::custom)?;
 
-        let user = match map.contains_key("user") {
-            true => map
-                .remove("user")
-                .ok_or_else(|| DeError::custom("expected user"))
-                .and_then(User::deserialize)
-                .map_err(DeError::custom)?,
-            false => member.as_ref().expect("expected user or member").user.clone(),
-        };
+        let user =
+            map.remove("user").map(User::deserialize).transpose().map_err(DeError::custom)?;
+
+        let user = user
+            .or_else(|| member.as_ref().map(|m| m.user.clone()))
+            .ok_or_else(|| DeError::custom("expected user or member"))?;
 
         let message = map
             .remove("message")
@@ -384,15 +390,11 @@ impl<'de> Deserialize<'de> for MessageComponentInteraction {
             .and_then(u8::deserialize)
             .map_err(DeError::custom)?;
 
-        let guild_locale = match map.contains_key("guild_locale") {
-            true => Some(
-                map.remove("guild_locale")
-                    .ok_or_else(|| DeError::custom("expected guild_locale"))
-                    .and_then(String::deserialize)
-                    .map_err(DeError::custom)?,
-            ),
-            false => None,
-        };
+        let guild_locale = map
+            .remove("guild_locale")
+            .map(String::deserialize)
+            .transpose()
+            .map_err(DeError::custom)?;
 
         let locale = map
             .remove("locale")
@@ -431,71 +433,6 @@ pub struct MessageComponentInteractionData {
     pub values: Vec<String>,
 }
 
-// A component.
-#[derive(Clone, Debug)]
-#[non_exhaustive]
-pub enum Component {
-    ActionRow(ActionRow),
-    Button(Button),
-    SelectMenu(SelectMenu),
-}
-
-impl<'de> Deserialize<'de> for Component {
-    fn deserialize<D: Deserializer<'de>>(deserializer: D) -> StdResult<Self, D::Error> {
-        let map = JsonMap::deserialize(deserializer)?;
-
-        let kind = map
-            .get("type")
-            .ok_or_else(|| DeError::custom("expected type"))
-            .and_then(ComponentType::deserialize)
-            .map_err(DeError::custom)?;
-
-        match kind {
-            ComponentType::ActionRow => serde_json::from_value::<ActionRow>(Value::Object(map))
-                .map(Component::ActionRow)
-                .map_err(DeError::custom),
-            ComponentType::Button => serde_json::from_value::<Button>(Value::Object(map))
-                .map(Component::Button)
-                .map_err(DeError::custom),
-            ComponentType::SelectMenu => serde_json::from_value::<SelectMenu>(Value::Object(map))
-                .map(Component::SelectMenu)
-                .map_err(DeError::custom),
-            ComponentType::Unknown => Err(DeError::custom("Unknown component type")),
-        }
-    }
-}
-
-impl Serialize for Component {
-    fn serialize<S>(&self, serializer: S) -> StdResult<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        match self {
-            Component::ActionRow(c) => ActionRow::serialize(c, serializer),
-            Component::Button(c) => Button::serialize(c, serializer),
-            Component::SelectMenu(c) => SelectMenu::serialize(c, serializer),
-        }
-    }
-}
-
-impl From<ActionRow> for Component {
-    fn from(component: ActionRow) -> Self {
-        Component::ActionRow(component)
-    }
-}
-
-impl From<Button> for Component {
-    fn from(component: Button) -> Self {
-        Component::Button(component)
-    }
-}
-
-impl From<SelectMenu> for Component {
-    fn from(component: SelectMenu) -> Self {
-        Component::SelectMenu(component)
-    }
-}
-
 /// The type of a component
 #[derive(Copy, Clone, Debug, Hash, Eq, PartialEq, PartialOrd, Ord)]
 #[non_exhaustive]
@@ -504,13 +441,15 @@ pub enum ComponentType {
     ActionRow = 1,
     Button = 2,
     SelectMenu = 3,
+    InputText = 4,
     Unknown = !0,
 }
 
 enum_number!(ComponentType {
     ActionRow,
     Button,
-    SelectMenu
+    SelectMenu,
+    InputText
 });
 
 /// An action row.
@@ -530,6 +469,7 @@ pub struct ActionRow {
 pub enum ActionRowComponent {
     Button(Button),
     SelectMenu(SelectMenu),
+    InputText(InputText),
 }
 
 impl<'de> Deserialize<'de> for ActionRowComponent {
@@ -543,12 +483,15 @@ impl<'de> Deserialize<'de> for ActionRowComponent {
             .map_err(DeError::custom)?;
 
         match kind {
-            ComponentType::Button => serde_json::from_value::<Button>(Value::Object(map))
+            ComponentType::Button => from_value::<Button>(Value::from(map))
                 .map(ActionRowComponent::Button)
                 .map_err(DeError::custom),
-            ComponentType::SelectMenu => serde_json::from_value::<SelectMenu>(Value::Object(map))
+            ComponentType::SelectMenu => from_value::<SelectMenu>(Value::from(map))
                 .map(ActionRowComponent::SelectMenu)
                 .map_err(DeError::custom),
+            ComponentType::InputText => from_value::<InputText>(Value::from(map))
+                .map(ActionRowComponent::InputText)
+                .map_err(DeError::custom),
             _ => Err(DeError::custom("Unknown component type")),
         }
     }
@@ -562,27 +505,7 @@ impl Serialize for ActionRowComponent {
         match self {
             ActionRowComponent::Button(c) => Button::serialize(c, serializer),
             ActionRowComponent::SelectMenu(c) => SelectMenu::serialize(c, serializer),
-        }
-    }
-}
-
-impl From<ActionRowComponent> for Component {
-    fn from(component: ActionRowComponent) -> Self {
-        match component {
-            ActionRowComponent::Button(b) => Component::Button(b),
-            ActionRowComponent::SelectMenu(s) => Component::SelectMenu(s),
-        }
-    }
-}
-
-impl TryFrom<Component> for ActionRowComponent {
-    type Error = Error;
-
-    fn try_from(value: Component) -> Result<Self> {
-        match value {
-            Component::ActionRow(_) => Err(Error::Model(ModelError::InvalidComponentType)),
-            Component::Button(b) => Ok(ActionRowComponent::Button(b)),
-            Component::SelectMenu(s) => Ok(ActionRowComponent::SelectMenu(s)),
+            ActionRowComponent::InputText(c) => InputText::serialize(c, serializer),
         }
     }
 }
@@ -675,3 +598,31 @@ pub struct SelectMenuOption {
     #[serde(default)]
     pub default: bool,
 }
+
+/// An input text component for modal interactions
+#[derive(Clone, Debug, Deserialize, Serialize)]
+pub struct InputText {
+    /// The component type, it will always be [`ComponentType::InputText`].
+    #[serde(rename = "type")]
+    pub kind: ComponentType,
+    /// An identifier defined by the developer for the select menu.
+    pub custom_id: String,
+    /// The input from the user
+    pub value: String,
+}
+
+/// The style of the input text
+#[derive(Copy, Clone, Debug, Hash, Eq, PartialEq, PartialOrd, Ord)]
+#[non_exhaustive]
+#[repr(u8)]
+pub enum InputTextStyle {
+    Short = 1,
+    Paragraph = 2,
+    Unknown = !0,
+}
+
+enum_number!(InputTextStyle {
+    Short,
+    Paragraph,
+    Unknown
+});
