@@ -1,19 +1,20 @@
 pub mod application_command;
 pub mod autocomplete;
 pub mod message_component;
+pub mod modal;
 pub mod ping;
 
 use application_command::ApplicationCommandInteraction;
 use autocomplete::AutocompleteInteraction;
-use bitflags::__impl_bitflags;
 use message_component::MessageComponentInteraction;
+use modal::ModalSubmitInteraction;
 use ping::PingInteraction;
 use serde::de::{Deserialize, Deserializer, Error as DeError};
 use serde::ser::{Serialize, Serializer};
-use serde_json::Value;
 
 use super::prelude::*;
 use crate::internal::prelude::*;
+use crate::json::{from_value, Value};
 
 #[derive(Clone, Debug)]
 pub enum Interaction {
@@ -21,60 +22,72 @@ pub enum Interaction {
     ApplicationCommand(ApplicationCommandInteraction),
     MessageComponent(MessageComponentInteraction),
     Autocomplete(AutocompleteInteraction),
+    ModalSubmit(ModalSubmitInteraction),
 }
 
 impl Interaction {
     /// Gets the interaction Id.
+    #[must_use]
     pub fn id(&self) -> InteractionId {
         match self {
             Interaction::Ping(i) => i.id,
             Interaction::ApplicationCommand(i) => i.id,
             Interaction::MessageComponent(i) => i.id,
             Interaction::Autocomplete(i) => i.id,
+            Interaction::ModalSubmit(i) => i.id,
         }
     }
 
     /// Gets the interaction type
+    #[must_use]
     pub fn kind(&self) -> InteractionType {
         match self {
             Interaction::Ping(_) => InteractionType::Ping,
             Interaction::ApplicationCommand(_) => InteractionType::ApplicationCommand,
             Interaction::MessageComponent(_) => InteractionType::MessageComponent,
             Interaction::Autocomplete(_) => InteractionType::Autocomplete,
+            Interaction::ModalSubmit(_) => InteractionType::ModalSubmit,
         }
     }
 
     /// Gets the interaction application Id
+    #[must_use]
     pub fn application_id(&self) -> ApplicationId {
         match self {
             Interaction::Ping(i) => i.application_id,
             Interaction::ApplicationCommand(i) => i.application_id,
             Interaction::MessageComponent(i) => i.application_id,
             Interaction::Autocomplete(i) => i.application_id,
+            Interaction::ModalSubmit(i) => i.application_id,
         }
     }
 
     /// Gets the interaction token.
+    #[must_use]
     pub fn token(&self) -> &str {
         match self {
             Interaction::Ping(ref i) => i.token.as_str(),
             Interaction::ApplicationCommand(i) => i.token.as_str(),
             Interaction::MessageComponent(i) => i.token.as_str(),
             Interaction::Autocomplete(i) => i.token.as_str(),
+            Interaction::ModalSubmit(i) => i.token.as_str(),
         }
     }
 
     /// Gets the invoked guild locale.
+    #[must_use]
     pub fn guild_locale(&self) -> Option<&str> {
         match self {
-            Interaction::Ping(i) => i.guild_locale.as_ref().map(String::as_str),
-            Interaction::ApplicationCommand(i) => i.guild_locale.as_ref().map(String::as_str),
-            Interaction::MessageComponent(i) => i.guild_locale.as_ref().map(String::as_str),
-            Interaction::Autocomplete(i) => i.guild_locale.as_ref().map(String::as_str),
+            Interaction::Ping(i) => i.guild_locale.as_deref(),
+            Interaction::ApplicationCommand(i) => i.guild_locale.as_deref(),
+            Interaction::MessageComponent(i) => i.guild_locale.as_deref(),
+            Interaction::Autocomplete(i) => i.guild_locale.as_deref(),
+            Interaction::ModalSubmit(i) => i.guild_locale.as_deref(),
         }
     }
 
     /// Converts this to a [`PingInteraction`]
+    #[must_use]
     pub fn ping(self) -> Option<PingInteraction> {
         match self {
             Interaction::Ping(i) => Some(i),
@@ -83,6 +96,7 @@ impl Interaction {
     }
 
     /// Converts this to an [`ApplicationCommandInteraction`]
+    #[must_use]
     pub fn application_command(self) -> Option<ApplicationCommandInteraction> {
         match self {
             Interaction::ApplicationCommand(i) => Some(i),
@@ -91,6 +105,7 @@ impl Interaction {
     }
 
     /// Converts this to a [`MessageComponentInteraction`]
+    #[must_use]
     pub fn message_component(self) -> Option<MessageComponentInteraction> {
         match self {
             Interaction::MessageComponent(i) => Some(i),
@@ -99,12 +114,22 @@ impl Interaction {
     }
 
     /// Converts this to a [`AutocompleteInteraction`]
+    #[must_use]
     pub fn autocomplete(self) -> Option<AutocompleteInteraction> {
         match self {
             Interaction::Autocomplete(i) => Some(i),
             _ => None,
         }
     }
+
+    /// Converts this to a [`ModalSubmitInteraction`]
+    #[must_use]
+    pub fn modal_submit(self) -> Option<ModalSubmitInteraction> {
+        match self {
+            Interaction::ModalSubmit(i) => Some(i),
+            _ => None,
+        }
+    }
 }
 
 impl<'de> Deserialize<'de> for Interaction {
@@ -118,24 +143,27 @@ impl<'de> Deserialize<'de> for Interaction {
             .map_err(DeError::custom)?;
 
         match kind {
-            InteractionType::Ping => serde_json::from_value::<PingInteraction>(Value::Object(map))
+            InteractionType::Ping => from_value::<PingInteraction>(Value::from(map))
                 .map(Interaction::Ping)
                 .map_err(DeError::custom),
             InteractionType::ApplicationCommand => {
-                serde_json::from_value::<ApplicationCommandInteraction>(Value::Object(map))
+                from_value::<ApplicationCommandInteraction>(Value::from(map))
                     .map(Interaction::ApplicationCommand)
                     .map_err(DeError::custom)
             },
             InteractionType::MessageComponent => {
-                serde_json::from_value::<MessageComponentInteraction>(Value::Object(map))
+                from_value::<MessageComponentInteraction>(Value::from(map))
                     .map(Interaction::MessageComponent)
                     .map_err(DeError::custom)
             },
             InteractionType::Autocomplete => {
-                serde_json::from_value::<AutocompleteInteraction>(Value::Object(map))
+                from_value::<AutocompleteInteraction>(Value::from(map))
                     .map(Interaction::Autocomplete)
                     .map_err(DeError::custom)
             },
+            InteractionType::ModalSubmit => from_value::<ModalSubmitInteraction>(Value::from(map))
+                .map(Interaction::ModalSubmit)
+                .map_err(DeError::custom),
             InteractionType::Unknown => Err(DeError::custom("Unknown interaction type")),
         }
     }
@@ -155,6 +183,7 @@ impl Serialize for Interaction {
                 MessageComponentInteraction::serialize(i, serializer)
             },
             Interaction::Autocomplete(i) => AutocompleteInteraction::serialize(i, serializer),
+            Interaction::ModalSubmit(i) => ModalSubmitInteraction::serialize(i, serializer),
         }
     }
 }
@@ -168,6 +197,7 @@ pub enum InteractionType {
     ApplicationCommand = 2,
     MessageComponent = 3,
     Autocomplete = 4,
+    ModalSubmit = 5,
     Unknown = !0,
 }
 
@@ -175,33 +205,19 @@ enum_number!(InteractionType {
     Ping,
     MessageComponent,
     ApplicationCommand,
-    Autocomplete
+    Autocomplete,
+    ModalSubmit
 });
 
-/// The flags for an interaction response.
-#[derive(Clone)]
-#[non_exhaustive]
-pub struct InteractionApplicationCommandCallbackDataFlags {
-    bits: u64,
-}
-
-__impl_bitflags! {
-    InteractionApplicationCommandCallbackDataFlags: u64 {
+bitflags! {
+    /// The flags for an interaction response.
+    #[derive(Default)]
+    pub struct InteractionApplicationCommandCallbackDataFlags: u64 {
+        /// Do not include any embeds when serializing this message.
+        const SUPPRESS_EMBEDS = 1 << 2;
         /// Interaction message will only be visible to sender and will
         /// be quickly deleted.
-        EPHEMERAL = 1 << 6;
-    }
-}
-
-impl<'de> Deserialize<'de> for InteractionApplicationCommandCallbackDataFlags {
-    fn deserialize<D: Deserializer<'de>>(deserializer: D) -> StdResult<Self, D::Error> {
-        Ok(Self::from_bits_truncate(u64::deserialize(deserializer)?))
-    }
-}
-
-impl Serialize for InteractionApplicationCommandCallbackDataFlags {
-    fn serialize<S: Serializer>(&self, serializer: S) -> StdResult<S::Ok, S::Error> {
-        serializer.serialize_u64(self.bits())
+        const EPHEMERAL = 1 << 6;
     }
 }
 
@@ -234,4 +250,5 @@ pub enum InteractionResponseType {
     DeferredUpdateMessage = 6,
     UpdateMessage = 7,
     Autocomplete = 8,
+    Modal = 9,
 }
