@@ -30,31 +30,30 @@ pub mod event;
 pub mod gateway;
 pub mod guild;
 pub mod id;
-#[cfg(feature = "unstable_discord_api")]
 pub mod interactions;
 pub mod invite;
+pub mod mention;
 pub mod misc;
 pub mod oauth2;
 pub mod permissions;
 pub mod prelude;
+pub mod sticker;
+pub mod timestamp;
 pub mod user;
 pub mod voice;
 pub mod webhook;
 
-use std::{
-    collections::HashMap,
-    fmt::{Display, Formatter, Result as FmtResult},
-    result::Result as StdResult,
-};
+use std::collections::HashMap;
+use std::result::Result as StdResult;
 
 use serde::de::Visitor;
 use serde::{Deserialize, Deserializer};
 #[cfg(feature = "voice-model")]
 pub use serenity_voice_model as voice_gateway;
+pub use timestamp::Timestamp;
 
 pub use self::error::Error as ModelError;
 pub use self::permissions::Permissions;
-use self::utils::*;
 use crate::internal::prelude::*;
 #[cfg(feature = "utils")]
 use crate::utils::Colour;
