@@ -41,11 +41,9 @@
 //! [Manage Roles]: Permissions::MANAGE_ROLES
 //! [Manage Webhooks]: Permissions::MANAGE_WEBHOOKS
 
-#[cfg(feature = "model")]
-use std::fmt::{Display, Formatter, Result as FmtResult};
+use std::fmt;
 
-use bitflags::__impl_bitflags;
-use serde::de::{Deserialize, Deserializer, Error as DeError};
+use serde::de::{Deserialize, Deserializer};
 use serde::ser::{Serialize, Serializer};
 
 /// This macro generates the [`Permissions::get_permission_names`] method.
@@ -87,6 +85,7 @@ macro_rules! generate_get_permission_names {
     {$ ($perm:ident: $name:expr),*} => {
         impl Permissions {
             /// Returns a list of names of all contained permissions.
+            #[must_use]
             pub fn get_permission_names(self) -> Vec<&'static str> {
                 let mut names = Vec::new();
 
@@ -111,11 +110,11 @@ macro_rules! generate_get_permission_names {
 /// - [Attach Files]
 /// - [Change Nickname]
 /// - [Connect]
-/// - [Create Invite]
+/// - [Create Instant Invite]
 /// - [Embed Links]
 /// - [Mention Everyone]
 /// - [Read Message History]
-/// - [Read Messages]
+/// - [View Channel]
 /// - [Send Messages]
 /// - [Send TTS Messages]
 /// - [Speak]
@@ -135,11 +134,11 @@ macro_rules! generate_get_permission_names {
 /// [Attach Files]: Permissions::ATTACH_FILES
 /// [Change Nickname]: Permissions::CHANGE_NICKNAME
 /// [Connect]: Permissions::CONNECT
-/// [Create Invite]: Permissions::CREATE_INVITE
+/// [Create Instant Invite]: Permissions::CREATE_INSTANT_INVITE
 /// [Embed Links]: Permissions::EMBED_LINKS
 /// [Mention Everyone]: Permissions::MENTION_EVERYONE
 /// [Read Message History]: Permissions::READ_MESSAGE_HISTORY
-/// [Read Messages]: Permissions::READ_MESSAGES
+/// [View Channel]: Permissions::VIEW_CHANNEL
 /// [Send Messages]: Permissions::SEND_MESSAGES
 /// [Send TTS Messages]: Permissions::SEND_TTS_MESSAGES
 /// [Speak]: Permissions::SPEAK
@@ -150,11 +149,11 @@ pub const PRESET_GENERAL: Permissions = Permissions {
         | Permissions::ATTACH_FILES.bits
         | Permissions::CHANGE_NICKNAME.bits
         | Permissions::CONNECT.bits
-        | Permissions::CREATE_INVITE.bits
+        | Permissions::CREATE_INSTANT_INVITE.bits
         | Permissions::EMBED_LINKS.bits
         | Permissions::MENTION_EVERYONE.bits
         | Permissions::READ_MESSAGE_HISTORY.bits
-        | Permissions::READ_MESSAGES.bits
+        | Permissions::VIEW_CHANNEL.bits
         | Permissions::SEND_MESSAGES.bits
         | Permissions::SEND_TTS_MESSAGES.bits
         | Permissions::SPEAK.bits
@@ -170,11 +169,11 @@ pub const PRESET_GENERAL: Permissions = Permissions {
 /// - [Add Reactions]
 /// - [Attach Files]
 /// - [Change Nickname]
-/// - [Create Invite]
+/// - [Create Instant Invite]
 /// - [Embed Links]
 /// - [Mention Everyone]
 /// - [Read Message History]
-/// - [Read Messages]
+/// - [View Channel]
 /// - [Send Messages]
 /// - [Send TTS Messages]
 /// - [Use External Emojis]
@@ -182,11 +181,11 @@ pub const PRESET_GENERAL: Permissions = Permissions {
 /// [Add Reactions]: Permissions::ADD_REACTIONS
 /// [Attach Files]: Permissions::ATTACH_FILES
 /// [Change Nickname]: Permissions::CHANGE_NICKNAME
-/// [Create Invite]: Permissions::CREATE_INVITE
+/// [Create Instant Invite]: Permissions::CREATE_INSTANT_INVITE
 /// [Embed Links]: Permissions::EMBED_LINKS
 /// [Mention Everyone]: Permissions::MENTION_EVERYONE
 /// [Read Message History]: Permissions::READ_MESSAGE_HISTORY
-/// [Read Messages]: Permissions::READ_MESSAGES
+/// [View Channel]: Permissions::VIEW_CHANNEL
 /// [Send Messages]: Permissions::SEND_MESSAGES
 /// [Send TTS Messages]: Permissions::SEND_TTS_MESSAGES
 /// [Use External Emojis]: Permissions::USE_EXTERNAL_EMOJIS
@@ -194,11 +193,11 @@ pub const PRESET_TEXT: Permissions = Permissions {
     bits: Permissions::ADD_REACTIONS.bits
         | Permissions::ATTACH_FILES.bits
         | Permissions::CHANGE_NICKNAME.bits
-        | Permissions::CREATE_INVITE.bits
+        | Permissions::CREATE_INSTANT_INVITE.bits
         | Permissions::EMBED_LINKS.bits
         | Permissions::MENTION_EVERYONE.bits
         | Permissions::READ_MESSAGE_HISTORY.bits
-        | Permissions::READ_MESSAGES.bits
+        | Permissions::VIEW_CHANNEL.bits
         | Permissions::SEND_MESSAGES.bits
         | Permissions::SEND_TTS_MESSAGES.bits
         | Permissions::USE_EXTERNAL_EMOJIS.bits,
@@ -220,51 +219,42 @@ pub const PRESET_VOICE: Permissions = Permissions {
     bits: Permissions::CONNECT.bits | Permissions::SPEAK.bits | Permissions::USE_VAD.bits,
 };
 
-/// A set of permissions that can be assigned to [`User`]s and [`Role`]s via
-/// [`PermissionOverwrite`]s, roles globally in a [`Guild`], and to
-/// [`GuildChannel`]s.
-///
-/// [`Guild`]: super::guild::Guild
-/// [`GuildChannel`]: super::channel::GuildChannel
-/// [`PermissionOverwrite`]: super::channel::PermissionOverwrite
-/// [`Role`]: super::guild::Role
-/// [`User`]: super::user::User
-#[derive(Copy, PartialEq, Eq, Clone, PartialOrd, Ord, Hash)]
-pub struct Permissions {
-    /// The flags making up the permissions.
+bitflags::bitflags! {
+    /// A set of permissions that can be assigned to [`User`]s and [`Role`]s via
+    /// [`PermissionOverwrite`]s, roles globally in a [`Guild`], and to
+    /// [`GuildChannel`]s.
     ///
-    /// # Note
-    /// Do not modify this yourself; use the provided methods.
-    /// Do the same when creating, unless you're absolutely certain that you're giving valid permission flags.
-    pub bits: u64,
-}
-
-__impl_bitflags! {
-    Permissions: u64 {
+    /// [`Guild`]: super::guild::Guild
+    /// [`GuildChannel`]: super::channel::GuildChannel
+    /// [`PermissionOverwrite`]: super::channel::PermissionOverwrite
+    /// [`Role`]: super::guild::Role
+    /// [`User`]: super::user::User
+    #[derive(Default)]
+    pub struct Permissions: u64 {
         /// Allows for the creation of [`RichInvite`]s.
         ///
         /// [`RichInvite`]: super::invite::RichInvite
-        CREATE_INVITE = 1 << 0;
+        const CREATE_INSTANT_INVITE = 1 << 0;
         /// Allows for the kicking of guild [member]s.
         ///
         /// [member]: super::guild::Member
-        KICK_MEMBERS = 1 << 1;
+        const KICK_MEMBERS = 1 << 1;
         /// Allows the banning of guild [member]s.
         ///
         /// [member]: super::guild::Member
-        BAN_MEMBERS = 1 << 2;
+        const BAN_MEMBERS = 1 << 2;
         /// Allows all permissions, bypassing channel [permission overwrite]s.
         ///
         /// [permission overwrite]: super::channel::PermissionOverwrite
-        ADMINISTRATOR = 1 << 3;
+        const ADMINISTRATOR = 1 << 3;
         /// Allows management and editing of guild [channel]s.
         ///
         /// [channel]: super::channel::GuildChannel
-        MANAGE_CHANNELS = 1 << 4;
+        const MANAGE_CHANNELS = 1 << 4;
         /// Allows management and editing of the [guild].
         ///
         /// [guild]: super::guild::Guild
-        MANAGE_GUILD = 1 << 5;
+        const MANAGE_GUILD = 1 << 5;
         /// [`Member`]s with this permission can add new [`Reaction`]s to a
         /// [`Message`]. Members can still react using reactions already added
         /// to messages without this permission.
@@ -272,101 +262,97 @@ __impl_bitflags! {
         /// [`Member`]: super::guild::Member
         /// [`Message`]: super::channel::Message
         /// [`Reaction`]: super::channel::Reaction
-        ADD_REACTIONS = 1 << 6;
+        const ADD_REACTIONS = 1 << 6;
         /// Allows viewing a guild's audit logs.
-        VIEW_AUDIT_LOG = 1 << 7;
+        const VIEW_AUDIT_LOG = 1 << 7;
         /// Allows the use of priority speaking in voice channels.
-        PRIORITY_SPEAKER = 1 << 8;
-        // Allows the user to go live
-        STREAM = 1 << 9;
-        /// Allows reading messages in a guild channel. If a user does not have
-        /// this permission, then they will not be able to see the channel.
-        READ_MESSAGES = 1 << 10;
+        const PRIORITY_SPEAKER = 1 << 8;
+        // Allows the user to go live.
+        const STREAM = 1 << 9;
+        /// Allows guild members to view a channel, which includes reading
+        /// messages in text channels and joining voice channels.
+        const VIEW_CHANNEL = 1 << 10;
         /// Allows sending messages in a guild channel.
-        SEND_MESSAGES = 1 << 11;
+        const SEND_MESSAGES = 1 << 11;
         /// Allows the sending of text-to-speech messages in a channel.
-        SEND_TTS_MESSAGES = 1 << 12;
+        const SEND_TTS_MESSAGES = 1 << 12;
         /// Allows the deleting of other messages in a guild channel.
         ///
         /// **Note**: This does not allow the editing of other messages.
-        MANAGE_MESSAGES = 1 << 13;
+        const MANAGE_MESSAGES = 1 << 13;
         /// Allows links from this user - or users of this role - to be
         /// embedded, with potential data such as a thumbnail, description, and
         /// page name.
-        EMBED_LINKS = 1 << 14;
+        const EMBED_LINKS = 1 << 14;
         /// Allows uploading of files.
-        ATTACH_FILES = 1 << 15;
+        const ATTACH_FILES = 1 << 15;
         /// Allows the reading of a channel's message history.
-        READ_MESSAGE_HISTORY = 1 << 16;
+        const READ_MESSAGE_HISTORY = 1 << 16;
         /// Allows the usage of the `@everyone` mention, which will notify all
         /// users in a channel. The `@here` mention will also be available, and
         /// can be used to mention all non-offline users.
         ///
         /// **Note**: You probably want this to be disabled for most roles and
         /// users.
-        MENTION_EVERYONE = 1 << 17;
+        const MENTION_EVERYONE = 1 << 17;
         /// Allows the usage of custom emojis from other guilds.
         ///
         /// This does not dictate whether custom emojis in this guild can be
         /// used in other guilds.
-        USE_EXTERNAL_EMOJIS = 1 << 18;
+        const USE_EXTERNAL_EMOJIS = 1 << 18;
         /// Allows for viewing guild insights.
-        VIEW_GUILD_INSIGHTS = 1 << 19;
+        const VIEW_GUILD_INSIGHTS = 1 << 19;
         /// Allows the joining of a voice channel.
-        CONNECT = 1 << 20;
+        const CONNECT = 1 << 20;
         /// Allows the user to speak in a voice channel.
-        SPEAK = 1 << 21;
+        const SPEAK = 1 << 21;
         /// Allows the muting of members in a voice channel.
-        MUTE_MEMBERS = 1 << 22;
+        const MUTE_MEMBERS = 1 << 22;
         /// Allows the deafening of members in a voice channel.
-        DEAFEN_MEMBERS = 1 << 23;
+        const DEAFEN_MEMBERS = 1 << 23;
         /// Allows the moving of members from one voice channel to another.
-        MOVE_MEMBERS = 1 << 24;
+        const MOVE_MEMBERS = 1 << 24;
         /// Allows the usage of voice-activity-detection in a [voice] channel.
         ///
         /// If this is disabled, then [`Member`]s must use push-to-talk.
         ///
         /// [`Member`]: super::guild::Member
         /// [voice]: super::channel::ChannelType::Voice
-        USE_VAD = 1 << 25;
+        const USE_VAD = 1 << 25;
         /// Allows members to change their own nickname in the guild.
-        CHANGE_NICKNAME = 1 << 26;
+        const CHANGE_NICKNAME = 1 << 26;
         /// Allows members to change other members' nicknames.
-        MANAGE_NICKNAMES = 1 << 27;
+        const MANAGE_NICKNAMES = 1 << 27;
         /// Allows management and editing of roles below their own.
-        MANAGE_ROLES = 1 << 28;
+        const MANAGE_ROLES = 1 << 28;
         /// Allows management of webhooks.
-        MANAGE_WEBHOOKS = 1 << 29;
+        const MANAGE_WEBHOOKS = 1 << 29;
         /// Allows management of emojis and stickers created without the use of an
         /// [`Integration`].
         ///
         /// [`Integration`]: super::guild::Integration
-        MANAGE_EMOJIS = 1 << 30;
+        const MANAGE_EMOJIS_AND_STICKERS = 1 << 30;
         /// Allows using slash commands.
-        USE_SLASH_COMMANDS = 1 << 31;
+        const USE_SLASH_COMMANDS = 1 << 31;
         /// Allows for requesting to speak in stage channels.
-        REQUEST_TO_SPEAK = 1 << 32;
+        const REQUEST_TO_SPEAK = 1 << 32;
+        /// Allows for creating, editing, and deleting scheduled events
+        const MANAGE_EVENTS = 1 << 33;
         /// Allows for deleting and archiving threads, and viewing all private threads.
-        MANAGE_THREADS = 1 << 34;
+        const MANAGE_THREADS = 1 << 34;
         /// Allows for creating threads.
-        CREATE_PUBLIC_THREADS = 1 << 35;
+        const CREATE_PUBLIC_THREADS = 1 << 35;
         /// Allows for creating private threads.
-        CREATE_PRIVATE_THREADS = 1 << 36;
+        const CREATE_PRIVATE_THREADS = 1 << 36;
         /// Allows the usage of custom stickers from other servers.
-        USE_EXTERNAL_STICKERS = 1 << 37;
+        const USE_EXTERNAL_STICKERS = 1 << 37;
         /// Allows for sending messages in threads
-        SEND_MESSAGES_IN_THREADS = 1 << 38;
+        const SEND_MESSAGES_IN_THREADS = 1 << 38;
         /// Allows for launching activities in a voice channel
-        START_EMBEDDED_ACTIVITIES = 1 << 39;
+        const USE_EMBEDDED_ACTIVITIES = 1 << 39;
         /// Allows for timing out users to prevent them from sending or reacting to messages in
         /// chat and threads, and from speaking in voice and stage channels.
-        MODERATE_MEMBERS = 1 << 40;
-        /// Allows for creating and participating in public threads.
-        #[deprecated(note = "This permission no longer exists")]
-        USE_PUBLIC_THREADS = 0b0010_0000_0000_0000_0000_0000_0000_0000_0000;
-        // Allows for creating and participating in private threads.
-        #[deprecated(note = "This permission no longer exists")]
-        USE_PRIVATE_THREADS = 0b0100_0000_0000_0000_0000_0000_0000_0000_0000;
+        const MODERATE_MEMBERS = 1 << 40;
     }
 }
 
@@ -378,33 +364,41 @@ generate_get_permission_names! {
     ban_members: "Ban Members",
     change_nickname: "Change Nickname",
     connect: "Connect",
-    create_invite: "Create Invite",
+    create_instant_invite: "Create Instant Invite",
+    create_private_threads: "Create Private Threads",
+    create_public_threads: "Create Public Threads",
     deafen_members: "Deafen Members",
     embed_links: "Embed Links",
     external_emojis: "Use External Emojis",
     kick_members: "Kick Members",
     manage_channels: "Manage Channels",
-    manage_emojis: "Manage Emojis",
+    manage_emojis_and_stickers: "Manage Emojis and Stickers",
     manage_guild: "Manage Guilds",
     manage_messages: "Manage Messages",
     manage_nicknames: "Manage Nicknames",
     manage_roles: "Manage Roles",
+    manage_threads: "Manage Threads",
     manage_webhooks: "Manage Webhooks",
     mention_everyone: "Mention Everyone",
+    moderate_members: "Moderate Members",
     move_members: "Move Members",
     mute_members: "Mute Members",
     priority_speaker: "Priority Speaker",
     read_message_history: "Read Message History",
     request_to_speak: "Request To Speak",
-    read_messages: "Read Messages",
     send_messages: "Send Messages",
+    send_messages_in_threads: "Send Messages in Threads",
     send_tts_messages: "Send TTS Messages",
     speak: "Speak",
     stream: "Stream",
+    use_embedded_activities: "Use Embedded Activities",
     use_external_emojis: "Use External Emojis",
+    use_external_stickers: "Use External Stickers",
     use_slash_commands: "Use Slash Commands",
     use_vad: "Use Voice Activity",
-    view_audit_log: "View Audit Log"
+    view_audit_log: "View Audit Log",
+    view_channel: "View Channel",
+    view_guild_insights: "View Guild Insights"
 }
 
 #[cfg(feature = "model")]
@@ -413,6 +407,7 @@ impl Permissions {
     /// [Add Reactions] permission.
     ///
     /// [Add Reactions]: Self::ADD_REACTIONS
+    #[must_use]
     pub fn add_reactions(self) -> bool {
         self.contains(Self::ADD_REACTIONS)
     }
@@ -421,6 +416,7 @@ impl Permissions {
     /// [Administrator] permission.
     ///
     /// [Administrator]: Self::ADMINISTRATOR
+    #[must_use]
     pub fn administrator(self) -> bool {
         self.contains(Self::ADMINISTRATOR)
     }
@@ -429,6 +425,7 @@ impl Permissions {
     /// [Attach Files] permission.
     ///
     /// [Attach Files]: Self::ATTACH_FILES
+    #[must_use]
     pub fn attach_files(self) -> bool {
         self.contains(Self::ATTACH_FILES)
     }
@@ -437,6 +434,7 @@ impl Permissions {
     /// [Ban Members] permission.
     ///
     /// [Ban Members]: Self::BAN_MEMBERS
+    #[must_use]
     pub fn ban_members(self) -> bool {
         self.contains(Self::BAN_MEMBERS)
     }
@@ -445,6 +443,7 @@ impl Permissions {
     /// [Change Nickname] permission.
     ///
     /// [Change Nickname]: Self::CHANGE_NICKNAME
+    #[must_use]
     pub fn change_nickname(self) -> bool {
         self.contains(Self::CHANGE_NICKNAME)
     }
@@ -453,6 +452,7 @@ impl Permissions {
     /// [Connect] permission.
     ///
     /// [Connect]: Self::CONNECT
+    #[must_use]
     pub fn connect(self) -> bool {
         self.contains(Self::CONNECT)
     }
@@ -461,14 +461,34 @@ impl Permissions {
     /// [View Audit Log] permission.
     ///
     /// [View Audit Log]: Self::VIEW_AUDIT_LOG
+    #[must_use]
     pub fn view_audit_log(self) -> bool {
         self.contains(Self::VIEW_AUDIT_LOG)
     }
 
+    /// Shorthand for checking that the set of permissions contains the
+    /// [View Channel] permission.
+    ///
+    /// [View Channel]: Self::VIEW_CHANNEL
+    #[must_use]
+    pub fn view_channel(self) -> bool {
+        self.contains(Self::VIEW_CHANNEL)
+    }
+
+    /// Shorthand for checking that the set of permissions contains the
+    /// [View Guild Insights] permission.
+    ///
+    /// [View Guild Insights]: Self::VIEW_GUILD_INSIGHTS
+    #[must_use]
+    pub fn view_guild_insights(self) -> bool {
+        self.contains(Self::VIEW_GUILD_INSIGHTS)
+    }
+
     /// Shorthand for checking that the set of permission contains the
     /// [Priority Speaker] permission.
     ///
     /// [Priority Speaker]: Self::PRIORITY_SPEAKER
+    #[must_use]
     pub fn priority_speaker(self) -> bool {
         self.contains(Self::PRIORITY_SPEAKER)
     }
@@ -477,22 +497,43 @@ impl Permissions {
     /// [Stream] permission.
     ///
     /// [Stream]: Self::STREAM
+    #[must_use]
     pub fn stream(self) -> bool {
         self.contains(Self::STREAM)
     }
 
     /// Shorthand for checking that the set of permissions contains the
-    /// [Create Invite] permission.
+    /// [Create Instant Invite] permission.
     ///
-    /// [Create Invite]: Self::CREATE_INVITE
-    pub fn create_invite(self) -> bool {
-        self.contains(Self::CREATE_INVITE)
+    /// [Create Instant Invite]: Self::CREATE_INSTANT_INVITE
+    #[must_use]
+    pub fn create_instant_invite(self) -> bool {
+        self.contains(Self::CREATE_INSTANT_INVITE)
+    }
+
+    /// Shorthand for checking that the set of permissions contains the
+    /// [Create Private Threads] permission.
+    ///
+    /// [Create Private Threads]: Self::CREATE_PRIVATE_THREADS
+    #[must_use]
+    pub fn create_private_threads(self) -> bool {
+        self.contains(Self::CREATE_PRIVATE_THREADS)
+    }
+
+    /// Shorthand for checking that the set of permissions contains the
+    /// [Create Public Threads] permission.
+    ///
+    /// [Create Public Threads]: Self::CREATE_PUBLIC_THREADS
+    #[must_use]
+    pub fn create_public_threads(self) -> bool {
+        self.contains(Self::CREATE_PUBLIC_THREADS)
     }
 
     /// Shorthand for checking that the set of permissions contains the
     /// [Deafen Members] permission.
     ///
     /// [Deafen Members]: Self::DEAFEN_MEMBERS
+    #[must_use]
     pub fn deafen_members(self) -> bool {
         self.contains(Self::DEAFEN_MEMBERS)
     }
@@ -501,6 +542,7 @@ impl Permissions {
     /// [Embed Links] permission.
     ///
     /// [Embed Links]: Self::EMBED_LINKS
+    #[must_use]
     pub fn embed_links(self) -> bool {
         self.contains(Self::EMBED_LINKS)
     }
@@ -509,6 +551,7 @@ impl Permissions {
     /// [Use External Emojis] permission.
     ///
     /// [Use External Emojis]: Self::USE_EXTERNAL_EMOJIS
+    #[must_use]
     pub fn external_emojis(self) -> bool {
         self.contains(Self::USE_EXTERNAL_EMOJIS)
     }
@@ -517,6 +560,7 @@ impl Permissions {
     /// [Kick Members] permission.
     ///
     /// [Kick Members]: Self::KICK_MEMBERS
+    #[must_use]
     pub fn kick_members(self) -> bool {
         self.contains(Self::KICK_MEMBERS)
     }
@@ -525,22 +569,25 @@ impl Permissions {
     /// [Manage Channels] permission.
     ///
     /// [Manage Channels]: Self::MANAGE_CHANNELS
+    #[must_use]
     pub fn manage_channels(self) -> bool {
         self.contains(Self::MANAGE_CHANNELS)
     }
 
     /// Shorthand for checking that the set of permissions contains the
-    /// [Manage Emojis] permission.
+    /// [Manage Emojis and Stickers] permission.
     ///
-    /// [Manage Emojis]: Self::MANAGE_EMOJIS
-    pub fn manage_emojis(self) -> bool {
-        self.contains(Self::MANAGE_EMOJIS)
+    /// [Manage Emojis and Stickers]: Self::MANAGE_EMOJIS_AND_STICKERS
+    #[must_use]
+    pub fn manage_emojis_and_stickers(self) -> bool {
+        self.contains(Self::MANAGE_EMOJIS_AND_STICKERS)
     }
 
     /// Shorthand for checking that the set of permissions contains the
     /// [Manage Guild] permission.
     ///
     /// [Manage Guild]: Self::MANAGE_GUILD
+    #[must_use]
     pub fn manage_guild(self) -> bool {
         self.contains(Self::MANAGE_GUILD)
     }
@@ -549,6 +596,7 @@ impl Permissions {
     /// [Manage Messages] permission.
     ///
     /// [Manage Messages]: Self::MANAGE_MESSAGES
+    #[must_use]
     pub fn manage_messages(self) -> bool {
         self.contains(Self::MANAGE_MESSAGES)
     }
@@ -557,6 +605,7 @@ impl Permissions {
     /// [Manage Nicknames] permission.
     ///
     /// [Manage Nicknames]: Self::MANAGE_NICKNAMES
+    #[must_use]
     pub fn manage_nicknames(self) -> bool {
         self.contains(Self::MANAGE_NICKNAMES)
     }
@@ -565,14 +614,25 @@ impl Permissions {
     /// [Manage Roles] permission.
     ///
     /// [Manage Roles]: Self::MANAGE_ROLES
+    #[must_use]
     pub fn manage_roles(self) -> bool {
         self.contains(Self::MANAGE_ROLES)
     }
 
+    /// Shorthand for checking that the set of permissions contains the
+    /// [Manage Threads] permission.
+    ///
+    /// [Manage Threads]: Self::MANAGE_THREADS
+    #[must_use]
+    pub fn manage_threads(self) -> bool {
+        self.contains(Self::MANAGE_THREADS)
+    }
+
     /// Shorthand for checking that the set of permissions contains the
     /// [Manage Webhooks] permission.
     ///
     /// [Manage Webhooks]: Self::MANAGE_WEBHOOKS
+    #[must_use]
     pub fn manage_webhooks(self) -> bool {
         self.contains(Self::MANAGE_WEBHOOKS)
     }
@@ -581,14 +641,25 @@ impl Permissions {
     /// [Mention Everyone] permission.
     ///
     /// [Mention Everyone]: Self::MENTION_EVERYONE
+    #[must_use]
     pub fn mention_everyone(self) -> bool {
         self.contains(Self::MENTION_EVERYONE)
     }
 
+    /// Shorthand for checking that the set of permissions contains the
+    /// [Moderate Members] permission.
+    ///
+    /// [Moderate Members]: Self::MODERATE_MEMBERS
+    #[must_use]
+    pub fn moderate_members(self) -> bool {
+        self.contains(Self::MODERATE_MEMBERS)
+    }
+
     /// Shorthand for checking that the set of permissions contains the
     /// [Move Members] permission.
     ///
     /// [Move Members]: Self::MOVE_MEMBERS
+    #[must_use]
     pub fn move_members(self) -> bool {
         self.contains(Self::MOVE_MEMBERS)
     }
@@ -597,6 +668,7 @@ impl Permissions {
     /// [Mute Members] permission.
     ///
     /// [Mute Members]: Self::MUTE_MEMBERS
+    #[must_use]
     pub fn mute_members(self) -> bool {
         self.contains(Self::MUTE_MEMBERS)
     }
@@ -605,30 +677,34 @@ impl Permissions {
     /// [Read Message History] permission.
     ///
     /// [Read Message History]: Self::READ_MESSAGE_HISTORY
+    #[must_use]
     pub fn read_message_history(self) -> bool {
         self.contains(Self::READ_MESSAGE_HISTORY)
     }
 
-    /// Shorthand for checking that the set of permissions contains the
-    /// [Read Messages] permission.
-    ///
-    /// [Read Messages]: Self::READ_MESSAGES
-    pub fn read_messages(self) -> bool {
-        self.contains(Self::READ_MESSAGES)
-    }
-
     /// Shorthand for checking that the set of permissions contains the
     /// [Send Messages] permission.
     ///
     /// [Send Messages]: Self::SEND_MESSAGES
+    #[must_use]
     pub fn send_messages(self) -> bool {
         self.contains(Self::SEND_MESSAGES)
     }
 
+    /// Shorthand for checking that the set of permissions contains the
+    /// [Send Messages in Threads] permission.
+    ///
+    /// [Send Messages in Threads]: Self::SEND_MESSAGES_IN_THREADS
+    #[must_use]
+    pub fn send_messages_in_threads(self) -> bool {
+        self.contains(Self::SEND_MESSAGES_IN_THREADS)
+    }
+
     /// Shorthand for checking that the set of permissions contains the
     /// [Send TTS Messages] permission.
     ///
     /// [Send TTS Messages]: Self::SEND_TTS_MESSAGES
+    #[must_use]
     pub fn send_tts_messages(self) -> bool {
         self.contains(Self::SEND_TTS_MESSAGES)
     }
@@ -637,6 +713,7 @@ impl Permissions {
     /// [Speak] permission.
     ///
     /// [Speak]: Self::SPEAK
+    #[must_use]
     pub fn speak(self) -> bool {
         self.contains(Self::SPEAK)
     }
@@ -645,22 +722,43 @@ impl Permissions {
     /// [Request To Speak] permission.
     ///
     /// [Request To Speak]: Self::REQUEST_TO_SPEAK
+    #[must_use]
     pub fn request_to_speak(self) -> bool {
         self.contains(Self::REQUEST_TO_SPEAK)
     }
 
+    /// Shorthand for checking that the set of permissions contains the
+    /// [Use Embedded Activities] permission.
+    ///
+    /// [Use Embedded Activities]: Self::USE_EMBEDDED_ACTIVITIES
+    #[must_use]
+    pub fn use_embedded_activities(self) -> bool {
+        self.contains(Self::USE_EMBEDDED_ACTIVITIES)
+    }
+
     /// Shorthand for checking that the set of permissions contains the
     /// [Use External Emojis] permission.
     ///
     /// [Use External Emojis]: Self::USE_EXTERNAL_EMOJIS
+    #[must_use]
     pub fn use_external_emojis(self) -> bool {
         self.contains(Self::USE_EXTERNAL_EMOJIS)
     }
 
+    /// Shorthand for checking that the set of permissions contains the
+    /// [Use External Stickers] permission.
+    ///
+    /// [Use External Stickers]: Self::USE_EXTERNAL_STICKERS
+    #[must_use]
+    pub fn use_external_stickers(self) -> bool {
+        self.contains(Self::USE_EXTERNAL_STICKERS)
+    }
+
     /// Shorthand for checking that the set of permissions contains the
     /// [Use Slash Commands] permission.
     ///
     /// [Use Slash Commands]: Self::USE_SLASH_COMMANDS
+    #[must_use]
     pub fn use_slash_commands(self) -> bool {
         self.contains(Self::USE_SLASH_COMMANDS)
     }
@@ -669,21 +767,32 @@ impl Permissions {
     /// [Use VAD] permission.
     ///
     /// [Use VAD]: Self::USE_VAD
+    #[must_use]
     pub fn use_vad(self) -> bool {
         self.contains(Self::USE_VAD)
     }
 }
 
-impl Default for Permissions {
-    fn default() -> Self {
-        Self::empty()
-    }
-}
-
 impl<'de> Deserialize<'de> for Permissions {
     fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
-        let str_u64 = String::deserialize(deserializer)?;
-        Ok(Permissions::from_bits_truncate(str_u64.parse::<u64>().map_err(D::Error::custom)?))
+        struct StringVisitor;
+
+        impl<'de> serde::de::Visitor<'de> for StringVisitor {
+            type Value = Permissions;
+
+            fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
+                formatter.write_str("permissions string")
+            }
+
+            fn visit_str<E>(self, v: &str) -> Result<Self::Value, E>
+            where
+                E: serde::de::Error,
+            {
+                let value = v.parse().map_err(E::custom)?;
+                Ok(Permissions::from_bits_truncate(value))
+            }
+        }
+        deserializer.deserialize_str(StringVisitor)
     }
 }
 
@@ -697,8 +806,8 @@ impl Serialize for Permissions {
 }
 
 #[cfg(feature = "model")]
-impl Display for Permissions {
-    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
+impl fmt::Display for Permissions {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let names = self.get_permission_names();
 
         let total = names.len();
@@ -717,3 +826,16 @@ impl Display for Permissions {
         Ok(())
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use serde_test::{assert_tokens, Token};
+
+    use super::*;
+
+    #[test]
+    fn permissions_serde() {
+        let value = Permissions::MANAGE_GUILD | Permissions::MANAGE_ROLES;
+        assert_tokens(&value, &[Token::Str("268435488")]);
+    }
+}
