@@ -15,14 +15,16 @@ pub use super::channel::*;
 pub use super::connection::*;
 pub use super::event::*;
 pub use super::gateway::*;
+pub use super::guild::audit_log::*;
 pub use super::guild::*;
 pub use super::id::*;
-#[cfg(feature = "unstable_discord_api")]
 pub use super::interactions::*;
 pub use super::invite::*;
+pub use super::mention::*;
 pub use super::misc::*;
 pub use super::oauth2::*;
 pub use super::permissions::*;
+pub use super::sticker::*;
 pub use super::user::*;
 pub use super::voice::*;
 pub use super::webhook::*;
