@@ -1,18 +1,12 @@
 //! User information-related models.
 
 use std::fmt;
-#[cfg(feature = "model")]
-use std::fmt::Write;
 
-use bitflags::__impl_bitflags;
 #[cfg(feature = "model")]
 use futures::future::{BoxFuture, FutureExt};
-use serde::{Deserialize, Deserializer, Serialize, Serializer};
-#[cfg(feature = "model")]
-use serde_json::json;
+use serde::{Deserialize, Serialize};
 
 use super::prelude::*;
-use super::utils::deserialize_u16;
 #[cfg(feature = "model")]
 use crate::builder::{CreateBotAuthParameters, CreateMessage, EditProfile};
 #[cfg(all(feature = "cache", feature = "model"))]
@@ -30,9 +24,145 @@ use crate::collector::{
 use crate::http::GuildPagination;
 #[cfg(feature = "model")]
 use crate::http::{CacheHttp, Http};
+use crate::internal::prelude::*;
 #[cfg(feature = "model")]
-use crate::utils;
-use crate::{internal::prelude::*, model::misc::Mentionable};
+use crate::json;
+#[cfg(feature = "model")]
+use crate::json::json;
+use crate::json::to_string;
+use crate::model::mention::Mentionable;
+
+/// Used with `#[serde(with|deserialize_with|serialize_with)]`
+///
+/// # Examples
+///
+/// ```rust,ignore
+/// #[derive(Deserialize, Serialize)]
+/// struct A {
+///     #[serde(with = "discriminator")]
+///     id: u16,
+/// }
+///
+/// #[derive(Deserialize)]
+/// struct B {
+///     #[serde(deserialize_with = "discriminator::deserialize")]
+///     id: u16,
+/// }
+///
+/// #[derive(Serialize)]
+/// struct C {
+///     #[serde(serialize_with = "discriminator::serialize")]
+///     id: u16,
+/// }
+/// ```
+pub(crate) mod discriminator {
+    use std::convert::TryFrom;
+    use std::fmt;
+
+    use serde::de::{Error, Visitor};
+    use serde::{Deserializer, Serializer};
+
+    pub fn deserialize<'de, D: Deserializer<'de>>(deserializer: D) -> Result<u16, D::Error> {
+        deserializer.deserialize_any(DiscriminatorVisitor)
+    }
+
+    #[allow(clippy::trivially_copy_pass_by_ref)]
+    pub fn serialize<S: Serializer>(value: &u16, serializer: S) -> Result<S::Ok, S::Error> {
+        serializer.collect_str(&format_args!("{:04}", value))
+    }
+
+    struct DiscriminatorVisitor;
+
+    impl<'de> Visitor<'de> for DiscriminatorVisitor {
+        type Value = u16;
+
+        fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
+            formatter.write_str("string or integer discriminator")
+        }
+
+        fn visit_u64<E: Error>(self, value: u64) -> Result<Self::Value, E> {
+            u16::try_from(value).map_err(Error::custom)
+        }
+
+        fn visit_str<E: Error>(self, s: &str) -> Result<Self::Value, E> {
+            s.parse().map_err(Error::custom)
+        }
+    }
+
+    /// Used with `#[serde(with|deserialize_with|serialize_with)]`
+    ///
+    /// # Examples
+    ///
+    /// ```rust,ignore
+    /// #[derive(Deserialize, Serialize)]
+    /// struct A {
+    ///     #[serde(with = "discriminator::option")]
+    ///     id: Option<u16>,
+    /// }
+    ///
+    /// #[derive(Deserialize)]
+    /// struct B {
+    ///     #[serde(deserialize_with = "discriminator::option::deserialize")]
+    ///     id: Option<u16>,
+    /// }
+    ///
+    /// #[derive(Serialize)]
+    /// struct C {
+    ///     #[serde(serialize_with = "discriminator::option::serialize")]
+    ///     id: Option<u16>,
+    /// }
+    /// ```
+    pub(crate) mod option {
+        use std::fmt;
+
+        use serde::de::{Error, Visitor};
+        use serde::{Deserializer, Serializer};
+
+        use super::DiscriminatorVisitor;
+
+        pub fn deserialize<'de, D: Deserializer<'de>>(
+            deserializer: D,
+        ) -> Result<Option<u16>, D::Error> {
+            deserializer.deserialize_option(OptionalDiscriminatorVisitor)
+        }
+
+        #[allow(clippy::trivially_copy_pass_by_ref)]
+        pub fn serialize<S: Serializer>(
+            value: &Option<u16>,
+            serializer: S,
+        ) -> Result<S::Ok, S::Error> {
+            match value {
+                Some(value) => serializer.serialize_some(&format_args!("{:04}", value)),
+                None => serializer.serialize_none(),
+            }
+        }
+
+        struct OptionalDiscriminatorVisitor;
+
+        impl<'de> Visitor<'de> for OptionalDiscriminatorVisitor {
+            type Value = Option<u16>;
+
+            fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
+                formatter.write_str("optional string or integer discriminator")
+            }
+
+            fn visit_none<E: Error>(self) -> Result<Self::Value, E> {
+                Ok(None)
+            }
+
+            fn visit_unit<E: Error>(self) -> Result<Self::Value, E> {
+                Ok(None)
+            }
+
+            fn visit_some<D: Deserializer<'de>>(
+                self,
+                deserializer: D,
+            ) -> Result<Self::Value, D::Error> {
+                Ok(Some(deserializer.deserialize_any(DiscriminatorVisitor)?))
+            }
+        }
+    }
+}
 
 /// Information about the current user.
 #[derive(Clone, Default, Debug, Deserialize, Serialize)]
@@ -42,7 +172,7 @@ pub struct CurrentUser {
     pub avatar: Option<String>,
     #[serde(default)]
     pub bot: bool,
-    #[serde(deserialize_with = "deserialize_u16")]
+    #[serde(with = "discriminator")]
     pub discriminator: u16,
     pub email: Option<String>,
     pub mfa_enabled: bool,
@@ -69,14 +199,12 @@ impl CurrentUser {
     ///
     /// ```rust,no_run
     /// # #[cfg(feature = "cache")]
-    /// # async fn run() {
+    /// # fn run() {
     /// # use serenity::cache::Cache;
-    /// # use tokio::sync::RwLock;
-    /// # use std::sync::Arc;
     /// #
     /// # let cache = Cache::default();
     /// // assuming the cache has been unlocked
-    /// let user = cache.current_user().await;
+    /// let user = cache.current_user();
     ///
     /// match user.avatar_url() {
     ///     Some(url) => println!("{}'s avatar can be found at {}", user.name, url),
@@ -85,6 +213,7 @@ impl CurrentUser {
     /// # }
     /// ```
     #[inline]
+    #[must_use]
     pub fn avatar_url(&self) -> Option<String> {
         avatar_url(self.id, self.avatar.as_ref())
     }
@@ -93,6 +222,7 @@ impl CurrentUser {
     ///
     /// This will produce a PNG URL.
     #[inline]
+    #[must_use]
     pub fn default_avatar_url(&self) -> String {
         default_avatar_url(self.discriminator)
     }
@@ -112,7 +242,7 @@ impl CurrentUser {
     /// # use serenity::model::user::CurrentUser;
     /// #
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// #     let http = Http::default();
+    /// #     let http = Http::new("token");
     /// #     let mut user = CurrentUser::default();
     /// let avatar = serenity::utils::read_image("./avatar.png")?;
     ///
@@ -121,8 +251,6 @@ impl CurrentUser {
     /// # }
     /// ```
     ///
-    /// [`EditProfile`]: crate::builder::EditProfile
-    ///
     /// # Errors
     ///
     /// Returns an [`Error::Http`] if an invalid value is set.
@@ -136,15 +264,15 @@ impl CurrentUser {
         F: FnOnce(&mut EditProfile) -> &mut EditProfile,
     {
         let mut map = HashMap::new();
-        map.insert("username", Value::String(self.name.clone()));
+        map.insert("username", Value::from(self.name.clone()));
 
         if let Some(email) = self.email.as_ref() {
-            map.insert("email", Value::String(email.clone()));
+            map.insert("email", Value::from(email.clone()));
         }
 
         let mut edit_profile = EditProfile(map);
         f(&mut edit_profile);
-        let map = utils::hashmap_to_json_map(edit_profile.0);
+        let map = json::hashmap_to_json_map(edit_profile.0);
 
         *self = http.as_ref().edit_profile(&map).await?;
 
@@ -157,6 +285,7 @@ impl CurrentUser {
     /// This will call [`Self::avatar_url`] first, and if that returns [`None`], it
     /// then falls back to [`Self::default_avatar_url`].
     #[inline]
+    #[must_use]
     pub fn face(&self) -> String {
         self.avatar_url().unwrap_or_else(|| self.default_avatar_url())
     }
@@ -173,7 +302,7 @@ impl CurrentUser {
     /// #
     /// # async fn run() {
     /// #     let user = CurrentUser::default();
-    /// #     let http = Http::default();
+    /// #     let http = Http::new("token");
     /// // assuming the user has been bound
     ///
     /// if let Ok(guilds) = user.guilds(&http).await {
@@ -198,8 +327,10 @@ impl CurrentUser {
             let mut pagination = http
                 .as_ref()
                 .get_guilds(
-                    &GuildPagination::After(guilds.last().map_or(GuildId(1), |g: &GuildInfo| g.id)),
-                    100,
+                    Some(&GuildPagination::After(
+                        guilds.last().map_or(GuildId(1), |g: &GuildInfo| g.id),
+                    )),
+                    Some(100),
                 )
                 .await?;
             let len = pagination.len();
@@ -230,7 +361,7 @@ impl CurrentUser {
     /// #
     /// # async fn run() {
     /// #     let user = CurrentUser::default();
-    /// #     let http = Http::default();
+    /// #     let http = Http::new("token");
     /// use serenity::model::Permissions;
     ///
     /// // assuming the user has been bound
@@ -259,12 +390,12 @@ impl CurrentUser {
     /// #
     /// # async fn run() {
     /// #     let user = CurrentUser::default();
-    /// #     let http = Http::default();
+    /// #     let http = Http::new("token");
     /// use serenity::model::Permissions;
     ///
     /// // assuming the user has been bound
     /// let permissions =
-    ///     Permissions::READ_MESSAGES | Permissions::SEND_MESSAGES | Permissions::EMBED_LINKS;
+    ///     Permissions::VIEW_CHANNEL | Permissions::SEND_MESSAGES | Permissions::EMBED_LINKS;
     /// let url = match user.invite_url(&http, permissions).await {
     ///     Ok(v) => v,
     ///     Err(why) => {
@@ -311,7 +442,7 @@ impl CurrentUser {
     /// #
     /// # async fn run() {
     /// #     let user = CurrentUser::default();
-    /// #     let http = Http::default();
+    /// #     let http = Http::new("token");
     /// use serenity::model::oauth2::OAuth2Scope;
     /// use serenity::model::Permissions;
     ///
@@ -334,6 +465,7 @@ impl CurrentUser {
     /// );
     /// # }
     /// ```
+    ///
     /// # Errors
     ///
     /// Returns an
@@ -380,6 +512,7 @@ impl CurrentUser {
     /// # }
     /// ```
     #[inline]
+    #[must_use]
     pub fn static_avatar_url(&self) -> Option<String> {
         static_avatar_url(self.id, self.avatar.as_ref())
     }
@@ -401,6 +534,7 @@ impl CurrentUser {
     /// # }
     /// ```
     #[inline]
+    #[must_use]
     pub fn tag(&self) -> String {
         tag(&self.name, self.discriminator)
     }
@@ -440,7 +574,7 @@ impl DefaultAvatar {
     ///
     /// [`Error::Json`]: crate::error::Error::Json
     pub fn name(self) -> Result<String> {
-        serde_json::to_string(&self).map_err(From::from)
+        to_string(&self).map_err(From::from)
     }
 }
 
@@ -469,6 +603,7 @@ pub enum OnlineStatus {
 }
 
 impl OnlineStatus {
+    #[must_use]
     pub fn name(&self) -> &str {
         match *self {
             OnlineStatus::DoNotDisturb => "dnd",
@@ -500,7 +635,7 @@ pub struct User {
     pub bot: bool,
     /// The account's discriminator to differentiate the user from others with
     /// the same [`Self::name`]. The name+discriminator pair is always unique.
-    #[serde(deserialize_with = "deserialize_u16")]
+    #[serde(with = "discriminator")]
     pub discriminator: u16,
     /// The account's username. Changing username will trigger a discriminator
     /// change if the username+discriminator pair becomes non-unique.
@@ -526,62 +661,40 @@ pub struct User {
     pub accent_colour: Option<u32>,
 }
 
-/// User's public flags
-#[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
-pub struct UserPublicFlags {
-    pub bits: u32,
-}
-
-__impl_bitflags! {
-    UserPublicFlags: u32 {
+bitflags! {
+    /// User's public flags
+    #[derive(Default)]
+    pub struct UserPublicFlags: u32 {
         /// User's flag as discord employee
-        DISCORD_EMPLOYEE = 1 << 0;
+        const DISCORD_EMPLOYEE = 1 << 0;
         /// User's flag as partnered server owner
-        PARTNERED_SERVER_OWNER = 1 << 1;
+        const PARTNERED_SERVER_OWNER = 1 << 1;
         /// User's flag as hypesquad events
-        HYPESQUAD_EVENTS = 1 << 2;
+        const HYPESQUAD_EVENTS = 1 << 2;
         /// User's flag as bug hunter level 1
-        BUG_HUNTER_LEVEL_1 = 1 << 3;
+        const BUG_HUNTER_LEVEL_1 = 1 << 3;
         /// User's flag as house bravery
-        HOUSE_BRAVERY = 1 << 6;
+        const HOUSE_BRAVERY = 1 << 6;
         /// User's flag as house brilliance
-        HOUSE_BRILLIANCE = 1 << 7;
+        const HOUSE_BRILLIANCE = 1 << 7;
         /// User's flag as house balance
-        HOUSE_BALANCE = 1 << 8;
+        const HOUSE_BALANCE = 1 << 8;
         /// User's flag as early supporter
-        EARLY_SUPPORTER = 1 << 9;
+        const EARLY_SUPPORTER = 1 << 9;
         /// User's flag as team user
-        TEAM_USER = 1 << 10;
+        const TEAM_USER = 1 << 10;
         /// User's flag as system
-        SYSTEM = 1 << 12;
+        const SYSTEM = 1 << 12;
         /// User's flag as bug hunter level 2
-        BUG_HUNTER_LEVEL_2 = 1 << 14;
+        const BUG_HUNTER_LEVEL_2 = 1 << 14;
         /// User's flag as verified bot
-        VERIFIED_BOT = 1 << 16;
+        const VERIFIED_BOT = 1 << 16;
         /// User's flag as early verified bot developer
-        EARLY_VERIFIED_BOT_DEVELOPER = 1 << 17;
+        const EARLY_VERIFIED_BOT_DEVELOPER = 1 << 17;
         /// User's flag as discord certified moderator
-        DISCORD_CERTIFIED_MODERATOR = 1 << 18;
+        const DISCORD_CERTIFIED_MODERATOR = 1 << 18;
         /// Bot's running with HTTP interactions
-        BOT_HTTP_INTERACTIONS = 1 << 19;
-    }
-}
-
-impl<'de> Deserialize<'de> for UserPublicFlags {
-    fn deserialize<D>(deserializer: D) -> StdResult<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
-        Ok(UserPublicFlags::from_bits_truncate(deserializer.deserialize_u32(U32Visitor)?))
-    }
-}
-
-impl Serialize for UserPublicFlags {
-    fn serialize<S>(&self, serializer: S) -> StdResult<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u32(self.bits())
+        const BOT_HTTP_INTERACTIONS = 1 << 19;
     }
 }
 
@@ -609,9 +722,6 @@ impl Default for User {
 
 use std::hash::{Hash, Hasher};
 
-#[cfg(feature = "model")]
-use chrono::{DateTime, Utc};
-
 impl PartialEq for User {
     fn eq(&self, other: &Self) -> bool {
         self.id == other.id
@@ -632,6 +742,7 @@ impl User {
     ///
     /// This will produce a WEBP image URL, or GIF if the user has a GIF avatar.
     #[inline]
+    #[must_use]
     pub fn avatar_url(&self) -> Option<String> {
         avatar_url(self.id, self.avatar.as_ref())
     }
@@ -643,6 +754,7 @@ impl User {
     /// **Note**: This will only be present if the user is fetched via Rest API,
     /// e.g. with [`Http::get_user`].
     #[inline]
+    #[must_use]
     pub fn banner_url(&self) -> Option<String> {
         banner_url(self.id, self.banner.as_ref())
     }
@@ -655,8 +767,6 @@ impl User {
     /// # Errors
     ///
     /// See [`UserId::create_dm_channel`] for what errors may be returned.
-    ///
-    /// [`UserId::create_dm_channel`]: crate::model::id::UserId::create_dm_channel
     #[inline]
     pub async fn create_dm_channel(&self, cache_http: impl CacheHttp) -> Result<PrivateChannel> {
         if self.bot {
@@ -668,7 +778,8 @@ impl User {
 
     /// Retrieves the time that this user was created at.
     #[inline]
-    pub fn created_at(&self) -> DateTime<Utc> {
+    #[must_use]
+    pub fn created_at(&self) -> Timestamp {
         self.id.created_at()
     }
 
@@ -676,6 +787,7 @@ impl User {
     ///
     /// This will produce a PNG URL.
     #[inline]
+    #[must_use]
     pub fn default_avatar_url(&self) -> String {
         default_avatar_url(self.discriminator)
     }
@@ -702,20 +814,15 @@ impl User {
     /// #   #[cfg(feature = "cache")]
     ///     async fn message(&self, ctx: Context, msg: Message) {
     ///         if msg.content == "~help" {
-    ///             let url = match ctx
-    ///                 .cache
-    ///                 .current_user()
-    ///                 .await
-    ///                 .invite_url(&ctx, Permissions::empty())
-    ///                 .await
-    ///             {
-    ///                 Ok(v) => v,
-    ///                 Err(why) => {
-    ///                     println!("Error creating invite url: {:?}", why);
+    ///             let url =
+    ///                 match ctx.cache.current_user().invite_url(&ctx, Permissions::empty()).await {
+    ///                     Ok(v) => v,
+    ///                     Err(why) => {
+    ///                         println!("Error creating invite url: {:?}", why);
     ///
-    ///                     return;
-    ///                 },
-    ///             };
+    ///                         return;
+    ///                     },
+    ///                 };
     ///
     ///             let help = format!("Helpful info here. Invite me with this link: <{}>", url,);
     ///
@@ -736,7 +843,8 @@ impl User {
     /// }
     ///
     /// # async fn run() -> Result<(), Box<dyn std::error::Error>> {
-    /// let mut client = Client::builder("token").event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder("token", GatewayIntents::default()).event_handler(Handler).await?;
     /// #     Ok(())
     /// # }
     /// # }
@@ -755,9 +863,9 @@ impl User {
     ///
     /// [`Error::Http`]: crate::error::Error::Http
     /// [`Error::Json`]: crate::error::Error::Json
-    pub async fn direct_message<F>(&self, cache_http: impl CacheHttp, f: F) -> Result<Message>
+    pub async fn direct_message<'a, F>(&self, cache_http: impl CacheHttp, f: F) -> Result<Message>
     where
-        for<'a, 'b> F: FnOnce(&'b mut CreateMessage<'a>) -> &'b mut CreateMessage<'a>,
+        for<'b> F: FnOnce(&'b mut CreateMessage<'a>) -> &'b mut CreateMessage<'a>,
     {
         self.create_dm_channel(&cache_http).await?.send_message(&cache_http.http(), f).await
     }
@@ -765,9 +873,9 @@ impl User {
     /// This is an alias of [`Self::direct_message`].
     #[allow(clippy::missing_errors_doc)]
     #[inline]
-    pub async fn dm<F>(&self, cache_http: impl CacheHttp, f: F) -> Result<Message>
+    pub async fn dm<'a, F>(&self, cache_http: impl CacheHttp, f: F) -> Result<Message>
     where
-        for<'a, 'b> F: FnOnce(&'b mut CreateMessage<'a>) -> &'b mut CreateMessage<'a>,
+        for<'b> F: FnOnce(&'b mut CreateMessage<'a>) -> &'b mut CreateMessage<'a>,
     {
         self.direct_message(cache_http, f).await
     }
@@ -777,6 +885,7 @@ impl User {
     ///
     /// This will call [`Self::avatar_url`] first, and if that returns [`None`], it
     /// then falls back to [`Self::default_avatar_url`].
+    #[must_use]
     pub fn face(&self) -> String {
         self.avatar_url().unwrap_or_else(|| self.default_avatar_url())
     }
@@ -839,7 +948,7 @@ impl User {
                     #[cfg(feature = "cache")]
                     {
                         if let Some(cache) = cache_http.cache() {
-                            if let Some(member) = cache.member(guild_id, self.id).await {
+                            if let Some(member) = cache.member(guild_id, self.id) {
                                 has_role = Some(member.roles.contains(&role));
                             }
                         }
@@ -867,8 +976,6 @@ impl User {
     /// # Errors
     ///
     /// See [`UserId::to_user`] for what errors may be returned.
-    ///
-    /// [`UserId::to_user`]: crate::model::id::UserId::to_user
     #[inline]
     pub async fn refresh(&mut self, cache_http: impl CacheHttp) -> Result<()> {
         *self = self.id.to_user(cache_http).await?;
@@ -880,6 +987,7 @@ impl User {
     ///
     /// This will always produce a WEBP image URL.
     #[inline]
+    #[must_use]
     pub fn static_avatar_url(&self) -> Option<String> {
         static_avatar_url(self.id, self.avatar.as_ref())
     }
@@ -916,13 +1024,15 @@ impl User {
     ///         }
     ///     }
     /// }
-    /// let mut client = Client::builder("token").event_handler(Handler).await?;
+    /// let mut client =
+    ///     Client::builder("token", GatewayIntents::default()).event_handler(Handler).await?;
     ///
     /// client.start().await?;
     /// #     Ok(())
     /// # }
     /// ```
     #[inline]
+    #[must_use]
     pub fn tag(&self) -> String {
         tag(&self.name, self.discriminator)
     }
@@ -941,7 +1051,7 @@ impl User {
         #[cfg(feature = "cache")]
         {
             if let Some(cache) = cache_http.cache() {
-                if let Some(guild) = guild_id.to_guild_cached(cache).await {
+                if let Some(guild) = guild_id.to_guild_cached(cache) {
                     if let Some(member) = guild.members.get(&self.id) {
                         return member.nick.clone();
                     }
@@ -954,37 +1064,31 @@ impl User {
 
     /// Returns a future that will await one message by this user.
     #[cfg(feature = "collector")]
-    pub fn await_reply<'a>(
-        &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> CollectReply<'a> {
+    pub fn await_reply(&self, shard_messenger: impl AsRef<ShardMessenger>) -> CollectReply {
         CollectReply::new(shard_messenger).author_id(self.id.0)
     }
 
     /// Returns a stream builder which can be awaited to obtain a stream of messages sent by this user.
     #[cfg(feature = "collector")]
-    pub fn await_replies<'a>(
+    pub fn await_replies(
         &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> MessageCollectorBuilder<'a> {
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> MessageCollectorBuilder {
         MessageCollectorBuilder::new(shard_messenger).author_id(self.id.0)
     }
 
     /// Await a single reaction by this user.
     #[cfg(feature = "collector")]
-    pub fn await_reaction<'a>(
-        &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> CollectReaction<'a> {
+    pub fn await_reaction(&self, shard_messenger: impl AsRef<ShardMessenger>) -> CollectReaction {
         CollectReaction::new(shard_messenger).author_id(self.id.0)
     }
 
     /// Returns a stream builder which can be awaited to obtain a stream of reactions sent by this user.
     #[cfg(feature = "collector")]
-    pub fn await_reactions<'a>(
+    pub fn await_reactions(
         &self,
-        shard_messenger: &'a impl AsRef<ShardMessenger>,
-    ) -> ReactionCollectorBuilder<'a> {
+        shard_messenger: impl AsRef<ShardMessenger>,
+    ) -> ReactionCollectorBuilder {
         ReactionCollectorBuilder::new(shard_messenger).author_id(self.id.0)
     }
 }
@@ -1018,7 +1122,9 @@ impl UserId {
         #[cfg(feature = "cache")]
         {
             if let Some(cache) = cache_http.cache() {
-                for channel in cache.private_channels().await.values() {
+                for channel_entry in cache.private_channels().iter() {
+                    let channel = channel_entry.value();
+
                     if channel.recipient.id == self {
                         return Ok(channel.clone());
                     }
@@ -1037,14 +1143,16 @@ impl UserId {
     #[cfg(feature = "cache")]
     #[inline]
     pub async fn to_user_cached(self, cache: impl AsRef<Cache>) -> Option<User> {
-        cache.as_ref().user(self).await
+        cache.as_ref().user(self)
     }
 
     /// First attempts to find a [`User`] by its Id in the cache,
     /// upon failure requests it via the REST API.
     ///
-    /// **Note**: If the cache is not enabled,
-    /// REST API will be used only.
+    /// **Note**: If the cache is not enabled, REST API will be used only.
+    ///
+    /// **Note**: If the cache is enabled, you might want to enable the `temp_cache` feature to
+    /// cache user data retrieved by this function for a short duration.
     ///
     /// # Errors
     ///
@@ -1061,13 +1169,22 @@ impl UserId {
         #[cfg(feature = "cache")]
         {
             if let Some(cache) = cache_http.cache() {
-                if let Some(user) = cache.user(self).await {
+                if let Some(user) = cache.user(self) {
                     return Ok(user);
                 }
             }
         }
 
-        cache_http.http().get_user(self.0).await
+        let user = cache_http.http().get_user(self.0).await?;
+
+        #[cfg(all(feature = "cache", feature = "temp_cache"))]
+        {
+            if let Some(cache) = cache_http.cache() {
+                cache.temp_users.insert(user.id, user.clone());
+            }
+        }
+
+        Ok(user)
     }
 }
 
@@ -1179,16 +1296,78 @@ fn tag(name: &str, discriminator: u16) -> String {
     let mut tag = String::with_capacity(37);
     tag.push_str(name);
     tag.push('#');
-
-    #[allow(clippy::let_underscore_must_use)]
-    let _ = write!(tag, "{:04}", discriminator);
+    tag.push_str(&format!("{:04}", discriminator));
 
     tag
 }
 
 #[cfg(test)]
-#[allow(clippy::unwrap_used)]
 mod test {
+    #[test]
+    fn test_discriminator_serde() {
+        use serde::{Deserialize, Serialize};
+        use serde_test::{assert_de_tokens, assert_tokens, Token};
+
+        use super::discriminator;
+
+        #[derive(Debug, PartialEq, Deserialize, Serialize)]
+        struct User {
+            #[serde(with = "discriminator")]
+            discriminator: u16,
+        }
+        #[derive(Debug, PartialEq, Deserialize, Serialize)]
+        struct UserOpt {
+            #[serde(with = "discriminator::option")]
+            discriminator: Option<u16>,
+        }
+
+        let user = User {
+            discriminator: 123,
+        };
+        assert_tokens(&user, &[
+            Token::Struct {
+                name: "User",
+                len: 1,
+            },
+            Token::Str("discriminator"),
+            Token::Str("0123"),
+            Token::StructEnd,
+        ]);
+        assert_de_tokens(&user, &[
+            Token::Struct {
+                name: "User",
+                len: 1,
+            },
+            Token::Str("discriminator"),
+            Token::U16(123),
+            Token::StructEnd,
+        ]);
+
+        let user = UserOpt {
+            discriminator: Some(123),
+        };
+        assert_tokens(&user, &[
+            Token::Struct {
+                name: "UserOpt",
+                len: 1,
+            },
+            Token::Str("discriminator"),
+            Token::Some,
+            Token::Str("0123"),
+            Token::StructEnd,
+        ]);
+        assert_de_tokens(&user, &[
+            Token::Struct {
+                name: "UserOpt",
+                len: 1,
+            },
+            Token::Str("discriminator"),
+            Token::Some,
+            Token::U16(123),
+            Token::StructEnd,
+        ]);
+    }
+
     #[cfg(feature = "model")]
     mod model {
         use crate::model::user::User;
