@@ -1,5 +1,8 @@
+use std::fmt;
+
 use super::ArgumentConvert;
-use crate::{model::prelude::*, prelude::*};
+use crate::model::prelude::*;
+use crate::prelude::*;
 
 /// Error that can be returned from [`Channel::convert`].
 #[non_exhaustive]
@@ -21,8 +24,8 @@ impl std::error::Error for ChannelParseError {
     }
 }
 
-impl std::fmt::Display for ChannelParseError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+impl fmt::Display for ChannelParseError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Self::Http(_) => f.write_str("Failed to request channel via HTTP"),
             Self::NotFoundOrMalformed => f.write_str("Channel not found or unknown format"),
@@ -38,18 +41,36 @@ fn channel_belongs_to_guild(channel: &Channel, guild: GuildId) -> bool {
     }
 }
 
-async fn lookup_channel_global(ctx: &Context, s: &str) -> Result<Channel, ChannelParseError> {
+async fn lookup_channel_global(
+    ctx: &Context,
+    guild_id: Option<GuildId>,
+    s: &str,
+) -> Result<Channel, ChannelParseError> {
     if let Some(channel_id) = s.parse::<u64>().ok().or_else(|| crate::utils::parse_channel(s)) {
         return ChannelId(channel_id).to_channel(ctx).await.map_err(ChannelParseError::Http);
     }
 
-    let channels = ctx.cache.channels.read().await;
-    if let Some(channel) =
-        channels.values().find(|channel| channel.name.eq_ignore_ascii_case(s)).cloned()
-    {
+    #[cfg(feature = "cache")]
+    if let Some(channel) = ctx.cache.channels.iter().find_map(|m| {
+        let channel = m.value();
+        if channel.name.eq_ignore_ascii_case(s) {
+            Some(channel.clone())
+        } else {
+            None
+        }
+    }) {
         return Ok(Channel::Guild(channel));
     }
 
+    if let Some(guild_id) = guild_id {
+        let channels = ctx.http.get_channels(guild_id.0).await.map_err(ChannelParseError::Http)?;
+        if let Some(channel) =
+            channels.into_iter().find(|channel| channel.name.eq_ignore_ascii_case(s))
+        {
+            return Ok(Channel::Guild(channel));
+        }
+    }
+
     Err(ChannelParseError::NotFoundOrMalformed)
 }
 
@@ -63,7 +84,6 @@ async fn lookup_channel_global(ctx: &Context, s: &str) -> Result<Channel, Channe
 /// 1. Lookup by ID.
 /// 2. [Lookup by mention](`crate::utils::parse_channel`).
 /// 3. Lookup by name.
-#[cfg(feature = "cache")]
 #[async_trait::async_trait]
 impl ArgumentConvert for Channel {
     type Err = ChannelParseError;
@@ -74,7 +94,7 @@ impl ArgumentConvert for Channel {
         _channel_id: Option<ChannelId>,
         s: &str,
     ) -> Result<Self, Self::Err> {
-        let channel = lookup_channel_global(ctx, s).await?;
+        let channel = lookup_channel_global(ctx, guild_id, s).await?;
 
         // Don't yield for other guilds' channels
         if let Some(guild_id) = guild_id {
@@ -104,14 +124,13 @@ impl std::error::Error for GuildChannelParseError {
     fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
         match self {
             Self::Http(e) => Some(e),
-            Self::NotFoundOrMalformed => None,
-            Self::NotAGuildChannel => None,
+            Self::NotFoundOrMalformed | Self::NotAGuildChannel => None,
         }
     }
 }
 
-impl std::fmt::Display for GuildChannelParseError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+impl fmt::Display for GuildChannelParseError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Self::Http(_) => f.write_str("Failed to request channel via HTTP"),
             Self::NotFoundOrMalformed => f.write_str("Channel not found or unknown format"),
@@ -125,7 +144,6 @@ impl std::fmt::Display for GuildChannelParseError {
 /// Lookup is done by the global cache, hence the cache feature needs to be enabled.
 ///
 /// For more information, see the ArgumentConvert implementation for [`Channel`]
-#[cfg(feature = "cache")]
 #[async_trait::async_trait]
 impl ArgumentConvert for GuildChannel {
     type Err = GuildChannelParseError;
@@ -164,14 +182,13 @@ impl std::error::Error for ChannelCategoryParseError {
     fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
         match self {
             Self::Http(e) => Some(e),
-            Self::NotFoundOrMalformed => None,
-            Self::NotAChannelCategory => None,
+            Self::NotFoundOrMalformed | Self::NotAChannelCategory => None,
         }
     }
 }
 
-impl std::fmt::Display for ChannelCategoryParseError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+impl fmt::Display for ChannelCategoryParseError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Self::Http(_) => f.write_str("Failed to request channel via HTTP"),
             Self::NotFoundOrMalformed => f.write_str("Channel not found or unknown format"),
@@ -185,7 +202,6 @@ impl std::fmt::Display for ChannelCategoryParseError {
 /// Lookup is done by the global cache, hence the cache feature needs to be enabled.
 ///
 /// For more information, see the ArgumentConvert implementation for [`Channel`]
-#[cfg(feature = "cache")]
 #[async_trait::async_trait]
 impl ArgumentConvert for ChannelCategory {
     type Err = ChannelCategoryParseError;
@@ -198,7 +214,7 @@ impl ArgumentConvert for ChannelCategory {
     ) -> Result<Self, Self::Err> {
         match Channel::convert(ctx, guild_id, channel_id, s).await {
             Ok(Channel::Category(channel)) => Ok(channel),
-            // TODO: accomodate issue #1352 somehow
+            // TODO: accommodate issue #1352 somehow
             Ok(_) => Err(ChannelCategoryParseError::NotAChannelCategory),
             Err(ChannelParseError::Http(e)) => Err(ChannelCategoryParseError::Http(e)),
             Err(ChannelParseError::NotFoundOrMalformed) => {
