@@ -1,5 +1,8 @@
+use std::fmt;
+
 use super::ArgumentConvert;
-use crate::{model::prelude::*, prelude::*};
+use crate::model::prelude::*;
+use crate::prelude::*;
 
 /// Error that can be returned from [`Member::convert`].
 #[non_exhaustive]
@@ -16,8 +19,8 @@ pub enum MemberParseError {
 
 impl std::error::Error for MemberParseError {}
 
-impl std::fmt::Display for MemberParseError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+impl fmt::Display for MemberParseError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Self::OutsideGuild => f.write_str("Tried to find member outside a guild"),
             Self::GuildNotInCache => f.write_str("Guild is not in cache"),
@@ -36,7 +39,6 @@ impl std::fmt::Display for MemberParseError {
 /// 3. [Lookup by name#discrim](`crate::utils::parse_user_tag`).
 /// 4. Lookup by name
 /// 5. Lookup by nickname
-#[cfg(feature = "cache")]
 #[async_trait::async_trait]
 impl ArgumentConvert for Member {
     type Err = MemberParseError;
@@ -47,17 +49,13 @@ impl ArgumentConvert for Member {
         _channel_id: Option<ChannelId>,
         s: &str,
     ) -> Result<Self, Self::Err> {
-        let guild = guild_id
-            .ok_or(MemberParseError::OutsideGuild)?
-            .to_guild_cached(ctx)
-            .await
-            .ok_or(MemberParseError::GuildNotInCache)?;
+        let guild_id = guild_id.ok_or(MemberParseError::OutsideGuild)?;
 
         // DON'T use guild.members: it's only populated when guild presences intent is enabled!
 
         // If string is a raw user ID or a mention
         if let Some(user_id) = s.parse().ok().or_else(|| crate::utils::parse_username(s)) {
-            if let Ok(member) = guild.member(ctx, UserId(user_id)).await {
+            if let Ok(member) = guild_id.member(ctx, UserId(user_id)).await {
                 return Ok(member);
             }
         }
@@ -66,7 +64,7 @@ impl ArgumentConvert for Member {
 
         // If string is a username+discriminator
         if let Some((name, discrim)) = crate::utils::parse_user_tag(s) {
-            if let Ok(member_results) = guild.search_members(ctx, name, Some(100)).await {
+            if let Ok(member_results) = guild_id.search_members(ctx, name, Some(100)).await {
                 if let Some(member) = member_results.into_iter().find(|m| {
                     m.user.name.eq_ignore_ascii_case(name) && m.user.discriminator == discrim
                 }) {
@@ -76,7 +74,7 @@ impl ArgumentConvert for Member {
         }
 
         // If string is username or nickname
-        if let Ok(member_results) = guild.search_members(ctx, s, Some(100)).await {
+        if let Ok(member_results) = guild_id.search_members(ctx, s, Some(100)).await {
             if let Some(member) = member_results.into_iter().find(|m| {
                 m.user.name.eq_ignore_ascii_case(s)
                     || m.nick.as_ref().map_or(false, |nick| nick.eq_ignore_ascii_case(s))
