@@ -1,14 +1,9 @@
-use std::{
-    default::Default,
-    fmt::{self, Display, Write},
-    ops::Add,
-};
+use std::fmt;
+use std::ops::Add;
 
-use crate::model::{
-    guild::Emoji,
-    id::{ChannelId, RoleId, UserId},
-    misc::Mentionable,
-};
+use crate::model::guild::Emoji;
+use crate::model::id::{ChannelId, RoleId, UserId};
+use crate::model::mention::Mentionable;
 
 /// The Message Builder is an ergonomic utility to easily build a message,
 /// by adding text and mentioning mentionable structs.
@@ -21,19 +16,9 @@ use crate::model::{
 /// value:
 ///
 /// ```rust,no_run
-/// # use serde_json::json;
 /// # use serenity::model::prelude::*;
 /// #
-/// # async fn run() {
-/// # let user = UserId(1);
-/// # let emoji = serde_json::from_value::<Emoji>(json!({
-/// #     "animated": false,
-/// #     "id": EmojiId(2),
-/// #     "name": "test",
-/// #     "managed": false,
-/// #     "require_colons": true,
-/// #     "roles": Vec::<Role>::new(),
-/// # })).unwrap();
+/// # fn run(user: UserId, emoji: Emoji) {
 /// #
 /// use serenity::utils::MessageBuilder;
 ///
@@ -65,6 +50,7 @@ impl MessageBuilder {
     /// // alternatively:
     /// let message = MessageBuilder::default();
     /// ```
+    #[must_use]
     pub fn new() -> MessageBuilder {
         MessageBuilder::default()
     }
@@ -133,10 +119,7 @@ impl MessageBuilder {
     }
 
     fn _channel(&mut self, channel: ChannelId) -> &mut Self {
-        #[allow(clippy::let_underscore_must_use)]
-        let _ = write!(self.0, "{}", channel.mention());
-        // should not error, ignoring
-
+        self.0.push_str(&channel.mention().to_string());
         self
     }
 
@@ -150,45 +133,30 @@ impl MessageBuilder {
     /// Mention an emoji in a message's content:
     ///
     /// ```rust
-    /// # use serde_json::json;
-    /// # use serenity::model::guild::Role;
-    /// #
-    /// # {
-    /// #
-    /// use serenity::model::guild::Emoji;
-    /// use serenity::model::id::EmojiId;
-    /// use serenity::utils::MessageBuilder;
+    /// # use serde_json::{json, from_value};
+    /// # use serenity::model::guild::Emoji;
+    /// # use serenity::model::id::EmojiId;
+    /// # use serenity::utils::MessageBuilder;
     ///
-    /// # let emoji = serde_json::from_value::<Emoji>(json!({
-    /// #     "animated": false,
+    /// # let emoji = from_value::<Emoji>(json!({
     /// #     "id": EmojiId(302516740095606785),
-    /// #     "managed": true,
-    /// #     "name": "smugAnimeFace".to_string(),
-    /// #     "require_colons": true,
-    /// #     "roles": Vec::<Role>::new(),
+    /// #     "name": "smugAnimeFace",
     /// # })).unwrap();
     ///
     /// let message = MessageBuilder::new().push("foo ").emoji(&emoji).push(".").build();
     ///
     /// assert_eq!(message, "foo <:smugAnimeFace:302516740095606785>.");
-    /// # }
     /// ```
     ///
     /// [Display implementation]: crate::model::guild::Emoji#impl-Display
     pub fn emoji(&mut self, emoji: &Emoji) -> &mut Self {
-        #[allow(clippy::let_underscore_must_use)]
-        let _ = write!(self.0, "{}", emoji);
-        // should not error, ignoring
-
+        self.0.push_str(&emoji.to_string());
         self
     }
 
     /// Mentions something that implements the [`Mentionable`] trait.
     pub fn mention<M: Mentionable>(&mut self, item: &M) -> &mut Self {
-        #[allow(clippy::let_underscore_must_use)]
-        let _ = write!(self.0, "{}", item.mention());
-        // should not error, ignoring
-
+        self.0.push_str(&item.mention().to_string());
         self
     }
 
@@ -215,7 +183,7 @@ impl MessageBuilder {
     /// );
     /// ```
     #[inline]
-    pub fn push<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push(&mut self, content: impl Into<Content>) -> &mut Self {
         self._push(&content.into().to_string())
     }
 
@@ -263,7 +231,11 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "```\nhello\n```");
     /// ```
-    pub fn push_codeblock<D: I>(&mut self, content: D, language: Option<&str>) -> &mut Self {
+    pub fn push_codeblock(
+        &mut self,
+        content: impl Into<Content>,
+        language: Option<&str>,
+    ) -> &mut Self {
         self.0.push_str("```");
 
         if let Some(language) = language {
@@ -301,7 +273,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, expected);
     /// ```
-    pub fn push_mono<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_mono(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push('`');
         self.0.push_str(&content.into().to_string());
         self.0.push('`');
@@ -330,7 +302,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, expected);
     /// ```
-    pub fn push_italic<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_italic(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push('_');
         self.0.push_str(&content.into().to_string());
         self.0.push('_');
@@ -339,7 +311,7 @@ impl MessageBuilder {
     }
 
     /// Pushes an inline bold text to the content.
-    pub fn push_bold<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_bold(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push_str("**");
         self.0.push_str(&content.into().to_string());
         self.0.push_str("**");
@@ -348,7 +320,7 @@ impl MessageBuilder {
     }
 
     /// Pushes an underlined inline text to the content.
-    pub fn push_underline<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_underline(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push_str("__");
         self.0.push_str(&content.into().to_string());
         self.0.push_str("__");
@@ -357,7 +329,7 @@ impl MessageBuilder {
     }
 
     /// Pushes a strikethrough inline text to the content.
-    pub fn push_strike<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_strike(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push_str("~~");
         self.0.push_str(&content.into().to_string());
         self.0.push_str("~~");
@@ -366,7 +338,7 @@ impl MessageBuilder {
     }
 
     /// Pushes a spoiler'd inline text to the content.
-    pub fn push_spoiler<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_spoiler(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push_str("||");
         self.0.push_str(&content.into().to_string());
         self.0.push_str("||");
@@ -375,7 +347,7 @@ impl MessageBuilder {
     }
 
     /// Pushes a quoted inline text to the content
-    pub fn push_quote<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_quote(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push_str("> ");
         self.0.push_str(&content.into().to_string());
 
@@ -395,7 +367,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "hello\nworld");
     /// ```
-    pub fn push_line<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_line(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push(content);
         self.0.push('\n');
 
@@ -415,7 +387,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "`hello`\nworld");
     /// ```
-    pub fn push_mono_line<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_mono_line(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_mono(content);
         self.0.push('\n');
 
@@ -435,7 +407,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "_hello_\nworld");
     /// ```
-    pub fn push_italic_line<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_italic_line(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_italic(content);
         self.0.push('\n');
 
@@ -455,7 +427,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "**hello**\nworld");
     /// ```
-    pub fn push_bold_line<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_bold_line(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_bold(content);
         self.0.push('\n');
 
@@ -475,7 +447,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "__hello__\nworld");
     /// ```
-    pub fn push_underline_line<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_underline_line(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_underline(content);
         self.0.push('\n');
 
@@ -495,7 +467,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "~~hello~~\nworld");
     /// ```
-    pub fn push_strike_line<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_strike_line(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_strike(content);
         self.0.push('\n');
 
@@ -515,7 +487,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "||hello||\nworld");
     /// ```
-    pub fn push_spoiler_line<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_spoiler_line(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_spoiler(content);
         self.0.push('\n');
 
@@ -535,7 +507,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "> hello\nworld");
     /// ```
-    pub fn push_quote_line<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_quote_line(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_quote(content);
         self.0.push('\n');
 
@@ -544,7 +516,7 @@ impl MessageBuilder {
 
     /// Pushes text to your message, but normalizing content - that means
     /// ensuring that there's no unwanted formatting, mention spam etc.
-    pub fn push_safe<C: I>(&mut self, content: C) -> &mut Self {
+    pub fn push_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         {
             let mut c = content.into();
             c.inner =
@@ -557,7 +529,11 @@ impl MessageBuilder {
     }
 
     /// Pushes a code-block to your message normalizing content.
-    pub fn push_codeblock_safe<D: I>(&mut self, content: D, language: Option<&str>) -> &mut Self {
+    pub fn push_codeblock_safe(
+        &mut self,
+        content: impl Into<Content>,
+        language: Option<&str>,
+    ) -> &mut Self {
         self.0.push_str("```");
 
         if let Some(language) = language {
@@ -576,7 +552,7 @@ impl MessageBuilder {
     }
 
     /// Pushes an inline monospaced text to the content normalizing content.
-    pub fn push_mono_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_mono_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push('`');
         {
             let mut c = content.into();
@@ -589,7 +565,7 @@ impl MessageBuilder {
     }
 
     /// Pushes an inline italicized text to the content normalizing content.
-    pub fn push_italic_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_italic_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push('_');
         {
             let mut c = content.into();
@@ -602,7 +578,7 @@ impl MessageBuilder {
     }
 
     /// Pushes an inline bold text to the content normalizing content.
-    pub fn push_bold_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_bold_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push_str("**");
         {
             let mut c = content.into();
@@ -615,7 +591,7 @@ impl MessageBuilder {
     }
 
     /// Pushes an underlined inline text to the content normalizing content.
-    pub fn push_underline_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_underline_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push_str("__");
         {
             let mut c = content.into();
@@ -628,7 +604,7 @@ impl MessageBuilder {
     }
 
     /// Pushes a strikethrough inline text to the content normalizing content.
-    pub fn push_strike_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_strike_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push_str("~~");
         {
             let mut c = content.into();
@@ -641,7 +617,7 @@ impl MessageBuilder {
     }
 
     /// Pushes a spoiler'd inline text to the content normalizing content.
-    pub fn push_spoiler_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_spoiler_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push_str("||");
         {
             let mut c = content.into();
@@ -654,7 +630,7 @@ impl MessageBuilder {
     }
 
     /// Pushes a quoted inline text to the content normalizing content.
-    pub fn push_quote_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_quote_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.0.push_str("> ");
         {
             let mut c = content.into();
@@ -679,7 +655,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "Hello @\u{200B}everyone\nHow are you?");
     /// ```
-    pub fn push_line_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_line_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_safe(content);
         self.0.push('\n');
 
@@ -700,7 +676,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "`'hello @\u{200B}everyone'`\nworld");
     /// ```
-    pub fn push_mono_line_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_mono_line_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_mono_safe(content);
         self.0.push('\n');
 
@@ -721,7 +697,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "_@\u{200B}everyone_\nIsn't a mention.");
     /// ```
-    pub fn push_italic_line_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_italic_line_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_italic_safe(content);
         self.0.push('\n');
 
@@ -742,7 +718,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "**@\u{200B}everyone**\nIsn't a mention.");
     /// ```
-    pub fn push_bold_line_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_bold_line_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_bold_safe(content);
         self.0.push('\n');
 
@@ -765,7 +741,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "__@\u{200B}everyone__\nIsn't a mention.");
     /// ```
-    pub fn push_underline_line_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_underline_line_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_underline_safe(content);
         self.0.push('\n');
 
@@ -787,7 +763,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "~~@\u{200B}everyone~~\nIsn't a mention.");
     /// ```
-    pub fn push_strike_line_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_strike_line_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_strike_safe(content);
         self.0.push('\n');
 
@@ -809,7 +785,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "||@\u{200B}everyone||\nIsn't a mention.");
     /// ```
-    pub fn push_spoiler_line_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_spoiler_line_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_spoiler_safe(content);
         self.0.push('\n');
 
@@ -831,7 +807,7 @@ impl MessageBuilder {
     ///
     /// assert_eq!(content, "> @\u{200B}everyone\nIsn't a mention.");
     /// ```
-    pub fn push_quote_line_safe<D: I>(&mut self, content: D) -> &mut Self {
+    pub fn push_quote_line_safe(&mut self, content: impl Into<Content>) -> &mut Self {
         self.push_quote_safe(content);
         self.0.push('\n');
 
@@ -856,10 +832,7 @@ impl MessageBuilder {
     /// [`Role`]: crate::model::guild::Role
     /// [Display implementation]: RoleId#impl-Display
     pub fn role<R: Into<RoleId>>(&mut self, role: R) -> &mut Self {
-        #[allow(clippy::let_underscore_must_use)]
-        let _ = write!(self.0, "{}", role.into().mention());
-        // should not error, ignoring
-
+        self.0.push_str(&role.into().mention().to_string());
         self
     }
 
@@ -874,15 +847,12 @@ impl MessageBuilder {
     /// [`User`]: crate::model::user::User
     /// [Display implementation]: UserId#impl-Display
     pub fn user<U: Into<UserId>>(&mut self, user: U) -> &mut Self {
-        #[allow(clippy::let_underscore_must_use)]
-        let _ = write!(self.0, "{}", user.into().mention());
-        // should not error, ignoring
-
+        self.0.push_str(&user.into().mention().to_string());
         self
     }
 }
 
-impl Display for MessageBuilder {
+impl fmt::Display for MessageBuilder {
     /// Formats the message builder into a string.
     ///
     /// This is done by simply taking the internal value of the tuple-struct and
@@ -895,6 +865,7 @@ impl Display for MessageBuilder {
     ///
     /// ```rust
     /// use serenity::utils::MessageBuilder;
+    /// ```
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         fmt::Display::fmt(&self.0, f)
     }
@@ -911,19 +882,13 @@ impl Display for MessageBuilder {
 /// Make a named link to Rust's GitHub organization:
 ///
 /// ```rust
-/// #[cfg(feature = "utils")]
-/// {
-///     use serenity::utils::{EmbedMessageBuilding, MessageBuilder};
+/// use serenity::utils::{EmbedMessageBuilding, MessageBuilder};
 ///
-///     let msg = MessageBuilder::new()
-///         .push_named_link("Rust's GitHub", "https://github.com/rust-lang")
-///         .build();
+/// let msg = MessageBuilder::new()
+///     .push_named_link("Rust's GitHub", "https://github.com/rust-lang")
+///     .build();
 ///
-///     assert_eq!(msg, "[Rust's GitHub](https://github.com/rust-lang)");
-/// }
-///
-/// #[cfg(not(feature = "utils"))]
-/// {}
+/// assert_eq!(msg, "[Rust's GitHub](https://github.com/rust-lang)");
 /// ```
 pub trait EmbedMessageBuilding {
     /// Pushes a named link to a message, intended for use in embeds.
@@ -933,22 +898,16 @@ pub trait EmbedMessageBuilding {
     /// Make a simple link to Rust's homepage for use in an embed:
     ///
     /// ```rust
-    /// #[cfg(feature = "utils")]
-    /// {
-    ///     use serenity::utils::{EmbedMessageBuilding, MessageBuilder};
+    /// use serenity::utils::{EmbedMessageBuilding, MessageBuilder};
     ///
-    ///     let mut msg = MessageBuilder::new();
-    ///     msg.push("Rust's website: ");
-    ///     msg.push_named_link("Homepage", "https://rust-lang.org");
-    ///     let content = msg.build();
+    /// let mut msg = MessageBuilder::new();
+    /// msg.push("Rust's website: ");
+    /// msg.push_named_link("Homepage", "https://rust-lang.org");
+    /// let content = msg.build();
     ///
-    ///     assert_eq!(content, "Rust's website: [Homepage](https://rust-lang.org)");
-    /// }
-    ///
-    /// #[cfg(not(feature = "utils"))]
-    /// {}
+    /// assert_eq!(content, "Rust's website: [Homepage](https://rust-lang.org)");
     /// ```
-    fn push_named_link<T: I, U: I>(&mut self, name: T, url: U) -> &mut Self;
+    fn push_named_link(&mut self, name: impl Into<Content>, url: impl Into<Content>) -> &mut Self;
 
     /// Pushes a named link intended for use in an embed, but with a normalized
     /// name to avoid escaping issues.
@@ -958,50 +917,48 @@ pub trait EmbedMessageBuilding {
     /// # Examples
     ///
     /// ```rust
-    /// #[cfg(feature = "utils")]
-    /// {
-    ///     use serenity::utils::{EmbedMessageBuilding, MessageBuilder};
+    /// use serenity::utils::{EmbedMessageBuilding, MessageBuilder};
     ///
-    ///     let mut msg = MessageBuilder::new();
-    ///     msg.push("A weird website name: ");
-    ///     msg.push_named_link_safe("Try to ] break links (](", "https://rust-lang.org");
-    ///     let content = msg.build();
+    /// let mut msg = MessageBuilder::new();
+    /// msg.push("A weird website name: ");
+    /// msg.push_named_link_safe("Try to ] break links (](", "https://rust-lang.org");
+    /// let content = msg.build();
     ///
-    ///     assert_eq!(
-    ///         content,
-    ///         "A weird website name: [Try to   break links ( (](https://rust-lang.org)"
-    ///     );
-    /// }
-    ///
-    /// #[cfg(not(feature = "utils"))]
-    /// {}
+    /// assert_eq!(content, "A weird website name: [Try to   break links ( (](https://rust-lang.org)");
     /// ```
-    fn push_named_link_safe<T: I, U: I>(&mut self, name: T, url: U) -> &mut Self;
+    fn push_named_link_safe(
+        &mut self,
+        name: impl Into<Content>,
+        url: impl Into<Content>,
+    ) -> &mut Self;
 }
 
 impl EmbedMessageBuilding for MessageBuilder {
-    fn push_named_link<T: I, U: I>(&mut self, name: T, url: U) -> &mut Self {
+    fn push_named_link(&mut self, name: impl Into<Content>, url: impl Into<Content>) -> &mut Self {
         let name = name.into().to_string();
         let url = url.into().to_string();
 
-        #[allow(clippy::let_underscore_must_use)]
-        let _ = write!(self.0, "[{}]({})", name, url);
+        self.0.push_str(&format!("[{}]({})", name, url));
         // error cannot be returned, ignoring instead
 
         self
     }
 
-    fn push_named_link_safe<T: I, U: I>(&mut self, name: T, url: U) -> &mut Self {
+    fn push_named_link_safe(
+        &mut self,
+        name: impl Into<Content>,
+        url: impl Into<Content>,
+    ) -> &mut Self {
         self.0.push('[');
         {
             let mut c = name.into();
-            c.inner = normalize(&c.inner).replace("]", " ");
+            c.inner = normalize(&c.inner).replace(']', " ");
             self.0.push_str(&c.to_string());
         }
         self.0.push_str("](");
         {
             let mut c = url.into();
-            c.inner = normalize(&c.inner).replace(")", " ");
+            c.inner = normalize(&c.inner).replace(')', " ");
             self.0.push_str(&c.to_string());
         }
         self.0.push(')');
@@ -1050,7 +1007,7 @@ impl<T: ToString> Add<T> for Content {
     type Output = Content;
 
     fn add(mut self, rhs: T) -> Content {
-        self.inner = self.inner + &rhs.to_string();
+        self.inner += &rhs.to_string();
 
         self
     }
@@ -1061,7 +1018,7 @@ impl<T: ToString> Add<T> for ContentModifier {
 
     fn add(self, rhs: T) -> Content {
         let mut nc = self.to_content();
-        nc.inner = nc.inner + &rhs.to_string();
+        nc.inner += &rhs.to_string();
 
         nc
     }
@@ -1122,6 +1079,7 @@ impl Content {
     }
 
     #[allow(clippy::inherent_to_string)]
+    #[must_use]
     pub fn to_string(&self) -> String {
         trait UnwrapWith {
             fn unwrap_with(&self, n: usize) -> usize;
@@ -1201,36 +1159,13 @@ impl Content {
     }
 }
 
-impl From<ContentModifier> for Content {
-    fn from(cm: ContentModifier) -> Content {
-        cm.to_content()
-    }
-}
-
-mod private {
-    use std::fmt;
-
-    use super::{Content, ContentModifier};
-
-    pub trait A {}
-
-    impl A for ContentModifier {}
-    impl A for Content {}
-    impl<T: fmt::Display> A for T {}
-}
-
-/// This trait exists for the purpose of bypassing the "conflicting implementations" error from the compiler.
-pub trait I: self::private::A {
-    fn into(self) -> Content;
-}
-
-impl<T: fmt::Display> I for T {
-    fn into(self) -> Content {
+impl<T: fmt::Display> From<T> for Content {
+    fn from(t: T) -> Content {
         Content {
             italic: false,
             bold: false,
             strikethrough: false,
-            inner: self.to_string(),
+            inner: t.to_string(),
             code: false,
             underline: false,
             spoiler: false,
@@ -1238,18 +1173,6 @@ impl<T: fmt::Display> I for T {
     }
 }
 
-impl I for ContentModifier {
-    fn into(self) -> Content {
-        self.to_content()
-    }
-}
-
-impl I for Content {
-    fn into(self) -> Content {
-        self
-    }
-}
-
 fn normalize(text: &str) -> String {
     // Remove invite links and popular scam websites, mostly to prevent the
     // current user from triggering various ad detectors and prevent embeds.
@@ -1274,10 +1197,8 @@ fn normalize(text: &str) -> String {
 
 #[cfg(test)]
 mod test {
-    use super::{
-        ContentModifier::{Bold, Code, Italic, Spoiler},
-        MessageBuilder,
-    };
+    use super::ContentModifier::{Bold, Code, Italic, Spoiler};
+    use super::MessageBuilder;
     use crate::model::prelude::*;
 
     macro_rules! gen {
